/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Authors: Jules Colding <colding@omesc.com>
 *
 *    Camel Brutus folder summary class.
 *
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* system includes */
#include <ctype.h>
#include <string.h>

/* Camel includes */
#include <camel/camel-i18n.h>
#include <camel/camel-operation.h>
#include <camel/camel-file-utils.h>
#include <camel/camel-mime-utils.h>
#include <camel/camel-string-utils.h>

#include <libedataserver/e-iconv.h>
#include <libedataserver/md5-utils.h>

/* camel-brutus includes */
#include "camel-brutus-folder-summary.h"
#include "camel-brutus-store.h"
#include "camel-brutus-folder.h"
#include "camel-brutus-private.h"

/* pure brutus includes */
#include <idl_output/IMAPIFolder.h>
#include <idl_output/return_codes.h>
#include <idl_output/types.h>

/* evolution-brutus includes */
#include <server/brutus_corba.h>
#include <server/brutus_mapi.h>
#include <server/brutus_smtp.h>
#include <server/brutus_cpid.h>
#include <server/brutus_util.h>
#include <server/brutus_conv.h>
#include <server/brutus.h>

#define CAMEL_BRUTUS_SUMMARY_VERSION (2)

static CamelFolderSummaryClass *camel_brutus_summary_parent;

// for the summary update thread pool
struct _folder_data {
	CamelStore *store;
	char *full_name;
};

/*
 * Will free all internally allocated memory from "info"
 */
static void
brutus_message_info_empty(CamelFolderSummary *s,
			  CamelMessageInfoBase *mi)
{
	g_async_queue_ref(wait_mem_queue);
	g_async_queue_push(wait_mem_queue, (gpointer)mi->uid);
	mi->uid = NULL;
	g_async_queue_unref(wait_mem_queue);

	camel_pstring_free(mi->subject);
	mi->subject = NULL;

	camel_pstring_free(mi->from);
	mi->from = NULL;

	camel_pstring_free(mi->to);
	mi->to = NULL;

	camel_pstring_free(mi->cc);
	mi->cc = NULL;

	camel_pstring_free(mi->mlist);
	mi->mlist = NULL;

	g_free(mi->references);
	mi->references = NULL;

	camel_flag_list_free(&mi->user_flags);

	camel_tag_list_free(&mi->user_tags);

	if (s && mi->content)
		camel_folder_summary_content_info_free(s, mi->content);
	mi->content = NULL;
}

/*
 * Adapted from "camel-folder-summary.c".
 * The return value is un-casted gchar* on pupose to detect any
 * difference between char and gchar at compile time. The original
 * e-d-s code is a mess between gchar returning functions and char
 * return values :-(
 */
static gchar*
brutus_camel_summary_format_string(struct _camel_header_raw *header,
				   const char *name,
				   const char *charset)
{
	const gchar *text;

	text = camel_header_raw_find(&header, name, NULL);
	if (!text)
		return NULL;

	while (isspace((int)*text))
		text++;

	return camel_header_decode_string(text, charset);
}

/*
 * Adapted from "camel-folder-summary.c"
 */
static gchar*
brutus_camel_summary_format_address(struct _camel_header_raw *header,
				    const char *name,
				    const char *charset)
{
	struct _camel_header_address *addr_list;
	const gchar *text;
	gchar *retv;

	text = camel_header_raw_find(&header, name, NULL);
	addr_list = camel_header_address_decode(text, charset);
	if (addr_list) {
		retv = camel_header_address_list_format(addr_list);
		camel_header_address_list_clear(&addr_list);
	} else {
		retv = g_strdup(text);
	}

	return retv;
}

static int
brutus_summary_header_load(CamelFolderSummary *s,
			   FILE *in)
{
	char *tmp = NULL;
	CamelBrutusSummary *cbs = CAMEL_BRUTUS_SUMMARY(s);

	if (-1 == camel_brutus_summary_parent->summary_header_load(s, in))
		return -1;

	if (-1 == camel_file_util_decode_uint32(in, &cbs->version))
		return -1;

	if (1 == cbs->version)
		return 0; // no more to load

	/*
	 * version 2 or later...
	 */

	// flags
	if (-1 == camel_file_util_decode_uint32(in, &cbs->flags))
		return -1;

	// get session key
	if (-1 == camel_file_util_decode_string(in, &tmp))
		return -1;

	memcpy((void*)cbs->session_key, (const void*)tmp, sizeof(cbs->session_key));
	g_free(tmp);

	if (2 == cbs->version)
		return 0; // no more to load

	return -1;
}

static int
brutus_summary_header_save(CamelFolderSummary *s,
			   FILE *out)
{
	CamelBrutusSummary *cbs = CAMEL_BRUTUS_SUMMARY(s);

	if (-1 == camel_brutus_summary_parent->summary_header_save(s, out))
		return -1;

	if (-1 == camel_file_util_encode_uint32(out, CAMEL_BRUTUS_SUMMARY_VERSION))
		return -1;

	if (-1 == camel_file_util_encode_uint32(out, cbs->flags))
		return -1;

	if (-1 == camel_file_util_encode_string(out, cbs->session_key))
		return -1;

	return 0;
}

static CamelMessageInfo *
brutus_message_info_clone(CamelFolderSummary *s,
			  const CamelMessageInfo *mi)
{
	CamelBrutusMessageInfo *to;
	const CamelBrutusMessageInfo *from = (CamelBrutusMessageInfo*)mi;

	to = (CamelBrutusMessageInfo*)camel_brutus_summary_parent->message_info_clone(s, mi);
	to->pr_record_key = g_strdup(from->pr_record_key);

	return (CamelMessageInfo*)to;
}

static CamelMessageInfo*
brutus_message_info_load(CamelFolderSummary *s,
			 FILE *in)
{
	CamelMessageInfo *info;
	CamelBrutusMessageInfo *brutus_info;

	info = camel_brutus_summary_parent->message_info_load(s, in);
	if (!info)
		goto err;

	brutus_info = (CamelBrutusMessageInfo*)info;
	brutus_info->pr_record_key = NULL;

	if (-1 == camel_file_util_decode_string(in, &brutus_info->pr_record_key))
		goto err;

	return info;

err:
	if (info)
		camel_message_info_free(info);

	return NULL;
}

static int
brutus_message_info_save(CamelFolderSummary *s,
			 FILE *out,
			 CamelMessageInfo *info)
{
	CamelBrutusMessageInfo *brutus_info = (CamelBrutusMessageInfo*)info;

	if (-1 == camel_brutus_summary_parent->message_info_save(s, out, info))
		return -1;

	return camel_file_util_encode_string(out, brutus_info->pr_record_key);
}

static void
brutus_message_info_free(CamelFolderSummary *s,
			 CamelMessageInfo *mi)
{
	CamelBrutusMessageInfo *bi = (CamelBrutusMessageInfo *)mi;

	g_free(bi->pr_record_key);
	((CamelFolderSummaryClass*)camel_brutus_summary_parent)->message_info_free(s, mi);
}

/* static gboolean brutus_info_set_flags(CamelMessageInfo *mi,  */
/*				      guint32 mask,  */
/*				      guint32 set); */

static void
camel_brutus_summary_class_init(CamelBrutusSummaryClass *klass)
{
	CamelFolderSummaryClass *parent_class = (CamelFolderSummaryClass*)klass;

	camel_brutus_summary_parent = CAMEL_FOLDER_SUMMARY_CLASS(camel_type_get_global_classfuncs(camel_folder_summary_get_type()));

	parent_class->message_info_clone = brutus_message_info_clone ;
	parent_class->summary_header_load = brutus_summary_header_load;
	parent_class->summary_header_save = brutus_summary_header_save;
	parent_class->message_info_load = brutus_message_info_load;
	parent_class->message_info_save = brutus_message_info_save;
	parent_class->message_info_free = brutus_message_info_free;
/*	parent_class->info_set_flags = brutus_info_set_flags;  */
/*	parent_class->content_info_load = brutus_content_info_load; */
/*	parent_class->content_info_save = brutus_content_info_save; */
}

static void
camel_brutus_summary_init(CamelBrutusSummary *brutus_summary)
{
	CamelFolderSummary *summary = (CamelFolderSummary *)brutus_summary;

	/* subclasses need to set the right instance data sizes */
	summary->message_info_size = sizeof(CamelBrutusMessageInfo);
	summary->content_info_size = 0; //sizeof(CamelBrutusMessageContentInfo);
}

CamelFolderSummary*
camel_brutus_summary_new(CamelFolder *folder,
			 const char *summary_filename)
{
	CamelFolderSummary *summary = CAMEL_FOLDER_SUMMARY(camel_object_new(camel_brutus_summary_get_type()));

	summary->flags = 0;
	summary->folder = folder ;
	camel_folder_summary_set_build_content(summary, FALSE);
	camel_folder_summary_set_filename(summary, summary_filename);

	// create the file if it doesn't exist
	if (-1 == camel_folder_summary_load(summary)) {
		camel_folder_summary_clear(summary);
		camel_folder_summary_touch(summary);
	}

	return summary;
}

/*
 * This function will walk the folder summary and update all message
 * infos build during a previous run of brutus_update_folder_summary_light()
 */
void
brutus_update_folder_summary_post_light(gpointer data,
					gpointer static_data)
{
	CamelException *ex = camel_exception_new();
	struct _folder_data *folder_data = (struct _folder_data*)data;
	CamelStore *parent_store = folder_data->store;
	char *full_name = folder_data->full_name;
	CamelFolder *folder = NULL;
	CamelBrutusFolder *brutus_folder = NULL;
	CamelBrutusSummary *cbs = NULL;
	CamelBrutusMessageInfo *msg_info_ptr = NULL;
	CamelBrutusMessageInfo *msg_info = NULL;
	CamelFolderChangeInfo *changes = NULL;
	GPtrArray *summary = NULL;
	struct _camel_header_raw *camel_header_raw = NULL;
	struct _camel_header_address *camel_addr_list = NULL;
	struct _camel_header_address *camel_addr = NULL;
	struct _camel_header_references *refs = NULL;
	struct _camel_header_references *irt = NULL;
	struct _camel_header_references *scan = NULL;
	CamelContentType *ct = NULL;
	const char *content = NULL;
	const char *charset = NULL;
	gboolean errors = TRUE;
	gboolean inbound = FALSE;
	gboolean aborted = FALSE;
	char *msg_id = NULL;
	char *smtp_header = NULL;
	char *tmp = NULL;
	guchar digest[16] = { 0 };
	int count = 0;
	CORBA_long n = 0;
	CORBA_unsigned_long r = 0;
	CORBA_unsigned_long k = 0;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_ENTRYID *sender_eid = NULL;
	BRUTUS_SRowSet *recp_row_set = NULL;
	BRUTUS_SPropValue *pv = NULL;
	BRUTUS_seq_SPropValue *prop_values = NULL;
	BRUTUS_seq_SPropValue *ex_prop_values = NULL;
	CORBA_Environment ev[1];
	static BRUTUS_BDEFINE recp_props[3] = {               // for an recipients table
		/* 0 */ BRUTUS_BRUTUS_PR_RECIPIENT_TYPE,             // MAPI_TO, MAPI_CC, MAPI_BCC
		/* 1 */ BRUTUS_BRUTUS_PR_EMAIL_ADDRESS,              // if not there use PR_DISPLAY_NAME
		/* 2 */ BRUTUS_BRUTUS_PR_DISPLAY_NAME,               // display name, possibly email adress if recipient does not resolve in the Exchange address book
	};
	static BRUTUS_SPropTagArray recp_proptag_array = {    // for an recipients table
		._maximum = 3,
		._length = 3,
		._buffer = recp_props,
	};
	static BRUTUS_BDEFINE msg_props[14] = {               // for a message object
		/* 0  */ BRUTUS_BRUTUS_PR_ENTRYID,                   // long-term entryid
		/* 1  */ BRUTUS_BRUTUS_PR_DISPLAY_CC,                // display name of CC recipients
		/* 2  */ BRUTUS_BRUTUS_PR_SENDER_EMAIL_ADDRESS,      // email address of sender
		/* 3  */ BRUTUS_BRUTUS_PR_TRANSPORT_MESSAGE_HEADERS, // smtp envelope of inbound messages
		/* 4  */ BRUTUS_BRUTUS_PR_INTERNET_CPID,             // Equivalent to RFC 2822 "Content-Type".
		/* 5  */ BRUTUS_BRUTUS_PR_SENDER_ADDRTYPE,           // MHS, PROFS, SMTP, X400 or EX
		/* 6  */ BRUTUS_BRUTUS_PR_CLIENT_SUBMIT_TIME,        // Equivalent to RFC 2822 "Date:" when inbound
		/* 7  */ BRUTUS_BRUTUS_PR_PROVIDER_SUBMIT_TIME,      // Equivalent to RFC 2822 "Date:" when outbound
		/* 8  */ BRUTUS_BRUTUS_PR_INTERNET_MESSAGE_ID,       // Equivalent to RFC 2822 "Message-Id:"
		/* 9  */ BRUTUS_BRUTUS_PR_INTERNET_REFERENCES,       // Equivalent to RFC 2822 "References:" (rarely seen)
		/* 10 */ BRUTUS_BRUTUS_PR_IN_REPLY_TO_ID,            // Equivalent to RFC 2822 "In-Reply-To:"
		/* 11 */ BRUTUS_BRUTUS_PR_RECEIVED_BY_ADDRTYPE,      // inbound if not found
		/* 12 */ BRUTUS_BRUTUS_PR_SENDER_ENTRYID,            // open and get PR_SMTP_ADDRESS if PR_SENDER_ADDRTYPE==EX
		/* 13 */ BRUTUS_BRUTUS_PR_SUBJECT,                   // subject
		/* 14 *//*  BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray msg_proptag_array = {     // for message object properties
		._maximum = 14,
		._length = 14,
		._buffer = msg_props,
	};
	static BRUTUS_SPropTagArray ex_msg_proptag_array = {  // for message object properties
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};

	folder = camel_brutus_folder_new(parent_store, full_name, 0, ex);
	camel_object_unref(CAMEL_OBJECT(parent_store));
	parent_store = NULL;
	g_free(full_name);
	full_name = NULL;
	g_free(folder_data);
	folder_data = NULL;
	if (camel_exception_is_set(ex)) {
		d("Could not create camel folder");
		camel_exception_free(ex);
		return;
	}
	camel_exception_free(ex);
	brutus_folder = CAMEL_BRUTUS_FOLDER(folder);

	BRUTUS_FOLDER_LOCK(brutus_folder, summary_mutex);

	d(folder->full_name);

	CORBA_exception_init(ev);

	if (!brutus_is_connected(brutus_folder->brutus_server,
				 brutus_folder->mapi_session))
		goto out_err;

	summary = camel_folder_get_summary(folder);
	if (!summary)
		goto out;

	// iterate over summary content (columns)
	dfmt2("Number of messages to parse in \"%s\" = %llu", folder->full_name, (long long unsigned int)summary->len);
	for (r = 0; r < summary->len; r++) {
		if (!brutus_is_connected(brutus_folder->brutus_server,
					 brutus_folder->mapi_session)) { // kill the thread
			aborted = TRUE;
			goto out_err; // force a later update to commence from the dawn of times
		}

		changes = camel_folder_change_info_new();
		if (!changes)
			goto out_err;

		if (!folder_summary_update_is_a_go(folder->full_name)) {
			aborted = TRUE; // due to a delete that is underway
			break;
		}

		// to catch goto's
		errors = TRUE;

		// get the cached message info pointer to retrieve the uid
		msg_info_ptr = g_ptr_array_index(summary, r);

		// get the real message info
		msg_info = (CamelBrutusMessageInfo*)camel_folder_summary_uid(folder->summary, msg_info_ptr->info.uid);
		msg_info_ptr = NULL;

		// get the uid
		eid = brutus_string_to_entryid(msg_info->info.uid);

		// get properties from message object
		br = BRUTUS_IMAPISession_QueryMessage(brutus_folder->mapi_session,
						      eid,
						      &msg_proptag_array,
						      &ex_msg_proptag_array,
						      &recp_proptag_array,
						      &prop_values,
						      &ex_prop_values,
						      &recp_row_set,
						      ev);
		CORBA_free(ex_prop_values); // we are not asking for any additional properties
		ex_prop_values = NULL;
		CORBA_free(eid);
		eid = NULL;
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto next;
		}
		if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
			d(brutus_bresult_to_str(br));
			goto next;
		}

		// inbound or outbound?
		if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[11].ulPropTag)) // PR_RECEIVED_BY_ADDRTYPE
			inbound = TRUE;
		else
			inbound = FALSE; // draft, sent or being edited

		// get full SMTP header if present, alternatively check MSGFLAG_ORIGIN_INTERNET first
		if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[3].ulPropTag)) { // PR_TRANSPORT_MESSAGE_HEADERS
			smtp_header = brutus_unfold_rfc822_header(prop_values->_buffer[3].Value._u.lpszA);
			if (smtp_header) {
				camel_header_raw = brutus_get_camel_header_raw(smtp_header);
				free(smtp_header);
				smtp_header = NULL;
			} else
				goto next;
		} else
			camel_header_raw = NULL;

		// get long-term ENTRYID
		brutus_sbinary_to_entryid(&prop_values->_buffer[0].Value._u.bin, &eid);
		if (!eid)
			goto next;

		msg_id = brutus_entryid_to_string(eid);
		CORBA_free(eid);
		eid = NULL;
		if (!msg_id)
			goto next;

		// charset and content type
		ct = NULL;
		content = NULL;
		charset = NULL;
		if (camel_header_raw) {
			if ((content = camel_header_raw_find(&camel_header_raw, "Content-Type", NULL))
			    && (ct = camel_content_type_decode(content))
			    && (charset = camel_content_type_param(ct, "charset"))
			    && (!g_ascii_strcasecmp(charset, "us-ascii")))
				charset = NULL;
		} else {
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[11].ulPropTag)) // PR_INTERNET_CPID
				charset = brutus_cpid_to_label(prop_values->_buffer[11].Value._u.l);
		}
		charset = charset ? e_iconv_charset_name(charset) : NULL;
		camel_content_type_unref(ct);

		// push the uid to the delayed free queue
		g_async_queue_ref(wait_mem_queue);
		g_async_queue_push(wait_mem_queue, (gpointer)msg_info->info.uid);
		msg_info->info.uid = msg_id;
		msg_id = NULL;
		g_async_queue_unref(wait_mem_queue);

		// Message-Id:
		if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[8].ulPropTag)) // PR_INTERNET_MESSAGE_ID
			msg_id = camel_header_msgid_decode(prop_values->_buffer[8].Value._u.lpszA);
		else if (camel_header_raw)
			msg_id = camel_header_msgid_decode(camel_header_raw_find(&camel_header_raw, "Message-Id", NULL));
		else {
			d("Invalid RFC 822 message header");
#ifndef BRUTUS_SPY
			goto rollback;
#endif
		}
		if (!msg_id) {
			d("No memory or invalid RFC 822 message header");
#ifdef BRUTUS_SPY
			memset(&msg_info->info.message_id, 0, sizeof(CamelSummaryMessageID));
#else
			goto rollback;
#endif
		} else {
			md5_get_digest(msg_id, strlen(msg_id), digest);
			memcpy(msg_info->info.message_id.id.hash, digest, sizeof(msg_info->info.message_id.id.hash));
			g_free(msg_id);
			msg_id = NULL;
		}

		// Subject:
		tmp = NULL;
		camel_pstring_free(msg_info->info.subject);
		msg_info->info.subject = NULL;
		if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[13].ulPropTag))
			tmp = camel_header_decode_string(prop_values->_buffer[13].Value._u.lpszA, charset);
		else if (camel_header_raw)
			tmp = brutus_camel_summary_format_string(camel_header_raw, "Subject", charset);
		if (tmp) {
			msg_info->info.subject = camel_pstring_strdup(tmp);
			g_free(tmp);
			tmp = NULL;
		}

		/*
		 * Recipients. The recipient is BCC if neither "To:" nor "CC:" is present
		 * in the recipient table.
		 */

		// To:
		camel_header_address_list_clear(&camel_addr_list);
		if (camel_header_raw) {
			tmp = brutus_camel_summary_format_address(camel_header_raw, "To", charset);
		} else if ((recp_row_set->_length) && (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(recp_row_set->_buffer[0]._buffer[0].ulPropTag))) { // we don't know if STORE_SUBMIT_OK is set
			for (k = 0; k<recp_row_set->_length; k++) { // loop all recipients
				if (BRUTUS_BRUTUS_MAPI_TO & recp_row_set->_buffer[k]._buffer[0].Value._u.l) {
					n = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_EMAIL_ADDRESS);
					if (-1 == n) // no PR_EMAIL_ADDRESS
						n = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_DISPLAY_NAME);

					camel_addr = camel_header_address_decode(recp_row_set->_buffer[k]._buffer[n].Value._u.lpszA, charset);
					camel_header_address_list_append(&camel_addr_list, camel_addr);
				}
			}
			if (camel_addr_list) {
				tmp = camel_header_address_list_format(camel_addr_list);
				camel_header_address_list_clear(&camel_addr_list);
			}
		}
		if (tmp) {
			camel_pstring_free(msg_info->info.to);
			msg_info->info.to = camel_pstring_strdup(tmp);
			g_free(tmp);
			tmp = NULL;
		}

		// CC:
		camel_header_address_list_clear(&camel_addr_list);
		camel_pstring_free(msg_info->info.cc);
		msg_info->info.cc = NULL;
		if (camel_header_raw) {
			tmp = brutus_camel_summary_format_address(camel_header_raw, "CC", charset);
		} else if ((recp_row_set->_length) && (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(recp_row_set->_buffer[0]._buffer[0].ulPropTag))) { // we don't know if STORE_SUBMIT_OK is set
			for (k = 0; k<recp_row_set->_length; k++) { // loop all recipients
				if (BRUTUS_BRUTUS_MAPI_CC & recp_row_set->_buffer[k]._buffer[0].Value._u.l) {
					n = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_EMAIL_ADDRESS);
					if (-1 == n) // no PR_EMAIL_ADDRESS
						n = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_DISPLAY_NAME);

					camel_addr = camel_header_address_decode(recp_row_set->_buffer[k]._buffer[n].Value._u.lpszA, charset);
					camel_header_address_list_append(&camel_addr_list, camel_addr);
				}
			}
			if (camel_addr_list) {
				tmp = camel_header_address_list_format(camel_addr_list);
				camel_header_address_list_clear(&camel_addr_list);
			}
		}
		if (tmp) {
			msg_info->info.to = camel_pstring_strdup(tmp);
			g_free(tmp);
			tmp = NULL;
		}

		// From:
		camel_header_address_list_clear(&camel_addr_list);
		if (camel_header_raw) {
			tmp = brutus_camel_summary_format_address(camel_header_raw, "From", charset);
		} else { // check for sender addrtype: SMTP=>Get SENDER_EMAIL_ADDRESS. EX=>Open sender and get PR_SMTP_ADDRESS.
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[5].ulPropTag)) { // PR_SENDER_ADDRTYPE
				if (!g_ascii_strcasecmp("SMTP", prop_values->_buffer[5].Value._u.lpszA)) {
					if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[2].ulPropTag)) { // PR_SENDER_EMAIL_ADDRESS
						camel_addr_list = camel_header_address_decode(prop_values->_buffer[2].Value._u.lpszA, charset);
						tmp = camel_header_address_list_format(camel_addr_list);
						camel_header_address_list_clear(&camel_addr_list);
					}
				} else if (!g_ascii_strcasecmp("EX", prop_values->_buffer[5].Value._u.lpszA)) {
					if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[12].ulPropTag)) { // PR_SENDER_ENTRYID
						brutus_sbinary_to_entryid(&prop_values->_buffer[12].Value._u.bin, &sender_eid);
						if (sender_eid) {
							pv = brutus_open_object_get_one_prop(brutus_folder->mapi_session,
											     CORBA_OBJECT_NIL,
											     sender_eid,
											     BRUTUS_BRUTUS_PR_SMTP_ADDRESS);
							if ((pv) && (BRUTUS_BRUTUS_PR_SMTP_ADDRESS == pv->ulPropTag)) {
								camel_addr_list = camel_header_address_decode(pv->Value._u.lpszA, charset);
								tmp = camel_header_address_list_format(camel_addr_list);
								camel_header_address_list_clear(&camel_addr_list);
							}
							CORBA_free(pv);
							pv = NULL;
							CORBA_free(sender_eid);
							sender_eid = NULL;
						}
					}
				}
			} else { // unknown address type
				if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[2].ulPropTag)) { // PR_SENDER_EMAIL_ADDRESS
					camel_addr_list = camel_header_address_decode(prop_values->_buffer[7].Value._u.lpszA, charset);
					tmp = camel_header_address_list_format(camel_addr_list);
					camel_header_address_list_clear(&camel_addr_list);
				}
			}
		}
		if (tmp) {
			camel_pstring_free(msg_info->info.from);
			msg_info->info.from = camel_pstring_strdup(tmp);
			g_free(tmp);
			tmp = NULL;
		}

		// Received/Sent:
		if (inbound) {
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[6].ulPropTag)) // PR_CLIENT_SUBMIT_TIME
				msg_info->info.date_sent = brutus_filetime_to_time_t(prop_values->_buffer[6].Value._u.ft);
			else if (camel_header_raw)
				msg_info->info.date_sent = camel_header_decode_date(camel_header_raw_find(&camel_header_raw, "Date", NULL), NULL);
			else
				msg_info->info.date_sent = 0;
		} else {
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[7].ulPropTag))        // PR_PROVIDER_SUBMIT_TIME
				msg_info->info.date_sent = brutus_filetime_to_time_t(prop_values->_buffer[7].Value._u.ft);
			else if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[6].ulPropTag))   // PR_CLIENT_SUBMIT_TIME
				msg_info->info.date_sent = brutus_filetime_to_time_t(prop_values->_buffer[6].Value._u.ft);
			else
				msg_info->info.date_sent = msg_info->info.date_received;
		}

		// Mailing list:
		msg_info->info.mlist = NULL;
		if (camel_header_raw) {
			tmp = camel_header_raw_check_mailing_list(&camel_header_raw);
			if (tmp) {
				msg_info->info.mlist = camel_pstring_strdup(tmp);
				g_free(tmp);
				tmp = NULL;
			}
		}

		// References:
		if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[9].ulPropTag)) // PR_INTERNET_REFERENCES
			refs = camel_header_references_decode(prop_values->_buffer[9].Value._u.lpszA);
		else if (camel_header_raw)
			refs = camel_header_references_decode(camel_header_raw_find(&camel_header_raw, "References", NULL));
		else
			refs = NULL;
		if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[10].ulPropTag)) // PR_IN_REPLY_TO_ID
			irt = camel_header_references_inreplyto_decode(prop_values->_buffer[10].Value._u.lpszA);
		else if (camel_header_raw)
			irt = camel_header_references_inreplyto_decode(camel_header_raw_find(&camel_header_raw, "In-Reply-To", NULL));
		else
			irt = NULL;
		if (refs || irt) { // straight from camel-folder-summary.c
			if (irt) {
				/* The References field is populated from the ``References'' and/or ``In-Reply-To''
				   headers. If both headers exist, take the first thing in the In-Reply-To header
				   that looks like a Message-ID, and append it to the References header. */
				if (refs)
					irt->next = refs;

				refs = irt;
			}

			count = camel_header_references_list_size(&refs);
			msg_info->info.references = g_malloc(sizeof(*msg_info->info.references) + ((count-1) * sizeof(msg_info->info.references->references[0])));
			count = 0;
			scan = refs;
			while (scan) {
				md5_get_digest(scan->id, strlen(scan->id), digest);
				memcpy(msg_info->info.references->references[count].id.hash, digest, sizeof(msg_info->info.message_id.id.hash));
				count++;
				scan = scan->next;
			}
			msg_info->info.references->size = count;
			camel_header_references_list_clear(&refs);
		}

		// all is well
		errors = FALSE;

		goto next;

	rollback:
		d("*************  ROLLBACK * ROLLBACK * ROLLBACK  **************");

		// msg_id msg_info->info.uid
		if (msg_id) {
			camel_folder_summary_remove_uid(folder->summary, msg_id);
			free(msg_id);
			msg_id = NULL;
		} else
			camel_folder_summary_remove_uid(folder->summary, camel_message_info_uid(&msg_info->info));
	next:
		camel_header_raw_clear(&camel_header_raw);

		CORBA_free(eid);
		eid = NULL;

		CORBA_free(prop_values);
		prop_values = NULL;

		CORBA_free(recp_row_set);
		recp_row_set = NULL;

		if (!errors) {
			camel_folder_change_info_change_uid(changes, camel_message_info_uid(&msg_info->info));
			camel_message_info_free(&msg_info->info);  // unref

			if (changes && camel_folder_change_info_changed(changes)) {
				camel_folder_summary_touch(folder->summary);
				camel_folder_summary_save(folder->summary);

				camel_object_trigger_event(folder, "folder_changed", changes);
			}
		}
		if (changes) {
			camel_folder_change_info_free(changes);
			changes = NULL;
		}
	}
	goto out;

out_err:
	folder->summary->time = 0;

out:
	CORBA_exception_free(ev);

	if (summary)
		camel_folder_free_summary(folder, summary);

	if (!aborted) {
		cbs = CAMEL_BRUTUS_SUMMARY(folder->summary);
		cbs->flags |= BRUTUS_FOLDER_SUMMARY_HAS_LONG_TERM_ENTRYIDS;
		if (((CamelOfflineFolder*)folder)->sync_offline)
			cbs->flags |= BRUTUS_FOLDER_SYNC_OFFLINE;
		else
			cbs->flags &= ~BRUTUS_FOLDER_SYNC_OFFLINE;

		camel_folder_summary_touch(folder->summary);
		camel_folder_summary_save(folder->summary);
	}
	if (changes) // aborted
		camel_folder_change_info_free(changes);

	BRUTUS_FOLDER_UNLOCK(brutus_folder, summary_mutex);

	folder_is_done(folder->full_name);
	camel_object_unref(CAMEL_OBJECT(folder));
}

/*
 * This function is only invoked if the summary count is 0 (zero).
 */
static void
brutus_update_folder_summary_light(CamelFolder *folder,
				   CamelException *ex)
{
	struct _folder_data *folder_data = NULL;
	CamelBrutusFolder *brutus_folder = CAMEL_BRUTUS_FOLDER(folder);
	CamelBrutusSummary *cbs = NULL;
	CamelBrutusMessageInfo *msg_info = NULL;
	CamelFolderChangeInfo *changes = NULL;
	gboolean errors = TRUE;
	gboolean do_break = FALSE;
	char *msg_id = NULL;
	char *tmp = NULL;
	CORBA_long n = 0;
	CORBA_unsigned_long r = 0;
	CORBA_unsigned_long processed = 0;
	CORBA_unsigned_long row_count = 0;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_IMAPITable content_table = CORBA_OBJECT_NIL;    // folder content
	BRUTUS_SRowSet *row_set = NULL;
	CORBA_Environment ev[1];
	static BRUTUS_BDEFINE content_props[11] = {            // for the contents table
		/* 0  */ BRUTUS_BRUTUS_PR_ENTRYID,                     // short-term entryid
		/* 1  */ BRUTUS_BRUTUS_PR_RECORD_KEY,                  // provider unique ID
		/* 2  */ BRUTUS_BRUTUS_PR_DISPLAY_TO,                  //
		/* 3  */ BRUTUS_BRUTUS_PR_SENDER_NAME,                 //
		/* 4  */ BRUTUS_BRUTUS_PR_SUBJECT,                     //
		/* 5  */ BRUTUS_BRUTUS_PR_MESSAGE_DELIVERY_TIME,       //
		/* 6  */ BRUTUS_BRUTUS_PR_MSG_STATUS,                  // message status
		/* 7  */ BRUTUS_BRUTUS_PR_MESSAGE_FLAGS,               //
		/* 8  */ BRUTUS_BRUTUS_PR_SENSITIVITY,                 //
		/* 9  */ BRUTUS_BRUTUS_PR_IMPORTANCE,                  //
		/* 10 */ BRUTUS_BRUTUS_PR_MESSAGE_SIZE,                //
	};
	static BRUTUS_SPropTagArray content_proptag_array = {  // for the contents table
		._maximum = 11,
		._length = 11,
		._buffer = content_props,
	};
	static BRUTUS_SSortOrderSet sort_order = {             // for the contents table
		.cCategories = 0,
		.cExpanded = 0,
		.aSort._maximum = 0,
		.aSort._length = 0,
		.aSort._buffer = NULL,
	};

	CORBA_exception_init(ev);

	d(folder->full_name);

	camel_operation_start(NULL, _("Fetching summary information for %s"), folder->name);
	camel_operation_progress(NULL, 0);

	folder->summary->time = brutus_get_server_utc_time(brutus_folder->brutus_server, 
							   brutus_folder->mapi_session, 
							   ((CamelService*)folder->parent_store)->url->host);

	// get contents table
	br = BRUTUS_IMAPIFolder_GetContentsTable(brutus_folder->mapi_folder,
						 0,
						 &content_table,
						 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out_err;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		d("Possibly this error: http://support.microsoft.com/default.aspx?scid=kb\%3Ben-us\%3B176001");
		goto out_err;
	}

	br = BRUTUS_IMAPITable_GetRowCount(content_table,
					   0,
					   &row_count,
					   ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out_err;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out_err;
	}
	if (!row_count)
		goto out;

	br = brutus_PrepareTable(content_table,
				 &content_proptag_array,
				 CORBA_OBJECT_NIL,
				 &sort_order,
				 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out_err;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out_err;
	}

	// iterate over content (columns)
	dfmt2("Number of messages to parse in \"%s\" = %llu", folder->full_name, (long long unsigned int)row_count);
	do {
		changes = camel_folder_change_info_new();
		if (!changes)
			goto out_err;

		br = brutus_QuerySomeRows(content_table,
					  (CORBA_long)BRUTUS_IMAPITABLE_ROW_COUNT_MAX,
					  &row_set,
					  ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_err;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			goto out_err;
		}

		for (r = 0; r < row_set->_length; r++) {

			// progress bar
			camel_operation_progress(NULL, (100*processed)/row_count);
			processed++;

			// to catch goto's
			errors = TRUE;

			// get short-term ENTRYID
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_ENTRYID);
			if (-1 == n) {
				d("No ENTRYID");
				continue;
			}
			brutus_sbinary_to_entryid(&row_set->_buffer[r]._buffer[n].Value._u.bin, &eid);
			if (!eid)
				continue;

			// create message id string
			msg_id = brutus_entryid_to_string(eid);
			CORBA_free(eid);
			eid = NULL;
			if (!msg_id)
				goto next;

			// create message info object
			msg_info = (CamelBrutusMessageInfo*)camel_message_info_new(folder->summary);
			msg_info->pr_record_key = NULL;

			// set id
			msg_info->info.uid = g_strdup(msg_id);
			free(msg_id);
			if (!msg_info->info.uid)
				goto rollback;

			// set record_id
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_RECORD_KEY);
			if (-1 != n)
				msg_info->pr_record_key = brutus_print_spropvalue(&row_set->_buffer[r]._buffer[n]);
			else {
				d("No PR_RECORD_KEY");
				goto rollback;
			}

			// Message flags
			msg_info->info.flags = 0;
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_MSG_STATUS);
			if (-1 != n) {
				if (BRUTUS_BRUTUS_MSGSTATUS_ANSWERED & row_set->_buffer[r]._buffer[n].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_ANSWERED;

				if (BRUTUS_BRUTUS_MSGSTATUS_DELMARKED & row_set->_buffer[r]._buffer[n].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_DELETED;

				if (BRUTUS_BRUTUS_MSGSTATUS_DRAFT & row_set->_buffer[r]._buffer[n].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_DRAFT;

				if (BRUTUS_BRUTUS_MSGSTATUS_HIGHLIGHTED & row_set->_buffer[r]._buffer[n].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_FLAGGED;
			}
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_MESSAGE_FLAGS);
			if (-1 != n) {
				if (BRUTUS_BRUTUS_MSGFLAG_READ & row_set->_buffer[r]._buffer[n].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_SEEN;

				if (BRUTUS_BRUTUS_MSGFLAG_HASATTACH & row_set->_buffer[r]._buffer[n].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_ATTACHMENTS;
			}

			// Subject:
			tmp = NULL;
			msg_info->info.subject = NULL;
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_SUBJECT);
			if (-1 != n)
				tmp = camel_header_decode_string(row_set->_buffer[r]._buffer[n].Value._u.lpszA, NULL);
			if (tmp) {
				msg_info->info.subject = camel_pstring_strdup(tmp);
				g_free(tmp);
				tmp = NULL;
			}

			/*
			 * Recipients. The recipient is BCC if neither "To:" nor "CC:" is present
			 * in the recipient table.
			 */

			// To:
			tmp = NULL;
			msg_info->info.to = NULL;
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_DISPLAY_TO);
			if (-1 != n)
				tmp = camel_header_decode_string(row_set->_buffer[r]._buffer[n].Value._u.lpszA, NULL);
			if (tmp) {
				msg_info->info.to = camel_pstring_strdup(tmp);
				g_free(tmp);
				tmp = NULL;
			}

			// From:
			tmp = NULL;
			msg_info->info.from = NULL;
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_SENDER_NAME);
			if (-1 != n)
				tmp = camel_header_decode_string(row_set->_buffer[r]._buffer[n].Value._u.lpszA, NULL);
			if (tmp) {
				msg_info->info.from = camel_pstring_strdup(tmp);
				g_free(tmp);
				tmp = NULL;
			}

			// Received/Sent:
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_MESSAGE_DELIVERY_TIME);
			if (-1 != n) {
				msg_info->info.date_received = brutus_filetime_to_time_t(row_set->_buffer[r]._buffer[n].Value._u.ft);
				msg_info->info.date_sent = msg_info->info.date_received;
			} else {
				msg_info->info.date_received = 0;
				msg_info->info.date_sent = 0;
			}

			// Size:
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_MESSAGE_SIZE);
			if (-1 != n)
				msg_info->info.size = row_set->_buffer[r]._buffer[n].Value._u.l;
			else
				msg_info->info.size = 0;

			// all is well
			errors = FALSE;
			goto next;

		rollback:
			d("*************  ROLLBACK * ROLLBACK * ROLLBACK  **************");

			camel_message_info_free(&msg_info->info);

		next:
			if (!errors) {
				camel_folder_summary_add(folder->summary,(CamelMessageInfo*)msg_info);
				camel_folder_change_info_add_uid(changes, camel_message_info_uid(&msg_info->info));
				camel_folder_change_info_recent_uid(changes, camel_message_info_uid(&msg_info->info));
			}

			errors = TRUE;
		}

		if (!row_set->_length)
			do_break = TRUE;

		CORBA_free(row_set);
		row_set = NULL;

		camel_folder_summary_touch(folder->summary);
		camel_folder_summary_save(folder->summary);

		if (changes && camel_folder_change_info_changed(changes))
			camel_object_trigger_event(folder, "folder_changed", changes);
		if (changes) {
			camel_folder_change_info_free(changes);
			changes = NULL;
		}

		if (do_break)
			break;
	} while (TRUE);

	// now set the session key
	cbs = CAMEL_BRUTUS_SUMMARY(folder->summary);
	if (((CamelOfflineFolder*)folder)->sync_offline)
		cbs->flags |= BRUTUS_FOLDER_SYNC_OFFLINE;
	else
		cbs->flags &= ~BRUTUS_FOLDER_SYNC_OFFLINE;
	memcpy((void*)cbs->session_key, (const void*)brutus_folder->session_key, sizeof(cbs->session_key));
	camel_folder_summary_touch(folder->summary);
	camel_folder_summary_save(folder->summary);

	goto out;

out_err:
	folder->summary->time = 0;

	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
			    _("Could not update folder summary"));
out:
	brutus_release_object(BRUTUS_IMAPITable_tc, &content_table, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	// start the asynchronous summary update
	if (row_count) {
		folder_data = g_new0(struct _folder_data, 1);
		if (!folder_data)
			goto out_no_mem;
		folder_data->full_name = g_strdup(folder->full_name);
		if (!folder_data->full_name) {
			g_free(folder_data);
			goto out_no_mem;
		}
		folder_data->store = camel_folder_get_parent_store(folder);
		camel_object_ref(CAMEL_OBJECT(folder_data->store));

		g_thread_pool_push(folder_summary_updates_thread_pool,
				   (gpointer)folder_data,
				   NULL);
	}

out_no_mem:
	camel_operation_progress(NULL, 100);
	camel_operation_end(NULL);
}

void
brutus_update_folder_summary(CamelFolder *folder,
			     gboolean force_update,
			     CamelException *ex)
{
	struct _folder_data *folder_data = NULL;
	CamelBrutusFolder *brutus_folder = CAMEL_BRUTUS_FOLDER(folder);
	CamelBrutusSummary *cbs = CAMEL_BRUTUS_SUMMARY(folder->summary);
	CamelBrutusMessageInfo *msg_info = NULL;
	CamelFolderChangeInfo *changes = NULL;
	time_t last_summary_update = 0;   // last time the folder summary was updated
	time_t msg_modification_time = 1; // greater than last_summary_update to make sure that changes are caught during the first run
	struct _camel_header_raw *camel_header_raw = NULL;
	struct _camel_header_address *camel_addr_list = NULL;
	struct _camel_header_address *camel_addr = NULL;
	struct _camel_header_references *refs = NULL;
	struct _camel_header_references *irt = NULL;
	struct _camel_header_references *scan = NULL;
	CamelContentType *ct = NULL;
	const char *content = NULL;
	const char *charset = NULL;
	gboolean exists = FALSE;
	gboolean errors = TRUE;
	gboolean inbound = FALSE;
	gboolean do_break = FALSE;
	GPtrArray *seen_msg_infos = NULL; // PR_RECORD_KEY array of CamelBrutusMessageInfo with corresponding remote message object
	GPtrArray *expunge_msg_infos = NULL; // message infos to be deleted due to missing corresponding remote message object
	char *pr_record_key = NULL;
	char *msg_id = NULL;
	char *smtp_header = NULL;
	char *tmp = NULL;
	guchar digest[16] = { 0 };
	int count = 0;
	CORBA_long n = 0;
	CORBA_unsigned_long r = 0;
	CORBA_unsigned_long k = 0;
	CORBA_unsigned_long processed = 0;
	CORBA_unsigned_long row_count = 0;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_ENTRYID *sender_eid = NULL;
	BRUTUS_IMAPITable content_table = CORBA_OBJECT_NIL; // folder content
	BRUTUS_SRowSet *row_set = NULL;
	BRUTUS_SRowSet *recp_row_set = NULL;
	BRUTUS_SPropValue *pv = NULL;
	BRUTUS_seq_SPropValue *prop_values = NULL;
	BRUTUS_seq_SPropValue *ex_prop_values = NULL;
	CORBA_Environment ev[1];
	static BRUTUS_BDEFINE content_props[4] = {            // for the contents table
		/* 0 */ BRUTUS_BRUTUS_PR_ENTRYID,                     // short-term entryid
		/* 1 */ BRUTUS_BRUTUS_PR_MSG_STATUS,                  // message status
		/* 2 */ BRUTUS_BRUTUS_PR_LAST_MODIFICATION_TIME,      // message modification time
		/* 3 */ BRUTUS_BRUTUS_PR_RECORD_KEY,                  // provider unique ID
	};
	static BRUTUS_SPropTagArray content_proptag_array = { // for the contents table
		._maximum = 4,
		._length = 4,
		._buffer = content_props,
	};
	static BRUTUS_SSortOrderSet sort_order = {            // for the contents table
		.cCategories = 0,
		.cExpanded = 0,
		.aSort._maximum = 0,
		.aSort._length = 0,
		.aSort._buffer = NULL,
	};
	static BRUTUS_BDEFINE recp_props[3] = {               // for an recipients table
		/* 0 */ BRUTUS_BRUTUS_PR_RECIPIENT_TYPE,             // MAPI_TO, MAPI_CC, MAPI_BCC
		/* 1 */ BRUTUS_BRUTUS_PR_EMAIL_ADDRESS,              // if not there use PR_DISPLAY_NAME
		/* 2 */ BRUTUS_BRUTUS_PR_DISPLAY_NAME,               // display name, possibly email adress if recipient does not resolve in the Exchange address book
	};
	static BRUTUS_SPropTagArray recp_proptag_array = {    // for an recipients table
		._maximum = 3,
		._length = 3,
		._buffer = recp_props,
	};
	static BRUTUS_BDEFINE msg_props[20] = {               // for a message object
		/* 0  */ BRUTUS_BRUTUS_PR_ENTRYID,                   // long-term entryid
		/* 1  */ BRUTUS_BRUTUS_PR_OBJECT_TYPE,               // must be MAPI_MESSAGE
		/* 2  */ BRUTUS_BRUTUS_PR_MESSAGE_FLAGS,             // flags
		/* 3  */ BRUTUS_BRUTUS_PR_SUBJECT,                   // subject
		/* 4  */ BRUTUS_BRUTUS_PR_DISPLAY_TO,                // display name of TO recipients
		/* 5  */ BRUTUS_BRUTUS_PR_DISPLAY_CC,                // display name of CC recipients
		/* 6  */ BRUTUS_BRUTUS_PR_SENDER_NAME,               // only display name of sender
		/* 7  */ BRUTUS_BRUTUS_PR_SENDER_EMAIL_ADDRESS,      // email address of sender
		/* 8  */ BRUTUS_BRUTUS_PR_TRANSPORT_MESSAGE_HEADERS, // smtp envelope of inbound messages
		/* 9  */ BRUTUS_BRUTUS_PR_MESSAGE_DELIVERY_TIME,     // PT_SYSTIME - sent_time if sent, recieved_time if inbound
		/* 10 */ BRUTUS_BRUTUS_PR_MESSAGE_SIZE,              // accumulated size of all properties on message
		/* 11 */ BRUTUS_BRUTUS_PR_INTERNET_CPID,             // Equivalent to RFC 2822 "Content-Type".
		/* 12 */ BRUTUS_BRUTUS_PR_SENDER_ADDRTYPE,           // MHS, PROFS, SMTP, X400 or EX
		/* 13 */ BRUTUS_BRUTUS_PR_CLIENT_SUBMIT_TIME,        // Equivalent to RFC 2822 "Date:" when inbound
		/* 14 */ BRUTUS_BRUTUS_PR_PROVIDER_SUBMIT_TIME,      // Equivalent to RFC 2822 "Date:" when outbound
		/* 15 */ BRUTUS_BRUTUS_PR_INTERNET_MESSAGE_ID,       // Equivalent to RFC 2822 "Message-Id:"
		/* 16 */ BRUTUS_BRUTUS_PR_INTERNET_REFERENCES,       // Equivalent to RFC 2822 "References:" (rarely seen)
		/* 17 */ BRUTUS_BRUTUS_PR_IN_REPLY_TO_ID,            // Equivalent to RFC 2822 "In-Reply-To:"
		/* 18 */ BRUTUS_BRUTUS_PR_RECEIVED_BY_ADDRTYPE,      // inbound if not found
		/* 19 */ BRUTUS_BRUTUS_PR_SENDER_ENTRYID,            // open and get PR_SMTP_ADDRESS if PR_SENDER_ADDRTYPE==EX
		/* 20 *//*  BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray msg_proptag_array = {     // for message object properties
		._maximum = 20,
		._length = 20,
		._buffer = msg_props,
	};
	static BRUTUS_SPropTagArray ex_msg_proptag_array = {     // for message object properties
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};

	if (!brutus_folder->is_a_folder)
		return;

	if (!brutus_is_connected(brutus_folder->brutus_server,
				 brutus_folder->mapi_session)) {
		camel_exception_set(ex,
				    CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
				    _("Not connected"));
		return;
	}

	CORBA_exception_init(ev);

	d(folder->full_name);

#ifdef BRUTUS_SPY
	force_update = TRUE;
#endif
	/* _light() and regular updates are protected by the folder mutex. 
	 * _post_light() are running with another folder object and are *NOT*
	 * protected by the folder mutex. 
	 * We will therefore never get past this mutex if a _light() or a 
	 * regular update is in progress. We can, OTOH, get past this if a 
	 * _post_light() update is in progress.
	 */
	BRUTUS_FOLDER_TRY_LOCK(brutus_folder, summary_mutex);

	/*
	 * OK, no _light() or another regular update is running so we are running 
	 * a new regular update, intercepting a _post_light() update or is looking 
	 * at this folder for the very first time.
	 */

	// we will not get past this if we the folder is being deleted or if
	// a _post_light() update is in progress
	if (folder_locked(folder->full_name))
		goto out_unlock;

	/*
	 * Now we know that we are not intercepting a _post_light() update or a
	 * delete in progress.
	 */

	// seeing this folder for the first time?
	if (0 >= camel_folder_get_message_count(folder)) {
		brutus_update_folder_summary_light(folder, ex);
		goto out_unlock;
	}

	/*
	 * Determine if we should use the lightweight functions or
	 * if we can progress from here
	 */
	if (!(cbs->flags & BRUTUS_FOLDER_SUMMARY_HAS_LONG_TERM_ENTRYIDS)) { // does the summary contain long-term entryids?
		d("");
		if (strcmp(cbs->session_key, brutus_folder->session_key)) { // the short-term entryids are *not* valid
			d("");
			camel_folder_summary_clear(folder->summary);
			memset((void*)cbs->session_key, '\0', sizeof(cbs->session_key));
			brutus_update_folder_summary_light(folder, ex);
		} else { // the short-term entryids *are* valid
			d("");
			if (folder_summary_post_update_may_commence(folder->full_name)) {
				folder_data = g_new0(struct _folder_data, 1);
				if (!folder_data)
					goto out_unlock;
				folder_data->full_name = g_strdup(folder->full_name);
				if (!folder_data->full_name) {
					g_free(folder_data);
					goto out_unlock;
				}
				folder_data->store = camel_folder_get_parent_store(folder);
				camel_object_ref(CAMEL_OBJECT(folder_data->store));

				g_thread_pool_push(folder_summary_updates_thread_pool,
						   (gpointer)folder_data,
						   NULL);
			}
		}
		goto out_unlock;
	}
	d("");

	camel_operation_start(NULL, _("Fetching summary information for %s"), folder->name);
	camel_operation_progress(NULL, 0);

	if (force_update) // using PR_LAST_MODIFICATION_TIME of content table to detect changes
		last_summary_update = 0;
	else
		last_summary_update = folder->summary->time;
	folder->summary->time = brutus_get_server_utc_time(brutus_folder->brutus_server,
							   brutus_folder->mapi_session,
							   ((CamelService*)folder->parent_store)->url->host);

	// get contents table
	br = BRUTUS_IMAPIFolder_GetContentsTable(brutus_folder->mapi_folder,
						 0,
						 &content_table,
						 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out_err;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		d("Possibly this error: http://support.microsoft.com/default.aspx?scid=kb\%3Ben-us\%3B176001");
		goto out_err;
	}

	br = BRUTUS_IMAPITable_GetRowCount(content_table,
					   0,
					   &row_count,
					   ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out_err;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out_err;
	}
	if (!row_count)
		goto out_end_progress;

	br = brutus_PrepareTable(content_table,
				 &content_proptag_array,
				 CORBA_OBJECT_NIL,
				 &sort_order,
				 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out_err;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out_err;
	}

	// for later use for figuring out if any messages has been deleted behind our back
	seen_msg_infos = g_ptr_array_sized_new(row_count);
	if (!seen_msg_infos)
		goto out_err;

	// iterate over content (columns)
	dfmt2("Number of messages to parse in \"%s\" = %llu", folder->full_name, (long long unsigned int)row_count);
	do {
		changes = camel_folder_change_info_new();
		if (!changes)
			goto out_err;

		br = brutus_QuerySomeRows(content_table,
					  (CORBA_long)BRUTUS_IMAPITABLE_ROW_COUNT_MAX,
					  &row_set,
					  ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_err;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			goto out_err;
		}

		for (r = 0; r < row_set->_length; r++) {

			if (camel_operation_cancel_check(NULL)) {
				d("Canceled");
				camel_exception_set(ex,
						    CAMEL_EXCEPTION_USER_CANCEL,
						    _("Fetching folder summary information canceled"));
				goto canceled;
			}

			if (!brutus_is_connected(brutus_folder->brutus_server,
						 brutus_folder->mapi_session)) {
				camel_exception_set(ex,
						    CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
						    _("Not connected"));
				goto disconnected;
			}

			// progress bar
			camel_operation_progress(NULL, (100*processed)/(2*row_count));
			processed++;

			// to catch goto's
			errors = TRUE;

			free(pr_record_key);
			pr_record_key = NULL;

			// add to the array of pr_record_key for existing remote messages
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_RECORD_KEY);
			if (-1 != n) {
				pr_record_key = brutus_print_spropvalue(&row_set->_buffer[r]._buffer[n]);
				g_ptr_array_add(seen_msg_infos, (gpointer)g_strdup(pr_record_key));
			} else
				d("No PR_RECORD_KEY");

			// check whether the message has been modified since the last summary update. If force_update is TRUE this check is skipped
			if (!force_update) {
				n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_LAST_MODIFICATION_TIME);
				if (-1 != n) {
					msg_modification_time = brutus_filetime_to_time_t(row_set->_buffer[r]._buffer[n].Value._u.ft);
					if (last_summary_update > msg_modification_time)
						continue;
				}
			}

			// get short-term ENTRYID
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_ENTRYID);
			if (-1 == n) {
				d("No ENTRYID");
				continue;
			}
			brutus_sbinary_to_entryid(&row_set->_buffer[r]._buffer[n].Value._u.bin, &eid);
			if (!eid)
				continue;

			// get properties from message object
			br = BRUTUS_IMAPISession_QueryMessage(brutus_folder->mapi_session,
							      eid,
							      &msg_proptag_array,
							      &ex_msg_proptag_array,
							      &recp_proptag_array,
							      &prop_values,
							      &ex_prop_values,
							      &recp_row_set,
							      ev);
			CORBA_free(ex_prop_values); // we are not asking for any additional properties
			ex_prop_values = NULL;
			CORBA_free(eid);
			eid = NULL;
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto next;
			}
			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				goto next;
			}

			// inbound or outbound?
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[18].ulPropTag)) // PR_RECEIVED_BY_ADDRTYPE
				inbound = TRUE;
			else
				inbound = FALSE; // draft, sent or being edited

			// get full SMTP header if present, alternatively check MSGFLAG_ORIGIN_INTERNET first
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[8].ulPropTag)) { // PR_TRANSPORT_MESSAGE_HEADERS
				smtp_header = brutus_unfold_rfc822_header(prop_values->_buffer[8].Value._u.lpszA);
				if (smtp_header) {
					camel_header_raw = brutus_get_camel_header_raw(smtp_header);
					free(smtp_header);
					smtp_header = NULL;
				} else
					goto next;
			} else
				camel_header_raw = NULL;

			// get long-term ENTRYID
			brutus_sbinary_to_entryid(&prop_values->_buffer[0].Value._u.bin, &eid);
			if (!eid)
				goto next;

			msg_id = brutus_entryid_to_string(eid);
			CORBA_free(eid);
			eid = NULL;
			if (!msg_id)
				goto next;

			// charset and content type
			ct = NULL;
			content = NULL;
			charset = NULL;
			if (camel_header_raw) {
				if ((content = camel_header_raw_find(&camel_header_raw, "Content-Type", NULL))
				    && (ct = camel_content_type_decode(content))
				    && (charset = camel_content_type_param(ct, "charset"))
				    && (!g_ascii_strcasecmp(charset, "us-ascii")))
					charset = NULL;
			} else {
				if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[11].ulPropTag)) // PR_INTERNET_CPID
					charset = brutus_cpid_to_label(prop_values->_buffer[11].Value._u.l);
			}
			charset = charset ? e_iconv_charset_name(charset) : NULL;
			camel_content_type_unref(ct);

			/* get or create message info object */
			msg_info = (CamelBrutusMessageInfo*)camel_folder_summary_uid(folder->summary, msg_id);
			if (msg_info) {
				exists = TRUE;
				if (msg_info->pr_record_key)
					g_free(msg_info->pr_record_key);
				brutus_message_info_empty(folder->summary, &msg_info->info);
			} else {
				exists = FALSE;
				msg_info = (CamelBrutusMessageInfo*)camel_message_info_new(folder->summary);
			}
			msg_info->pr_record_key = NULL;

			// Message flags
			msg_info->info.flags = 0;
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_MSG_STATUS); // required column in folder content tables
			if (-1 != n) {
				if (BRUTUS_BRUTUS_MSGSTATUS_ANSWERED & row_set->_buffer[r]._buffer[n].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_ANSWERED;

				if (BRUTUS_BRUTUS_MSGSTATUS_DELMARKED & row_set->_buffer[r]._buffer[n].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_DELETED;

				if (BRUTUS_BRUTUS_MSGSTATUS_DRAFT & row_set->_buffer[r]._buffer[n].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_DRAFT;

				if (BRUTUS_BRUTUS_MSGSTATUS_HIGHLIGHTED & row_set->_buffer[r]._buffer[n].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_FLAGGED;
			}
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[2].ulPropTag)) { // PR_MESSAGE_FLAGS
				if (BRUTUS_BRUTUS_MSGFLAG_READ & prop_values->_buffer[2].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_SEEN;

				if (BRUTUS_BRUTUS_MSGFLAG_HASATTACH & prop_values->_buffer[2].Value._u.l)
					msg_info->info.flags |= CAMEL_MESSAGE_ATTACHMENTS;
			}

			msg_info->info.uid = g_strdup(msg_id);
			if (!msg_info->info.uid)
				goto rollback;
			free(msg_id);
			msg_id = NULL;

			// PR_RECORD_KEY
			msg_info->pr_record_key = g_strdup(pr_record_key);
			free(pr_record_key);
			pr_record_key = NULL;

			// Message-Id:
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[15].ulPropTag)) // PR_INTERNET_MESSAGE_ID
				msg_id = camel_header_msgid_decode(prop_values->_buffer[15].Value._u.lpszA);
			else if (camel_header_raw)
				msg_id = camel_header_msgid_decode(camel_header_raw_find(&camel_header_raw, "Message-Id", NULL));
			else {
				d("Invalid RFC 822 message header");
#ifndef BRUTUS_SPY
				goto rollback;
#endif
			}
			if (!msg_id) {
				d("No memory or invalid RFC 822 message header");
#ifdef BRUTUS_SPY
				memset(&msg_info->info.message_id, 0, sizeof(CamelSummaryMessageID));
#else
				goto rollback;
#endif
			} else {
				md5_get_digest(msg_id, strlen(msg_id), digest);
				memcpy(msg_info->info.message_id.id.hash, digest, sizeof(msg_info->info.message_id.id.hash));
				g_free(msg_id);
				msg_id = NULL;
			}

			// Subject:
			tmp = NULL;
			msg_info->info.subject = NULL;
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[3].ulPropTag))
				tmp = camel_header_decode_string(prop_values->_buffer[3].Value._u.lpszA, charset);
			else if (camel_header_raw)
				tmp = brutus_camel_summary_format_string(camel_header_raw, "Subject", charset);
			if (tmp) {
				msg_info->info.subject = camel_pstring_strdup(tmp);
				g_free(tmp);
				tmp = NULL;
			}

			/*
			 * Recipients. The recipient is BCC if neither "To:" nor "CC:" is present
			 * in the recipient table.
			 */

			// To:
			camel_header_address_list_clear(&camel_addr_list);
			msg_info->info.to = NULL;
			if (camel_header_raw) {
				tmp = brutus_camel_summary_format_address(camel_header_raw, "To", charset);
			} else if ((recp_row_set->_length) && (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(recp_row_set->_buffer[0]._buffer[0].ulPropTag))) { // we don't know if STORE_SUBMIT_OK is set
				for (k = 0; k<recp_row_set->_length; k++) { // loop all recipients
					if (BRUTUS_BRUTUS_MAPI_TO & recp_row_set->_buffer[k]._buffer[0].Value._u.l) {
						n = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_EMAIL_ADDRESS);
						if (-1 == n) // no PR_EMAIL_ADDRESS
							n = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_DISPLAY_NAME);

						camel_addr = camel_header_address_decode(recp_row_set->_buffer[k]._buffer[n].Value._u.lpszA, charset);
						camel_header_address_list_append(&camel_addr_list, camel_addr);
					}
				}
				if (camel_addr_list) {
					tmp = camel_header_address_list_format(camel_addr_list);
					camel_header_address_list_clear(&camel_addr_list);
				}
			}
			if (tmp) {
				msg_info->info.to = camel_pstring_strdup(tmp);
				g_free(tmp);
				tmp = NULL;
			}

			// CC:
			camel_header_address_list_clear(&camel_addr_list);
			msg_info->info.cc = NULL;
			if (camel_header_raw) {
				tmp = brutus_camel_summary_format_address(camel_header_raw, "CC", charset);
			} else if ((recp_row_set->_length) && (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(recp_row_set->_buffer[0]._buffer[0].ulPropTag))) { // we don't know if STORE_SUBMIT_OK is set
				for (k = 0; k<recp_row_set->_length; k++) { // loop all recipients
					if (BRUTUS_BRUTUS_MAPI_CC & recp_row_set->_buffer[k]._buffer[0].Value._u.l) {
						n = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_EMAIL_ADDRESS);
						if (-1 == n) // no PR_EMAIL_ADDRESS
							n = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_DISPLAY_NAME);

						camel_addr = camel_header_address_decode(recp_row_set->_buffer[k]._buffer[n].Value._u.lpszA, charset);
						camel_header_address_list_append(&camel_addr_list, camel_addr);
					}
				}
				if (camel_addr_list) {
					tmp = camel_header_address_list_format(camel_addr_list);
					camel_header_address_list_clear(&camel_addr_list);
				}
			}
			if (tmp) {
				msg_info->info.to = camel_pstring_strdup(tmp);
				g_free(tmp);
				tmp = NULL;
			}

			// From:
			camel_header_address_list_clear(&camel_addr_list);
			msg_info->info.from = NULL;
			if (camel_header_raw) {
				tmp = brutus_camel_summary_format_address(camel_header_raw, "From", charset);
			} else { // check for sender addrtype: SMTP=>Get SENDER_EMAIL_ADDRESS. EX=>Open sender and get PR_SMTP_ADDRESS.
				if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[12].ulPropTag)) { // PR_SENDER_ADDRTYPE
					if (!g_ascii_strcasecmp("SMTP", prop_values->_buffer[12].Value._u.lpszA)) {
						if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[7].ulPropTag)) { // PR_SENDER_EMAIL_ADDRESS
							camel_addr_list = camel_header_address_decode(prop_values->_buffer[7].Value._u.lpszA, charset);
							tmp = camel_header_address_list_format(camel_addr_list);
							camel_header_address_list_clear(&camel_addr_list);
						} else if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[6].ulPropTag)) { // PR_SENDER_NAME
							tmp = camel_header_decode_string(prop_values->_buffer[6].Value._u.lpszA, charset);
						}
					} else if (!g_ascii_strcasecmp("EX", prop_values->_buffer[12].Value._u.lpszA)) {
						if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[19].ulPropTag)) { // PR_SENDER_ENTRYID
							brutus_sbinary_to_entryid(&prop_values->_buffer[19].Value._u.bin, &sender_eid);
							if (sender_eid) {
								pv = brutus_open_object_get_one_prop(brutus_folder->mapi_session,
												     CORBA_OBJECT_NIL,
												     sender_eid,
												     BRUTUS_BRUTUS_PR_SMTP_ADDRESS);
								if ((pv) && (BRUTUS_BRUTUS_PR_SMTP_ADDRESS == pv->ulPropTag)) {
									camel_addr_list = camel_header_address_decode(pv->Value._u.lpszA, charset);
									tmp = camel_header_address_list_format(camel_addr_list);
									camel_header_address_list_clear(&camel_addr_list);
								} else  if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[6].ulPropTag)) { // PR_SENDER_NAME
									tmp = camel_header_decode_string(prop_values->_buffer[6].Value._u.lpszA, charset);
								}
								CORBA_free(pv);
								pv = NULL;
								CORBA_free(sender_eid);
								sender_eid = NULL;
							} else if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[6].ulPropTag)) { // PR_SENDER_NAME
								tmp = camel_header_decode_string(prop_values->_buffer[6].Value._u.lpszA, charset);
							}
						}
					} else { // for all other address types than SMTP or EX use the sender name
						if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[6].ulPropTag)) { // PR_SENDER_NAME
							tmp = camel_header_decode_string(prop_values->_buffer[6].Value._u.lpszA, charset);
						}
					}
				} else { // unknown address type
					if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[7].ulPropTag)) { // PR_SENDER_EMAIL_ADDRESS
						camel_addr_list = camel_header_address_decode(prop_values->_buffer[7].Value._u.lpszA, charset);
						tmp = camel_header_address_list_format(camel_addr_list);
						camel_header_address_list_clear(&camel_addr_list);
					} else if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[6].ulPropTag)) { // PR_SENDER_NAME
						tmp = camel_header_decode_string(prop_values->_buffer[6].Value._u.lpszA, charset);
					}
				}
			}
			if (tmp) {
				msg_info->info.from = camel_pstring_strdup(tmp);
				g_free(tmp);
				tmp = NULL;
			}

			// Received/Sent:
			if (inbound) {
				if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[9].ulPropTag)) // PR_MESSAGE_DELIVERY_TIME
					msg_info->info.date_received = brutus_filetime_to_time_t(prop_values->_buffer[9].Value._u.ft);
				else
					msg_info->info.date_received = 0;

				if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[13].ulPropTag)) // PR_CLIENT_SUBMIT_TIME
					msg_info->info.date_sent = brutus_filetime_to_time_t(prop_values->_buffer[13].Value._u.ft);
				else if (camel_header_raw)
					msg_info->info.date_sent = camel_header_decode_date(camel_header_raw_find(&camel_header_raw, "Date", NULL), NULL);
				else
					msg_info->info.date_sent = 0;
			} else {
				if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[9].ulPropTag)) // PR_MESSAGE_DELIVERY_TIME
					msg_info->info.date_received = brutus_filetime_to_time_t(prop_values->_buffer[9].Value._u.ft);
				else
					msg_info->info.date_received = 0;

				if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[14].ulPropTag))        // PR_PROVIDER_SUBMIT_TIME
					msg_info->info.date_sent = brutus_filetime_to_time_t(prop_values->_buffer[14].Value._u.ft);
				else if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[13].ulPropTag))   // PR_CLIENT_SUBMIT_TIME
					msg_info->info.date_sent = brutus_filetime_to_time_t(prop_values->_buffer[13].Value._u.ft);
				else if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[9].ulPropTag))    // PR_MESSAGE_DELIVERY_TIME
					msg_info->info.date_sent = brutus_filetime_to_time_t(prop_values->_buffer[9].Value._u.ft);
				else
					msg_info->info.date_sent = 0;
			}

			// Size:
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[10].ulPropTag)) // PR_MESSAGE_SIZE
				msg_info->info.size = prop_values->_buffer[10].Value._u.l;
			else
				msg_info->info.size = 0;

			// Mailing list:
			msg_info->info.mlist = NULL;
			if (camel_header_raw) {
				tmp = camel_header_raw_check_mailing_list(&camel_header_raw);
				if (tmp) {
					msg_info->info.mlist = camel_pstring_strdup(tmp);
					g_free(tmp);
					tmp = NULL;
				}
			}

			// References:
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[16].ulPropTag)) // PR_INTERNET_REFERENCES
				refs = camel_header_references_decode(prop_values->_buffer[16].Value._u.lpszA);
			else if (camel_header_raw)
				refs = camel_header_references_decode(camel_header_raw_find(&camel_header_raw, "References", NULL));
			else
				refs = NULL;
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[17].ulPropTag)) // PR_IN_REPLY_TO_ID
				irt = camel_header_references_inreplyto_decode(prop_values->_buffer[17].Value._u.lpszA);
			else if (camel_header_raw)
				irt = camel_header_references_inreplyto_decode(camel_header_raw_find(&camel_header_raw, "In-Reply-To", NULL));
			else
				irt = NULL;
			if (refs || irt) { // straight from camel-folder-summary.c
				if (irt) {
					/* The References field is populated from the ``References'' and/or ``In-Reply-To''
					   headers. If both headers exist, take the first thing in the In-Reply-To header
					   that looks like a Message-ID, and append it to the References header. */
					if (refs)
						irt->next = refs;

					refs = irt;
				}

				count = camel_header_references_list_size(&refs);
				msg_info->info.references = g_malloc(sizeof(*msg_info->info.references) + ((count-1) * sizeof(msg_info->info.references->references[0])));
				count = 0;
				scan = refs;
				while (scan) {
					md5_get_digest(scan->id, strlen(scan->id), digest);
					memcpy(msg_info->info.references->references[count].id.hash, digest, sizeof(msg_info->info.message_id.id.hash));
					count++;
					scan = scan->next;
				}
				msg_info->info.references->size = count;
				camel_header_references_list_clear(&refs);
			}

			// all is well
			errors = FALSE;

			goto next;

		rollback:
			d("*************  ROLLBACK * ROLLBACK * ROLLBACK  **************");

			// msg_id msg_info->info.uid
			if (!exists) {
				camel_message_info_free(&msg_info->info);
			} else {
				if (msg_id) {
					camel_folder_summary_remove_uid(folder->summary, msg_id);
					free(msg_id);
					msg_id = NULL;
				} else
					camel_folder_summary_remove_uid(folder->summary, camel_message_info_uid(&msg_info->info));
			}
		next:
			camel_header_raw_clear(&camel_header_raw);

			CORBA_free(eid);
			eid = NULL;

			CORBA_free(prop_values);
			prop_values = NULL;

			CORBA_free(recp_row_set);
			recp_row_set = NULL;

			if (!errors) {
				if (exists) {
					camel_folder_change_info_change_uid(changes, camel_message_info_uid(&msg_info->info));
					camel_message_info_free(&msg_info->info);  // unref
				} else { // don't unref here
					camel_folder_summary_add(folder->summary,(CamelMessageInfo*)msg_info);
					camel_folder_change_info_add_uid(changes, camel_message_info_uid(&msg_info->info));
					camel_folder_change_info_recent_uid(changes, camel_message_info_uid(&msg_info->info));
				}
			}
		}

		if (!row_set->_length)
			do_break = TRUE;

		CORBA_free(row_set);
		row_set = NULL;

		// report the additions and modifications
		camel_folder_summary_touch(folder->summary);
		camel_folder_summary_save(folder->summary);
		if (changes && camel_folder_change_info_changed(changes))
			camel_object_trigger_event(folder, "folder_changed", changes);
		if (changes) {
			camel_folder_change_info_free(changes);
			changes = NULL;
		}

		if (do_break)
			break;
	} while (TRUE);

	// from an early loop exit
	free(pr_record_key);
	pr_record_key = NULL;

	// cleanup for deleted or moved message objects
	expunge_msg_infos = g_ptr_array_sized_new(0);
	if (!expunge_msg_infos)
		goto out_err;

	for (k = 0; k < folder->summary->messages->len; k++) {

		// progress bar
		camel_operation_progress(NULL, (100*(processed+k))/(2*row_count));

		exists = FALSE;
		msg_info = ((CamelBrutusMessageInfo*)g_ptr_array_index(folder->summary->messages, k));
		for (r = 0; r < seen_msg_infos->len; r++) { // compare by pr_record_key
			if (!g_ascii_strcasecmp((gchar*)g_ptr_array_index(seen_msg_infos, r), msg_info->pr_record_key)) {
				exists = TRUE;
				break;
			}
		}
		if (!exists) // there is no remote corresponding message object for this info and it must be deleted by uid
			g_ptr_array_add(expunge_msg_infos, (gpointer)camel_message_info_uid(&((CamelBrutusMessageInfo*)g_ptr_array_index(folder->summary->messages, k))->info));
	}

	for (r = 0; r < expunge_msg_infos->len; r++) {
		msg_id = g_ptr_array_index(expunge_msg_infos, r);
		if (!msg_id) {
			d("ghost uid");
			continue;
		}
		camel_folder_change_info_remove_uid(changes, (const char*)msg_id);
		camel_folder_summary_remove_uid(folder->summary, (const char*)msg_id);
	}
	msg_id = NULL;

	// now set the session key
	cbs = CAMEL_BRUTUS_SUMMARY(folder->summary);
	if (((CamelOfflineFolder*)folder)->sync_offline)
		cbs->flags |= BRUTUS_FOLDER_SYNC_OFFLINE;
	else
		cbs->flags &= ~BRUTUS_FOLDER_SYNC_OFFLINE;
	memcpy((void*)cbs->session_key, (const void*)brutus_folder->session_key, sizeof(cbs->session_key));
	camel_folder_summary_touch(folder->summary);
	camel_folder_summary_save(folder->summary);

	// report the deletions
	if (changes && camel_folder_change_info_changed(changes))
		camel_object_trigger_event(folder, "folder_changed", changes);
	if (changes) {
		camel_folder_change_info_free(changes);
		changes = NULL;
	}

	goto out_end_progress;

out_err:
	d("");
	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
			    _("Could not update folder summary"));
canceled:
disconnected:
	folder->summary->time = last_summary_update;

out_end_progress:
	camel_operation_progress(NULL, 100);
	camel_operation_end(NULL);

out_unlock:
	BRUTUS_FOLDER_UNLOCK(brutus_folder, summary_mutex);
out:
	brutus_release_object(BRUTUS_IMAPITable_tc, &content_table, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	if (seen_msg_infos) {
		g_ptr_array_foreach(seen_msg_infos, brutus_for_each_g_free, NULL);
		g_ptr_array_free(seen_msg_infos, TRUE);
	}

	if (expunge_msg_infos)
		g_ptr_array_free(expunge_msg_infos, TRUE);
}

CamelType
camel_brutus_summary_get_type (void)
{
	static CamelType type = CAMEL_INVALID_TYPE;

	if (type == CAMEL_INVALID_TYPE) {
		type = camel_type_register(
			camel_folder_summary_get_type(),
			"CamelBrutusSummary",
			sizeof(CamelBrutusSummary),
			sizeof(CamelBrutusSummaryClass),
			(CamelObjectClassInitFunc)camel_brutus_summary_class_init,
			NULL,
			(CamelObjectInitFunc)camel_brutus_summary_init,
			NULL);
	}

	return type;
}
