/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Authors: Jules Colding <colding@omesc.com>
 *
 *    Camel Brutus store class.
 *
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* system includes */
#include <string.h>
#include <uuid/uuid.h>
#include <orbit/orbit.h>
#include <sys/time.h>
#include <time.h>
#include <glib.h>
#include <glib/gstdio.h>

/* Camel includes */
#include <camel/camel-i18n.h>
#include <camel/camel-session.h>
#include <camel/camel-store.h>
#include <camel/camel-file-utils.h>
#include <camel/camel-exception.h>
#include <camel/camel-stream-mem.h>

#include <libedataserver/md5-utils.h>
#include <libedataserver/e-account-list.h>
#include <libedataserver/e-list-iterator.h>

/* camel-brutus includes */
#include "camel-brutus-private.h"
#include "camel-brutus-store.h"
#include "camel-brutus-folder.h"
#include "camel-brutus-folder-summary.h"

/* pure brutus includes */
#include <idl_output/BrutusLogOn.h>
#include <idl_output/return_codes.h>
#include <idl_output/types.h>
#include <idl_output/uid.h>
#include <servant_impl/BrutusCheck_impl.c>

/* evolution-brutus includes */
#include <server/brutus_conv.h>
#include <server/brutus_mapi.h>
#include <server/brutus_corba.h>
#include <server/brutus_string.h>
#include <server/brutus_util.h>
#include <server/brutus.h>
#include <session/brutusd.h>

// used as full_name when caching folder information
#define TOP_FOLDER_CACHE_ID "f1b7b625-e52f-4353-b834-6e02acf1a78b"

// where to store the folder infomation cache
#define FOLDER_INFO_CACHE "folder_info_cache"

/*
  CamelFolderInfo cache blob memory layout:

  Size                   | Content
  ----------------------------------

  Fixed sized part, use 'struct fi_cache_FixedSizedPart*' to access:
  sizeof(size_t)       | strlen(uri)
  sizeof(size_t)       | strlen(name)
  sizeof(size_t)       | strlen(full_name)
  sizeof(guint32)      | flags
  sizeof(guint32)      | unread
  sizeof(guint32)      | total
  sizeof(digest_str_t) | md5_get_digest(next->full_name)   // (next == NULL)   => digest = { '\0' }
  sizeof(digest_str_t) | md5_get_digest(parent->full_name) // (parent == NULL) => digest = { '\0' }
  sizeof(digest_str_t) | md5_get_digest(child->full_name)  // (child == NULL)  => digest = { '\0' }

  Variable sized part:
  strlen(uri)       | uri
  strlen(name)      | name
  strlen(full_name) | full_name
*/
struct fi_cache_FixedSizedPart {
	size_t uri_len;
	size_t name_len;
	size_t full_name_len;
	guint32 flags;
#if (EDS_EPOCH >= 110)
	guint32 unread; // must be gint32 when my patch (http://bugzilla.gnome.org/show_bug.cgi?id=377908) is accepted
	guint32 total;  // same...
#else
	guint32 unread;
	guint32 total;
#endif
	digest_str_t next_full_name_digest;
	digest_str_t parent_full_name_digest;
	digest_str_t child_full_name_digest;
};


static CamelOfflineStoreClass *parent_class = NULL;
extern CamelServiceAuthType camel_brutus_password_authtype;
#ifdef BRUTUS_HAVE_ENCRYPTED_AUTH
extern CamelServiceAuthType camel_brutus_encrypted_authtype;
#endif

/******************************
 *        thread pool         *
 *****************************/

GThreadPool *folder_summary_updates_thread_pool = NULL;


/******************************
 *   delayed freeing thread   *
 *****************************/

// queue to hold entry that must be freed in a delayed manner
GAsyncQueue *wait_mem_queue = NULL;

// delay before freeing memory that has been put in the delay queue
static struct timespec delay = {
	.tv_sec = 30,
	.tv_nsec = 0,
};

static GThread *delayed_free_thread = NULL;

/*
 * The wait queue will be totally flushed whenever new data arrives.
 * This is done on the expectation that data is being pushed on the
 * queue in bursts. Like:
 *
 * ...waiting ... LOTS_OF_DATA ... waiting ... waiting ... LOTS_OF_DATA ...
 *
 * The assumption is that the duration of any one burst is significantly
 * smaller than the delay time.
 */
static gpointer
delayed_free_thread_func(gpointer data)
{
	gpointer mem = NULL;

	d("");
	g_async_queue_ref(wait_mem_queue);

	while (TRUE) {
		mem = g_async_queue_pop(wait_mem_queue);
		nanosleep(&delay, NULL);
		do {
			g_free(mem);
			mem = g_async_queue_try_pop(wait_mem_queue);
		} while (mem);
	}

	// we'll never get here anyway but...
	g_async_queue_unref(wait_mem_queue);
}


/*******************************
 *   public helper functions   *
 ******************************/

/*
 * Public syncronization functions to prevent asynchronous folder summary
 * updates to collide with synchronous folder deletions.
 */
static GStaticMutex folder_busy_mutex = G_STATIC_MUTEX_INIT;

static GPtrArray *folder_busy_list = NULL;

struct folder_delete_status {
	gboolean deleting; // folder delete is pending
	gboolean updating; // a sumamry update is pending - may be cancelled by a pending delete
	char *full_name;
};

/*
 * Callee must invoke this function repeatedly until the
 * update has been cancelled
 */
static gboolean
folder_delete_is_a_go(const char *full_name)
{
	guint n;
	gboolean retv = FALSE;
	gboolean found = FALSE;
	struct folder_delete_status *folder_status;

	d("");
	g_static_mutex_lock(&folder_busy_mutex);
	for (n = 0; n < folder_busy_list->len; n++) {
		folder_status = (struct folder_delete_status*)g_ptr_array_index(folder_busy_list, n);

		if (!strcmp(full_name, folder_status->full_name)) {
			found = TRUE;

			folder_status->deleting = TRUE;

			if (folder_status->updating)
				retv = FALSE;
			else
				retv = TRUE;
			break;
		}
	}
	if (!found) {
		retv = TRUE;
		folder_status = (struct folder_delete_status*)g_new(struct folder_delete_status, 1);
		folder_status->full_name = g_strdup(full_name);
		folder_status->deleting = TRUE;
		folder_status->updating = FALSE;

		g_ptr_array_add(folder_busy_list, folder_status);
	}
	g_static_mutex_unlock(&folder_busy_mutex);

	return retv;
}

/*
 * Callee must invoke this function to get clearance for
 * folder summary update. Callee must invoke folder_is_done()
 * if the update is cancelled due to this function returning FALSE.
 */
gboolean
folder_summary_update_is_a_go(const char *full_name)
{
	guint n;
	gboolean retv = FALSE;
	gboolean found = FALSE;
	struct folder_delete_status *folder_status;

	g_static_mutex_lock(&folder_busy_mutex);
	for (n = 0; n < folder_busy_list->len; n++) {
		folder_status = (struct folder_delete_status*)g_ptr_array_index(folder_busy_list, n);

		if (!strcmp(full_name, folder_status->full_name)) {
			found = TRUE;

			folder_status->updating = TRUE;

			if (folder_status->deleting)
				retv = FALSE;
			else
				retv = TRUE;
			break;
		}
	}
	if (!found) {
		retv = TRUE;
		folder_status = (struct folder_delete_status*)g_new(struct folder_delete_status, 1);
		folder_status->full_name = g_strdup(full_name);
		folder_status->deleting = FALSE;
		folder_status->updating = TRUE;

		g_ptr_array_add(folder_busy_list, folder_status);
	}
	g_static_mutex_unlock(&folder_busy_mutex);

	return retv;
}

/*
 * Callee must invoke this function to get clearance for
 * starting a folder summary post update.
 */
gboolean
folder_summary_post_update_may_commence(const char *full_name)
{
	guint n;
	gboolean retv = TRUE;
	struct folder_delete_status *folder_status;

	d("");
	g_static_mutex_lock(&folder_busy_mutex);
	for (n = 0; n < folder_busy_list->len; n++) {
		folder_status = (struct folder_delete_status*)g_ptr_array_index(folder_busy_list, n);

		if (!strcmp(full_name, folder_status->full_name)) {
			retv = FALSE;
			break;
		}
	}
	if (retv) {
		folder_status = (struct folder_delete_status*)g_new(struct folder_delete_status, 1);
		folder_status->full_name = g_strdup(full_name);
		folder_status->deleting = FALSE;
		folder_status->updating = TRUE;

		g_ptr_array_add(folder_busy_list, folder_status);
	}
	g_static_mutex_unlock(&folder_busy_mutex);

	return retv;
}

/*
 * Will return TRUE if the folder is updating or being deleted.
 * Will return FALSE if no such operations are in progress.
 */
gboolean
folder_locked(const char *full_name)
{
	guint n;
	gboolean retv = FALSE;
	struct folder_delete_status *folder_status;

	g_static_mutex_lock(&folder_busy_mutex);
	for (n = 0; n < folder_busy_list->len; n++) {
		folder_status = (struct folder_delete_status*)g_ptr_array_index(folder_busy_list, n);

		if (!strcmp(full_name, folder_status->full_name)) {
			retv = TRUE;
			break;
		}
	}
	g_static_mutex_unlock(&folder_busy_mutex);

	return retv;
}

void
folder_is_done(const char *full_name)
{
	guint n;
	struct folder_delete_status *folder_status = NULL;

	d("");
	g_static_mutex_lock(&folder_busy_mutex);
	for (n = 0; n < folder_busy_list->len; n++) {
		folder_status = (struct folder_delete_status*)g_ptr_array_index(folder_busy_list, n);

		if (!strcmp(full_name, folder_status->full_name)) {
			g_ptr_array_remove_index(folder_busy_list, n);
			break;
		}
	}
	g_static_mutex_unlock(&folder_busy_mutex);

	if (folder_status) {
		g_free(folder_status->full_name);
		g_free(folder_status);
	}
}

static void
free_folder_busy_list(void)
{
	guint n;
	struct folder_delete_status *folder_status = NULL;

	d("");
	g_static_mutex_lock(&folder_busy_mutex);
	if (folder_busy_list) {
		for (n = 0; n < folder_busy_list->len; n++) {
			folder_status = (struct folder_delete_status*)g_ptr_array_index(folder_busy_list, n);
			if (folder_status) {
				g_free(folder_status->full_name);
				g_free(folder_status);
			}
		}
		g_ptr_array_free(folder_busy_list, TRUE);
		folder_busy_list = NULL;
	}
	g_static_mutex_unlock(&folder_busy_mutex);
}

/*******************************
 *   static helper functions   *
 ******************************/

static void
get_random_guid(char guid_str[37])
{
	uuid_t guid = { 0 };

	uuid_generate(guid);
	uuid_unparse(guid, guid_str);
}

static const EAccount*
brutus_find_account(EAccountList *accounts, const char *url)
{
	EIterator *it;
	const EAccount *account = NULL;

	d("");
	if (!url)
		return NULL;

	for (it = e_list_get_iterator ((EList *)accounts); e_iterator_is_valid (it); e_iterator_next (it)) {
		account = (const EAccount *)e_iterator_get(it);

		if (!strcmp(url, account->source->url))
			return account;
	}
	g_object_unref(it);

	return NULL;
}

static int
brutus_get_auto_check_timeout(const char *url)
{
	int retv = -1;
	const EAccount *account;
	EAccountList *account_list = NULL;
	GConfClient *gconf_client = NULL;

	d("");
	gconf_client = gconf_client_get_default();
	if (!gconf_client)
		return -1;

	account_list = e_account_list_new(gconf_client);
	if (!account_list)
		goto out;

	account = brutus_find_account(account_list, url);
	if (account)
		retv = account->source->auto_check_time;

out:
	if (account_list)
		g_object_unref(account_list);

	if (gconf_client)
		g_object_unref(gconf_client);

	return retv;
}

/*
 * Must be called with the folder_tree_rw_mutex read locked
 */
#ifdef BRUTUS_DEBUG
static void
brutus_print_folder_tree(CamelFolderInfo *top)
{
	int n = 0;
	int level = 0;
	CamelFolderInfo *fi = NULL;

	if (!top)
		return;

	if (!getenv(ENV_DEBUG))
		return;

	fi = top;
	while (TRUE) {
		fi = fi->parent;
		if (fi)
			level++;
		else break;
	}

	fi = top;
	printf("\n");
	while (TRUE) {
		if (level >= 1) {
			for (n = 0; n < level-1; n++)
				printf("\t");
			printf("|----- ");
		}

		if (!fi->child)
			printf("%s (%d)\n", fi->name, fi->total);
		else {
			printf("%s (%d)", fi->name, fi->total);
			brutus_print_folder_tree(fi->child);
		}

		fi = fi->next;
		if (!fi) {
			printf("\n");
			break;
		}
	}
}
#else
#define brutus_print_folder_tree(__top__) G_STMT_START { } G_STMT_END
#endif

static void
brutus_folder_info_to_blob(const CamelFolderInfo *fi,
			   size_t *len,
			   void **blob)
{
	char *pos = NULL;
	struct fi_cache_FixedSizedPart *fixed_part = NULL;

	*len = sizeof(struct fi_cache_FixedSizedPart)
		+ strlen(fi->uri)
		+ strlen(fi->name)
		+ strlen(fi->full_name);
	*blob = g_try_malloc0(*len);
	if (!*blob) {
		*len = 0;
		return;
	}
	fixed_part = (struct fi_cache_FixedSizedPart*) *blob;
	fixed_part->uri_len = strlen(fi->uri);
	fixed_part->name_len = strlen(fi->name);
	fixed_part->full_name_len = strlen(fi->full_name);
	fixed_part->flags = fi->flags;
	fixed_part->unread = fi->unread;
	fixed_part->total = fi->total;
	if (fi->next && fi->next->full_name) {
		brutus_get_digest_str(fi->next->full_name, fixed_part->next_full_name_digest);
	}
	if (fi->parent && fi->parent->full_name) {
		brutus_get_digest_str(fi->parent->full_name, fixed_part->parent_full_name_digest);
	}
	if (fi->child && fi->child->full_name) {
		brutus_get_digest_str(fi->child->full_name, fixed_part->child_full_name_digest);
	}

	pos = (char*)*blob + sizeof(struct fi_cache_FixedSizedPart);
	memcpy((void*)pos, (void*)fi->uri, fixed_part->uri_len);

	pos += fixed_part->uri_len;
	memcpy((void*)pos, (void*)fi->name, fixed_part->name_len);

	pos += fixed_part->name_len;
	memcpy((void*)pos, (void*)fi->full_name, fixed_part->full_name_len);
}

static gboolean
brutus_get_from_fi_cache(CamelDataCache *cache,
			 const char *key,
			 CamelFolderInfo *fi,
			 digest_str_t next_full_name_digest,
			 digest_str_t parent_full_name_digest,
			 digest_str_t child_full_name_digest)
{
	struct fi_cache_FixedSizedPart fixed_part = {
		.uri_len = 0,
		.name_len = 0,
		.full_name_len = 0,
		.flags = 0,
		.unread = 0,
		.total = 0,
		.next_full_name_digest = { '\0' },
		.parent_full_name_digest = { '\0' },
		.child_full_name_digest = { '\0' },
	};
	gboolean retv = FALSE;
	CamelStream *cache_stream = NULL;
	CamelStreamMem *mem_stream = NULL;

	d(key);

	fi->uri = NULL;
	fi->name = NULL;
	fi->full_name = NULL;
	fi->next = NULL;
	fi->parent = NULL;
	fi->child = NULL;

	cache_stream  = camel_data_cache_get(cache, "cache", key, NULL);
	if (!cache_stream)
		return FALSE;

	mem_stream = (CamelStreamMem*)camel_stream_mem_new();
	if (!mem_stream)
		goto out;

	camel_stream_write_to_stream(cache_stream, (CamelStream*)mem_stream);
	camel_stream_reset((CamelStream*)mem_stream);
	camel_object_unref(CAMEL_OBJECT(cache_stream));
	cache_stream = NULL;
	camel_stream_read((CamelStream*)mem_stream, (void*) &fixed_part, sizeof(struct fi_cache_FixedSizedPart));

	fi->flags = fixed_part.flags;
	fi->unread = fixed_part.unread;
	fi->total = fixed_part.total;

	fi->uri = (char*)g_try_malloc(fixed_part.uri_len + 1);
	if (!fi->uri)
		goto out;
	fi->uri[fixed_part.uri_len] = '\0';
	if (fixed_part.uri_len != camel_stream_read((CamelStream*)mem_stream, (void*) fi->uri, fixed_part.uri_len))
		goto out;

	fi->name = (char*)g_try_malloc(fixed_part.name_len + 1);
	if (!fi->name)
		goto out;
	fi->name[fixed_part.name_len] = '\0';
	if (fixed_part.name_len != camel_stream_read((CamelStream*)mem_stream, (void*) fi->name, fixed_part.name_len))
		goto out;

	fi->full_name = (char*)g_try_malloc(fixed_part.full_name_len + 1);
	if (!fi->full_name)
		goto out;
	fi->full_name[fixed_part.full_name_len] = '\0';
	if (fixed_part.full_name_len != camel_stream_read((CamelStream*)mem_stream, (void*) fi->full_name, fixed_part.full_name_len))
		goto out;

	memcpy((void*)next_full_name_digest, (void*)fixed_part.next_full_name_digest, sizeof(digest_str_t));
	memcpy((void*)parent_full_name_digest, (void*)fixed_part.parent_full_name_digest, sizeof(digest_str_t));
	memcpy((void*)child_full_name_digest, (void*)fixed_part.child_full_name_digest, sizeof(digest_str_t));

	retv = TRUE;
out:
	if (cache_stream)
		camel_object_unref(CAMEL_OBJECT(cache_stream));

	if (mem_stream)
		camel_object_unref(CAMEL_OBJECT(mem_stream));

	return retv;

}

static gboolean
brutus_add_to_fi_cache(CamelDataCache *cache,
		       const char *key,
		       const CamelFolderInfo *fi)
{
	gboolean retv = FALSE;
	size_t len = 0;
	void *fi_blob = NULL;
	digest_str_t digest_str = { '\0' };
	CamelStream *cache_stream = NULL;
	CamelStreamMem *mem_stream = NULL;

	if (!cache || !key || !fi)
		goto out;

	brutus_get_digest_str(key, digest_str);
	brutus_folder_info_to_blob(fi, &len, &fi_blob);
	if (!len)
		goto out;

	mem_stream = (CamelStreamMem*)camel_stream_mem_new_with_buffer((const char*)fi_blob, len);
	if (!mem_stream)
		goto out;
	camel_stream_reset((CamelStream*)mem_stream);
	g_free(fi_blob);
	fi_blob = NULL;

	cache_stream = camel_data_cache_add(cache, "cache", digest_str, NULL);
	if (!cache_stream)
		goto out;

	if (-1 == camel_stream_write_to_stream((CamelStream*)mem_stream, cache_stream))
		goto out;

	if (-1 == camel_stream_flush(cache_stream)) { // no space on device?
		d("Could not flush to folder information cache");
		camel_data_cache_remove(cache, "cache", digest_str, NULL);
		goto out;
	}

	retv = TRUE;
out:
	if (cache_stream) {
		camel_stream_close(cache_stream);
		camel_object_unref(CAMEL_OBJECT(cache_stream));
	}

	if (mem_stream)
		camel_object_unref(CAMEL_OBJECT(mem_stream));

	g_free(fi_blob);

	return retv;
}


/*
 * Callee must hold folder_tree_rw_mutex
 */
static void
brutus_save_folder_tree(CamelBrutusStorePrivate *priv,
			CamelFolderInfo *top,
			CamelException *ex)
{
	const char *key = NULL;
	CamelFolderInfo *fi = NULL;

	if (!priv->folder_tree)
		return;

	if (!top) {
		fi = priv->folder_tree;
		key = TOP_FOLDER_CACHE_ID;
	} else {
		fi = top;
		key = fi->full_name;
	}

	while (TRUE) {
		d(key);
		if (!brutus_add_to_fi_cache(priv->fi_cache, key, fi)) {
			if (!camel_exception_is_set(ex)) {
				camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
						    _("Errors building folder information cache"));
			}
			break;
		}
		if (fi->child)
			brutus_save_folder_tree(priv, fi->child, ex);

		fi = fi->next;
		if (!fi)
			break;
		key = fi->full_name;
	}
}


/*
 * Will add a CamelFolderInfo to the hash tables.
 * Must be called with the folder_tree_rw_mutex held.
 */
static void
brutus_add_to_hash_tables(CamelBrutusStorePrivate *priv,
			  CamelFolderInfo *fi,
			  BRUTUS_ENTRYID *eid)
{
	if (!priv || !fi)
		return;

	d(fi->full_name);
	if (eid)
		g_hash_table_insert(priv->fi_from_entryid_hash,
				    (gpointer)eid,
				    (gpointer)fi);

	g_hash_table_insert(priv->fi_from_full_name_hash,
			    (gpointer)fi->full_name,
			    (gpointer)fi);

	if (eid)
		g_hash_table_insert(priv->entryid_from_full_name_hash,
				    (gpointer)fi->full_name,
				    (gpointer)eid);
}

/*
 * Will remove an entry from all hash tables.
 * Must be called with the folder_tree_rw_mutex write locked.
 */
static void
brutus_delete_from_hash_tables(CamelBrutusStorePrivate *priv,
			       const char *full_name,
			       const BRUTUS_ENTRYID *eid)
{
	d("");
	if (eid)
		g_hash_table_steal(priv->entryid_from_full_name_hash,
				   full_name);

	g_hash_table_steal(priv->fi_from_full_name_hash,
			   full_name);

	if (eid)
		g_hash_table_remove(priv->fi_from_entryid_hash,
				    eid); // will g_free eid
}

static void
brutus_hash_table_data_free(gpointer data)
{
	CORBA_free(data);
}

/*
 * Will delete all children and all next's to fi.
 * Must be called with the folder_tree_rw_mutex write locked.
 */
static void
brutus_folder_info_free(CamelBrutusStorePrivate *priv,
			CamelFolderInfo *fi)
{
	const BRUTUS_ENTRYID *eid = NULL;

	d("");
	if (!fi)
		return;

	if (fi->full_name) {
		eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, fi->full_name);
		brutus_delete_from_hash_tables(priv, fi->full_name, eid);
	}
	brutus_folder_info_free(priv, fi->next);
	brutus_folder_info_free(priv, fi->child);

	g_free(fi->name);
	g_free(fi->uri);

	// Work around a bug where up-source is using these
	// pointers after they have been freed. This will lead
	// to false positives when valgrind checks for leaks.

	if (fi->full_name)
		g_async_queue_push(wait_mem_queue, (gpointer)fi->full_name);
	g_async_queue_push(wait_mem_queue, (gpointer)fi);
}

/*
 * Will delete a branch from the folder info tree, free all
 * related memory and re-link the pruned tree.
 * Must be called with the folder_tree_rw_mutex write locked.
 */
static void
brutus_delete_branch(CamelBrutusStorePrivate *priv,
		     CamelFolderInfo *fi)
{
	CamelFolderInfo *parent = NULL;
	CamelFolderInfo *this = NULL;
	CamelFolderInfo *prev = NULL;

	if (!fi)
		return;

	d(fi->full_name);

	parent = fi->parent;
	if (parent)
		this = parent->child; // start with foremost child to parent
	else
		this = fi; // fi is a top-level node

	// re-link folder info tree
	if (fi == this) { // fi is top-level or the first child of parent
		if (parent)
			parent->child = this->next;
	} else do {
			prev = this;
			this = this->next;
			if (fi == this) { // take fi out of the tree
				prev->next = this->next;
				break;
			}
		} while (TRUE);

	// isolate fi
	fi->parent = NULL;
	fi->next = NULL;

	// now, as the branch has been cut loose, free it
	brutus_folder_info_free(priv, fi);
}

/*
 * Builds a store URI
 */
static gchar*
brutus_build_uri(const gchar *base_url,
		 const gchar *full_name,
		 CamelException *ex)
{
	CamelURL *url = NULL;
	gchar *uri = NULL;

	d("");
	url = camel_url_new(base_url, ex);
	if (!url)
		return NULL;

	g_free(url->path);
	url->path = g_strdup_printf("/%s", full_name ? full_name : "");
	uri = camel_url_to_string(url, CAMEL_URL_HIDE_ALL);
	camel_url_free(url);

	return uri;
}

#ifdef BRUTUS_SPY
static CamelFolderInfo*
brutus_build_root_folder_info(CamelStore *store,
			      BRUTUS_IMAPIFolder folder,
			      CamelFolderInfo *parent,
			      CamelException *ex)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	CamelFolderInfo *info = g_new0(CamelFolderInfo, 1);
	char *tmp = NULL;

	d("");
	info->next = NULL; // there is no next for root folders
	info->parent = parent;
	info->child = NULL; // to be created later

	tmp = brutus_get_property_as_string((BRUTUS_IMAPIProp)folder, BRUTUS_BRUTUS_PR_DISPLAY_NAME);
	if (!tmp || ('\0' == *tmp)) {
		info->name = g_strdup("Root Folder");
	} else
		info->name = g_strdup(tmp);
	free(tmp);

	info->full_name = g_strdup_printf("%s/%s", parent->full_name, info->name);
	info->uri = brutus_build_uri(priv->base_url, info->full_name, ex);

	info->flags = CAMEL_FOLDER_CHILDREN | CAMEL_FOLDER_SYSTEM;
	info->unread = 0;
	info->total = 0;

	return info;
}
#endif

/*
 * Must be called with the folder_tree_rw_mutex write locked.
 * The child sub-tree is populated by this function. Everything
 * else must be set prior to this function. "parent_folder" is
 * the parent to all folders added here. "this" is the parent
 * CamelFolderInfo to all CamelFolderInfos created here and
 * refers to "parent_folder"..
 */
static void
brutus_build_folder_hierachy(CamelStore *store,
			     BRUTUS_IMAPIFolder parent_folder,
			     CamelFolderInfo *this,
			     gboolean release_folder,
			     CamelException *ex)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	CamelFolderInfo *previous_info = NULL;
	CamelFolderInfo *current_info = NULL;
	char *tmp = NULL;
	gchar *tmp_name = NULL;
	gboolean name_mangled = FALSE;
	int mangle_level = 1;
	gchar *mangled_name = NULL;
	CORBA_boolean cb = CORBA_FALSE;
	CORBA_long n = 0;
	CORBA_unsigned_long r = 0;
	BRUTUS_SRowSet *row_set = NULL;
	BRUTUS_SPropValue *pv = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_IMAPITable hierachy = CORBA_OBJECT_NIL;
	BRUTUS_IMAPIFolder tmp_folder = CORBA_OBJECT_NIL;
	BRUTUS_BDEFINE obj_type;
	BRUTUS_ENTRYID *eid = NULL;
	CORBA_Environment ev[1];
	static BRUTUS_BDEFINE props[9] = {
		BRUTUS_BRUTUS_PR_LONGTERM_ENTRYID_FROM_TABLE,
		BRUTUS_BRUTUS_PR_ENTRYID, // short-term entryid fallback
		BRUTUS_BRUTUS_PR_DISPLAY_NAME,
		BRUTUS_BRUTUS_PR_SUBFOLDERS,
		BRUTUS_BRUTUS_PR_FOLDER_TYPE,
		BRUTUS_BRUTUS_PR_STATUS,
		BRUTUS_BRUTUS_PR_CONTENT_COUNT,
		BRUTUS_BRUTUS_PR_CONTENT_UNREAD,
		BRUTUS_BRUTUS_PR_CONTAINER_CLASS
	};
	static BRUTUS_SPropTagArray proptag_array = {
		._maximum = 9,
		._length = 9,
		._buffer = props,
	};
	static BRUTUS_SSortOrderSet sort_order = {
		.cCategories = 0,
		.cExpanded = 0,
		.aSort._maximum = 0,
		.aSort._length = 0,
		.aSort._buffer = NULL,
	};

	d("");
	if (!this || !store)
		goto out;
	this->child = NULL;

	CORBA_exception_init(ev);

	cb = CORBA_Object_is_nil(parent_folder, ev);
	if (!ORBIT2_EX(ev) && cb)
		goto out;

	// get hierachy table
	br = BRUTUS_IMAPIFolder_GetHierarchyTable(parent_folder, 0, &hierachy, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br)
		goto out;

	// get all rows
	br = brutus_QueryAllRows(hierachy,
				 &proptag_array,
				 CORBA_OBJECT_NIL,
				 &sort_order,
				 0,
				 &row_set,
				 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br)
		goto out;
	cb = CORBA_Object_is_nil(hierachy, ev);
	if (!ORBIT2_EX(ev) && !cb) {
		BRUTUS_IMAPITable_Destroy(hierachy, 0, ev);
		PRINT_IF_ORBIT2_EX(ev);

		CORBA_Object_release(hierachy, ev);
		hierachy = CORBA_OBJECT_NIL;
	}
	if (!row_set->_length)
		goto out;

	// get data for all the parent_folder children
	previous_info = NULL;
	current_info = g_new0(CamelFolderInfo, 1);
	for (r = 0; r < row_set->_length; r++) {

		// in case of a previous rollback
		if (!current_info)
			current_info = g_new0(CamelFolderInfo, 1);

		/*
		 * link the tree
		 */
		current_info->parent = this;
		current_info->child = NULL;
		current_info->next = NULL;

#ifndef BRUTUS_SPY
		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_CONTAINER_CLASS);
		if (-1 != n) { // do not show these folders in the mail component in non-SPY mode
			if (!g_ascii_strcasecmp(row_set->_buffer[r]._buffer[n].Value._u.lpszA, "IPF.Appointment")) {
				d("rollback - Do not show Calendar folders");
				goto rollback;
			}
			if (!g_ascii_strcasecmp(row_set->_buffer[r]._buffer[n].Value._u.lpszA, "IPF.Contact")) {
				d("rollback - Do not show Contacts folders");
				goto rollback;
			}
			if (!g_ascii_strcasecmp(row_set->_buffer[r]._buffer[n].Value._u.lpszA, "IPF.Journal")) {
				d("rollback - Do not show Journal folders");
				goto rollback;
			}
			if (!g_ascii_strcasecmp(row_set->_buffer[r]._buffer[n].Value._u.lpszA, "IPF.StickyNote")) {
				d("rollback - Do not show Notes folders");
				goto rollback;
			}
			if (!g_ascii_strcasecmp(row_set->_buffer[r]._buffer[n].Value._u.lpszA, "IPF.Task")) {
				d("rollback - Do not show Task folders");
				goto rollback;
			}
		}
#endif
		/*
		 * strings
		 */

		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_DISPLAY_NAME);
		if (-1 == n) {
			d("rollback");
			goto rollback;
		}
		current_info->name = g_strdup(row_set->_buffer[r]._buffer[n].Value._u.lpszA);

		/* mangle names containing '/' */
		tmp = strchr(current_info->name, '/');
		if (tmp) {
			name_mangled = TRUE;
			tmp_name = g_strdup(current_info->name);
			tmp = strchr(tmp_name, '/');
			*(tmp++) = '~';
			*tmp = '\0';
			mangled_name = g_strdup_printf("%s%d", tmp_name, mangle_level++);
			g_free(tmp_name);
		} else {
			name_mangled = FALSE;
			mangled_name = current_info->name;
		}
		current_info->full_name = g_strdup_printf("%s/%s", current_info->parent->full_name, mangled_name);
		if (name_mangled)
			g_free(mangled_name);
		current_info->uri = brutus_build_uri(priv->base_url, current_info->full_name, ex);
		d(current_info->full_name);

		/*
		 * counts
		 */

		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_CONTENT_UNREAD);
		if (-1 == n)
			current_info->unread = -1;
		else
			current_info->unread = (guint32)row_set->_buffer[r]._buffer[n].Value._u.l;

		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_CONTENT_COUNT);
		if (-1 == n)
			current_info->total = -1;
		else
			current_info->total = (guint32)row_set->_buffer[r]._buffer[n].Value._u.l;

		/*
		 * flags
		 */

		current_info->flags = 0;

		/* mark folders containing '/' characters */
		if (name_mangled)
			current_info->flags |= CAMEL_FOLDER_NOSELECT | CAMEL_FOLDER_SYSTEM;

		/* children? */
		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_SUBFOLDERS);
		if (-1 == n) {
			d("rollback");
			goto rollback;
		}
		if ((guint32)row_set->_buffer[r]._buffer[n].Value._u.b)
			current_info->flags |= CAMEL_FOLDER_CHILDREN;
		else
			current_info->flags |= CAMEL_FOLDER_NOCHILDREN;

		/* search result folder? */
		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_FOLDER_TYPE);
		if (-1 == n) {
			d("rollback");
			goto rollback;
		}
		if (row_set->_buffer[r]._buffer[n].Value._u.l & BRUTUS_BRUTUS_FOLDER_SEARCH)
			current_info->flags |= CAMEL_FOLDER_VIRTUAL;

		/* hidden system folder? */
		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_STATUS);
		if (-1 != n) {
			if (row_set->_buffer[r]._buffer[n].Value._u.l & BRUTUS_BRUTUS_FLDSTATUS_HIDDEN) {
#ifdef BRUTUS_SPY
				current_info->flags |= CAMEL_FOLDER_SYSTEM;
#else
				d("rollback");
				goto rollback; // do not show hidden folders unless spying is on
#endif
			}
		}

		/*
		 * hash tables
		 */
		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_LONGTERM_ENTRYID_FROM_TABLE);
		if (-1 == n) { // use short-term entryid fallback property
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_ENTRYID);
			if (-1 == n) {
				d("rollback");
				goto rollback;
			} else {
				// get long-term ENTRYID and register it in the hash tables
				brutus_sbinary_to_entryid(&row_set->_buffer[r]._buffer[n].Value._u.bin, &eid);
				pv = brutus_open_object_get_one_prop(CORBA_OBJECT_NIL,
								     parent_folder,
								     eid,
								     BRUTUS_BRUTUS_PR_ENTRYID);
				CORBA_free(eid);
				brutus_sbinary_to_entryid(&pv->Value._u.bin, &eid);
				CORBA_free(pv);
			}
		} else {
			brutus_sbinary_to_entryid(&row_set->_buffer[r]._buffer[n].Value._u.bin, &eid);
		}
		dfmt("Adding into folder tree: %s", current_info->name);
		brutus_add_to_hash_tables(priv, current_info, eid);

		// link children to parent node
		if (!this->child)
			this->child = current_info;

		// add new info struct
		if (previous_info)
			previous_info->next = current_info;

		if (r == (row_set->_length - 1))
			break;

		previous_info = current_info;
		current_info->next = g_new0(CamelFolderInfo, 1);
		current_info = current_info->next;


		continue;

	rollback:
		d("*************  ROLLBACK * ROLLBACK * ROLLBACK  **************");
		g_free(current_info->name);
		g_free(current_info->full_name);
		g_free(current_info->uri);
		g_free(current_info);
		current_info = NULL;
		if (previous_info)
			previous_info->next = NULL;
	}
	CORBA_free(row_set);
	row_set = NULL;

	/* recursively build the child nodes */
	current_info = this->child;
	while (current_info) {
		if (current_info->flags & CAMEL_FOLDER_NOCHILDREN)
			goto next_sibling;

		eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, current_info->full_name);
		br = BRUTUS_IMAPIFolder_OpenEntry(parent_folder,
						  eid,
						  "",
						  BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
						  &obj_type,
						  &tmp_folder,
						  ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto next_sibling;
		}
		if (BRUTUS_BRUTUS_S_OK != br)
			goto next_sibling;

		// will relase tmp_folder
		brutus_build_folder_hierachy(store, tmp_folder, current_info, TRUE, ex);

	next_sibling:
		current_info = current_info->next;
	}

out:
	CORBA_free(row_set);

	/* release hierachy table */
	cb = CORBA_Object_is_nil(hierachy, ev);
	if (!ORBIT2_EX(ev) && !cb) {
		BRUTUS_IMAPITable_Destroy(hierachy, 0, ev);
		PRINT_IF_ORBIT2_EX(ev);

		CORBA_Object_release(hierachy, ev);
	}

	/* release input folder */
	if (release_folder) {
		cb = CORBA_Object_is_nil(parent_folder, ev);
		if (!ORBIT2_EX(ev) && !cb) {
			BRUTUS_IMAPIFolder_Destroy(parent_folder, 0, ev);
			PRINT_IF_ORBIT2_EX(ev);
			CORBA_Object_release(parent_folder, ev);
		}
	}
	CORBA_exception_free(ev);

	return;
}


/*
 * Must be called with the folder_tree_rw_mutex write locked
 */
static CamelFolderInfo*
brutus_build_folder_tree(CamelStore *store,
			 CamelException *ex)
{
	guint n = 0;
	size_t msg_store_count = 0;
	guint *msg_store_map = NULL;
	char *tmp = NULL;
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	CamelService *service = (CamelService*)brutus_store;
	CamelFolderInfo *top;
	CamelFolderInfo *current = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_IMAPIFolder folder;
	BRUTUS_IMsgStore msg_store = CORBA_OBJECT_NIL;
	BRUTUS_ENTRYID *eid = NULL;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	top = NULL;

	d("");
	if (!brutus_is_connected(priv->brutus_server,
				 priv->mapi_session)) {
		camel_exception_set(ex,
				    CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
				    _("Not connected"));
		goto out;
	}
	if (!priv->msg_stores->len)
		goto out;

	msg_store_map = g_malloc0(priv->msg_stores->len * sizeof(guint));
	if (!msg_store_map)
		goto out;

	top = g_new0(CamelFolderInfo, 1);
	priv->folder_tree = top;

	/* initialize top */
	top->next = NULL;
	top->parent = NULL;
	top->child = NULL;
	top->uri = brutus_build_uri(priv->base_url, NULL, ex);
	top->name = g_strdup(service->url->host);
	top->full_name = g_strdup(top->uri);
	top->flags = CAMEL_FOLDER_NOSELECT | CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_CHILDREN;
	top->unread = -1;
	top->total = -1;

	/* start by populating the message store infos */
	top->child = g_new0(CamelFolderInfo, 1);
	current = top->child;
	for (n = 0; n < priv->msg_stores->len; n++) {
		current->next = NULL;
		current->parent = top;
		current->child = NULL;
#ifndef BRUTUS_SPY
		// we _really_ need a unique ENTRYID
		if (!(BRUTUS_BRUTUS_STORE_ENTRYID_UNIQUE & ((struct BrutusMsgStore*)g_ptr_array_index(priv->msg_stores, n))->support_mask)) {
			d("ENTRYIDs not unique in this store");
			continue; // do not use unappropiate message stores unless we are in spy mode
		}
#endif
		// the store
		msg_store = ((struct BrutusMsgStore*)g_ptr_array_index(priv->msg_stores, n))->msg_store;

		/* determine message store type */
		current->flags = CAMEL_FOLDER_NOSELECT | CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_CHILDREN;
		tmp = brutus_get_guid_str((BRUTUS_IMAPIProp)msg_store, BRUTUS_BRUTUS_PR_MDB_PROVIDER);

		if (!g_ascii_strcasecmp(tmp, BRUTUS_BRUTUS_pbExchangeProviderPrimaryUserGuid))
			;
		else if (!g_ascii_strcasecmp(tmp, BRUTUS_BRUTUS_pbExchangeProviderPublicGuid))
			current->flags |= CAMEL_FOLDER_SHARED_TO_ME;
#ifndef BRUTUS_SPY
		else {
			free(tmp);
			continue; // we only want the private and public message stores
		}
#else
		else if (!g_ascii_strcasecmp(tmp, BRUTUS_BRUTUS_pbExchangeProviderDelegateGuid))
			current->flags |= CAMEL_FOLDER_SHARED_TO_ME;
		else if (!g_ascii_strcasecmp(tmp, BRUTUS_BRUTUS_pbExchangeProviderXportGuid))
			;
		else
			d("unknown message store type");
#endif
		free(tmp);

		tmp = brutus_get_property_as_string((BRUTUS_IMAPIProp)msg_store, BRUTUS_BRUTUS_PR_DISPLAY_NAME);
		if (!tmp)
			continue;
		current->name = g_strdup(tmp); // this double allocation is to make sure that I can do g_free() when freeing the memory later
		free(tmp);
		current->full_name = g_strdup(current->name);
		current->uri = brutus_build_uri(priv->base_url, current->full_name, ex);

		current->unread = 0;
		current->total = 0;

		/* add to the hash tables */
		eid = brutus_get_entryid((BRUTUS_IMAPIProp)msg_store, BRUTUS_BRUTUS_PR_ENTRYID);
		if (!eid) {
			g_free(current->uri);
			g_free(current->name);
			g_free(current->full_name);
			continue;
		}
		brutus_add_to_hash_tables(priv, current, eid);
		msg_store_map[msg_store_count++] = n;

		if (n == (priv->msg_stores->len - 1))
			break;

		current->next = g_new0(CamelFolderInfo, 1);
		current = current->next;
	}

	/* now get all root folders, and their folder hierachy, for all message stores, starting with the first child to top */
	current = top->child;
	for (n = 0; n < msg_store_count; n++) {
		msg_store = ((struct BrutusMsgStore*)g_ptr_array_index(priv->msg_stores, msg_store_map[n]))->msg_store;
#ifdef BRUTUS_SPY
		br = brutus_get_root_folder(msg_store, &folder, ev);
#else
		br = BRUTUS_IMAPISession_GetOutlookFolder(priv->mapi_session,
							  BRUTUS_OUTLOOK_PUBLIC_FOLDERS_ROOT,
							  msg_store,
							  &folder,
							  ev);
#endif
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			continue;
		}
		if (BRUTUS_BRUTUS_S_OK != br)
			continue;
#ifdef BRUTUS_SPY
		eid = brutus_get_entryid((BRUTUS_IMAPIProp)folder, BRUTUS_BRUTUS_PR_ENTRYID);
		if (!eid)
			continue;

		current->child = brutus_build_root_folder_info(store, folder, current, ex);

		/* add to the hash tables */
		brutus_add_to_hash_tables(priv, current->child, eid);

		/* will release the folder */
		brutus_build_folder_hierachy(store, folder, current->child, TRUE, ex);
#else
		brutus_build_folder_hierachy(store, folder, current, TRUE, ex);
#endif
		d(current->full_name);
		current = current->next;
	}

out:
	g_free(msg_store_map);
	CORBA_exception_free(ev);
	brutus_print_folder_tree(priv->folder_tree);

	return top;
}

/*
 * Must be called with the folder_tree_rw_mutex write locked.
 */
static void
brutus_build_folder_hierachy_from_cache(CamelBrutusStorePrivate *priv,
					CamelFolderInfo *top,
					digest_str_t next_full_name_digest,
					digest_str_t child_full_name_digest,
					const gboolean add_to_hash_table)
{
	digest_str_t next_digest;
	digest_str_t parent_digest;
	digest_str_t child_digest;

//	d(top->full_name);

	while (TRUE) {
		// insert into hash table
		if (add_to_hash_table) {
			d(top->full_name);
			brutus_add_to_hash_tables(priv, top, NULL);
		}

		// is there a child?
		if ('\0' != child_full_name_digest[0]) {
			top->child = g_new0(CamelFolderInfo, 1);
			if (!brutus_get_from_fi_cache(priv->fi_cache,
						      child_full_name_digest,
						      top->child,
						      next_digest,
						      parent_digest,
						      child_digest))
				break;
			top->child->parent = top;

			brutus_build_folder_hierachy_from_cache(priv,
								top->child,
								next_digest,
								child_digest,
								TRUE);
		}

		// is there a sibling?
		if ('\0' == next_full_name_digest[0]) {
			d("");
			top->next = NULL;
			break;
		}

		top->next = g_new0(CamelFolderInfo, 1);
		if (!brutus_get_from_fi_cache(priv->fi_cache,
					      next_full_name_digest,
					      top->next,
					      next_full_name_digest,
					      parent_digest,
					      child_full_name_digest))
			break;
		top->next->parent = top;
		top = top->next;
	}

	return;
}


/*
 * Must be called with the folder_tree_rw_mutex write locked
 */
static CamelFolderInfo*
brutus_build_folder_tree_from_cache(CamelStore *store)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	digest_str_t next_full_name_digest;
	digest_str_t parent_full_name_digest;
	digest_str_t child_full_name_digest;
	digest_str_t digest_str = { '\0' };

	d("");

	// compute cache key for top folder
	brutus_get_digest_str(TOP_FOLDER_CACHE_ID, digest_str);
	priv->folder_tree = g_new0(CamelFolderInfo, 1);
	if (!priv->folder_tree)
		return NULL;

	// initialize top
	if (!brutus_get_from_fi_cache(priv->fi_cache,
				      digest_str,
				      priv->folder_tree,
				      next_full_name_digest,   // <== { '\0' }
				      parent_full_name_digest, // <== { '\0' }
				      child_full_name_digest)) {
		brutus_folder_info_free(priv, priv->folder_tree);
		priv->folder_tree = NULL;
		return NULL;
	}

	// get all subfolders
	brutus_build_folder_hierachy_from_cache(priv,
						priv->folder_tree, // top
						next_full_name_digest,
						child_full_name_digest,
						FALSE);
	brutus_print_folder_tree(priv->folder_tree);

	return priv->folder_tree;
}

/* only invoked from brutus_connect(), when the store is connect locked */
static void
brutus_load_mapi_stores(CamelBrutusStorePrivate *priv)
{
	char *tmp = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_BDEFINE flags = BRUTUS_BRUTUS_MAPI_BEST_ACCESS | BRUTUS_BRUTUS_MDB_ONLINE;
	BRUTUS_SRowSet *row_set = NULL;
	BRUTUS_ENTRYID *eid = NULL;
	CORBA_unsigned_long n = 0;
	struct BrutusMsgStore *msg_store_struct = NULL;
	BRUTUS_SPropValue *pv = NULL;
	CORBA_Environment ev[1];
	BRUTUS_BDEFINE props[1] = { // Only retrieve entry id.
		BRUTUS_BRUTUS_PR_ENTRYID,
	};
	BRUTUS_SPropTagArray proptag_array = {
		._maximum = 1,
		._length = 1,
		._buffer = props,
	};
	BRUTUS_SSortOrderSet sort_order = {
		.cCategories = 0,
		.cExpanded = 0,
		.aSort._maximum = 0,
		.aSort._length = 0,
		.aSort._buffer = NULL,
	};

	d("");
	CORBA_exception_init(ev);

	// get the single row
	br = brutus_QueryAllRows(priv->mapi_msg_store_table,
				 &proptag_array,
				 CORBA_OBJECT_NIL,
				 &sort_order,
				 0,
				 &row_set,
				 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto exit;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto exit;
	}

	priv->msg_stores = g_ptr_array_new();
	for (n = 0; n < row_set->_length; n++) {
		msg_store_struct = g_new(struct BrutusMsgStore, 1);
		if (!msg_store_struct) {
			d("No memory");
			goto exit;
		}
		msg_store_struct->msg_store = CORBA_OBJECT_NIL;
		msg_store_struct->support_mask = 0;

		brutus_sbinary_to_entryid(&row_set->_buffer[n]._buffer[0].Value._u.bin, &eid);

		// open the message store - check for valid flags
		do {
			br = BRUTUS_IMAPISession_OpenMsgStore(priv->mapi_session,
							      eid,
							      "",
							      flags,
							      &msg_store_struct->msg_store,
							      ev);
			if (ORBIT2_EX(ev))
				break;

			if (BRUTUS_BRUTUS_S_OK == br)
				break;

			if (BRUTUS_BRUTUS_MAPI_E_UNKNOWN_FLAGS != br)
				break;

			// Exchange 5.5/2000 does not know this flag, disable it
			flags &= ~BRUTUS_BRUTUS_MDB_ONLINE;
			br = BRUTUS_IMAPISession_OpenMsgStore(priv->mapi_session,
							      eid,
							      "",
							      flags,
							      &msg_store_struct->msg_store,
							      ev);
		} while (FALSE);
		CORBA_free(eid);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			g_free(msg_store_struct);
			continue;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			g_free(msg_store_struct);
			continue;
		}
#ifndef BRUTUS_SPY
		free(tmp);
		tmp = brutus_get_guid_str((BRUTUS_IMAPIProp)msg_store_struct->msg_store, BRUTUS_BRUTUS_PR_MDB_PROVIDER);
		if (g_ascii_strcasecmp(tmp, BRUTUS_BRUTUS_pbExchangeProviderPrimaryUserGuid)
		    && g_ascii_strcasecmp(tmp, BRUTUS_BRUTUS_pbExchangeProviderPublicGuid)) {
			brutus_release_object(BRUTUS_IMsgStore_tc, &msg_store_struct->msg_store, CORBA_OBJECT_NIL, ev);
			PRINT_IF_ORBIT2_EX(ev);
			g_free(msg_store_struct);
			free(tmp);
			tmp = NULL;
			continue;
		}
		free(tmp);
		tmp = NULL;
#endif
		br = brutus_GetOneProp(msg_store_struct->msg_store,
				       BRUTUS_BRUTUS_PR_STORE_SUPPORT_MASK,
				       &pv,
				       ev);
		if (BRUTUS_BRUTUS_S_OK == br)
			msg_store_struct->support_mask = pv->Value._u.l;
		if (pv) {
			CORBA_free(pv);
			pv = NULL;
		}
		g_ptr_array_add(priv->msg_stores, (gpointer)msg_store_struct);
	}

exit:
	free(tmp);
	CORBA_exception_free(ev);
	CORBA_free(row_set);

	return;
}

/* only invoked from brutus_(dis)connect() and from finalize() when the store is connect locked */
static void
brutus_mapi_shutdown(CamelBrutusStore *brutus_store)
{
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_Environment ev[1];
	CORBA_exception_init(ev);

	d("");

	/* message store table */
	cb = CORBA_Object_is_nil(priv->mapi_msg_store_table, ev);
	if (!ORBIT2_EX(ev) && !cb) {
		BRUTUS_IMAPITable_Destroy(priv->mapi_msg_store_table, 0, ev);
		PRINT_IF_ORBIT2_EX(ev);

		CORBA_Object_release(priv->mapi_msg_store_table, ev);
	}
	priv->mapi_msg_store_table = CORBA_OBJECT_NIL;

	/* message stores */
	if (priv->msg_stores) {
		guint n = 0;
		BRUTUS_IMsgStore imsg_store = CORBA_OBJECT_NIL;
		for (n = 0; n < priv->msg_stores->len; n++) {
			imsg_store = ((struct BrutusMsgStore*)g_ptr_array_index(priv->msg_stores, n))->msg_store;
			cb = CORBA_Object_is_nil(imsg_store, ev);
			if (!ORBIT2_EX(ev) && !cb) {
				BRUTUS_IMsgStore_Destroy(imsg_store, 0, ev);
				PRINT_IF_ORBIT2_EX(ev);

				CORBA_Object_release(imsg_store, ev);
			}
			g_free(g_ptr_array_index(priv->msg_stores, n));
		}
		g_ptr_array_free(priv->msg_stores, TRUE);
		priv->msg_stores = NULL;
	}

	/* mapi session and instance ID */
	cb = CORBA_Object_is_nil(priv->mapi_session, ev);
	if (!ORBIT2_EX(ev) && !cb) {
		BRUTUS_IMAPISession_Destroy(priv->mapi_session, priv->instance_id, ev);
		PRINT_IF_ORBIT2_EX(ev);

		CORBA_Object_release(priv->mapi_session, ev);
	}
	priv->mapi_session = CORBA_OBJECT_NIL;
	priv->instance_id = 0;

	memset((void*)priv->session_key, '\0', sizeof(priv->session_key));

	CORBA_exception_free(ev);
}

static void
brutus_free_lifeline(BRUTUS_BrutusProxy proxy,
		     struct BrutusCheckProxy *lifeline)
{
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_Environment ev[1];
	CORBA_exception_init(ev);

	d("");
	cb = CORBA_Object_is_nil((CORBA_Object)lifeline->wrapper, ev);
	if (!ORBIT2_EX(ev) && !cb) {
		BRUTUS_BrutusProxy_killWrapper(proxy, lifeline->wrapper, ev);
		CORBA_Object_release((CORBA_Object)lifeline->wrapper, ev);
	}

	cb = CORBA_Object_is_nil((CORBA_Object)lifeline->filling, ev);
	if (!ORBIT2_EX(ev) && !cb)
		CORBA_Object_release((CORBA_Object)lifeline->filling, ev);

	CORBA_exception_free(ev);
	lifeline->wrapper = CORBA_OBJECT_NIL;
	lifeline->filling = CORBA_OBJECT_NIL;
}

static gboolean
brutus_orb_init(CamelBrutusStorePrivate *priv,
		const char *brutus_server,
		const guint16 brutus_port)
{
	gboolean retv = FALSE;
	static guint orb_num = 0;
	char *orb_name = NULL;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_Environment ev[1];

	d("");
	CORBA_exception_init(ev);

	// Hmm... I am unsure as to whether the Camel sub system will invoke the same backend
	// object more than once, so I put this here to avoid identical ORB names for certainty.
	orb_num++;
	orb_name = g_strdup_printf("%d-%s",
				   orb_num,
				   "Brutus-CamelStore-ORB_orbit-io-thread");
	if (!orb_name) {
		d("No memory");
		goto out;
	}

	d("Invoking ORB_Init()");
	priv->orb = brutus_create_orb(0,
				      NULL,
				      priv->use_ssliop,
				      "BrutusLogOn",
				      BRUTUS_BrutusLogOn_IOR_TABLE_KEY,
				      brutus_server,
				      brutus_port,
				      "Brutus-CamelStore-ORB_orbit-io-thread",
				      ev);
	g_free(orb_name);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	cb = CORBA_Object_is_nil((CORBA_Object) priv->orb, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb)
		goto out;

	d("Getting root POA");
	priv->root_poa = brutus_get_root_poa(priv->orb, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	cb = CORBA_Object_is_nil((CORBA_Object) priv->root_poa, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb) {
		d("Could not get the root POA");
		goto out;
	}

	retv = TRUE;

out:
	CORBA_exception_free(ev);

	if (retv)
		d("The ORB is initialized");
	else
		d("Could not initialize the ORB");

	return retv;
}

/****************************************
 *   service virtual method overloads   *
 ***************************************/

static void brutus_construct(CamelService *service,
			     CamelSession *session,
			     CamelProvider *provider,
			     CamelURL *url,
			     CamelException *ex);

static GList *brutus_query_auth_types(CamelService *service,
				      CamelException *ex);

static char *brutus_get_name(CamelService *service,
			     gboolean brief);

static gboolean brutus_connect(CamelService *service,
			       CamelException *ex);

static gboolean brutus_disconnect(CamelService *service,
				  gboolean clean,
				  CamelException *ex);

/**************************************
 *   store virtual method overloads   *
 *************************************/

static CamelFolder *brutus_get_folder(CamelStore *store,
				      const char *folder_name,
				      guint32 flags,
				      CamelException *ex);

static CamelFolder *brutus_get_inbox(CamelStore *store,
				     CamelException *ex);

static CamelFolder *brutus_get_trash(CamelStore *store,
				     CamelException *ex);

/* static CamelFolder *brutus_get_junk(CamelStore *store, */
/*				    CamelException *ex); */

static CamelFolderInfo *brutus_create_folder(CamelStore *store,
					     const char *parent_name,
					     const char *folder_name,
					     CamelException *ex);

static void brutus_delete_folder(CamelStore *store,
				 const char *folder_name,
				 CamelException *ex);

static void brutus_rename_folder(CamelStore *store,
				 const char *old_name,
				 const char *new_name,
				 CamelException *ex);

/* static void brutus_sync(CamelStore *store, */
/*			int expunge, */
/*			CamelException *ex); */

static CamelFolderInfo *brutus_get_folder_info(CamelStore *store,
					       const char *top,
					       guint32 flags,
					       CamelException *ex);

static gboolean brutus_folder_subscribed(CamelStore *store,
					 const char *folder_name);

static void brutus_subscribe_folder(CamelStore *store,
				    const char *folder_name,
				    CamelException *ex);

static void brutus_unsubscribe_folder(CamelStore *store,
				      const char *folder_name,
				      CamelException *ex);

/**********************************************
 ************ IMPLEMENTATIONS *****************
 *********************************************/

static void
camel_brutus_store_class_init(CamelBrutusStoreClass *camel_brutus_store_class)
{
	CamelServiceClass *camel_service_class = CAMEL_SERVICE_CLASS(camel_brutus_store_class);
	CamelStoreClass *camel_store_class = CAMEL_STORE_CLASS(camel_brutus_store_class);

	d("");

	// initiate gthreads
	if (!g_thread_supported())
		g_thread_init(NULL);

	parent_class = CAMEL_OFFLINE_STORE_CLASS(camel_type_get_global_classfuncs(camel_offline_store_get_type()));

	/* service virtual method overloading */
	camel_service_class->construct = brutus_construct;
	camel_service_class->query_auth_types = brutus_query_auth_types;
	camel_service_class->get_name = brutus_get_name;
	camel_service_class->connect = brutus_connect;
	camel_service_class->disconnect = brutus_disconnect;

	/* store virtual method overloading */
	camel_store_class->get_folder = brutus_get_folder;
	camel_store_class->get_inbox = brutus_get_inbox;
	camel_store_class->get_trash = brutus_get_trash;
//	camel_store_class->get_junk = brutus_get_junk;

	camel_store_class->create_folder = brutus_create_folder;
	camel_store_class->delete_folder = brutus_delete_folder;
	camel_store_class->rename_folder = brutus_rename_folder;

//	camel_store_class->sync = brutus_sync;

	camel_store_class->get_folder_info = brutus_get_folder_info;
	camel_store_class->free_folder_info = camel_store_free_folder_info_nop;

	camel_store_class->folder_subscribed = brutus_folder_subscribed;
	camel_store_class->subscribe_folder = brutus_subscribe_folder;
	camel_store_class->unsubscribe_folder = brutus_unsubscribe_folder;
}

/* TODO: Enable Exchange Trash and Junk if needed */
static void
brutus_construct(CamelService *service,
		 CamelSession *session,
		 CamelProvider *provider,
		 CamelURL *url,
		 CamelException *ex)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(service);
	CamelStore *store = CAMEL_STORE(service);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	CORBA_boolean cb = CORBA_TRUE;
	gchar *path = NULL;

	d("");
	CAMEL_SERVICE_CLASS(parent_class)->construct(service, session, provider, url, ex);
	if (camel_exception_is_set(ex))
		return;

	if (!(url->host
	      && url->user
	      && camel_url_get_param(url, "iiop_port")
	      && camel_url_get_param(url, "exchange_mailbox")
	      && camel_url_get_param(url, "brutus_server"))) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_INVALID,
				    _("Brutus Account not configured - required data not available in url"));
		return;
	}

	priv->brutus_server = brutus_strip_trailing_and_leading_whitespace(camel_url_get_param(url, "brutus_server"));
	if (!priv->brutus_server) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("No memory or invalid Brutus Server name in account configuration"));
		return;
	}

	/* SSLIOP */
#ifdef BRUTUS_HAVE_SSL
	priv->use_ssliop = camel_url_get_param(url, "use_ssliop") ? TRUE : FALSE;
#endif
	/* base URL */
	priv->base_url = camel_url_to_string(service->url, (CAMEL_URL_HIDE_PASSWORD | CAMEL_URL_HIDE_PARAMS | CAMEL_URL_HIDE_AUTH));
	if (!priv->base_url)
		goto out_no_mem;

	/* storage path */
	priv->storage_path = camel_session_get_storage_path(session, service, ex);
	if (!priv->storage_path)
		goto out_no_mem;

	/* create evolution-brutus subdirectories */
	path = g_strdup_printf("%s/%s", priv->storage_path, FOLDER_SUMMARIES);
	if (!path) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("No Camel storage path"));
		return;
	}
	if (-1 == g_mkdir_with_parents(path, 0700)) { // for folder summaries
		g_free(path);
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not create Camel Folder Summary directory"));
		return;
	}
	g_free(path);
	path = g_strdup_printf("%s/%s", priv->storage_path, FOLDER_MESSAGE_CACHE);
	if (!path) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("No Camel storage path"));
		return;
	}
	if (-1 == g_mkdir_with_parents(path, 0700)) { // for the folder message cache
		g_free(path);
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not create Camel Folder Summary directory"));
		return;
	}
	g_free(path);
	path = g_strdup_printf("%s/%s", priv->storage_path, FOLDER_INFO_CACHE);
	if (!path) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("No Camel storage path"));
		return;
	}
	if (-1 == g_mkdir_with_parents(path, 0700)) { // for the folder information cache
		g_free(path);
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not create Camel Folder Information directory"));
		return;
	}
	g_free(path);

	/* filter */
	if (camel_url_get_param(url, "filter"))
		store->flags |= CAMEL_STORE_FILTER_INBOX;

	/* hash tables */

	// key is BRUTUS_ENTRYID* - value is CamelFolderInfo*
	priv->fi_from_entryid_hash = g_hash_table_new_full(g_direct_hash,
							   brutus_entryid_cmp,
							   brutus_hash_table_data_free, // key destroy
							   NULL);
	if (!priv->fi_from_entryid_hash)
		goto out_no_mem;

	// key is string - value is CamelFolderInfo*
	priv->fi_from_full_name_hash = g_hash_table_new_full(g_str_hash,
							     g_str_equal,
							     NULL,
							     NULL);
	if (!priv->fi_from_full_name_hash)
		goto out_no_mem;

	// key is string - value is BRUTUS_ENTRYID*
	priv->entryid_from_full_name_hash = g_hash_table_new_full(g_str_hash,
								  g_str_equal,
								  NULL,
								  NULL); // value is a duplicated pointer and is destroyed in fi_from_entryid_hash
	if (!priv->entryid_from_full_name_hash)
		goto out_no_mem;

	// create folder information cache
	path = g_strdup_printf("%s/%s", priv->storage_path, FOLDER_INFO_CACHE);
	if (!path) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("No Camel storage path"));
		return;
	}
	if (-1 == g_mkdir_with_parents(path, 0700)) { // for the folder information cache
		g_free(path);
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not create Camel Folder Information directory"));
		return;
	}
	priv->fi_cache = camel_data_cache_new(path, 0, ex);
	g_free(path);
	if (!priv->fi_cache) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not create Camel Folder Information cache"));
		return;
	}
	camel_data_cache_set_expire_age(priv->fi_cache, -1);

	/* TODO - Enable Exchange Junk and Trash */
	store->flags &= ~CAMEL_STORE_VJUNK;
	store->flags &= ~CAMEL_STORE_VTRASH;

	d("Initializing CORBA");
	{
		gboolean orb_ok = FALSE;
		guint64 iiop_port = 951;
		guint64 ssliop_port = 952;
		guint16 brutus_port;

		if (!brutus_get_xbit_number(16, camel_url_get_param(url, "ssliop_port"), &ssliop_port)) {
			camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_INVALID,
					    _("Invalid SSLIOP port in url"));
			return;
		}
		if (!brutus_get_xbit_number(16, camel_url_get_param(url, "iiop_port"), &iiop_port)) {
			camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_INVALID,
					    _("Invalid IIOP port in url"));
			return;
		}
		brutus_port = priv->use_ssliop ? (guint16)ssliop_port : (guint16)iiop_port;

		priv->brutus_logon_str = brutus_build_corbaloc_string(priv->brutus_server,
								      BRUTUS_BrutusLogOn_IOR_TABLE_KEY,
								      priv->use_ssliop,
								      brutus_port);
		orb_ok = brutus_orb_init(priv,
					 priv->brutus_server,
					 brutus_port);

		if (!orb_ok) {
			camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
					    _("Could not initiate ORB"));
			return;
		}
	}

	d("Getting root POAManager, activating the RootPOA and instantiating the lifeline");
	{
		PortableServer_POAManager root_poa_manager = CORBA_OBJECT_NIL;
		gboolean use_local_proxy = !(ORBit_proto_use("IPv4") || ORBit_proto_use("IPv6"));
		CORBA_Environment ev[1];
		CORBA_exception_init(ev);

		root_poa_manager = PortableServer_POA__get_the_POAManager(priv->root_poa, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		}

		cb = CORBA_Object_is_nil((CORBA_Object)root_poa_manager, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		}
		if (cb) {
			d("Could not get the root POAManager");
			goto out_ex;
		}
		PortableServer_POAManager_activate(root_poa_manager, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		}

		CORBA_Object_release((CORBA_Object)root_poa_manager, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		}

		// get brutus proxy
		{
			char *home_dir = getenv("HOME");
			gchar *ref_path = g_strconcat(home_dir, "/", BRUTUSD_REF_FILE_NAME, NULL);

			priv->brutus_proxy = brutus_object_ref_from_file(priv->orb, ref_path, ev);
			g_free(ref_path);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto out_ex;
			}
			cb = CORBA_Object_is_nil((CORBA_Object)priv->brutus_proxy, ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto out_ex;
			}
			if (cb) {
				d("Could not get the Brutus proxy reference");
				goto out_ex;
			}
		}

		// the lifeline
		priv->lifeline.filling = impl_BRUTUS_BrutusCheck__create(priv->root_poa, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		}
		if (use_local_proxy) {
			d("Using local network capable Brutus proxy");
			BRUTUS_BrutusProxy_wrapObject(priv->brutus_proxy,
						      BRUTUS_BrutusCheck_tc,
						      priv->lifeline.filling,
						      &priv->lifeline.wrapper,
						      ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto out_ex;
			}
		} else {
			/* this will enable us to use "wrapper" unconditionally */
			priv->lifeline.wrapper = CORBA_Object_duplicate(priv->lifeline.filling, ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto out_ex;
			}
		}
		d("Testing primary BrutusCheck object...");
		BRUTUS_BrutusCheck_ping(priv->lifeline.filling, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		} else
			d("priv->lifeline.filling - OK");
		d("Testing BrutusCheck object wrapper...");
		BRUTUS_BrutusCheck_ping(priv->lifeline.wrapper, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		} else
			d("priv->lifeline.wrapper - OK");

		goto out;

	out_ex:
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not construct the store - CORBA exception caught"));
		goto out;

	out_no_mem:
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not construct the store - No memory"));

	out:
		CORBA_exception_free(ev);
	}

	// create the memory wait queue
	wait_mem_queue = g_async_queue_new();
	if (!wait_mem_queue) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not construct the store - No memory"));
		return;
	}

	// create delayed free thread
	delayed_free_thread = g_thread_create((GThreadFunc)delayed_free_thread_func,
					      NULL,
					      FALSE,
					      NULL);
	if (!delayed_free_thread)
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not construct the store - No memory"));

	folder_busy_list = g_ptr_array_new();
	if (!folder_busy_list) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not construct the store - No memory"));
		return;
	}

	memset((void*)priv->session_key, '\0', sizeof(priv->session_key));
	d("");

	// create thread pool for folder summary updates
	folder_summary_updates_thread_pool = g_thread_pool_new(brutus_update_folder_summary_post_light,
							       NULL,
							       1,
							       TRUE,
							       NULL);
	if (!folder_summary_updates_thread_pool) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not construct the store - No memory"));
		return;
	}
}

static GList*
brutus_query_auth_types(CamelService *service,
			CamelException *ex)
{
	GList *auth_types = NULL;

	d("");
	auth_types = g_list_prepend(auth_types, &camel_brutus_password_authtype
#ifdef BRUTUS_HAVE_ENCRYPTED_AUTH
				    , &camel_brutus_encrypted_authtype
#endif
		);

	return auth_types;
}

static char*
brutus_get_name(CamelService *service,
		gboolean brief)
{
	if (brief) {
		return g_strdup_printf(_("Exchange server %s"),
				       service->url->host);
	} else {
		return g_strdup_printf(_("Exchange account for %s on %s"),
				       service->url->user,
				       service->url->host);
	}
}

static gboolean
brutus_connect(CamelService *service,
	       CamelException *ex)
{
	static gboolean got_time_zones = FALSE;
	CamelBrutusStore *store = CAMEL_BRUTUS_STORE(service);
	CamelBrutusStorePrivate *priv = store->priv;
	BRUTUS_BrutusLogOn brutus_logon = CORBA_OBJECT_NIL;
	BRUTUS_BrutusLogOn_BrutusVersion brutus_server_version = {
		.major = 0,
		.minor = 0,
		.micro = 0,
		.release_tag = 0
	};
	CORBA_boolean cb = CORBA_TRUE;
	gboolean connected = FALSE;
	gboolean get_session_key = TRUE;
	const char *tmp = NULL;
	char *account_name = NULL;
	gchar *windows_user = NULL;
	gchar *windows_domain = NULL;
	gchar *exchange_mailbox = NULL;
	gchar *exchange_mailbox_dn = NULL;
	gchar *exchange_mailbox_email = NULL;
	gchar *exchange_server = NULL;
	char *prompt = NULL;
	char *err_prompt = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	CamelException ex_internal; // sometimes this function is called with (ex == NULL)
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);
	camel_exception_init(&ex_internal);

	/* check connection status */
	connected = brutus_is_connected(priv->brutus_server,
					priv->mapi_session);
	if (connected) {
		get_session_key = FALSE;
		goto out; // connected
	}

#ifndef HAVE_ORBIT2_TIMEOUT
	if (!brutus_server_ping(priv->brutus_server)) {
		get_session_key = FALSE;
		goto exit; // no reason to even try...
	}
#endif
	BRUTUS_STORE_LOCK(store, connect_mutex);
	brutus_mapi_shutdown(store); // reset connection

	windows_domain = brutus_strip_trailing_and_leading_whitespace(camel_url_get_param(service->url, "windows_domain"));
	if (!windows_domain)
		windows_domain = g_strdup("");

	exchange_mailbox = brutus_strip_trailing_and_leading_whitespace(camel_url_get_param(service->url, "exchange_mailbox"));
	exchange_mailbox_dn = brutus_strip_trailing_and_leading_whitespace(camel_url_get_param(service->url, "exchange_mailbox_dn"));
	exchange_mailbox_email = brutus_strip_trailing_and_leading_whitespace(camel_url_get_param(service->url, "exchange_mailbox_email"));
	exchange_server = brutus_strip_trailing_and_leading_whitespace(service->url->host);
	windows_user = brutus_strip_trailing_and_leading_whitespace(service->url->user);

	/* we are not connected - proceed */
	d("Attempting to query a session from the proxy");
	{
		BRUTUS_BrutusProxy_QueryState query_state = BRUTUS_BrutusProxy_offerInProgressState;
		struct timespec t = {
			.tv_sec = 0,
			.tv_nsec = 10000000 // 0.01 second
		};

		d(exchange_mailbox);
		d(exchange_server);
		do {
			query_state = BRUTUS_BrutusProxy_querySession(priv->brutus_proxy,
								      exchange_mailbox,
								      exchange_server,
								      &priv->mapi_session,
								      ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto proceed;
			}
			switch (query_state) {
			case BRUTUS_BrutusProxy_mustOfferState :
				goto proceed;
			case BRUTUS_BrutusProxy_offerInProgressState :
				nanosleep(&t, NULL); // sleep for 0.01 second
				break;
			case BRUTUS_BrutusProxy_queryOK : // got session
				break;
			default:
				d("Unknown query response");
				goto proceed;
			}
		} while (BRUTUS_BrutusProxy_queryOK != query_state);

		cb = CORBA_Object_is_nil((CORBA_Object)priv->mapi_session, ev);
		if (!ORBIT2_EX(ev) && !cb) {
			BRUTUS_IMAPISession_ping(priv->mapi_session, ev);
			if (!ORBIT2_EX(ev)) {
				priv->instance_id = BRUTUS_IMAPISession_addLifeline(priv->mapi_session, priv->lifeline.wrapper, ev);
				if (ORBIT2_EX(ev)) {
					d("Could not add lifeline");
					goto proceed;
				}

				d("Query successful");
				connected = TRUE; // we are connected, no query lock to release
				goto out_get_msgstore_tables;
			} else { // could not ping
				CORBA_Object_release((CORBA_Object)priv->mapi_session, ev);
				priv->mapi_session = CORBA_OBJECT_NIL;
			}
		}
	}

proceed:
	d("Could not get session from proxy, must log on explicitly");
	BRUTUS_BrutusProxy_getProfile(priv->brutus_proxy,
				      exchange_mailbox,
				      exchange_server,
				      &priv->mapi_profile,
				      ev);
	if (ORBIT2_EX(ev))
		PRINT_ORBIT2_EX(ev);
	else if (!strlen(priv->mapi_profile)) {
		d("No stored MAPI Profile - generating a new one");
		char guid_str[37] = { 0 };
		uuid_t guid = { 0 };

		CORBA_free(priv->mapi_profile);

		uuid_generate(guid);
		uuid_unparse(guid, guid_str);
		priv->mapi_profile = CORBA_string_dup(guid_str);
	}
	dfmt("Using this MAPI Profile: \"%s\"", priv->mapi_profile);

	d("Getting reference to BrutusLogOn object");
	{
		brutus_logon = (BRUTUS_BrutusLogOn)CORBA_ORB_resolve_initial_references(priv->orb, "BrutusLogOn", ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			brutus_logon = CORBA_OBJECT_NIL;
		}

		cb = CORBA_Object_is_nil(brutus_logon, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			cb = TRUE;
		}
		if (cb) {
			d("Could not resolve (resolve_initial_references) BrutusLogOn, falling back to corbaloc...");
			brutus_logon = (BRUTUS_BrutusLogOn)CORBA_ORB_string_to_object(priv->orb, priv->brutus_logon_str, ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				camel_exception_set(&ex_internal, CAMEL_EXCEPTION_SYSTEM,
						    _("CORBA error"));
				goto out_release_lock;
			}

			cb = CORBA_Object_is_nil(brutus_logon, ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				cb = TRUE;
			}
			if (cb) {
				d("Could not resolve (string_to_object) BrutusLogOn, NIL object returned");
				camel_exception_set(&ex_internal, CAMEL_EXCEPTION_SYSTEM,
						    _("CORBA error"));
				goto out_release_lock;
			} else
				d("BrutusLogOn resolved");
		}
		d("BrutusLogOn reference has been acquired");

		d("");
		BRUTUS_BrutusLogOn_GetVersion(brutus_logon, &brutus_server_version, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			camel_exception_set(&ex_internal, CAMEL_EXCEPTION_SYSTEM,
					    _("Could not retrieve Brutus Server version - Communications error"));
			goto out_release_lock;
		}
		if (!brutus_server_version_OK((int)brutus_server_version.major,
					      (int)brutus_server_version.minor,
					      (int)brutus_server_version.micro,
					      (int)brutus_server_version.release_tag)) {
			camel_exception_set(&ex_internal, CAMEL_EXCEPTION_SYSTEM,
					    _("Unsupported version of Brutus Server. You must have Brutus Server version "BRUTUS_SERVER_REQ_VERSION_STRING));
			goto out_release_lock;
		}
		d("Brutus Server version check completed");
	}

	prompt = g_strdup_printf(_("Please enter Windows domain password for \"%s\\%s\" (Exchange mailbox: %s)"),
				 windows_domain,
				 windows_user,
				 exchange_mailbox);
	err_prompt = g_strdup_printf(_("Authentication attempt failed - Please reenter Windows domain password for \"%s\\%s\" (Exchange mailbox: %s)"),
				     windows_domain,
				     windows_user,
				     exchange_mailbox);
	if (!prompt || !err_prompt)
		d("No memory");
	else
		connected = brutus_logon_loop(brutus_logon,
					      priv->lifeline.wrapper,
					      &priv->mapi_session,
					      &priv->instance_id,
					      &priv->mapi_profile,
					      prompt,
					      err_prompt,
					      windows_user,
					      windows_domain,
					      exchange_mailbox,
					      exchange_mailbox_dn,
					      exchange_mailbox_email,
					      exchange_server);

	if (!connected) {
		d("Could not connect to Brutus server");
		camel_exception_set(&ex_internal, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not connect to Brutus server"));
	} else {
		tmp = camel_url_get_param(service->url, "create_calendar_account");
		if (tmp) {
			account_name = g_strdup_printf("%s %s", exchange_mailbox, _("Calendar"));
			if (!account_name)
				d("No memory");
			else {
				if (!brutus_add_account(E_CAL_SOURCE_TYPE_EVENT,
							account_name,
							priv->mapi_profile,
							exchange_server,
							windows_user,
							exchange_mailbox,
							exchange_mailbox_dn,
							exchange_mailbox_email,
							windows_domain,
							priv->brutus_server,
							priv->use_ssliop ? camel_url_get_param(service->url, "ssliop_port") : camel_url_get_param(service->url, "iiop_port"),
							priv->use_ssliop,
							brutus_get_auto_check_timeout((const char*)camel_url_to_string(service->url, 0))))
					d("Could not create Calendar gconf source");
				g_free(account_name);
			}
		}

		tmp = camel_url_get_param(service->url, "create_todo_account");
		if (tmp) {
			account_name = g_strdup_printf("%s %s", exchange_mailbox, _("Tasks"));
			if (!account_name)
				d("No memory");
			else {
				if (!brutus_add_account(E_CAL_SOURCE_TYPE_TODO,
							account_name,
							priv->mapi_profile,
							exchange_server,
							windows_user,
							exchange_mailbox,
							exchange_mailbox_dn,
							exchange_mailbox_email,
							windows_domain,
							priv->brutus_server,
							priv->use_ssliop ? camel_url_get_param(service->url, "ssliop_port") : camel_url_get_param(service->url, "iiop_port"),
							priv->use_ssliop,
							brutus_get_auto_check_timeout((const char*)camel_url_to_string(service->url, 0))))
					d("Could not create Calendar gconf source");
				g_free(account_name);
			}
		}
	}

out_release_lock:
	if (connected) {
		d("Offering IMAPISession to Brutus proxy");
		BRUTUS_BrutusProxy_offerSession(priv->brutus_proxy,
						priv->mapi_session,
						exchange_mailbox,
						exchange_server,
						priv->mapi_profile,
						ev);
		PRINT_IF_ORBIT2_EX(ev);

	out_get_msgstore_tables:
		d("Getting message store table");
		d(camel_url_to_string(service->url, 0));
		dfmt("auto-check-timeout = %d", brutus_get_auto_check_timeout((const char*)camel_url_to_string(service->url, 0)));

		br = BRUTUS_IMAPISession_GetMsgStoresTable(priv->mapi_session, 0, &priv->mapi_msg_store_table, ev);
		if (!ORBIT2_EX(ev)) {
			if (BRUTUS_BRUTUS_S_OK != br) {
				d("Could not get message store table");
				priv->mapi_msg_store_table = CORBA_OBJECT_NIL;
			} else
				brutus_load_mapi_stores(priv);
		} else
			PRINT_ORBIT2_EX(ev);
	} else {
		d("Could not connect to Brutus server");
		if (!camel_exception_is_set(&ex_internal)) {
			camel_exception_set(&ex_internal, CAMEL_EXCEPTION_SYSTEM,
					    _("Could not connect to Brutus server"));
		}

		d("No IMAPISession offer - releasing query lock");
		BRUTUS_BrutusProxy_offerSession(priv->brutus_proxy,
						CORBA_OBJECT_NIL,
						"",
						"",
						"",
						ev);
		PRINT_IF_ORBIT2_EX(ev);
	}

	g_free(prompt);
	g_free(err_prompt);

	cb = CORBA_Object_is_nil(brutus_logon, ev);
	if (!ORBIT2_EX(ev) && !cb)
		CORBA_Object_release(brutus_logon, ev);

	if (!connected)
		memset((void*)priv->session_key, '\0', sizeof(priv->session_key));
	else if (get_session_key)
		get_random_guid(priv->session_key);
	d(priv->session_key);


	BRUTUS_STORE_UNLOCK(store, connect_mutex);
	d("");

out:
	do { // get time zone information - needed at all?
		if (!got_time_zones && connected) {
			struct timeval tv;
			struct timezone tz;
			gchar *tmp_exchange_server = NULL;
			BRUTUS_TIME_OF_DAY_INFO exchange_tod;

			tmp_exchange_server = g_strjoin(NULL, "\\\\", exchange_server, NULL);
			d(tmp_exchange_server);
			if (!tmp_exchange_server)
				break;

			cb = BRUTUS_IMAPISession_GetServerTime(priv->mapi_session, tmp_exchange_server, &exchange_tod, ev);
			g_free(tmp_exchange_server);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				break;
			}

			if (cb && (exchange_tod.tod_timezone != -1)) {
				if (gettimeofday(&tv, &tz)) // must not fail!
					break;

				priv->exchange_time_zone = (long)exchange_tod.tod_timezone;
				priv->client_time_zone = (long)tz.tz_minuteswest;
				got_time_zones = TRUE;

				dfmt("Exchange time zone = %ld", priv->exchange_time_zone);
				dfmt("Client time zone = %ld", priv->exchange_time_zone);
			}
		}
	} while (0);

#ifndef HAVE_ORBIT2_TIMEOUT
exit:
#endif
	CORBA_exception_free(ev);

	g_free(exchange_server);
	g_free(exchange_mailbox);
	g_free(exchange_mailbox_dn);
	g_free(exchange_mailbox_email);
	g_free(windows_domain);
	g_free(windows_user);

	d("");

	camel_exception_xfer(ex, &ex_internal);

	return connected;
}

static gboolean
brutus_disconnect(CamelService *service,
		  gboolean clean,
		  CamelException *ex)
{
	CamelBrutusStore *store = CAMEL_BRUTUS_STORE(service);
	CamelBrutusStorePrivate *priv = store->priv;

	d("");
	if (brutus_is_connected(priv->brutus_server,
				priv->mapi_session)) {
		BRUTUS_STORE_LOCK(store, connect_mutex);
		brutus_mapi_shutdown(store);
		BRUTUS_STORE_UNLOCK(store, connect_mutex);
	}

	return TRUE;
}

static CamelFolder*
brutus_get_outlook_folder_by_enum(CamelStore *store,
				  const BRUTUS_OUTLOOK_FOLDER which,
				  CamelException *ex)
{
	CamelFolder *retv = NULL;
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	BRUTUS_IMAPIFolder mapi_folder = CORBA_OBJECT_NIL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	const char *msg = NULL;
	CORBA_Environment ev[1];

	if ((BRUTUS_OUTLOOK_NONE == which) || (BRUTUS_OUTLOOK_DEVNULL == which))
		return NULL;

	CORBA_exception_init(ev);

	brutus_connect((CamelService *)brutus_store, ex);
	if (camel_exception_is_set(ex))
		goto out;

	br = BRUTUS_IMAPISession_GetOutlookFolder(priv->mapi_session,
						  which,
						  CORBA_OBJECT_NIL,
						  &mapi_folder,
						  ev);
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		switch (which) {
		case BRUTUS_OUTLOOK_INBOX :
			msg = _("Could not retrieve the Inbox folder");
			break;
		case BRUTUS_OUTLOOK_OUTBOX :
			msg = _("Could not retrieve the Outbox folder");
			break;
		case BRUTUS_OUTLOOK_WASTEBASKET :
			msg = _("Could not retrieve the Trash folder");
			break;
		case BRUTUS_OUTLOOK_SENTMAIL :
			msg = _("Could not retrieve the Sent folder");
			break;
		case BRUTUS_OUTLOOK_VIEWS :
			msg = _("Could not retrieve the Views folder");
			break;
		case BRUTUS_OUTLOOK_COMMON_VIEWS :
			msg = _("Could not retrieve the Common Views folder");
			break;
		case BRUTUS_OUTLOOK_FINDER :
			msg = _("Could not retrieve the Finder folder");
			break;
		case BRUTUS_OUTLOOK_CALENDAR :
			msg = _("Could not retrieve the Calendar folder");
			break;
		case BRUTUS_OUTLOOK_CONTACTS :
			msg = _("Could not retrieve the Contacts folder");
			break;
		case BRUTUS_OUTLOOK_JOURNAL :
			msg = _("Could not retrieve the Journal folder");
			break;
		case BRUTUS_OUTLOOK_NOTES :
			msg = _("Could not retrieve the Notes folder");
			break;
		case BRUTUS_OUTLOOK_TASKS :
			msg = _("Could not retrieve the Task folder");
			break;
		case BRUTUS_OUTLOOK_REMINDERS :
			msg = _("Could not retrieve the Reminders folder");
			break;
		case BRUTUS_OUTLOOK_DRAFTS :
			msg = _("Could not retrieve the Drafts folder");
			break;
		default :
			g_assert(FALSE);
		}
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM, msg);
		goto out;
	}
	retv = camel_brutus_folder_wrap(store, mapi_folder, ex);
out:
	CORBA_exception_free(ev);

	return retv;
}

static CamelFolder*
brutus_get_folder(CamelStore *store,
		  const char *folder_name,
		  guint32 flags,
		  CamelException *ex)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);

	if (!folder_name || !strlen(folder_name))
		return NULL;
	d(folder_name);

	if (brutus_connect((CamelService*)brutus_store, ex))
		return camel_brutus_folder_new(store, folder_name, flags, ex);
	else
		return camel_brutus_folder_new_offline(store, folder_name, flags, ex);
}

static CamelFolder*
brutus_get_inbox(CamelStore *store,
		 CamelException *ex)
{
	CamelFolder *retv = NULL;

	retv = brutus_get_outlook_folder_by_enum(store, BRUTUS_OUTLOOK_INBOX, ex);
	if (!retv)
		return NULL;

	// TODO: is this right?
	if (camel_url_get_param(((CamelService*)store)->url, "filter"))
		retv->folder_flags |= CAMEL_FOLDER_FILTER_RECENT;
	if (camel_url_get_param (((CamelService*)store)->url, "filter_junk"))
		retv->folder_flags |= CAMEL_FOLDER_FILTER_JUNK;

	return retv;
}

static CamelFolder*
brutus_get_trash(CamelStore *store,
		 CamelException *ex)
{
	return brutus_get_outlook_folder_by_enum(store, BRUTUS_OUTLOOK_WASTEBASKET, ex);
}

/* static CamelFolder* */
/* brutus_get_junk(CamelStore *store, */
/*		CamelException *ex) */
/* { */
/*	CamelFolder *retv = NULL; */
/*	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store); */

/*	d(""); */
/*	brutus_connect((CamelService *)brutus_store, ex); */
/*	if (camel_exception_is_set(ex)) */
/*		goto out; */

/*	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM, */
/*			    _("No Brutus Junk folder yet")); */

/* out: */
/*	return retv; */
/* } */

static CamelFolderInfo*
brutus_create_folder(CamelStore *store,
		     const char *parent_name,
		     const char *folder_name,
		     CamelException *ex)
{
	CamelFolderInfo *retv = NULL;
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	const BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_IMAPIFolder parent_mapi_folder = CORBA_OBJECT_NIL;
	BRUTUS_IMAPIFolder new_mapi_folder = CORBA_OBJECT_NIL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_BDEFINE obj_type = 0;
	CORBA_boolean cb = CORBA_TRUE;
	char *name = NULL;
	char *full_name = NULL;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	d(parent_name);
	d(folder_name);

	if (!parent_name) {
		d("FIXME!!");
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM, "Can not create a top-level folder");
		CORBA_exception_free(ev);
		return NULL;
	}

	brutus_connect((CamelService *)brutus_store, ex);
	if (camel_exception_is_set(ex))
		goto out;

	BRUTUS_PRIV_READ_LOCK(priv, folder_tree_rw_mutex);
	eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, parent_name);
	if (!eid) {
		BRUTUS_PRIV_READ_UNLOCK(priv, folder_tree_rw_mutex);
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Internal data corruption - Shutdown Evolution with \"evolution --force-shutdown\" in a shell, then restart Evolution"));
		goto out;
	}

	// prepare child folder name
	name = strdup(folder_name);
	if (!name) {
		BRUTUS_PRIV_READ_UNLOCK(priv, folder_tree_rw_mutex);
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("No memory"));
		goto out;
	}
	brutus_strip_trailing_char('/', name);
	full_name = g_strjoin(NULL, parent_name, "/", name, NULL);
	if (!full_name) {
		BRUTUS_PRIV_READ_UNLOCK(priv, folder_tree_rw_mutex);
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("No memory"));
		goto out;
	}

	// open parent folder
	br = BRUTUS_IMAPISession_OpenEntry(priv->mapi_session,
					   eid,
					   "",
					   BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
					   &obj_type,
					   &parent_mapi_folder,
					   ev);
	BRUTUS_PRIV_READ_UNLOCK(priv, folder_tree_rw_mutex);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Communication error"));
		goto out;
	}
	if (BRUTUS_BRUTUS_MAPI_E_NOT_FOUND == br) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Internal data corruption - Shutdown Evolution with \"evolution --force-shutdown\" in a shell, then restart Evolution"));
		goto out;
	}
	if (BRUTUS_BRUTUS_MAPI_E_NO_ACCESS == br) {
		camel_exception_set(ex, CAMEL_EXCEPTION_FOLDER_INSUFFICIENT_PERMISSION,
				    _("Could not open folder - Access Denied"));
		goto out;
	}

	if (!brutus_create_mapi_folder(parent_mapi_folder, "IPF.Note", name)) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not create folder"));
		goto out;
	}

	// open the new child folder
	new_mapi_folder = brutus_open_folder_by_name(parent_mapi_folder, name);
	cb = CORBA_Object_is_nil(new_mapi_folder, ev);
	if (ORBIT2_EX(ev) || cb)
		goto out;

	// add new branch
	brutus_insert_in_folder_tree(store,
				     parent_name,
				     full_name,
				     new_mapi_folder,
				     ex);

	BRUTUS_PRIV_READ_LOCK(priv, folder_tree_rw_mutex);
	retv = (CamelFolderInfo*)g_hash_table_lookup(priv->fi_from_full_name_hash, full_name);
	camel_object_trigger_event(CAMEL_OBJECT(store), "folder_created", retv);
	BRUTUS_PRIV_READ_UNLOCK(priv, folder_tree_rw_mutex);

out:
	brutus_release_object(BRUTUS_IMAPIFolder_tc, &parent_mapi_folder, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	brutus_release_object(BRUTUS_IMAPIFolder_tc, &new_mapi_folder, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	free(name);
	g_free(full_name);

	return retv;
}

static void
brutus_delete_folder(CamelStore *store,
		     const char *folder_name,
		     CamelException *ex)
{
	CamelFolderInfo *fi = NULL;
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	const BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_BDEFINE obj_type = 0;
	BRUTUS_IMAPIFolder parent_mapi_folder = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];
	int i;
	char *parent = NULL;
	char *tmp = NULL;
	char *summary_file_path = NULL;
	char *cache_file_path = NULL;
	guchar digest[16] = { 0 }; // md5 hash of long term ENTRYID
	gchar digest_str[33] = { '\0' }; // stringified version of digest
	struct timespec t = {
		.tv_sec = 0,
		.tv_nsec = 10000000 // 0.01 second
	};

	d(folder_name);
	brutus_connect((CamelService *)brutus_store, ex);
	if (camel_exception_is_set(ex))
		return;
	CORBA_exception_init(ev);

	// is the folder empty?
	fi = brutus_get_folder_info(store, folder_name, 0, ex);
	if (!fi) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Internal data corruption - Shutdown Evolution with \"evolution --force-shutdown\" in a shell, then restart Evolution"));
		goto out_no_lock;
	}
	if ((fi->child) || (fi->total))
		goto out_no_lock;

	// get parent name
	parent = strdup(folder_name);
	if (!parent) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("No memory"));
		goto out_no_lock;
	}
	brutus_strip_trailing_char('/', parent);
	tmp = strrchr(parent, '/');
	if (tmp)
		*tmp = '\0';
	else {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Can not delete top folder"));
		goto out_no_lock; // can not delete top folder
	}

	BRUTUS_PRIV_WRITE_LOCK(priv, folder_tree_rw_mutex);

	// avoid collision with an asynchronous folder summary update in progress
	while (!folder_delete_is_a_go(folder_name))
		nanosleep(&t, NULL); // sleep for 0.01 second

	// get parent eid
	eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, parent);
	if (!eid) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Internal data corruption - Shutdown Evolution with \"evolution --force-shutdown\" in a shell, then restart Evolution"));
		goto out;
	}

	// open parent folder
	br = BRUTUS_IMAPISession_OpenEntry(priv->mapi_session,
					   eid,
					   "",
					   BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
					   &obj_type,
					   &parent_mapi_folder,
					   ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto ex;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto ex;
	}

	// get child eid
	eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, folder_name);
	if (!eid) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Internal data corruption - Shutdown Evolution with \"evolution --force-shutdown\" in a shell, then restart Evolution"));
		goto out;
	}

	// delete the child
	br = BRUTUS_IMAPIFolder_DeleteFolder(parent_mapi_folder,
					     eid,
					     CORBA_OBJECT_NIL,
					     BRUTUS_BRUTUS_DEL_FOLDERS | BRUTUS_BRUTUS_DEL_MESSAGES,
					     ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto ex;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto ex;
	}

	// remove cache files
	md5_get_digest(folder_name, strlen(folder_name), digest);
	for (i = 0; i < sizeof(digest); i++)
		snprintf(&digest_str[2*i], 3, "%02x", (guint8)digest[i]);

	// construct the paths
	summary_file_path = g_strdup_printf("%s/%s/%s", priv->storage_path, FOLDER_SUMMARIES, digest_str);
	cache_file_path = g_strdup_printf("%s/%s/%s", priv->storage_path, FOLDER_MESSAGE_CACHE, digest_str);

	// delete the summary and message cache files
	tmp = g_strdup_printf("/usr/bin/rm -rf %s", summary_file_path);
	if (-1 == system(tmp))
		d("Could not delete folder summary cache");
	g_free(tmp);

	tmp = g_strdup_printf("/usr/bin/rm -rf %s", cache_file_path);
	if (-1 == system(tmp))
		d("Could not delete message cache");
	g_free(tmp);

	g_free(summary_file_path);
	g_free(cache_file_path);

	// sent event
	camel_object_trigger_event(store, "folder_deleted", fi);

	// update folder info tree
	brutus_delete_branch(priv, fi);

	goto out;
ex:
	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
			    _("Could not delete folder"));
out:
	folder_is_done(folder_name);
	BRUTUS_PRIV_WRITE_UNLOCK(priv, folder_tree_rw_mutex);

out_no_lock:
	brutus_release_object(BRUTUS_IMAPIFolder_tc, &parent_mapi_folder, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	free(parent);

	return;
}

static void
brutus_folder_rename(CamelBrutusStore *brutus_store,
		     const char *old_name,
		     const char *new_name,
		     CamelException *ex)
{
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	const char *name = NULL;
	char *tmp = NULL;
	BRUTUS_BDEFINE obj_type = 0;
	BRUTUS_IMAPIFolder folder = CORBA_OBJECT_NIL;
	CamelFolderInfo *fi = NULL;
	const BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_ENTRYID *tmp_eid = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_SPropProblemArray *mapi_problems = NULL;
	BRUTUS_SPropValue pv = {
		.ulPropTag = BRUTUS_BRUTUS_PR_DISPLAY_NAME,
		.Value._d = BRUTUS_BRUTUS_PT_STRING8,
		.Value._u.lpszA = NULL,
	};
	const BRUTUS_seq_SPropValue prop_values = {
		._maximum = 1,
		._length = 1,
		._buffer = &pv,
	};
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	BRUTUS_PRIV_WRITE_LOCK(priv, folder_tree_rw_mutex);

	eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, old_name);
	br = BRUTUS_IMAPISession_OpenEntry(priv->mapi_session,
					   eid,
					   "",
					   BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
					   &obj_type,
					   &folder,
					   ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto ex;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto ex;
	}

	tmp = strdup(new_name);
	brutus_strip_trailing_char('/', tmp); // no terminating '/'

	name = strrchr(tmp, '/');
	if (name)
		name++;
	else
		name = tmp;
	pv.Value._u.lpszA = CORBA_string_dup(name);

	mapi_problems = BRUTUS_seq_SPropProblem__alloc();
	mapi_problems->_maximum = 0;
	mapi_problems->_length = 0;
	mapi_problems->_buffer = NULL;
	br = BRUTUS_IMAPIFolder_SetProps(folder,
					 &prop_values,
					 FALSE,
					 &mapi_problems,
					 ev);
	CORBA_free(mapi_problems);
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto ex;
	}

	br = BRUTUS_IMAPIFolder_SaveChanges(folder, 0, ev);
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto ex;
	}

	// change the name in the folder info tree
	fi = (CamelFolderInfo*)g_hash_table_lookup(priv->fi_from_full_name_hash, old_name);
	tmp_eid = brutus_entryid_dup(eid);
	brutus_delete_from_hash_tables(priv, old_name, eid);

	g_free(fi->name);
	fi->name = g_strdup(name);
	g_free(fi->full_name);
	fi->full_name = g_strdup(tmp);

	brutus_add_to_hash_tables(priv, fi, tmp_eid);

	goto out;
ex:
	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
			    _("Could not rename folder"));
out:
	BRUTUS_PRIV_WRITE_UNLOCK(priv, folder_tree_rw_mutex);

	brutus_release_object(BRUTUS_IMAPIFolder_tc, &folder, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_free(pv.Value._u.lpszA);
	free(tmp);

	CORBA_exception_free(ev);
}

static void
brutus_folder_move(CamelBrutusStore *brutus_store,
		   const char *old_name,
		   const char *new_name, // no trailing '/'
		   CamelException *ex)
{
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_BDEFINE obj_type = 0;
	BRUTUS_IMAPIFolder dest_folder = CORBA_OBJECT_NIL;
	BRUTUS_IMAPIFolder moved_folder = CORBA_OBJECT_NIL;
	CamelFolderInfo *fi = NULL;
	CORBA_boolean cb = CORBA_TRUE;
	const BRUTUS_ENTRYID *new_parent_eid = NULL;
	const BRUTUS_ENTRYID *eid = NULL;
	CORBA_Environment ev[1];
	char *full_parent_name = NULL;
	char *base_name = NULL;
	char *tmp = NULL;

	CORBA_exception_init(ev);

	tmp = g_strdup(new_name);
	brutus_strip_trailing_char('/', tmp); // no terminating '/'

	// basename() - it _must_ be a subfolder
	base_name = strrchr(tmp, '/');
	if (base_name)
		base_name++;
	else
		g_assert(FALSE);

	// get new parent folder name
	full_parent_name = tmp;
	tmp = strrchr(full_parent_name, '/');
	*tmp = '\0';

	BRUTUS_PRIV_WRITE_LOCK(priv, folder_tree_rw_mutex);

	new_parent_eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, full_parent_name);
	eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, old_name);

	// open the destination folder
	br = BRUTUS_IMAPISession_OpenEntry(priv->mapi_session,
					   new_parent_eid,
					   "",
					   BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
					   &obj_type,
					   &dest_folder,
					   ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto ex;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto ex;
	}

	// move the child
	br = BRUTUS_IMAPIFolder_CopyFolder(dest_folder,
					   eid,
					   "",
					   new_parent_eid,
					   base_name,
					   CORBA_OBJECT_NIL,
					   BRUTUS_BRUTUS_FOLDER_MOVE,
					   ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto ex;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto ex;
	}

	// delete old branch
	fi = (CamelFolderInfo*)g_hash_table_lookup(priv->fi_from_full_name_hash, old_name);
	brutus_delete_branch(priv, fi);

	// get moved_folder reference
	moved_folder = brutus_open_folder_by_name(dest_folder, base_name);
	cb = CORBA_Object_is_nil(moved_folder, ev);
	if (ORBIT2_EX(ev) || cb)
		goto ex;

	BRUTUS_PRIV_WRITE_UNLOCK(priv, folder_tree_rw_mutex);

	// add new branch
	brutus_insert_in_folder_tree((CamelStore*)brutus_store,
				     full_parent_name,
				     new_name,
				     moved_folder,
				     ex);
	goto out; // all is well

ex:
	BRUTUS_PRIV_WRITE_UNLOCK(priv, folder_tree_rw_mutex);
	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
			    _("Could not move folder"));

out:

	brutus_release_object(BRUTUS_IMAPIFolder_tc, &dest_folder, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	brutus_release_object(BRUTUS_IMAPIFolder_tc, &moved_folder, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	g_free(full_parent_name);

	CORBA_exception_free(ev);
}

static void
brutus_rename_folder(CamelStore *store,
		     const char *old_name,
		     const char *new_name,
		     CamelException *ex)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);

	d("");
	if ((!old_name) || (!new_name) || !strlen(old_name) || !strlen(new_name)) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Invalid folder name"));
		return;
	}

	brutus_connect((CamelService *)brutus_store, ex);
	if (camel_exception_is_set(ex))
		return;

	// a real move or just a plain rename?
	if (brutus_is_siblings(old_name, new_name))
		brutus_folder_rename(brutus_store, old_name, new_name, ex);
	else
		brutus_folder_move(brutus_store, old_name, new_name, ex);
}

/* static void */
/* brutus_sync(CamelStore *store, */
/*	    int expunge, */
/*	    CamelException *ex) */
/* { */
/*	return; */
/* } */

static CamelFolderInfo*
brutus_get_folder_info(CamelStore *store,
		       const char *top,
		       guint32 flags,
		       CamelException *ex)
{

	static GStaticMutex get_fi_mutex = G_STATIC_MUTEX_INIT;
	static gboolean prev_connect_state = FALSE;
	gboolean connected = FALSE;
	CamelFolderInfo *retv = NULL;
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;

	d(top);
	g_static_mutex_lock(&get_fi_mutex);

	d(top);
	connected = brutus_connect((CamelService *)brutus_store, ex);
	if (!connected) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
				    _("Warning: Using cached folder information"));
	}

	BRUTUS_PRIV_WRITE_LOCK(priv, folder_tree_rw_mutex);

	// Rebuild hash tables from scratch when switching from the
	// offline state to the online one. This is the only way to
	// catch changes to the store that happened behind our back
	// when we were offline
	if (!prev_connect_state && connected && priv->folder_tree) {
		brutus_delete_branch(priv, priv->folder_tree);
		priv->folder_tree = NULL;
	}
	prev_connect_state = connected;

	if (!priv->folder_tree) {
		if (connected) {
			brutus_build_folder_tree(store, ex);
			brutus_save_folder_tree(priv, NULL, ex);
		} else {
			brutus_build_folder_tree_from_cache(store);
		}
	}
	BRUTUS_PRIV_WRITE_UNLOCK(priv, folder_tree_rw_mutex);

	BRUTUS_PRIV_READ_LOCK(priv, folder_tree_rw_mutex);
	if (!priv->folder_tree)
		goto out;

	if (!top || ('\0' == *top))
		retv = priv->folder_tree;
	else
		retv = (CamelFolderInfo*)g_hash_table_lookup(priv->fi_from_full_name_hash, (gpointer)top);
out:
	BRUTUS_PRIV_READ_UNLOCK(priv, folder_tree_rw_mutex);
	g_static_mutex_unlock(&get_fi_mutex);

	return retv;
}

void
brutus_rebuild_folder_tree(CamelStore *store,
			   CamelException *ex)
{
	brutus_get_folder_info(store, NULL, 0, ex);

	return;
}

static gboolean
brutus_folder_subscribed(CamelStore *store,
			 const char *folder_name)
{
	gboolean retv = FALSE;
	CamelException *ex = camel_exception_new();
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);

	d("");
	brutus_connect((CamelService *)brutus_store, ex);
	if (camel_exception_is_set(ex))
		goto out;
out:
	camel_exception_free(ex);

	return retv;
}

static void
brutus_subscribe_folder(CamelStore *store,
			const char *folder_name,
			CamelException *ex)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);

	d("");
	brutus_connect((CamelService *)brutus_store, ex);
	if (camel_exception_is_set(ex))
		return;

	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
			    _("No support"));
}

static void
brutus_unsubscribe_folder(CamelStore *store,
			  const char *folder_name,
			  CamelException *ex)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);

	d("");
	brutus_connect((CamelService *)brutus_store, ex);
	if (camel_exception_is_set(ex))
		return;

	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
			    _("No support"));
}

/* Frees the private structure */
static void
camel_brutus_store_finalize(CamelObject *object)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(object);
	CamelBrutusStorePrivate *priv = brutus_store->priv;

	d("");
	if (priv) {
		CORBA_boolean cb = CORBA_TRUE;
		CORBA_Environment ev[1];
		CORBA_exception_init(ev);

		if (priv->mapi_profile)
			CORBA_free(priv->mapi_profile);

		BRUTUS_PRIV_WRITE_LOCK(priv, folder_tree_rw_mutex);
		if (priv->folder_tree) {
			brutus_delete_branch(priv, priv->folder_tree);
			priv->folder_tree = NULL;
		}
		if (priv->fi_from_entryid_hash) {
			g_hash_table_destroy(priv->fi_from_entryid_hash);
			priv->fi_from_entryid_hash= NULL;
		}
		if (priv->fi_from_full_name_hash) {
			g_hash_table_destroy(priv->fi_from_full_name_hash);
			priv->fi_from_full_name_hash = NULL;
		}
		if (priv->entryid_from_full_name_hash) {
			g_hash_table_destroy(priv->entryid_from_full_name_hash);
			priv->entryid_from_full_name_hash = NULL;
		}
		if (priv->fi_cache) {
			camel_object_unref(CAMEL_OBJECT(priv->fi_cache));
			priv->fi_cache = NULL;
		}
		BRUTUS_PRIV_WRITE_UNLOCK(priv, folder_tree_rw_mutex);

		// free the MAPI session
		d("Shutting down MAPI");
		if (brutus_is_connected(priv->brutus_server,
					priv->mapi_session)) {
			BRUTUS_STORE_LOCK(brutus_store, connect_mutex);
			brutus_mapi_shutdown(brutus_store);
			BRUTUS_STORE_UNLOCK(brutus_store, connect_mutex);
		}

		// destroy the lifeline
		brutus_free_lifeline(priv->brutus_proxy,
				     &priv->lifeline);

		// destroy RootPOA
		cb = CORBA_Object_is_nil((CORBA_Object)priv->root_poa, ev);
		if (!ORBIT2_EX(ev) && !cb) {
			CORBA_Object_release((CORBA_Object)priv->root_poa, ev);
			PRINT_IF_ORBIT2_EX(ev);
		}
		priv->root_poa = CORBA_OBJECT_NIL;

		// destroy ORB
		cb = CORBA_Object_is_nil((CORBA_Object)priv->orb, ev);
		if (!ORBIT2_EX(ev) && !cb) {
			CORBA_Object_release((CORBA_Object)priv->orb, ev);
			PRINT_IF_ORBIT2_EX(ev);
		}
		priv->orb = CORBA_OBJECT_NIL;

		CORBA_exception_free(ev);
		d("CORBA is down");

		// mutexes
		g_mutex_free(priv->connect_mutex);
		g_static_rw_lock_free(&priv->folder_tree_rw_mutex);

		// strings
		g_free(priv->base_url);
		g_free(priv->storage_path);
		g_free(priv->brutus_logon_str);

		g_free(priv);
		brutus_store->priv = NULL;
	}

	g_thread_pool_free(folder_summary_updates_thread_pool, TRUE, TRUE);

	free_folder_busy_list();
	g_async_queue_unref(wait_mem_queue);
}

static void
camel_brutus_store_init(gpointer object,
			gpointer klass)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(object);
	CamelBrutusStorePrivate *priv = g_new0(CamelBrutusStorePrivate, 1);

	d("");
	priv->orb = CORBA_OBJECT_NIL;
	priv->root_poa = CORBA_OBJECT_NIL;

	priv->use_ssliop = FALSE;

	priv->mapi_profile = NULL;
	priv->mapi_session = CORBA_OBJECT_NIL;
	priv->instance_id = 0;

	priv->brutus_proxy = CORBA_OBJECT_NIL;
	priv->lifeline.wrapper = CORBA_OBJECT_NIL;
	priv->lifeline.filling = CORBA_OBJECT_NIL;

	priv->exchange_time_zone = 0;
	priv->client_time_zone = 0;

	priv->mapi_msg_store_table = CORBA_OBJECT_NIL;
	priv->msg_stores = NULL;

	priv->folder_tree = NULL;
	priv->entryid_from_full_name_hash = NULL;
	priv->fi_from_entryid_hash = NULL;
	priv->fi_from_full_name_hash = NULL;

	priv->base_url = NULL;
	priv->storage_path = NULL;
	priv->brutus_logon_str = NULL;

	priv->fi_cache = NULL;

	priv->connect_mutex = g_mutex_new();
	g_static_rw_lock_init(&priv->folder_tree_rw_mutex);

	brutus_store->priv = priv;
}

CamelType
camel_brutus_store_get_type(void)
{
	static CamelType camel_brutus_store_type = CAMEL_INVALID_TYPE;

	if (camel_brutus_store_type == CAMEL_INVALID_TYPE) {
		camel_brutus_store_type =
			camel_type_register(camel_offline_store_get_type(),
					    "CamelBrutusStore",
					    sizeof(CamelBrutusStore),
					    sizeof(CamelBrutusStoreClass),
					    (CamelObjectClassInitFunc)camel_brutus_store_class_init,
					    NULL,
					    (CamelObjectInitFunc)camel_brutus_store_init,
					    (CamelObjectFinalizeFunc)camel_brutus_store_finalize);
	}

	return camel_brutus_store_type;
}

gboolean
brutus_insert_in_folder_tree(CamelStore *store,
			     const char *parent_name,
			     const char *full_name,
			     BRUTUS_IMAPIFolder mapi_folder,
			     CamelException *ex)
{
	gboolean retv = FALSE;
	char *name = NULL; // PR_ENTRYID by definition
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	CamelFolderInfo *fi = NULL;
	CamelFolderInfo *self = NULL;
	CORBA_long n = 0;
	CORBA_unsigned_long r = 0;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_BDEFINE obj_type;
	const BRUTUS_ENTRYID *parent_eid = NULL;
	BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_IMAPIFolder parent_folder = CORBA_OBJECT_NIL;
	BRUTUS_IMAPITable hierachy = CORBA_OBJECT_NIL;
	BRUTUS_SRowSet *row_set = NULL;
	BRUTUS_SPropValue *pv = NULL;
	CORBA_Environment ev[1];
	static BRUTUS_BDEFINE props[8] = {
		BRUTUS_BRUTUS_PR_LONGTERM_ENTRYID_FROM_TABLE,
		BRUTUS_BRUTUS_PR_ENTRYID, // short-term fallback
		BRUTUS_BRUTUS_PR_DISPLAY_NAME,
		BRUTUS_BRUTUS_PR_SUBFOLDERS,
		BRUTUS_BRUTUS_PR_FOLDER_TYPE,
		BRUTUS_BRUTUS_PR_STATUS,
		BRUTUS_BRUTUS_PR_CONTENT_COUNT,
		BRUTUS_BRUTUS_PR_CONTENT_UNREAD
	};
	static BRUTUS_SPropTagArray proptag_array = {
		._maximum = 8,
		._length = 8,
		._buffer = props,
	};
	static BRUTUS_SSortOrderSet sort_order = {
		.cCategories = 0,
		.cExpanded = 0,
		.aSort._maximum = 0,
		.aSort._length = 0,
		.aSort._buffer = NULL,
	};

	CORBA_exception_init(ev);

	if (!brutus_is_connected(priv->brutus_server,
				 priv->mapi_session)) {
		camel_exception_set(ex,
				    CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
				    _("Not connected"));
		goto out_no_lock;
	}

	name = strrchr(full_name, '/');
	if (name)
		name++;
	else
		name = (char*)full_name;

	BRUTUS_PRIV_WRITE_LOCK(priv, folder_tree_rw_mutex);

	fi = (CamelFolderInfo*)g_hash_table_lookup(priv->fi_from_full_name_hash, parent_name);
	if (!fi) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Internal data corruption - Shutdown Evolution with \"evolution --force-shutdown\" in a shell, then restart Evolution"));
		goto out;
	}

	// get parent eid
	parent_eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, parent_name);
	if (!parent_eid) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Internal data corruption - Shutdown Evolution with \"evolution --force-shutdown\" in a shell, then restart Evolution"));
		goto out;
	}

	// open parent folder
	br = BRUTUS_IMAPISession_OpenEntry(priv->mapi_session,
					   parent_eid,
					   "",
					   BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
					   &obj_type,
					   &parent_folder,
					   ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br)
		goto out;

	// get hierachy table
	br = BRUTUS_IMAPIFolder_GetHierarchyTable(parent_folder, 0, &hierachy, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br)
		goto out;

	// get all rows
	br = brutus_QueryAllRows(hierachy,
				 &proptag_array,
				 CORBA_OBJECT_NIL,
				 &sort_order,
				 0,
				 &row_set,
				 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);

		brutus_release_object(BRUTUS_IMAPITable_tc, &hierachy, CORBA_OBJECT_NIL, ev);
		PRINT_IF_ORBIT2_EX(ev);

		goto out;
	}
	brutus_release_object(BRUTUS_IMAPITable_tc, &hierachy, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);
	if (BRUTUS_BRUTUS_S_OK != br)
		goto out;


	if (!row_set->_length)
		goto out;

	// locate the correct child
	for (r = 0;  r <= row_set->_length; r++) {
		if (r == row_set->_length)
			goto out; // not found

		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_DISPLAY_NAME);
		if (-1 == n)
			continue;

		if (!g_ascii_strcasecmp(name, row_set->_buffer[r]._buffer[n].Value._u.lpszA))
			break;
	}

	/* r is now pointing at the correct entry in the hierarchy table */

	/* hidden system folder? */
	n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_STATUS);
	if (-1 != n) {
		if (row_set->_buffer[r]._buffer[n].Value._u.l & BRUTUS_BRUTUS_FLDSTATUS_HIDDEN) {
#ifdef BRUTUS_SPY
			self = g_new0(CamelFolderInfo, 1);
			self->flags = CAMEL_FOLDER_SYSTEM;
#else
			goto out; // do not show hidden folders unless spying is on
#endif
		}
	} else {
		self = g_new0(CamelFolderInfo, 1);
		self->flags = 0;
	}

	// put the child in before the other children
	self->next = fi->child;
	fi->child = self;
	self->parent = fi;
	self->uri = brutus_build_uri(priv->base_url, full_name, ex);
	self->name = g_strdup(name);
	self->full_name = g_strdup(full_name);
	self->child = NULL;

	n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_CONTENT_UNREAD);
	if (-1 == n)
		self->unread = -1;
	else
		self->unread = (guint32)row_set->_buffer[r]._buffer[n].Value._u.l;

	n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_CONTENT_COUNT);
	if (-1 == n)
		self->total = -1;
	else
		self->total = (guint32)row_set->_buffer[r]._buffer[n].Value._u.l;

	n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_FOLDER_TYPE);
	if (-1 != n) {
		if (row_set->_buffer[r]._buffer[n].Value._u.l & BRUTUS_BRUTUS_FOLDER_SEARCH)
			self->flags |= CAMEL_FOLDER_VIRTUAL;
	}

	n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_LONGTERM_ENTRYID_FROM_TABLE);
	if (-1 == n) { // use fallback property
		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_ENTRYID);
		if (-1 == n) {
			g_free(self->uri);
			g_free(self->name);
			g_free(self->full_name);
			g_free(self);
			camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
					    _("Communication error with server"));
			goto out;
		} else {
			brutus_sbinary_to_entryid(&row_set->_buffer[r]._buffer[n].Value._u.bin, &eid);
			pv = brutus_open_object_get_one_prop(CORBA_OBJECT_NIL,
							     parent_folder,
							     eid,
							     BRUTUS_BRUTUS_PR_ENTRYID);
			CORBA_free(eid);
			brutus_sbinary_to_entryid(&pv->Value._u.bin, &eid);
			CORBA_free(pv);
		}
	} else {
		brutus_sbinary_to_entryid(&row_set->_buffer[r]._buffer[n].Value._u.bin, &eid);
	}
	brutus_add_to_hash_tables(priv, self, eid);

	/* children? */
	n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_SUBFOLDERS);
	if (-1 == n)
		goto out;
	if ((guint32)row_set->_buffer[r]._buffer[n].Value._u.b) {
		self->flags |= CAMEL_FOLDER_CHILDREN;
		brutus_build_folder_hierachy(store, mapi_folder, self, FALSE, ex);
	} else
		self->flags |= CAMEL_FOLDER_NOCHILDREN;

out:
	BRUTUS_PRIV_WRITE_UNLOCK(priv, folder_tree_rw_mutex);

out_no_lock:
	brutus_release_object(BRUTUS_IMAPIFolder_tc, &parent_folder, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	return retv;
}
