/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Authors: Jules Colding <colding@omesc.com>
 *
 *    Camel Brutus transport class.
 *
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* system includes */
#include <string.h>
#include <errno.h>

/* camel includes */
#include <camel/camel-i18n.h>
#include <camel/camel-session.h>
#include <camel/camel-service.h>
#include <camel/camel-stream.h>
#include <camel/camel-stream-mem.h>
#include <camel/camel-medium.h>
#include <camel/camel-data-wrapper.h>
#include <camel/camel-mime-message.h>
#include <camel/camel-multipart.h>
#include <camel/camel-mime-utils.h>

/* camel-brutus includes */
#include "camel-brutus-transport.h"
#include "camel-brutus-store.h"

/* pure brutus includes */
#include <idl_output/IMAPISession.h>
#include <idl_output/IMAPIFolder.h>
#include <idl_output/IMessage.h>

/* evolution-brutus includes */
#include <server/brutus.h>
#include <server/brutus_mapi.h>
#include <server/brutus_conv.h>
#include <server/brutus_corba.h>
#include <server/brutus_camel.h>

static CamelTransportClass *parent_class = NULL;

/******************************
 *   static helper functions  *
 *****************************/

static void
brutus_scan_message(CamelMimeMessage *msg, CamelMimePart *part)
{
	int i;
	int parts;
	CamelDataWrapper *dw = NULL;
	CamelContentType *type = NULL;

	dw = camel_medium_get_content_object(CAMEL_MEDIUM(part));
	if (!dw) {
		d("");
		return;
	}

	if (CAMEL_IS_MIME_MESSAGE(dw)) {
		d("");
		brutus_scan_message(msg, CAMEL_MIME_PART(dw));
	} else if (CAMEL_IS_MULTIPART(dw)) {

		parts = camel_multipart_get_number(CAMEL_MULTIPART(dw));
		dfmt("%d", parts);

		for (i = 0; i < parts; i++)
			brutus_scan_message(msg, camel_multipart_get_part(CAMEL_MULTIPART(dw), i));
 	} else {
		type = camel_mime_part_get_content_type(part);
		dfmt2("%s/%s", type->type, type->subtype);
	}
}

/******************************************************
 *   transport and service virtual method overloads   *
 *****************************************************/

static char *brutus_transport_get_name(CamelService *service,
				       gboolean brief);

static void brutus_transport_construct(CamelService *service,
				       CamelSession *session,
				       CamelProvider *provider,
				       CamelURL *url,
				       CamelException *ex);

static gboolean brutus_send_to(CamelTransport *transport,
			       CamelMimeMessage *message,
			       CamelAddress *from,
			       CamelAddress *recipients,
			       CamelException *ex);

/**********************************************
 ************ IMPLEMENTATIONS *****************
 *********************************************/

static void
camel_brutus_transport_class_init (CamelBrutusTransportClass *camel_brutus_transport_class)
{
	CamelTransportClass *camel_transport_class = CAMEL_TRANSPORT_CLASS(camel_brutus_transport_class);
	CamelServiceClass *camel_service_class = CAMEL_SERVICE_CLASS(camel_brutus_transport_class);

	parent_class = CAMEL_TRANSPORT_CLASS(camel_type_get_global_classfuncs(camel_transport_get_type()));

	/* virtual method overload */
	camel_service_class->get_name = brutus_transport_get_name;
	camel_service_class->construct = brutus_transport_construct;
	camel_transport_class->send_to = brutus_send_to;
}

static void
brutus_transport_construct(CamelService *service,
			   CamelSession *session,
			   CamelProvider *provider,
			   CamelURL *url,
			   CamelException *ex)
{
	CAMEL_SERVICE_CLASS(parent_class)->construct(service, session, provider, url, ex);
}

CamelType
camel_brutus_transport_get_type(void)
{
	static CamelType camel_brutus_transport_type = CAMEL_INVALID_TYPE;

	if (camel_brutus_transport_type == CAMEL_INVALID_TYPE) {
		camel_brutus_transport_type =
			camel_type_register(CAMEL_TRANSPORT_TYPE,
					    "CamelBrutusTransport",
					    sizeof(CamelBrutusTransport),
					    sizeof(CamelBrutusTransportClass),
					    (CamelObjectClassInitFunc)camel_brutus_transport_class_init,
					    NULL,
					    (CamelObjectInitFunc)NULL,
					    NULL);
	}

	return camel_brutus_transport_type;
}

static char*
brutus_transport_get_name(CamelService *service,
			  gboolean brief)
{
	if (brief) 
		return g_strdup (_("Exchange"));
	else 
		return g_strdup_printf(_("Exchange mail delivery for %s on %s"),
				       service->url->user,
				       service->url->host);
}

static gboolean
brutus_send_to(CamelTransport *transport,
	       CamelMimeMessage *message,
	       CamelAddress *from,
	       CamelAddress *recipients,
	       CamelException *ex)
{
	CamelService *service = CAMEL_SERVICE(transport);
	CamelStore *store =  NULL;
	CamelBrutusStore *brutus_store = NULL;
	CamelBrutusStorePrivate *priv = NULL;
	gboolean retv = FALSE;
	char *url = NULL;
	CORBA_Environment ev[1];
	BRUTUS_ENTRYID *msg_eid = NULL;
	BRUTUS_ENTRYID *dest_eid = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_IMAPIFolder outbox = CORBA_OBJECT_NIL;
	BRUTUS_IMessage mapi_message = CORBA_OBJECT_NIL;
	BRUTUS_seq_ENTRYID msg_id_list = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};

#ifdef BRUTUS_DEBUG
	brutus_scan_message(message, CAMEL_MIME_PART(message));
#endif

	CORBA_exception_init(ev);

	//
	// preparing to send
	//

	if (!camel_session_is_online(service->session)) {
		d("Offline");
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
				    _("Cannot send message: Evolution is offline"));
		goto out;
	}

	url = camel_url_to_string(service->url,
				  (CAMEL_URL_HIDE_PASSWORD
				   | CAMEL_URL_HIDE_PARAMS
				   | CAMEL_URL_HIDE_AUTH));
	store = camel_session_get_store(service->session, url, ex);
	g_free(url);
	if (!store) {
		d("ERROR: Could not get a pointer to the store");
		camel_exception_set(ex, CAMEL_EXCEPTION_STORE_INVALID,
				    _("Cannot get folder: Invalid operation on this store"));
		goto out;
	}
	brutus_store = CAMEL_BRUTUS_STORE(store);
	priv = brutus_store->priv;

	if (!brutus_is_connected(priv->brutus_server,
				 priv->mapi_session)) {
		d("Not connected");
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
				    _("Cannot send message: Service not connected"));
		goto out;
	}

	br = BRUTUS_IMAPISession_GetOutlookFolder(priv->mapi_session,
						  BRUTUS_OUTLOOK_OUTBOX,
						  CORBA_OBJECT_NIL,
						  &outbox,
						  ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Cannot send message: Communication error"));
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Cannot send message: Server error"));
		goto out; // outbox is nil here
	}

	//
	// starting send operations
	//

	camel_operation_start(NULL, _("Sending Message"));

	br = BRUTUS_IMAPIFolder_CreateMessage(outbox, "", 0, &mapi_message, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Cannot send message: Communication error"));
		goto out_end;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Cannot send message: Server error"));
		goto out_end;
	}

	// convert to MAPI
	if (!brutus_camel_mime_message_to_mapi(message, mapi_message, priv->mapi_session, priv->root_poa, NULL, ex))
		goto out_end;

	// Submit the message. SaveChanges() is called internally.
	br = BRUTUS_IMessage_SubmitMessage(mapi_message, 0, BRUTUS_OUTLOOK_SENTMAIL, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Cannot send message: Communication error"));
		goto out_end;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not submit outgoing message"));
		goto out_end;
	}
	
out_end:
	camel_operation_end(NULL);
out:
	if (store)
		camel_object_unref(CAMEL_OBJECT(store));

        brutus_free_seq_entryid_content(&msg_id_list);

	CORBA_free(msg_eid);
	CORBA_free(dest_eid);

	brutus_release_object(BRUTUS_IMAPIFolder_tc, &outbox, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	brutus_release_object(BRUTUS_IMessage_tc, &mapi_message, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	return retv;
}
