/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus keyring test program.
 *    Copyright (C) 2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib/gstdio.h>

#include <server/brutus.h>
#include <brutus-keyring/brutus-keyring.h>
#include <brutus-keyring/brutus-keyringd.h>

// Reads all of file_name into *secret.
static gboolean
read_file(const char *file_name,
	  BRUTUS_BrutusKeyring_KeyringData *secret)
{
	gboolean retv = FALSE;
	int fd = -1;
	ssize_t len;
	struct stat fstatus;

	if (stat(file_name, &fstatus))
		return FALSE;

	secret->_maximum = fstatus.st_size;
	secret->_length = secret->_maximum;
	secret->_buffer = BRUTUS_BrutusKeyring_KeyringData_allocbuf(secret->_maximum);
	if (!secret->_buffer) {
		if (!fstatus.st_size)
			retv = TRUE;
		goto out;
	}

	fd = open(file_name, O_RDONLY | O_NOCTTY | O_NOFOLLOW, 0);
	if (-1 == fd)
		goto out;

	len = read(fd, (void*)secret->_buffer, secret->_length);
	if (-1 != len)
		retv = TRUE;

out:
	if (!retv) {
		CORBA_free(secret->_buffer);
		secret->_maximum = 0;
		secret->_length = 0;
		secret->_buffer = NULL;
	}
	close(fd);

	return retv;
}

// Reads all of file_name into *secret.
static gboolean
write_file(const char *file_name,
	   const BRUTUS_BrutusKeyring_KeyringData *secret)
{
	gboolean retv = FALSE;
	int fd = -1;
	ssize_t len;

	fd = open(file_name, O_CREAT | O_TRUNC | O_WRONLY | O_NOCTTY | O_NOFOLLOW, 0);
	if (-1 == fd)
		goto out;

	len = write(fd, (void*)secret->_buffer, secret->_length);
	if (len == secret->_length)
		retv = TRUE;

out:
	close(fd);

	return retv;
}

static CORBA_ORB
create_orb(void)
{
	CORBA_ORB orb = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];
	int init_argc = 2;
	char **init_argv = (char**)malloc(sizeof(char*) * (init_argc));
	if (!init_argv)
		return CORBA_OBJECT_NIL;

	CORBA_exception_init(ev);

	memset((void*)init_argv, 0, init_argc);
	init_argv[0] = strdup("DUMMY_ARGV0");

	// Explicitly force ORBit2 to be local only
	init_argv[1] = "--ORBLocalOnly=1";

	// initialize the ORB
	orb = CORBA_ORB_init(&init_argc, init_argv, "brutus-query_orb_orbit-io-thread", ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		orb = CORBA_OBJECT_NIL;
	}

	free(init_argv[0]);
	free(init_argv);

	CORBA_exception_free(ev);

	return orb;
}

static CORBA_Object
brutus_object_ref_from_file(CORBA_ORB orb,
			    gchar *filename,
			    CORBA_Environment *ev)
{
	CORBA_Object obj = CORBA_OBJECT_NIL;
	CORBA_char *objref = NULL;
	FILE *file = NULL;
	struct stat st;
	size_t c = 0;

	file = g_fopen(filename, "r");
	if (!file)
		return CORBA_OBJECT_NIL;

	if (g_stat(filename, &st))
		goto out;

	objref = g_malloc0((st.st_size) * sizeof(gchar));
	if (!objref)
		goto out;

	// must work even if sizeof(char) != sizeof(CORBA_char)
	c = fread((void*)objref, sizeof(CORBA_char), (size_t)(st.st_size/sizeof(CORBA_char)), file);
	if (c != st.st_size)
		goto out;

	obj = (CORBA_Object)CORBA_ORB_string_to_object(orb, objref, ev);

out:
	g_free(objref);
	fclose(file);

	return obj;
}

static BRUTUS_BrutusKeyring
get_keyring_ref(CORBA_ORB orb)
{
	char *home_dir = getenv("HOME");
	gchar *ref_path = NULL;
	BRUTUS_BrutusKeyring keyring = CORBA_OBJECT_NIL;
	CORBA_boolean cb = CORBA_FALSE;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	ref_path = g_strconcat(home_dir, "/", BRUTUS_KEYRINGD_REF_FILE_NAME, NULL);
	if (!ref_path)
		goto out;
	keyring = brutus_object_ref_from_file(orb, ref_path, ev);
	g_free(ref_path);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	cb = CORBA_Object_is_nil((CORBA_Object)keyring, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb) {
		g_printf("Could not get the Brutus Keyring reference\n");
		goto out;
	}

out:
	CORBA_exception_free(ev);

	return keyring;

}

static void print_legalese(const char *prog_name)
{
	g_print("\n");
	g_print("                      -*- DISCLAIMER START -*-\n");
	g_print("\n");
	g_print("  * %s - A sample program to test the Brutus Keyring daemon.\n", prog_name);
	g_print("  * Copyright (C) 2007 OMC Denmark ApS\n");
	g_print("  * \n");
	g_print("  * This program is free software; you can redistribute it and/or modify\n");
	g_print("  * it under the terms of the GNU General Public License as published by\n");
	g_print("  * the Free Software Foundation; either version 2 of the License, or\n");
	g_print("  * (at your option) any later version.\n");
	g_print("  * \n");
	g_print("  * This program is distributed in the hope that it will be useful,\n");
	g_print("  * but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
	g_print("  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the\n");
	g_print("  * GNU General Public License for more details.\n");
	g_print("  * \n");
	g_print("  * You should have received a copy of the GNU General Public License\n");
	g_print("  * along with this program; if not, write to the Free Software\n");
	g_print("  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.\n");
	g_print("\n");
	g_print("                       -*- DISCLAIMER END -*-\n");
	g_print("\n");
}
int
main(int argc, char *argv[])
{
	int retv = EXIT_FAILURE;
	BRUTUS_BrutusKeyring_KeyringResult keyring_retv = BRUTUS_BrutusKeyring_failed;
	BRUTUS_BrutusKeyring_KeyringData *psecret = NULL;
	BRUTUS_BrutusKeyring_KeyringData secret;
	BRUTUS_BrutusKeyring keyring = CORBA_OBJECT_NIL;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_ORB orb = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];

	g_type_init();
	g_thread_init(NULL);

	print_legalese(argv[0]);

	secret._buffer = NULL;
	CORBA_exception_init(ev);

	// get keyring reference
	orb = create_orb();
	keyring = get_keyring_ref(orb);
	cb = CORBA_Object_is_nil((CORBA_Object)keyring, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb) {
		g_print("The Brutus Keyring daemon is not running\n");
		goto out;
	}

	if (2 >= argc)
		goto err_input;

	if (!strcmp("-encrypt", argv[1])) {
		if (4 != argc)
			goto err_input;
		if (!read_file(argv[2], &secret))
			goto out;

		keyring_retv = BRUTUS_BrutusKeyring_setSecret(keyring, argv[3], &secret, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BrutusKeyring_set_passphrase == keyring_retv) {
			g_print("Keyring locked, please set the passphrase\n");
			goto out;
		}
		if (BRUTUS_BrutusKeyring_nop == keyring_retv) {
			g_print("Empty file\n");
			goto out;
		}
		if (BRUTUS_BrutusKeyring_bad_passphrase == keyring_retv) {
			g_print("Keyring still locked, please set the correct passphrase\n");
			goto out;
		}
		if (BRUTUS_BrutusKeyring_failed == keyring_retv) {
			g_print("Failed\n");
			goto out;
		}
		if (BRUTUS_BrutusKeyring_success == keyring_retv) {
			g_print("Success\n");
			retv = EXIT_SUCCESS;
		}
	} else if (!strcmp("-decrypt", argv[1])) {
		if (4 != argc)
			goto err_input;

		keyring_retv = BRUTUS_BrutusKeyring_getSecret(keyring, argv[2], &psecret, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BrutusKeyring_set_passphrase == keyring_retv) {
			g_print("Keyring locked, please set the passphrase\n");
			goto out;
		}
		if (BRUTUS_BrutusKeyring_nss == keyring_retv) {
			g_print("No such secret\n");
			goto out;
		}
		if (BRUTUS_BrutusKeyring_bad_passphrase == keyring_retv) {
			g_print("Keyring still locked, please set the correct passphrase\n");
			goto out;
		}
		if (BRUTUS_BrutusKeyring_failed == keyring_retv) {
			g_print("Failed\n");
			goto out;
		}
		if (BRUTUS_BrutusKeyring_success == keyring_retv) {
			retv = write_file(argv[3], psecret) ? EXIT_SUCCESS : EXIT_FAILURE;
			if (EXIT_SUCCESS == retv)
				g_print("Success\n");
		}

	} else if (!strcmp("-set_passphrase", argv[1])) {
		if (3 != argc)
			goto err_input;

		keyring_retv = BRUTUS_BrutusKeyring_setPassPhrase(keyring, argv[2], ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BrutusKeyring_failed == keyring_retv) {
			g_print("Failed\n");
			goto out;
		}
		if (BRUTUS_BrutusKeyring_bad_passphrase == keyring_retv) {
			g_print("Bad passhhrase\n");
			goto out;
		}
		if (BRUTUS_BrutusKeyring_success == keyring_retv) {
			g_print("Success\n");
			retv = EXIT_SUCCESS;
		}
	} else
		goto err_input;

	goto out;

err_input:
	g_print("Usage: %s -encrypt [PATH OF INPUT FILE] [NAME OF SECRET]\n", argv[0]);
	g_print("Usage: %s -decrypt [NAME OF SECRET] [PATH OF OUTPUT FILE]\n", argv[0]);
	g_print("Usage: %s -set_passphrase [PASSPHRASE]\n", argv[0]);
	retv = EXIT_SUCCESS;

out:
	if (secret._buffer)
		CORBA_free(secret._buffer);

	if (psecret && psecret->_buffer)
		CORBA_free(psecret->_buffer);

	CORBA_Object_release((CORBA_Object)keyring, ev);

	CORBA_ORB_shutdown(orb, FALSE, ev);
	CORBA_ORB_destroy(orb, ev);
	CORBA_Object_release((CORBA_Object)orb, ev);

	CORBA_exception_free(ev);

	return retv;
}
