/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus account password helper query program.
 *
 *    Authors:
 *        Jules Colding <colding@omesc.com>
 *
 *    Copyright (C) 2006-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

/*
 * Most of the gtk+ code in this file has been copied from
 * e-passwords.c which resides in the evolution-data-server code
 * base.
 */


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <string.h>
#include <gtk/gtk.h>
#include <glib/gstdio.h>
#include <glib/gi18n-lib.h>

#include <brutus-keyring/brutus-keyring.h>
#include <brutus-keyring/brutus-keyringd.h>
#include <server/brutus.h>

enum ACTION {
	act_none,
	act_query_password,
	act_set_passphrase,
	act_change_passphrase,
};

struct _cb_data {
	enum ACTION task;
	const char *key;
	GtkWindow *parent;
	GtkEntry *entry;
	GtkEntry *validator_entry;
};

static CORBA_ORB
create_orb(void)
{
	CORBA_ORB orb = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];
	int init_argc = 2;
	char **init_argv = (char**)malloc(sizeof(char*) * (init_argc));
	if (!init_argv)
		return CORBA_OBJECT_NIL;

	CORBA_exception_init(ev);

	memset((void*)init_argv, 0, init_argc);
	init_argv[0] = strdup("DUMMY_ARGV0");

	// Explicitly force ORBit2 to be local only
	init_argv[1] = "--ORBLocalOnly=1";

	// initialize the ORB
	orb = CORBA_ORB_init(&init_argc, init_argv, "brutus-query_orb_orbit-io-thread", ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		orb = CORBA_OBJECT_NIL;
	}

	free(init_argv[0]);
	free(init_argv);

	CORBA_exception_free(ev);

	return orb;
}

static CORBA_Object
brutus_object_ref_from_file(CORBA_ORB orb,
			    gchar *filename,
			    CORBA_Environment *ev)
{
	CORBA_Object obj = CORBA_OBJECT_NIL;
	CORBA_char *objref = NULL;
	FILE *file = NULL;
	struct stat st;
	size_t c = 0;

	file = g_fopen(filename, "r");
	if (!file)
		return CORBA_OBJECT_NIL;

	if (g_stat(filename, &st))
		goto out;

	objref = g_malloc0((st.st_size) * sizeof(gchar));
	if (!objref)
		goto out;

	// must work even if sizeof(char) != sizeof(CORBA_char)
	c = fread((void*)objref, sizeof(CORBA_char), (size_t)(st.st_size/sizeof(CORBA_char)), file);
	if (c != st.st_size)
		goto out;

	obj = (CORBA_Object)CORBA_ORB_string_to_object(orb, objref, ev);

out:
	g_free(objref);
	fclose(file);

	return obj;
}

static BRUTUS_BrutusKeyring
get_keyring_ref(CORBA_ORB orb)
{
	char *home_dir = getenv("HOME");
	gchar *ref_path = NULL;
	BRUTUS_BrutusKeyring keyring = CORBA_OBJECT_NIL;
	CORBA_boolean cb = CORBA_FALSE;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	ref_path = g_strconcat(home_dir, "/", BRUTUS_KEYRINGD_REF_FILE_NAME, NULL);
	if (!ref_path)
		goto out;
	keyring = brutus_object_ref_from_file(orb, ref_path, ev);
	g_free(ref_path);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	cb = CORBA_Object_is_nil((CORBA_Object)keyring, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb) {
		d("Could not get the Brutus Keyring reference");
		goto out;
	}

out:
	CORBA_exception_free(ev);

	return keyring;

}

static gboolean
passwd_to_keyring(const enum ACTION task,
		  const char *password,
		  const char *key_name)
{
	gboolean retv = FALSE;
	BRUTUS_BrutusKeyring_KeyringResult keyring_retv = BRUTUS_BrutusKeyring_failed;
	BRUTUS_BrutusKeyring_KeyringData secret;
	BRUTUS_BrutusKeyring keyring = CORBA_OBJECT_NIL;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_ORB orb = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];

	secret._maximum = 0;
	secret._length = 0;
	secret._buffer = NULL;
	CORBA_exception_init(ev);

	// get orb reference
	orb = create_orb();
	cb = CORBA_Object_is_nil((CORBA_Object)orb, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb)
		goto out;

	// get keyring reference
	keyring = get_keyring_ref(orb);
	cb = CORBA_Object_is_nil((CORBA_Object)keyring, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb)
		goto out;

	switch (task) {
	case act_query_password :
		secret._maximum = strlen(password)+1;
		secret._length = secret._maximum;
		secret._buffer = BRUTUS_BrutusKeyring_KeyringData_allocbuf(secret._maximum);
		if (!secret._buffer)
			goto out;
		memcpy((void*)secret._buffer, (const void*)password, secret._maximum);

		keyring_retv = BRUTUS_BrutusKeyring_setSecret(keyring, key_name, &secret, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BrutusKeyring_success == keyring_retv)
			retv = EXIT_SUCCESS;
		break;
	case act_set_passphrase :
		keyring_retv = BRUTUS_BrutusKeyring_setPassPhrase(keyring, password, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BrutusKeyring_success == keyring_retv)
			retv = EXIT_SUCCESS;
		break;
	case act_change_passphrase : /* not supported yet */
/*		keyring_retv = BRUTUS_BrutusKeyring_newPassPhrase(keyring, password, new_password, ev); */
/*		if (ORBIT2_EX(ev)) { */
/*			PRINT_ORBIT2_EX(ev); */
/*			goto out; */
/*		} */
/*		if (BRUTUS_BrutusKeyring_success == keyring_retv) */
/*			retv = EXIT_SUCCESS; */
		break;
	default :
		break;
	}

	retv = TRUE;
out:
	if (secret._buffer)
		CORBA_free(secret._buffer);

	cb = CORBA_Object_is_nil((CORBA_Object)keyring, ev);
	if (!ORBIT2_EX(ev) && !cb)
		CORBA_Object_release((CORBA_Object)keyring, ev);

	cb = CORBA_Object_is_nil((CORBA_Object)orb, ev);
	if (!ORBIT2_EX(ev) && !cb) {
		CORBA_ORB_shutdown(orb, FALSE, ev);
		CORBA_ORB_destroy(orb, ev);
		CORBA_Object_release((CORBA_Object)orb, ev);
	}

	CORBA_exception_free(ev);

	return retv;
}

static void
message(GtkWindow *parent,
	const char *msg)
{
	GtkWidget *msg_dialog = NULL;

	msg_dialog = gtk_message_dialog_new(parent,
					    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
					    GTK_MESSAGE_ERROR,
					    GTK_BUTTONS_CLOSE,
					    "%s",
					    msg);
	gtk_dialog_run(GTK_DIALOG(msg_dialog));
	gtk_widget_destroy(msg_dialog);
}

static void
dialog_cb(GtkDialog *dialog,
	  int response,
	  void *hook)
{
	int exit_val = EXIT_FAILURE;
	struct _cb_data *data = (struct _cb_data*)hook;
	const char *str = NULL;
	const char *validator_str = NULL;

	if (GTK_RESPONSE_OK != response)
		goto out;

	str = gtk_entry_get_text(data->entry);
	if (!str)
		goto out;
	if (!strlen(str)) {
		message(data->parent, _("Empty password"));
		goto out;
	}
	if (data->validator_entry) {
		validator_str = gtk_entry_get_text(data->validator_entry);
		if (!validator_str || !strlen(validator_str) || strcmp(str, validator_str)) {
			message(data->parent, _("The passphrases did not match"));
			goto out;
		}
	}

	exit_val = system("brutus-keyringd");
	if (EXIT_SUCCESS != exit_val)
		goto out;
	
	passwd_to_keyring(data->task, str, data->key);

out:
	gtk_main_quit();
}

int
main(int argc, char *argv[])
{
	GtkDialog *password_dialog = NULL;
	GtkWidget *vbox = NULL;
	GtkEntry *entry = NULL;
	GtkEntry *validator_entry = NULL;
	AtkObject *a11y = NULL;
	AtkObject *validator_a11y = NULL;
	struct _cb_data cb_data = {
		.task = act_none,
		.key = NULL,
		.parent = NULL,
		.entry = NULL,
		.validator_entry = NULL,
	};

	if ((2 != argc) && (4 != argc))
		goto err_input;

	if ((4 == argc) && !strcmp("-set_password", argv[1])) {
		cb_data.task = act_query_password;
		cb_data.key = argv[3];
	} else if ((2 == argc) && !strcmp("-set_passphrase", argv[1])) {
		cb_data.task = act_set_passphrase;
	} else if ((2 == argc) && !strcmp("-change_passphrase", argv[1])) {
		cb_data.task = act_change_passphrase; /* not supported yet */
		goto err_input;
	} else
		goto err_input;

	g_type_init();
	g_thread_init(NULL);
	gtk_init (&argc, &argv);

	password_dialog = (GtkDialog *)gtk_message_dialog_new(NULL,
							      0,
							      GTK_MESSAGE_QUESTION,
							      GTK_BUTTONS_OK_CANCEL,
							      "%s",
							      (act_query_password == cb_data.task) ? argv[2] : _("Enter Brutus Keyring passphrase twice"));
	cb_data.parent = GTK_WINDOW(password_dialog);
	gtk_window_set_title(GTK_WINDOW(password_dialog), "Brutus Keyring");

	gtk_widget_ensure_style(GTK_WIDGET(password_dialog));
	gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(password_dialog)->vbox), 0);
	gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(password_dialog)->action_area), 12);

#if !GTK_CHECK_VERSION (2,4,0)
	gtk_dialog_set_has_separator(password_dialog, FALSE);
#endif
	gtk_dialog_set_default_response(password_dialog, GTK_RESPONSE_OK);

	vbox = gtk_vbox_new(FALSE, 12);
	gtk_widget_show(vbox);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(password_dialog)->vbox), vbox, TRUE, FALSE, 0);
	gtk_container_set_border_width((GtkContainer*)vbox, 12);

	entry = (GtkEntry*)gtk_entry_new();
	cb_data.entry = entry;
	gtk_entry_set_invisible_char(entry, '*');
	gtk_entry_set_visibility(entry, FALSE);
	gtk_entry_set_max_length(entry, 128);
	gtk_entry_set_activates_default(entry, TRUE);
	a11y = gtk_widget_get_accessible((GtkWidget*)entry);
	atk_object_set_description(a11y, _("Brutus Keyring text entry field"));
	gtk_box_pack_start(GTK_BOX(vbox), (GtkWidget*)entry, TRUE, FALSE, 3);
	gtk_widget_show((GtkWidget*)entry);
	gtk_widget_grab_focus((GtkWidget*)entry);

	if (act_set_passphrase == cb_data.task) {
		validator_entry = (GtkEntry*)gtk_entry_new();
		cb_data.validator_entry = validator_entry;
		gtk_entry_set_invisible_char(validator_entry, '*');
		gtk_entry_set_visibility(validator_entry, FALSE);
		gtk_entry_set_max_length(validator_entry, 128);

		gtk_entry_set_activates_default(entry, FALSE);
		gtk_entry_set_activates_default(validator_entry, TRUE);

		validator_a11y = gtk_widget_get_accessible((GtkWidget*)validator_entry);
		atk_object_set_description(validator_a11y, _("Brutus Keyring secondary text entry field"));

		gtk_box_pack_start(GTK_BOX(vbox), (GtkWidget*)validator_entry, TRUE, FALSE, 3);
		gtk_widget_show((GtkWidget*)validator_entry);
	}
	g_signal_connect(password_dialog, "response", G_CALLBACK(dialog_cb), (gpointer)&cb_data);

	gtk_window_set_keep_above((GtkWindow*)password_dialog, TRUE);
	gtk_widget_show((GtkWidget*)password_dialog);
	gtk_window_set_urgency_hint((GtkWindow*)password_dialog, TRUE);

	gtk_main();
	gtk_widget_destroy((GtkWidget*)password_dialog);
	goto out;

err_input:
	g_print("Usage: %s -set_password [PROMPT] [NAME OF PASSWORD IN KEYRING]\n", argv[0]);
	g_print("Usage: %s -set_passphrase\n", argv[0]);
/*	g_print("Usage: %s -change_passphrase \n", argv[0]); */

out:
	return 0;
}
