/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus source file for implementing to/from Camel conversion functions.
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "brutus_camel.h"

/* system includes */
#include <string.h>
#include <strings.h>

/* camel includes */
#include <camel/camel-i18n.h>
#include <camel/camel-multipart.h>
#include <camel/camel-mime-utils.h>
#include <camel/camel-stream.h>
#include <camel/camel-stream-mem.h>

/* pure brutus includes */
#include <idl_output/types.h>
#include <idl_output/uid.h>
#include <idl_output/defines.h>
#include <idl_output/IAddrBook.h>

/* evolution-brutus includes */
#include <server/brutus.h>
#include <server/brutus_corba.h>
#include <server/brutus_mapi.h>
#include <server/brutus_cpid.h>
#include <server/brutus_conv.h>

#define BODY_TEXT_PLAIN 0x01
#define BODY_TEXT_HTML  0x02

static CORBA_char*
brutus_mapi_address_type(const char *addr)
{
	if ((!addr) || !strlen(addr) || (('[' != *addr) && ('/' != *addr)))
		return CORBA_string_dup("SMTP");

	if ('/' == *addr)
		return CORBA_string_dup("EX");

	if (g_strstr_len(addr, strlen("[SMTP:"), "SMTP:"))
		return CORBA_string_dup("SMTP");

	if (g_strstr_len(addr, strlen("[EX:"), "EX:"))
		return CORBA_string_dup("EX");

	if (g_strstr_len(addr, strlen("[X400:"), "X400:"))
		return CORBA_string_dup("X400");

	if (g_strstr_len(addr, strlen("[X500:"), "X500:"))
		return CORBA_string_dup("X500");

	if (g_strstr_len(addr, strlen("[3COM:"), "3COM:"))
		return CORBA_string_dup("3COM");

	if (g_strstr_len(addr, strlen("[ATT:"), "ATT:"))
		return CORBA_string_dup("ATT");

	if (g_strstr_len(addr, strlen("[FAX:"), "FAX:"))
		return CORBA_string_dup("FAX");

	if (g_strstr_len(addr, strlen("[MSFAX:"), "MSFAX:"))
		return CORBA_string_dup("MSFAX");

	if (g_strstr_len(addr, strlen("[MCI:"), "MCI:"))
		return CORBA_string_dup("MCI");

	if (g_strstr_len(addr, strlen("[MHS:"), "MHS:"))
		return CORBA_string_dup("MHS");

	if (g_strstr_len(addr, strlen("[MS:"), "MS:"))
		return CORBA_string_dup("MS");

	if (g_strstr_len(addr, strlen("[MSA:"), "MSA:"))
		return CORBA_string_dup("MSA");

	if (g_strstr_len(addr, strlen("[MSN:"), "MSN:"))
		return CORBA_string_dup("MSN");

	if (g_strstr_len(addr, strlen("[PROFS:"), "PROFS:"))
		return CORBA_string_dup("PROFS");

	if (g_strstr_len(addr, strlen("[SNADS:"), "SNADS:"))
		return CORBA_string_dup("SNADS");

	if (g_strstr_len(addr, strlen("[TELEX:"), "TELEX:"))
		return CORBA_string_dup("TELEX");

	return CORBA_string_dup("SMTP");
}

static gboolean
brutus_set_recipients(CamelMimeMessage *message,
		      BRUTUS_IMessage mapi_message,
		      BRUTUS_IMAPISession mapi_session,
		      PortableServer_POA poa,
		      CamelException *ex)
{
	unsigned int n;
	int to_cnt = 0;
	int cc_cnt = 0;
	int bcc_cnt = 0;
	CORBA_long mapi_recip_type;
	gboolean retv = FALSE;
	gboolean need_resolve = FALSE;
	const CamelInternetAddress *to_addr = NULL;
	const CamelInternetAddress *cc_addr = NULL;
	const CamelInternetAddress *bcc_addr = NULL;
	const char *name = NULL;
	const char *addr = NULL;
	char *msg = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_IAddrBook addr_book = CORBA_OBJECT_NIL;
	BRUTUS_ADRLIST addr_list = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	// extract recipients
	to_addr = camel_mime_message_get_recipients(message, CAMEL_RECIPIENT_TYPE_TO);
	cc_addr = camel_mime_message_get_recipients(message, CAMEL_RECIPIENT_TYPE_CC);
	bcc_addr = camel_mime_message_get_recipients(message, CAMEL_RECIPIENT_TYPE_BCC);

	// initiate MAPI address list
	addr_list._maximum =
		(to_addr ? camel_address_length(CAMEL_ADDRESS(to_addr)) : 0)
		+ (cc_addr ? camel_address_length(CAMEL_ADDRESS(cc_addr)) : 0)
		+ (bcc_addr ? camel_address_length(CAMEL_ADDRESS(bcc_addr)) : 0);
	addr_list._length = addr_list._maximum;
	addr_list._buffer = BRUTUS_ADRLIST_allocbuf(addr_list._maximum);

	// transfer addresses
	mapi_recip_type = BRUTUS_BRUTUS_MAPI_TO;
	for (n = 0; n < addr_list._length; n++) {
		name = NULL;
		addr = NULL;
		if (BRUTUS_BRUTUS_MAPI_TO == mapi_recip_type) {
			if (!camel_internet_address_get(to_addr, to_cnt++, &name, &addr))
				mapi_recip_type = BRUTUS_BRUTUS_MAPI_CC;
		}
		if (BRUTUS_BRUTUS_MAPI_CC == mapi_recip_type) {
			if (!camel_internet_address_get(cc_addr, cc_cnt++, &name, &addr))
				mapi_recip_type = BRUTUS_BRUTUS_MAPI_BCC;
		}
		if (BRUTUS_BRUTUS_MAPI_BCC == mapi_recip_type) {
			if (!camel_internet_address_get(bcc_addr, bcc_cnt++, &name, &addr))
				break;
		}

		// with or without PR_EMAIL_ADDRESS
		if (!need_resolve)
			need_resolve = !(addr || strlen(addr));

		addr_list._buffer[n].ulReserved1 = 0;
		addr_list._buffer[n].rgPropVals._maximum = 4 + !!(addr && strlen(addr)); // make optionally room for PR_EMAIL_ADDRESS
		addr_list._buffer[n].rgPropVals._length = addr_list._buffer[n].rgPropVals._maximum;
		addr_list._buffer[n].rgPropVals._buffer = BRUTUS_seq_SPropValue_allocbuf(addr_list._buffer[n].rgPropVals._maximum),

			addr_list._buffer[n].rgPropVals._buffer[0].ulPropTag = BRUTUS_BRUTUS_PR_RECIPIENT_TYPE;
		addr_list._buffer[n].rgPropVals._buffer[0].Value._d = BRUTUS_BRUTUS_PT_LONG;
		addr_list._buffer[n].rgPropVals._buffer[0].Value._u.l = mapi_recip_type;

		addr_list._buffer[n].rgPropVals._buffer[1].ulPropTag = BRUTUS_BRUTUS_PR_DISPLAY_NAME; // the name, the address or the empty string (in that order)
		addr_list._buffer[n].rgPropVals._buffer[1].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		addr_list._buffer[n].rgPropVals._buffer[1].Value._u.lpszA = (name && strlen(name)) ?
			CORBA_string_dup(name)
			: ((addr && strlen(addr)) ? CORBA_string_dup(addr) : CORBA_string_dup(""));
		d(name);
		d(addr_list._buffer[n].rgPropVals._buffer[1].Value._u.lpszA);
		addr_list._buffer[n].rgPropVals._buffer[2].ulPropTag = BRUTUS_BRUTUS_PR_ADDRTYPE;
		addr_list._buffer[n].rgPropVals._buffer[2].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		addr_list._buffer[n].rgPropVals._buffer[2].Value._u.lpszA = brutus_mapi_address_type(addr);

		addr_list._buffer[n].rgPropVals._buffer[3].ulPropTag = BRUTUS_BRUTUS_PR_SEND_RICH_INFO;
		addr_list._buffer[n].rgPropVals._buffer[3].Value._d = BRUTUS_BRUTUS_PT_BOOLEAN;
		addr_list._buffer[n].rgPropVals._buffer[3].Value._u.b = FALSE;

		if (addr_list._buffer[n].rgPropVals._length <= 4)
			continue;

		addr_list._buffer[n].rgPropVals._buffer[addr_list._buffer[n].rgPropVals._length - 1].ulPropTag = BRUTUS_BRUTUS_PR_EMAIL_ADDRESS;
		addr_list._buffer[n].rgPropVals._buffer[addr_list._buffer[n].rgPropVals._length - 1].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		addr_list._buffer[n].rgPropVals._buffer[addr_list._buffer[n].rgPropVals._length - 1].Value._u.lpszA = CORBA_string_dup(addr);
	}

	if (need_resolve) { // some transport providers requires a call to ResolveName() regardless. I'll wait for that bug report...
		br = BRUTUS_IMAPISession_OpenAddressBook(mapi_session, "", BRUTUS_BRUTUS_MAPI_ACCESS_MODIFY, &addr_book, ev);
		if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
			d("Could not open address book");
			camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
					    _("Could not open the Exchange address book for address resolution"));
			goto out;
		}

		br = BRUTUS_IAddrBook_ResolveName(addr_book, 0, &addr_list, ev);
		brutus_release_object(BRUTUS_IAddrBook_tc, &addr_book, poa, ev);

		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			msg = g_strdup("");
			for (n = 0; n < addr_list._length; n++) {
				if (addr_list._buffer[n].rgPropVals._length >= 3)
					msg = g_strjoin("\n", msg, addr_list._buffer[n].rgPropVals._buffer[1].Value._u.lpszA, NULL); // display name
			}
			camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
					     "%s:%s",
					     _("Some recipients could not be resolved"), msg);
			g_free(msg);
			goto out;
		}
	}

	// set the recipients
	br = BRUTUS_IMessage_ModifyRecipients(mapi_message, 0, &addr_list, ev);
	if (BRUTUS_BRUTUS_S_OK != br) {
		d("Could not set recipients");
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not set recipients on outgoing message"));
		goto out;
	}

	retv = TRUE;

out:
	brutus_free_addrlist_content(&addr_list, poa);
	CORBA_exception_free(ev);
	return retv;
}

static gboolean
brutus_append_attachment(BRUTUS_IMessage mapi_message,
			 CamelMimePart *mime_part,
			 gboolean inline_attachment,
			 CamelException *ex)
{
	gboolean retv = FALSE;
	CamelDataWrapper *dw = NULL;
	CamelStreamMem *stream = NULL;
	char *tmp = NULL;	
	char *pos = NULL;
	size_t attach_len = 0;
	const char *description = NULL;
	const char *disposition = NULL;
	const char *filename = NULL;
	const char *content_md5 = NULL;
	const char *content_location = NULL;
	char *mime_content_type = NULL;
	char *content_id = NULL;
	char **content_id_vec = NULL;
	CORBA_unsigned_long cb_written = 0;
	CORBA_unsigned_long attach_num = 0;
	BRUTUS_IAttach mapi_attach = CORBA_OBJECT_NIL;
	BRUTUS_IStream mapi_stream = CORBA_OBJECT_NIL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_SPropProblemArray *mapi_problems = NULL;
	int prop_count = 0;
	int n = 0;
	BRUTUS_seq_SPropValue msg_prop_values = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	BRUTUS_seq_octet buf = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	BRUTUS_seq_string rfc822_name_fields = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	BRUTUS_seq_string rfc822_body_fields = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	// get the streamed content
	dw = camel_medium_get_content_object(CAMEL_MEDIUM(mime_part));
	BRUTUS_CHECK_ALLOC(dw, ex);
	stream = (CamelStreamMem *)camel_stream_mem_new();
	BRUTUS_CHECK_ALLOC(stream, ex);
	if (-1 == camel_data_wrapper_write_to_stream(dw, (CamelStream*)stream)) {
		d("No memory");
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("No memory"));
		goto out;
	}
	if (!stream->buffer->len) {
		d("Empty attachment");
		retv = TRUE;
		goto out;
	}


	//
	// get MIME header data
	//

	description = camel_mime_part_get_description(mime_part);
	BRUTUS_NULL_IF_EMPTY_STRING(description);

	disposition = camel_mime_part_get_disposition(mime_part);
	BRUTUS_NULL_IF_EMPTY_STRING(disposition);

	filename = camel_mime_part_get_filename(mime_part);
	BRUTUS_NULL_IF_EMPTY_STRING(filename);

	content_id = (char*)camel_mime_part_get_content_id(mime_part);
	BRUTUS_NULL_IF_EMPTY_STRING(content_id);
	if (content_id) {
		content_id_vec = g_strsplit_set(content_id, "<>", -1);
		if (!content_id_vec[1])
			content_id = g_strdup(content_id);
		else
			content_id = g_strdup(content_id_vec[1]);
		g_strfreev(content_id_vec);
	}

	content_md5 = camel_mime_part_get_content_MD5(mime_part);
	BRUTUS_NULL_IF_EMPTY_STRING(content_md5);

	content_location = camel_mime_part_get_content_location(mime_part);
	BRUTUS_NULL_IF_EMPTY_STRING(content_location);

	// create attachment
	br = BRUTUS_IMessage_CreateAttach(mapi_message,
					  BRUTUS_BRUTUS_IID_IAttachment,
					  0,
					  &attach_num,
					  &mapi_attach,
					  ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Could not create attachment: Communication error"));
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not create attachment"));
		goto out;
	}

	// allocate SPropValue sequence:
	//
	// Streamed - done above :
	// buf (PR_ATTACH_DATA_BIN)
	//
	// SPropValue:
	// description (PR_DISPLAY_NAME)
	// filename (PR_ATTACH_LONG_FILENAME)
	// content_id (PR_ATTACH_CONTENT_ID)
	// content_location (PR_ATTACH_CONTENT_LOCATION)
	//
	// Named properties - will be done at the end:
	// content_md5 ("Content-MD5" MIME header)
	//
	// Flags: ( PR_ATTACHMENT_HIDDEN)
	//     disposition (inline or not?)
	//     mime_content_type "text/html ; alternative"
	//
	prop_count = (!!description) + (!!filename) + (!!content_id) + (!!content_location);
	prop_count += 4; // PR_ATTACH_MIME_TAG, PR_ATTACHMENT_HIDDEN, PR_ATTACH_METHOD, PR_ATTACH_RENDERING_POSITION
	msg_prop_values._maximum = prop_count;
	msg_prop_values._length = msg_prop_values._maximum;
	msg_prop_values._buffer = BRUTUS_seq_SPropValue_allocbuf(msg_prop_values._maximum);
	BRUTUS_CHECK_ALLOC(msg_prop_values._buffer, ex);

	n = 0;

	if (description) {
		msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_DISPLAY_NAME;
		msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		msg_prop_values._buffer[n].Value._u.lpszA = CORBA_string_dup(description);
		BRUTUS_CHECK_ALLOC(msg_prop_values._buffer[n].Value._u.lpszA, ex);
		n++;
	}
	if (filename) {
		msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_ATTACH_LONG_FILENAME;
		msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		msg_prop_values._buffer[n].Value._u.lpszA = CORBA_string_dup(filename);
		BRUTUS_CHECK_ALLOC(msg_prop_values._buffer[n].Value._u.lpszA, ex);
		n++;
	}
	if (content_id) {
		msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_ATTACH_CONTENT_ID;
		msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		msg_prop_values._buffer[n].Value._u.lpszA = CORBA_string_dup(content_id);
		BRUTUS_CHECK_ALLOC(msg_prop_values._buffer[n].Value._u.lpszA, ex);
		n++;
	}
	if (content_location) { // this might not work according to Volker
		msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_ATTACH_CONTENT_LOCATION;
		msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		msg_prop_values._buffer[n].Value._u.lpszA = CORBA_string_dup(content_location);
		BRUTUS_CHECK_ALLOC(msg_prop_values._buffer[n].Value._u.lpszA, ex);
		n++;
	}

	mime_content_type = camel_data_wrapper_get_mime_type(dw);
	BRUTUS_CHECK_ALLOC(mime_content_type, ex);
	msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_ATTACH_MIME_TAG;
	msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_STRING8;
	msg_prop_values._buffer[n].Value._u.lpszA = CORBA_string_dup(mime_content_type);
	BRUTUS_CHECK_ALLOC(msg_prop_values._buffer[n].Value._u.lpszA, ex);
	n++;

	// inline attachment 
	msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_ATTACHMENT_HIDDEN;
	msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_BOOLEAN;
	if (!inline_attachment && (disposition)) {
		tmp = strchr(disposition, ';');
		pos = strcasestr(disposition, "inline");
		do {
			if (!pos)
				break;

			if (!tmp) {
				inline_attachment = TRUE;
				break;
			}

			if (pos < tmp)
				inline_attachment = TRUE;
		} while (FALSE);
	}
	if (!inline_attachment && !g_ascii_strcasecmp(mime_content_type, "text/html"))
		inline_attachment = TRUE;
	if (inline_attachment)
		msg_prop_values._buffer[n].Value._u.b = inline_attachment ? CORBA_TRUE : CORBA_FALSE;
	n++;

	msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_ATTACH_METHOD;
	msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[n].Value._u.l = BRUTUS_BRUTUS_ATTACH_BY_VALUE;
	n++;

	msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_RENDERING_POSITION;
	msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[n].Value._u.l = -1;
	n++;

	// let Exchange know that this is a MIME attachment
	br = BRUTUS_IAttach_SetAttachTag(mapi_attach, BRUTUS_BRUTUS_OID_MIMETAG, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Could not set attach tag on attachment object: Communication error"));
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not set attach tag on attachment object"));
		goto out;
	}

	// set properties on attachment obejct
	mapi_problems = BRUTUS_seq_SPropProblem__alloc();
	mapi_problems->_maximum = 0;
	mapi_problems->_length = 0;
	mapi_problems->_buffer = NULL;
	br = BRUTUS_IAttach_SetProps(mapi_attach, &msg_prop_values, FALSE, &mapi_problems, ev);
	CORBA_free(mapi_problems);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Could not set properties on attachment object: Communication error"));
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not set properties on attachment object"));
		goto out;
	}

	// open stream for attachment data
	br = BRUTUS_IAttach_OpenProperty(mapi_attach,
					 BRUTUS_BRUTUS_PR_ATTACH_DATA_BIN,
					 BRUTUS_BRUTUS_IID_IStream,
					 BRUTUS_BRUTUS_STGM_WRITE,
					 BRUTUS_BRUTUS_MAPI_MODIFY | BRUTUS_BRUTUS_MAPI_CREATE,
					 &mapi_stream,
					 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Could not open attachment object: Communication error"));
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not open attachment object"));
		goto out;
	}

	// decode attachment data
	tmp = g_memdup(stream->buffer->data, stream->buffer->len);
	BRUTUS_CHECK_ALLOC(tmp, ex);
	attach_len = camel_base64_decode_simple(tmp,  stream->buffer->len);

	// Transfer data to attachment object
	buf._maximum = attach_len;
	buf._length = buf._maximum;
	buf._buffer = CORBA_sequence_CORBA_octet_allocbuf(buf._maximum);
	BRUTUS_CHECK_ALLOC(buf._buffer, ex);
	memcpy(buf._buffer, tmp, buf._maximum);
	g_free(tmp);
	br = BRUTUS_IStream_Write(mapi_stream,
				  &buf,
				  buf._length,
				  &cb_written,
				  ev);
	CORBA_free(buf._buffer);
	buf._maximum = 0;
	buf._buffer = NULL;
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Could not write to attachment object: Communication error"));
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not write to attachment object"));
		goto out;
	}
	if (cb_written != buf._length) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Incomplete write of data to attachment object"));
		goto out;
	}
	buf._length = 0;

	// commit
	br = BRUTUS_IStream_Commit(mapi_stream,
				   BRUTUS_BRUTUS_STGC_DEFAULT,
				   ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Could not commit to attachment object: Communication error"));
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not commit to attachment object"));
		goto out;
	}

	// set Content-MD5 MIME header
	if (content_md5) {
		rfc822_name_fields._maximum = 1;
		rfc822_name_fields._length = rfc822_name_fields._maximum;
		rfc822_name_fields._buffer = BRUTUS_seq_string_allocbuf(rfc822_name_fields._maximum);
		BRUTUS_CHECK_ALLOC(rfc822_name_fields._buffer, ex);
		rfc822_name_fields._buffer[0] = CORBA_string_dup("Content-MD5");
		BRUTUS_CHECK_ALLOC(rfc822_name_fields._buffer[0], ex);

		rfc822_body_fields._maximum = 1;
		rfc822_body_fields._length = rfc822_body_fields._maximum;
		rfc822_body_fields._buffer = BRUTUS_seq_string_allocbuf(rfc822_body_fields._maximum);
		BRUTUS_CHECK_ALLOC(rfc822_body_fields._buffer, ex);
		rfc822_body_fields._buffer[0] = CORBA_string_dup(content_md5);
		BRUTUS_CHECK_ALLOC(rfc822_body_fields._buffer[0], ex);

		// add rfc822 header
		br = BRUTUS_IAttach_SetRFC822Headers(mapi_attach, &rfc822_name_fields, &rfc822_body_fields, ev);		
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
					    _("Could not set Content-MD5 header on attachment object: Communication error"));
			goto out;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
					    _("Could not set Content-MD5 header on attachment object"));
			goto out;
		}
	}

	// save changes to attachment obejct
	br = BRUTUS_IAttach_SaveChanges(mapi_attach, BRUTUS_BRUTUS_FORCE_SAVE, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Could not save changes to attachment object: Communication error"));
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not save changes to attachment object"));
		goto out;
	}

	// all is well
	retv = TRUE;

out:
	brutus_release_object(BRUTUS_IStream_tc, &mapi_stream, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	brutus_release_object(BRUTUS_IAttach_tc, &mapi_attach, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);

	brutus_free_seq_string_content(&rfc822_name_fields);

	brutus_free_seq_string_content(&rfc822_body_fields);

	g_free(content_id);

	g_free(mime_content_type);

	if (stream)
		camel_object_unref(CAMEL_OBJECT(stream));

	CORBA_exception_free(ev);

	return retv;
}

/*
 * The function will parse the multipart and append content as attachments as needed.
 */ 
static void
brutus_parse_multipart(BRUTUS_IMessage mapi_message,
		       CamelMultipart *multi_part, 
		       gboolean top_multi_part, // is multi_part the first multi part in the message or has we been called recursively by ourself?
		       CamelException *ex)
{
	/*contains multiple parts*/
	unsigned int n;
	int base_index;
	guint part_count;
	CamelMimePart *mime_part;
	CamelDataWrapper *dw = NULL;
	char *mime_type = NULL;

	/*
	 * Assuming the first part always is the actual message
	 * and an attachment otherwise.....
	 */
	if (top_multi_part) {
		mime_part = camel_multipart_get_part(multi_part, 0);
		dw = camel_medium_get_content_object(CAMEL_MEDIUM(mime_part));
		mime_type = camel_data_wrapper_get_mime_type(dw);
		
		if (g_ascii_strcasecmp(mime_type, "text/plain") && g_ascii_strcasecmp(mime_type, "text/html"))
			brutus_append_attachment(mapi_message, mime_part, FALSE, ex);		
		
		g_free(mime_type);
	}

	base_index = top_multi_part ? 1 : 0; // we have treated case '0' above
	part_count = camel_multipart_get_number(multi_part);

	for (n = base_index; n < part_count; n++) {
		mime_part = camel_multipart_get_part(multi_part, n);
		dw = camel_medium_get_content_object(CAMEL_MEDIUM(mime_part));

		if (CAMEL_IS_MULTIPART(dw)) 
			brutus_parse_multipart(mapi_message, CAMEL_MULTIPART(dw), FALSE, ex);
		else
			brutus_append_attachment(mapi_message, mime_part, FALSE, ex);		
	} 
}

gboolean
brutus_camel_mime_message_to_mapi(CamelMimeMessage *message,
				  BRUTUS_IMessage mapi_message,
				  BRUTUS_IMAPISession mapi_session,
				  PortableServer_POA poa,
				  const BRUTUS_seq_SPropValue *ex_prop_values,
				  CamelException *ex)
{
	unsigned int n;
	int bare_prop_count; // later adjusted to allow for text/[plain,html] or multipart/alternative
	const int base_prop_count = 5;
	size_t msg_prop_count;
	gboolean retv = FALSE;
	guint8 body_flags = 0x00; // plain, html or both
	CORBA_string text_plain = NULL;  // plain body text
	CORBA_string text_html = NULL;   // html body text
	const CamelInternetAddress *reply_to_addr = NULL;
	char *addr = NULL;
	//
	// used for mime parts
	CamelMultipart *multi = NULL;
	CamelMimePart *part = NULL;
	CamelStreamMem *content = NULL;
	CamelContentType *ct = NULL;
	CamelDataWrapper *dw = NULL;
	const char *charset = NULL;
	//
	// MAPI properties and interfaces
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_SPropProblemArray *mapi_problems = NULL;
	BRUTUS_seq_SPropValue msg_prop_values = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	BRUTUS_seq_string rfc822_name_fields = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	BRUTUS_seq_string rfc822_body_fields = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	// set recipients on MAPI message
	if (!brutus_set_recipients(message, mapi_message, mapi_session, poa, ex))
		goto out;

	// set Reply-To MIME header
	reply_to_addr = camel_mime_message_get_reply_to(message);
	if (reply_to_addr) {
		addr = camel_address_encode((CamelAddress *)reply_to_addr);

		rfc822_name_fields._maximum = 1;
		rfc822_name_fields._length = rfc822_name_fields._maximum;
		rfc822_name_fields._buffer = BRUTUS_seq_string_allocbuf(rfc822_name_fields._maximum);
		BRUTUS_CHECK_ALLOC(rfc822_name_fields._buffer, ex);
		rfc822_name_fields._buffer[0] = CORBA_string_dup("Reply-To");
		BRUTUS_CHECK_ALLOC(rfc822_name_fields._buffer[0], ex);

		rfc822_body_fields._maximum = 1;
		rfc822_body_fields._length = rfc822_body_fields._maximum;
		rfc822_body_fields._buffer = BRUTUS_seq_string_allocbuf(rfc822_body_fields._maximum);
		BRUTUS_CHECK_ALLOC(rfc822_body_fields._buffer, ex);
		rfc822_body_fields._buffer[0] = CORBA_string_dup(addr);		
		BRUTUS_CHECK_ALLOC(rfc822_body_fields._buffer[0], ex);

		// add Reply-To header
		br = BRUTUS_IAttach_SetRFC822Headers(mapi_message, &rfc822_name_fields, &rfc822_body_fields, ev);		
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
					    _("Could not set Reply-To header on email: Communication error"));
			goto out;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
					    _("Could not set Reply-To header on email"));
			goto out;
		}
	}

	//
	// message body
	//

	/** Get the mime parts from CamelMimemessge **/
	multi = (CamelMultipart*)camel_medium_get_content_object(CAMEL_MEDIUM(message));
	if (!multi) {
		d("Could not get content object");
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Mal-formatted CamelMimeMessage: No content object"));
		return FALSE;
	}

	//
	// now get the message body and eventual attachments
	// 

	if (CAMEL_IS_MULTIPART(multi)) {
		brutus_parse_multipart(mapi_message, multi, TRUE, ex);
		part = camel_multipart_get_part(multi, 0); // the first part is always the actual message
	} else 
		part = CAMEL_MIME_PART(message);

	ct = camel_mime_part_get_content_type(part);
	dw = camel_medium_get_content_object(CAMEL_MEDIUM(part));
	charset = camel_content_type_param(dw->mime_type, "charset");

	content = (CamelStreamMem*)camel_stream_mem_new();
	camel_data_wrapper_write_to_stream(dw, (CamelStream *)content);

	// body text: plain or html?
	if (!g_ascii_strcasecmp(ct->type, "text")) {
		do { // testing for supported text subtypes
			if (!g_ascii_strcasecmp(ct->subtype, "plain")) {
				body_flags |= BODY_TEXT_PLAIN;
				break;
			}

			if (!g_ascii_strcasecmp(ct->subtype, "html")) {
				body_flags |= BODY_TEXT_HTML;
				break;
			}
		} while (0);
	} else {
		; /* only attachments are present */
	}

	// copy body text, plain or html not both
	if (body_flags & BODY_TEXT_PLAIN) {
		text_plain = CORBA_string_alloc(content->buffer->len);
		memcpy(text_plain, content->buffer->data, content->buffer->len);
		text_plain[content->buffer->len] = '\0';
	} else if (body_flags & BODY_TEXT_HTML) {
		text_html = CORBA_string_alloc(content->buffer->len);
		memcpy(text_html, content->buffer->data, content->buffer->len);
		text_html[content->buffer->len] = '\0';
	} else {
		// no body text - copy attachment data
	}
	camel_object_unref(CAMEL_OBJECT(content));

	// initiate final msg prop sequence
	bare_prop_count = base_prop_count + !!(body_flags & BODY_TEXT_PLAIN) + !!(body_flags & BODY_TEXT_HTML); // allow simultaneous html and plain body text
	msg_prop_count = bare_prop_count + (ex_prop_values ? ex_prop_values->_length : 0);

	msg_prop_values._maximum = msg_prop_count;
	msg_prop_values._length = bare_prop_count;
	msg_prop_values._buffer = BRUTUS_seq_SPropValue_allocbuf(msg_prop_values._maximum);

	// message class
	msg_prop_values._buffer[0].ulPropTag = BRUTUS_BRUTUS_PR_MESSAGE_CLASS;
	msg_prop_values._buffer[0].Value._d = BRUTUS_BRUTUS_PT_STRING8;
	msg_prop_values._buffer[0].Value._u.lpszA = CORBA_string_dup("IPM.Note"); // http://support.microsoft.com/?kbid=313523

	// subject
	msg_prop_values._buffer[1].ulPropTag = BRUTUS_BRUTUS_PR_SUBJECT;
	msg_prop_values._buffer[1].Value._d = BRUTUS_BRUTUS_PT_STRING8;
	msg_prop_values._buffer[1].Value._u.lpszA = CORBA_string_dup(camel_mime_message_get_subject(message));

	// charset encoding
	msg_prop_values._buffer[2].ulPropTag = BRUTUS_BRUTUS_PR_INTERNET_CPID;
	msg_prop_values._buffer[2].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[2].Value._u.l = charset ? brutus_label_to_cpid(charset) : brutus_label_to_cpid("UTF-8"); // defaulting to UTF-8

	// body text format
	msg_prop_values._buffer[3].ulPropTag = BRUTUS_BRUTUS_PR_MSG_EDITOR_FORMAT;
	msg_prop_values._buffer[3].Value._d = BRUTUS_BRUTUS_PT_LONG;
	if ((body_flags & BODY_TEXT_PLAIN) && (body_flags & BODY_TEXT_HTML))
		msg_prop_values._buffer[3].Value._u.l = BRUTUS_BRUTUS_EDITOR_FORMAT_DONTKNOW; // what else ?
	else if (body_flags & BODY_TEXT_PLAIN)
		msg_prop_values._buffer[3].Value._u.l = BRUTUS_BRUTUS_EDITOR_FORMAT_PLAINTEXT;
	else if (body_flags & BODY_TEXT_HTML)
		msg_prop_values._buffer[3].Value._u.l = BRUTUS_BRUTUS_EDITOR_FORMAT_HTML;

	// force MIME encoding
	msg_prop_values._buffer[4].ulPropTag = BRUTUS_BRUTUS_PR_INETMAIL_OVERRIDE_FORMAT;
	msg_prop_values._buffer[4].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[4].Value._u.l = BRUTUS_BRUTUS_ENCODEMIME;

	// body text
	if ((body_flags & BODY_TEXT_PLAIN) && (body_flags & BODY_TEXT_HTML)) {
		msg_prop_values._buffer[msg_prop_values._length - 2].ulPropTag = BRUTUS_BRUTUS_PR_BODY;
		msg_prop_values._buffer[msg_prop_values._length - 2].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		msg_prop_values._buffer[msg_prop_values._length - 2].Value._u.lpszA = text_plain;
		msg_prop_values._buffer[msg_prop_values._length - 1].ulPropTag = BRUTUS_BRUTUS_PR_BODY_HTML;
		msg_prop_values._buffer[msg_prop_values._length - 1].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		msg_prop_values._buffer[msg_prop_values._length - 1].Value._u.lpszA = text_html;
	} else if (body_flags & BODY_TEXT_PLAIN) {
		msg_prop_values._buffer[msg_prop_values._length - 1].ulPropTag = BRUTUS_BRUTUS_PR_BODY;
		msg_prop_values._buffer[msg_prop_values._length - 1].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		msg_prop_values._buffer[msg_prop_values._length - 1].Value._u.lpszA = text_plain;
	} else if (body_flags & BODY_TEXT_HTML) {
		msg_prop_values._buffer[msg_prop_values._length - 1].ulPropTag = BRUTUS_BRUTUS_PR_BODY_HTML;
		msg_prop_values._buffer[msg_prop_values._length - 1].Value._d = BRUTUS_BRUTUS_PT_STRING8;
		msg_prop_values._buffer[msg_prop_values._length - 1].Value._u.lpszA = text_html;
	}

	// append the additional properties
	if (ex_prop_values && ex_prop_values->_length) {
		for (n = 0; n < ex_prop_values->_length; n++)
			brutus_copy_spropvalue(&msg_prop_values._buffer[n + msg_prop_values._length], &ex_prop_values->_buffer[n]);

		msg_prop_values._length = msg_prop_values._maximum;
	}

	mapi_problems = BRUTUS_seq_SPropProblem__alloc();
	mapi_problems->_maximum = 0;
	mapi_problems->_length = 0;
	mapi_problems->_buffer = NULL;
	br = BRUTUS_IMessage_SetProps(mapi_message, &msg_prop_values, FALSE, &mapi_problems, ev);
	CORBA_free(mapi_problems);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
				    _("Could not set properties on outgoing message: Communication error"));
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not set properties on outgoing message"));
		goto out;
	}

	// all is well...
	retv = TRUE;

out:
	g_free(addr);
	brutus_free_seq_string_content(&rfc822_name_fields);
	brutus_free_seq_string_content(&rfc822_body_fields);
	brutus_free_seq_spropvalue_content(&msg_prop_values, poa);

	CORBA_exception_free(ev);
	return retv;
}
