/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus source file for implementing type conversion helper functions.
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "brutus_conv.h"
#include <glib/gprintf.h>

/*
 * Number of minutes that the UNIX epoch (1970-01-01 00:00:00) is
 * offset from the WIN32 epoch (1601-01-01 00:00:00). I can't use
 * seconds as that would give a number which is to big for a 32 bit
 * "long" type. 32 bits sucks...
 */
#define TIME_T_EPOCH_OFFSET 194074560UL
#define SECONDS_TO_100NS 10000000

time_t
brutus_filetime_to_time_t(const BRUTUS_FILETIME ft)
{
	const guint64 filetime = ((guint64)ft.dwHighDateTime << 32) + ft.dwLowDateTime;

	return (time_t)((filetime/SECONDS_TO_100NS) - (TIME_T_EPOCH_OFFSET * 60));
}

void
brutus_time_t_to_filetime(const time_t tt,
			  BRUTUS_FILETIME *ft)
{
	const guint64 filetime = (guint64)(((guint64)tt + (guint64)((guint64)TIME_T_EPOCH_OFFSET * 60)) * (guint64)SECONDS_TO_100NS);

	ft->dwHighDateTime = (CORBA_unsigned_long)(filetime >> 32);
	ft->dwLowDateTime = (CORBA_unsigned_long)(filetime & 0x00000000FFFFFFFF);
}

void
brutus_string_to_time_t(const char *str, // NULL ==> BOOM!
			const char *format,
			time_t *timep)
{
	struct tm stm;

	strptime(str, format, &stm);
	*timep = mktime(&stm);
}

void
brutus_time_t_to_string(const time_t timep,
			const size_t len,
			const char *format,
			char *str)
{
	struct tm stm;

	gmtime_r(&timep, &stm);
	strftime(str, len, format, &stm);
}

const gchar*
brutus_bresult_to_str(const BRUTUS_BRESULT br)
{
	switch (br) {
	case BRUTUS_BRUTUS_INTERNAL_ERROR :
		return "BRUTUS_INTERNAL_ERROR";
	case BRUTUS_BRUTUS_UNKNOWN_ERROR :
		return "BRUTUS_UNKNOWN_ERROR";
	case BRUTUS_BRUTUS_NO_MAPI_SESSION :
		return "BRUTUS_NO_MAPI_SESSION";
	case BRUTUS_BRUTUS_EXCEPTION_FROM_CLIENT :
		return "BRUTUS_EXCEPTION_FROM_CLIENT";
	case BRUTUS_BRUTUS_LIFELINE_TIMEOUT :
		return "BRUTUS_LIFELINE_TIMEOUT";
	case BRUTUS_BRUTUS_S_OK :
		return "S_OK";
	case BRUTUS_BRUTUS_MAPI_E_CALL_FAILED :
		return "MAPI_E_CALL_FAILED";
	case BRUTUS_BRUTUS_MAPI_E_NOT_ENOUGH_MEMORY :
		return "MAPI_E_NOT_ENOUGH_MEMORY";
	case BRUTUS_BRUTUS_MAPI_E_INVALID_PARAMETER :
		return "MAPI_E_INVALID_PARAMETER";
	case BRUTUS_BRUTUS_MAPI_E_INTERFACE_NOT_SUPPORTED :
		return "MAPI_E_INTERFACE_NOT_SUPPORTED";
	case BRUTUS_BRUTUS_MAPI_E_NO_ACCESS :
		return "MAPI_E_NO_ACCESS";
	case BRUTUS_BRUTUS_MAPI_E_NO_SUPPORT :
		return "MAPI_E_NO_SUPPORT";
	case BRUTUS_BRUTUS_MAPI_E_BAD_CHARWIDTH :
		return "MAPI_E_BAD_CHARWIDTH";
	case BRUTUS_BRUTUS_MAPI_E_STRING_TOO_LONG :
		return "MAPI_E_STRING_TOO_LONG";
	case BRUTUS_BRUTUS_MAPI_E_UNKNOWN_FLAGS :
		return "MAPI_E_UNKNOWN_FLAGS";
	case BRUTUS_BRUTUS_MAPI_E_INVALID_ENTRYID :
		return "MAPI_E_INVALID_ENTRYID";
	case BRUTUS_BRUTUS_MAPI_E_INVALID_OBJECT :
		return "MAPI_E_INVALID_OBJECT";
	case BRUTUS_BRUTUS_MAPI_E_OBJECT_CHANGED :
		return "MAPI_E_OBJECT_CHANGED";
	case BRUTUS_BRUTUS_MAPI_E_OBJECT_DELETED :
		return "MAPI_E_OBJECT_DELETED";
	case BRUTUS_BRUTUS_MAPI_E_BUSY :
		return "MAPI_E_BUSY";
	case BRUTUS_BRUTUS_MAPI_E_NOT_ENOUGH_DISK :
		return "MAPI_E_NOT_ENOUGH_DISK";
	case BRUTUS_BRUTUS_MAPI_E_NOT_ENOUGH_RESOURCES :
		return "MAPI_E_NOT_ENOUGH_RESOURCES";
	case BRUTUS_BRUTUS_MAPI_E_NOT_FOUND :
		return "MAPI_E_NOT_FOUND";
	case BRUTUS_BRUTUS_MAPI_E_VERSION :
		return "MAPI_E_VERSION";
	case BRUTUS_BRUTUS_MAPI_E_LOGON_FAILED :
		return "MAPI_E_LOGON_FAILED";
	case BRUTUS_BRUTUS_MAPI_E_SESSION_LIMIT :
		return "MAPI_E_SESSION_LIMIT";
	case BRUTUS_BRUTUS_MAPI_E_USER_CANCEL :
		return "MAPI_E_USER_CANCEL";
	case BRUTUS_BRUTUS_MAPI_E_UNABLE_TO_ABORT :
		return "MAPI_E_UNABLE_TO_ABORT";
	case BRUTUS_BRUTUS_MAPI_E_NETWORK_ERROR :
		return "MAPI_E_NETWORK_ERROR";
	case BRUTUS_BRUTUS_MAPI_E_DISK_ERROR :
		return "MAPI_E_DISK_ERROR";
	case BRUTUS_BRUTUS_MAPI_E_TOO_COMPLEX :
		return "MAPI_E_TOO_COMPLEX";
	case BRUTUS_BRUTUS_MAPI_E_BAD_COLUMN :
		return "MAPI_E_BAD_COLUMN";
	case BRUTUS_BRUTUS_MAPI_E_EXTENDED_ERROR :
		return "MAPI_E_EXTENDED_ERROR";
	case BRUTUS_BRUTUS_MAPI_E_COMPUTED :
		return "MAPI_E_COMPUTED";
	case BRUTUS_BRUTUS_MAPI_E_CORRUPT_DATA :
		return "MAPI_E_CORRUPT_DATA";
	case BRUTUS_BRUTUS_MAPI_E_UNCONFIGURED :
		return "MAPI_E_UNCONFIGURED";
	case BRUTUS_BRUTUS_MAPI_E_FAILONEPROVIDER :
		return "MAPI_E_FAILONEPROVIDER";
	case BRUTUS_BRUTUS_MAPI_E_UNKNOWN_CPID :
		return "MAPI_E_UNKNOWN_CPID";
	case BRUTUS_BRUTUS_MAPI_E_UNKNOWN_LCID :
		return "MAPI_E_UNKNOWN_LCID";
	case BRUTUS_BRUTUS_MAPI_E_PASSWORD_CHANGE_REQUIRED :
		return "MAPI_E_PASSWORD_CHANGE_REQUIRED";
	case BRUTUS_BRUTUS_MAPI_E_PASSWORD_EXPIRED :
		return "MAPI_E_PASSWORD_EXPIRED";
	case BRUTUS_BRUTUS_MAPI_E_INVALID_WORKSTATION_ACCOUNT :
		return "MAPI_E_INVALID_WORKSTATION_ACCOUNT";
	case BRUTUS_BRUTUS_MAPI_E_INVALID_ACCESS_TIME :
		return "MAPI_E_INVALID_ACCESS_TIME";
	case BRUTUS_BRUTUS_MAPI_E_ACCOUNT_DISABLED :
		return "MAPI_E_ACCOUNT_DISABLED";
	case BRUTUS_BRUTUS_MAPI_E_END_OF_SESSION :
		return "MAPI_E_END_OF_SESSION";
	case BRUTUS_BRUTUS_MAPI_E_UNKNOWN_ENTRYID :
		return "MAPI_E_UNKNOWN_ENTRYID";
	case BRUTUS_BRUTUS_MAPI_E_MISSING_REQUIRED_COLUMN :
		return "MAPI_E_MISSING_REQUIRED_COLUMN";
	case BRUTUS_BRUTUS_MAPI_W_NO_SERVICE :
		return "MAPI_W_NO_SERVICE";
	case BRUTUS_BRUTUS_MAPI_E_BAD_VALUE :
		return "MAPI_E_BAD_VALUE";
	case BRUTUS_BRUTUS_MAPI_E_INVALID_TYPE :
		return "MAPI_E_INVALID_TYPE";
	case BRUTUS_BRUTUS_MAPI_E_TYPE_NO_SUPPORT :
		return "MAPI_E_TYPE_NO_SUPPORT";
	case BRUTUS_BRUTUS_MAPI_E_UNEXPECTED_TYPE :
		return "MAPI_E_UNEXPECTED_TYPE";
	case BRUTUS_BRUTUS_MAPI_E_TOO_BIG :
		return "MAPI_E_TOO_BIG";
	case BRUTUS_BRUTUS_MAPI_E_DECLINE_COPY :
		return "MAPI_E_DECLINE_COPY";
	case BRUTUS_BRUTUS_MAPI_E_UNEXPECTED_ID :
		return "MAPI_E_UNEXPECTED_ID";
	case BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED :
		return "MAPI_W_ERRORS_RETURNED";
	case BRUTUS_BRUTUS_MAPI_E_UNABLE_TO_COMPLETE :
		return "MAPI_E_UNABLE_TO_COMPLETE";
	case BRUTUS_BRUTUS_MAPI_E_TIMEOUT :
		return "MAPI_E_TIMEOUT";
	case BRUTUS_BRUTUS_MAPI_E_TABLE_EMPTY :
		return "MAPI_E_TABLE_EMPTY";
	case BRUTUS_BRUTUS_MAPI_E_TABLE_TOO_BIG :
		return "MAPI_E_TABLE_TOO_BIG";
	case BRUTUS_BRUTUS_MAPI_E_INVALID_BOOKMARK :
		return "MAPI_E_INVALID_BOOKMARK";
	case BRUTUS_BRUTUS_MAPI_W_POSITION_CHANGED :
		return "MAPI_W_POSITION_CHANGED";
	case BRUTUS_BRUTUS_MAPI_W_APPROX_COUNT :
		return "MAPI_W_APPROX_COUNT";
	case BRUTUS_BRUTUS_MAPI_E_WAIT :
		return "MAPI_E_WAIT";
	case BRUTUS_BRUTUS_MAPI_E_CANCEL :
		return "MAPI_E_CANCEL";
	case BRUTUS_BRUTUS_MAPI_E_NOT_ME :
		return "MAPI_E_NOT_ME";
	case BRUTUS_BRUTUS_MAPI_W_CANCEL_MESSAGE :
		return "MAPI_W_CANCEL_MESSAGE";
	case BRUTUS_BRUTUS_MAPI_E_CORRUPT_STORE :
		return "MAPI_E_CORRUPT_STORE";
	case BRUTUS_BRUTUS_MAPI_E_NOT_IN_QUEUE :
		return "MAPI_E_NOT_IN_QUEUE";
	case BRUTUS_BRUTUS_MAPI_E_NO_SUPPRESS :
		return "MAPI_E_NO_SUPPRESS";
	case BRUTUS_BRUTUS_MAPI_E_COLLISION :
		return "MAPI_E_COLLISION";
	case BRUTUS_BRUTUS_MAPI_E_NOT_INITIALIZED :
		return "MAPI_E_NOT_INITIALIZED";
	case BRUTUS_BRUTUS_MAPI_E_NON_STANDARD :
		return "MAPI_E_NON_STANDARD";
	case BRUTUS_BRUTUS_MAPI_E_NO_RECIPIENTS :
		return "MAPI_E_NO_RECIPIENTS";
	case BRUTUS_BRUTUS_MAPI_E_SUBMITTED :
		return "MAPI_E_SUBMITTED";
	case BRUTUS_BRUTUS_MAPI_E_HAS_FOLDERS :
		return "MAPI_E_HAS_FOLDERS";
	case BRUTUS_BRUTUS_MAPI_E_HAS_MESSAGES :
		return "MAPI_E_HAS_MESSAGES";
	case BRUTUS_BRUTUS_MAPI_E_FOLDER_CYCLE :
		return "MAPI_E_FOLDER_CYCLE";
	case BRUTUS_BRUTUS_MAPI_W_PARTIAL_COMPLETION :
		return "MAPI_W_PARTIAL_COMPLETION";
	case BRUTUS_BRUTUS_MAPI_E_AMBIGUOUS_RECIP :
		return "MAPI_E_AMBIGUOUS_RECIP";
	case BRUTUS_BRUTUS_OLEOBJ_S_CANNOT_DOVERB_NOW :
		return "OLEOBJ_S_CANNOT_DOVERB_NOW";
	case BRUTUS_BRUTUS_STG_E_INVALIDFUNCTION :
		return "STG_E_INVALIDFUNCTION";
	case BRUTUS_BRUTUS_STG_E_FILENOTFOUND :
		return "STG_E_FILENOTFOUND";
	case BRUTUS_BRUTUS_STG_E_PATHNOTFOUND :
		return "STG_E_PATHNOTFOUND";
	case BRUTUS_BRUTUS_STG_E_TOOMANYOPENFILES :
		return "STG_E_TOOMANYOPENFILES";
	case BRUTUS_BRUTUS_STG_E_ACCESSDENIED :
		return "STG_E_ACCESSDENIED";
	case BRUTUS_BRUTUS_STG_E_INVALIDHANDLE :
		return "STG_E_INVALIDHANDLE";
	case BRUTUS_BRUTUS_STG_E_INSUFFICIENTMEMORY :
		return "STG_E_INSUFFICIENTMEMORY";
	case BRUTUS_BRUTUS_STG_E_INVALIDPOINTER :
		return "STG_E_INVALIDPOINTER";
	case BRUTUS_BRUTUS_STG_E_NOMOREFILES :
		return "STG_E_NOMOREFILES";
	case BRUTUS_BRUTUS_STG_E_DISKISWRITEPROTECTED :
		return "STG_E_DISKISWRITEPROTECTED";
	case BRUTUS_BRUTUS_STG_E_SEEKERROR :
		return "STG_E_SEEKERROR";
	case BRUTUS_BRUTUS_STG_E_WRITEFAULT :
		return "STG_E_WRITEFAULT";
	case BRUTUS_BRUTUS_STG_E_READFAULT :
		return "STG_E_READFAULT";
	case BRUTUS_BRUTUS_STG_E_SHAREVIOLATION :
		return "STG_E_SHAREVIOLATION";
	case BRUTUS_BRUTUS_STG_E_LOCKVIOLATION :
		return "STG_E_LOCKVIOLATION";
	case BRUTUS_BRUTUS_STG_E_FILEALREADYEXISTS :
		return "STG_E_FILEALREADYEXISTS";
	case BRUTUS_BRUTUS_STG_E_INVALIDPARAMETER :
		return "STG_E_INVALIDPARAMETER";
	case BRUTUS_BRUTUS_STG_E_MEDIUMFULL :
		return "STG_E_MEDIUMFULL";
	case BRUTUS_BRUTUS_STG_E_PROPSETMISMATCHED :
		return "STG_E_PROPSETMISMATCHED";
	case BRUTUS_BRUTUS_STG_E_ABNORMALAPIEXIT :
		return "STG_E_ABNORMALAPIEXIT";
	case BRUTUS_BRUTUS_STG_E_INVALIDHEADER :
		return "STG_E_INVALIDHEADER";
	case BRUTUS_BRUTUS_STG_E_INVALIDNAME :
		return "STG_E_INVALIDNAME";
	case BRUTUS_BRUTUS_STG_E_UNKNOWN :
		return "STG_E_UNKNOWN";
	case BRUTUS_BRUTUS_STG_E_UNIMPLEMENTEDFUNCTION :
		return "STG_E_UNIMPLEMENTEDFUNCTION";
	case BRUTUS_BRUTUS_STG_E_INVALIDFLAG :
		return "STG_E_INVALIDFLAG";
	case BRUTUS_BRUTUS_STG_E_INUSE :
		return "STG_E_INUSE";
	case BRUTUS_BRUTUS_STG_E_NOTCURRENT :
		return "STG_E_NOTCURRENT";
	case BRUTUS_BRUTUS_STG_E_REVERTED :
		return "STG_E_REVERTED";
	case BRUTUS_BRUTUS_STG_E_CANTSAVE :
		return "STG_E_CANTSAVE";
	case BRUTUS_BRUTUS_STG_E_OLDFORMAT :
		return "STG_E_OLDFORMAT";
	case BRUTUS_BRUTUS_STG_E_OLDDLL :
		return "STG_E_OLDDLL";
	case BRUTUS_BRUTUS_STG_E_SHAREREQUIRED :
		return "STG_E_SHAREREQUIRED";
	case BRUTUS_BRUTUS_STG_E_NOTFILEBASEDSTORAGE :
		return "STG_E_NOTFILEBASEDSTORAGE";
	case BRUTUS_BRUTUS_STG_E_EXTANTMARSHALLINGS :
		return "STG_E_EXTANTMARSHALLINGS";
	case BRUTUS_BRUTUS_STG_E_DOCFILECORRUPT :
		return "STG_E_DOCFILECORRUPT";
	case BRUTUS_BRUTUS_STG_E_BADBASEADDRESS :
		return "STG_E_BADBASEADDRESS";
	case BRUTUS_BRUTUS_STG_E_INCOMPLETE :
		return "STG_E_INCOMPLETE";
	case BRUTUS_BRUTUS_STG_E_TERMINATED :
		return "STG_E_TERMINATED";
	case BRUTUS_BRUTUS_STG_S_CONVERTED :
		return "STG_S_CONVERTED";
	case BRUTUS_BRUTUS_STG_S_BLOCK :
		return "STG_S_BLOCK";
	case BRUTUS_BRUTUS_STG_S_RETRYNOW :
		return "STG_S_RETRYNOW";
	case BRUTUS_BRUTUS_STG_S_MONITORING :
		return "STG_S_MONITORING";
	case BRUTUS_BRUTUS_STG_S_MULTIPLEOPENS :
		return "STG_S_MULTIPLEOPENS";
	case BRUTUS_BRUTUS_STG_S_CONSOLIDATIONFAILED :
		return "STG_S_CONSOLIDATIONFAILED";
	case BRUTUS_BRUTUS_STG_S_CANNOTCONSOLIDATE :
		return "STG_S_CANNOTCONSOLIDATE";
	case BRUTUS_BRUTUS_E_PENDING :
		return "E_PENDING";
	case BRUTUS_BRUTUS_SYNC_E_OBJECT_DELETED :
		return "SYNC_E_OBJECT_DELETED";
	case BRUTUS_BRUTUS_SYNC_E_IGNORE :
		return "SYNC_E_IGNORE";
	case BRUTUS_BRUTUS_SYNC_E_CONFLICT :
		return "SYNC_E_CONFLICT";
	case BRUTUS_BRUTUS_SYNC_E_NO_PARENT :
		return "SYNC_E_NO_PARENT";
	case BRUTUS_BRUTUS_SYNC_E_INCEST :
		return "SYNC_E_INCEST";
	case BRUTUS_BRUTUS_SYNC_E_UNSYNCHRONIZED :
		return "SYNC_E_UNSYNCHRONIZED";
	case BRUTUS_BRUTUS_SYNC_W_PROGRESS :
		return "SYNC_W_PROGRESS";
	case BRUTUS_BRUTUS_SYNC_W_CLIENT_CHANGE_NEWER :
		return "SYNC_W_CLIENT_CHANGE_NEWER";
	case BRUTUS_BRUTUS_EC_EDK_E_FAIL :
		return "EC_EDK_E_FAIL";
	case BRUTUS_BRUTUS_EC_EDK_E_OUTOFMEMORY :
		return "EC_EDK_E_OUTOFMEMORY";
	case BRUTUS_BRUTUS_EC_EDK_E_INVALIDARG :
		return "EC_EDK_E_INVALIDARG";
	case BRUTUS_BRUTUS_EC_EDK_E_NOTIMPL :
		return "EC_EDK_E_NOTIMPL";
	case BRUTUS_BRUTUS_EC_EDK_E_NOINTERFACE :
		return "EC_EDK_E_NOINTERFACE";
	case BRUTUS_BRUTUS_EC_EDK_E_ACCESSDENIED :
		return "EC_EDK_E_ACCESSDENIED";
	case BRUTUS_BRUTUS_EC_EDK_E_UNKNOWN :
		return "EC_EDK_E_UNKNOWN";
	case BRUTUS_BRUTUS_EC_EDK_E_NOT_FOUND :
		return "EC_EDK_E_NOT_FOUND";
	case BRUTUS_BRUTUS_EDK_E_NOT_FOUND :
		return "EDK_E_NOT_FOUND";
	case BRUTUS_BRUTUS_EC_EDK_E_SHUTDOWN_SERVICE :
		return "EC_EDK_E_SHUTDOWN_SERVICE";
	case BRUTUS_BRUTUS_EDK_E_SHUTDOWN_SERVICE :
		return "EDK_E_SHUTDOWN_SERVICE";
	case BRUTUS_BRUTUS_EC_EDK_E_ALREADY_EXISTS :
		return "EC_EDK_E_ALREADY_EXISTS";
	case BRUTUS_BRUTUS_EDK_E_ALREADY_EXISTS :
		return "EDK_E_ALREADY_EXISTS";
	case BRUTUS_BRUTUS_EC_EDK_E_END_OF_FILE :
		return "EC_EDK_E_END_OF_FILE";
	case BRUTUS_BRUTUS_EDK_E_END_OF_FILE :
		return "EDK_E_END_OF_FILE";
	case BRUTUS_BRUTUS_EC_EDK_E_AMBIGUOUS :
		return "EC_EDK_E_AMBIGUOUS";
	case BRUTUS_BRUTUS_EDK_E_AMBIGUOUS :
		return "EDK_E_AMBIGUOUS";
	case BRUTUS_BRUTUS_EC_EDK_E_PARSE :
		return "EC_EDK_E_PARSE";
	case BRUTUS_BRUTUS_EDK_E_PARSE :
		return "EDK_E_PARSE";
	default :
		return "ERROR: Unknown return code";
	};
}

const gchar*
brutus_proptype_to_str(const BRUTUS_PT_TYPE PropType)
{
	// Property types
	switch (PropType) {
	case BRUTUS_BRUTUS_PT_UNSPECIFIED :
		return "PT_UNSPECIFIED";
	case BRUTUS_BRUTUS_PT_NULL :
		return "PT_NULL";
	case BRUTUS_BRUTUS_PT_SHORT :
		return "PT_SHORT";
	case BRUTUS_BRUTUS_PT_USHORT :
		return "PT_USHORT";
	case BRUTUS_BRUTUS_PT_LONG :
		return "PT_LONG";
	case BRUTUS_BRUTUS_PT_ULONG :
		return "PT_ULONG";
	case BRUTUS_BRUTUS_PT_FLOAT :
		return "PT_FLOAT";
	case BRUTUS_BRUTUS_PT_DOUBLE :
		return "PT_DOUBLE";
	case BRUTUS_BRUTUS_PT_BOOLEAN :
		return "PT_BOOLEAN";
	case BRUTUS_BRUTUS_PT_CURRENCY :
		return "PT_CURRENCY";
	case BRUTUS_BRUTUS_PT_APPTIME :
		return "PT_APPTIME";
	case BRUTUS_BRUTUS_PT_SYSTIME :
		return "PT_SYSTIME";
	case BRUTUS_BRUTUS_PT_STRING8 :
		return "PT_STRING8";
	case BRUTUS_BRUTUS_PT_BINARY :
		return "PT_BINARY";
	case BRUTUS_BRUTUS_PT_UNICODE :
		return "PT_UNICODE";
	case BRUTUS_BRUTUS_PT_CLSID :
		return "PT_CLSID";
	case BRUTUS_BRUTUS_PT_LONGLONG :
		return "PT_LONGLONG";
	case BRUTUS_BRUTUS_PT_MV_SHORT :
		return "PT_MV_SHORT";
	case BRUTUS_BRUTUS_PT_MV_LONG :
		return "PT_MV_LONG";
	case BRUTUS_BRUTUS_PT_MV_FLOAT :
		return "PT_MV_FLOAT";
	case BRUTUS_BRUTUS_PT_MV_DOUBLE :
		return "PT_MV_DOUBLE";
	case BRUTUS_BRUTUS_PT_MV_CURRENCY :
		return "PT_MV_CURRENCY";
	case BRUTUS_BRUTUS_PT_MV_APPTIME :
		return "PT_MV_APPTIME";
	case BRUTUS_BRUTUS_PT_MV_SYSTIME :
		return "PT_MV_SYSTIME";
	case BRUTUS_BRUTUS_PT_MV_BINARY :
		return "PT_MV_BINARY";
	case BRUTUS_BRUTUS_PT_MV_STRING8 :
		return "PT_MV_STRING8";
	case BRUTUS_BRUTUS_PT_MV_UNICODE :
		return "PT_MV_UNICODE";
	case BRUTUS_BRUTUS_PT_MV_CLSID :
		return "PT_MV_CLSID";
	case BRUTUS_BRUTUS_PT_MV_LONGLONG :
		return "PT_MV_LONGLONG";
	case BRUTUS_BRUTUS_PT_MVI_SHORT :
		return "PT_MVI_SHORT";
	case BRUTUS_BRUTUS_PT_MVI_LONG :
		return "PT_MVI_LONG";
	case BRUTUS_BRUTUS_PT_MVI_FLOAT :
		return "PT_MVI_FLOAT";
	case BRUTUS_BRUTUS_PT_MVI_DOUBLE :
		return "PT_MVI_DOUBLE";
	case BRUTUS_BRUTUS_PT_MVI_CURRENCY :
		return "PT_MVI_CURRENCY";
	case BRUTUS_BRUTUS_PT_MVI_APPTIME :
		return "PT_MVI_APPTIME";
	case BRUTUS_BRUTUS_PT_MVI_SYSTIME :
		return "PT_MVI_SYSTIME";
	case BRUTUS_BRUTUS_PT_MVI_BINARY :
		return "PT_MVI_BINARY";
	case BRUTUS_BRUTUS_PT_MVI_STRING8 :
		return "PT_MVI_STRING8";
	case BRUTUS_BRUTUS_PT_MVI_UNICODE :
		return "PT_MVI_UNICODE";
	case BRUTUS_BRUTUS_PT_MVI_CLSID :
		return "PT_MVI_CLSID";
	case BRUTUS_BRUTUS_PT_MVI_LONGLONG :
		return "PT_MVI_LONGLONG";
	case BRUTUS_BRUTUS_PT_ERROR :
		return "PT_ERROR";
	case BRUTUS_BRUTUS_PT_OBJECT :
		return "PT_OBJECT";
	default :
		return "ERROR: Unknown property type";
	};
}

/* Duplicate values has been commented out */
const gchar*
brutus_proptag_to_str(const BRUTUS_BDEFINE PropTag)
{
	static gchar unknown_tag[256] = { '\0' };

	// Property tags
	switch (PropTag) {
	case BRUTUS_BRUTUS_PR_ACKNOWLEDGEMENT_MODE :
		return "PR_ACKNOWLEDGEMENT_MODE";
	case BRUTUS_BRUTUS_PR_ALTERNATE_RECIPIENT_ALLOWED :
		return "PR_ALTERNATE_RECIPIENT_ALLOWED";
	case BRUTUS_BRUTUS_PR_AUTHORIZING_USERS :
		return "PR_AUTHORIZING_USERS";
	case BRUTUS_BRUTUS_PR_AUTO_FORWARD_COMMENT :
		return "PR_AUTO_FORWARD_COMMENT/BRUTUS_PR_AUTO_FORWARD_COMMENT_A";
	case BRUTUS_BRUTUS_PR_AUTO_FORWARD_COMMENT_W :
		return "PR_AUTO_FORWARD_COMMENT_W";
	case BRUTUS_BRUTUS_PR_AUTO_FORWARDED :
		return "PR_AUTO_FORWARDED";
	case BRUTUS_BRUTUS_PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID :
		return "PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID";
	case BRUTUS_BRUTUS_PR_CONTENT_CORRELATOR :
		return "PR_CONTENT_CORRELATOR";
	case BRUTUS_BRUTUS_PR_CONTENT_IDENTIFIER :
		return "PR_CONTENT_IDENTIFIER/BRUTUS_PR_CONTENT_IDENTIFIER_A";
	case BRUTUS_BRUTUS_PR_CONTENT_IDENTIFIER_W :
		return "PR_CONTENT_IDENTIFIER_W";
	case BRUTUS_BRUTUS_PR_CONTENT_LENGTH :
		return "PR_CONTENT_LENGTH";
	case BRUTUS_BRUTUS_PR_CONTENT_RETURN_REQUESTED :
		return "PR_CONTENT_RETURN_REQUESTED";
	case BRUTUS_BRUTUS_PR_CONVERSATION_KEY :
		return "PR_CONVERSATION_KEY";
	case BRUTUS_BRUTUS_PR_CONVERSION_EITS :
		return "PR_CONVERSION_EITS";
	case BRUTUS_BRUTUS_PR_CONVERSION_WITH_LOSS_PROHIBITED :
		return "PR_CONVERSION_WITH_LOSS_PROHIBITED";
	case BRUTUS_BRUTUS_PR_CONVERTED_EITS :
		return "PR_CONVERTED_EITS";
	case BRUTUS_BRUTUS_PR_DEFERRED_DELIVERY_TIME :
		return "PR_DEFERRED_DELIVERY_TIME";
	case BRUTUS_BRUTUS_PR_DELIVER_TIME :
		return "PR_DELIVER_TIME";
	case BRUTUS_BRUTUS_PR_DISCARD_REASON :
		return "PR_DISCARD_REASON";
	case BRUTUS_BRUTUS_PR_DISCLOSURE_OF_RECIPIENTS :
		return "PR_DISCLOSURE_OF_RECIPIENTS";
	case BRUTUS_BRUTUS_PR_DL_EXPANSION_HISTORY :
		return "PR_DL_EXPANSION_HISTORY";
	case BRUTUS_BRUTUS_PR_DL_EXPANSION_PROHIBITED :
		return "PR_DL_EXPANSION_PROHIBITED";
	case BRUTUS_BRUTUS_PR_EXPIRY_TIME :
		return "PR_EXPIRY_TIME";
	case BRUTUS_BRUTUS_PR_IMPLICIT_CONVERSION_PROHIBITED :
		return "PR_IMPLICIT_CONVERSION_PROHIBITED";
	case BRUTUS_BRUTUS_PR_IMPORTANCE :
		return "PR_IMPORTANCE";
	case BRUTUS_BRUTUS_PR_IPM_ID :
		return "PR_IPM_ID";
	case BRUTUS_BRUTUS_PR_LATEST_DELIVERY_TIME :
		return "PR_LATEST_DELIVERY_TIME";
	case BRUTUS_BRUTUS_PR_MESSAGE_CLASS :
		return "PR_MESSAGE_CLASS/BRUTUS_PR_MESSAGE_CLASS_A";
	case BRUTUS_BRUTUS_PR_MESSAGE_CLASS_W :
		return "PR_MESSAGE_CLASS_W";
	case BRUTUS_BRUTUS_PR_MESSAGE_DELIVERY_ID :
		return "PR_MESSAGE_DELIVERY_ID";
	case BRUTUS_BRUTUS_PR_MESSAGE_SECURITY_LABEL :
		return "PR_MESSAGE_SECURITY_LABEL";
	case BRUTUS_BRUTUS_PR_OBSOLETED_IPMS :
		return "PR_OBSOLETED_IPMS";
	case BRUTUS_BRUTUS_PR_ORIGINALLY_INTENDED_RECIPIENT_NAME :
		return "PR_ORIGINALLY_INTENDED_RECIPIENT_NAME";
	case BRUTUS_BRUTUS_PR_ORIGINAL_EITS :
		return "PR_ORIGINAL_EITS";
	case BRUTUS_BRUTUS_PR_ORIGINATOR_CERTIFICATE :
		return "PR_ORIGINATOR_CERTIFICATE";
	case BRUTUS_BRUTUS_PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED :
		return "PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED";
	case BRUTUS_BRUTUS_PR_ORIGINATOR_RETURN_ADDRESS :
		return "PR_ORIGINATOR_RETURN_ADDRESS";
	case BRUTUS_BRUTUS_PR_PARENT_KEY :
		return "PR_PARENT_KEY";
	case BRUTUS_BRUTUS_PR_PRIORITY :
		return "PR_PRIORITY";
	case BRUTUS_BRUTUS_PR_ORIGIN_CHECK :
		return "PR_ORIGIN_CHECK";
	case BRUTUS_BRUTUS_PR_PROOF_OF_SUBMISSION_REQUESTED :
		return "PR_PROOF_OF_SUBMISSION_REQUESTED";
	case BRUTUS_BRUTUS_PR_READ_RECEIPT_REQUESTED :
		return "PR_READ_RECEIPT_REQUESTED";
	case BRUTUS_BRUTUS_PR_RECEIPT_TIME :
		return "PR_RECEIPT_TIME";
	case BRUTUS_BRUTUS_PR_RECIPIENT_REASSIGNMENT_PROHIBITED :
		return "PR_RECIPIENT_REASSIGNMENT_PROHIBITED";
	case BRUTUS_BRUTUS_PR_REDIRECTION_HISTORY :
		return "PR_REDIRECTION_HISTORY";
	case BRUTUS_BRUTUS_PR_RELATED_IPMS :
		return "PR_RELATED_IPMS";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENSITIVITY :
		return "PR_ORIGINAL_SENSITIVITY";
	case BRUTUS_BRUTUS_PR_LANGUAGES :
		return "PR_LANGUAGES/BRUTUS_PR_LANGUAGES_A";
	case BRUTUS_BRUTUS_PR_LANGUAGES_W :
		return "PR_LANGUAGES_W";
	case BRUTUS_BRUTUS_PR_REPLY_TIME :
		return "PR_REPLY_TIME";
	case BRUTUS_BRUTUS_PR_REPORT_TAG :
		return "PR_REPORT_TAG";
	case BRUTUS_BRUTUS_PR_REPORT_TIME :
		return "PR_REPORT_TIME";
	case BRUTUS_BRUTUS_PR_RETURNED_IPM :
		return "PR_RETURNED_IPM";
	case BRUTUS_BRUTUS_PR_SECURITY :
		return "PR_SECURITY";
	case BRUTUS_BRUTUS_PR_INCOMPLETE_COPY :
		return "PR_INCOMPLETE_COPY";
	case BRUTUS_BRUTUS_PR_SENSITIVITY :
		return "PR_SENSITIVITY";
	case BRUTUS_BRUTUS_PR_SUBJECT :
		return "PR_SUBJECT/BRUTUS_PR_SUBJECT_A";
	case BRUTUS_BRUTUS_PR_SUBJECT_W :
		return "PR_SUBJECT_W";
	case BRUTUS_BRUTUS_PR_SUBJECT_IPM :
		return "PR_SUBJECT_IPM";
	case BRUTUS_BRUTUS_PR_CLIENT_SUBMIT_TIME :
		return "PR_CLIENT_SUBMIT_TIME";
	case BRUTUS_BRUTUS_PR_REPORT_NAME :
		return "PR_REPORT_NAME/BRUTUS_PR_REPORT_NAME_A";
	case BRUTUS_BRUTUS_PR_REPORT_NAME_W :
		return "PR_REPORT_NAME_W";
	case BRUTUS_BRUTUS_PR_SENT_REPRESENTING_SEARCH_KEY :
		return "PR_SENT_REPRESENTING_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_X400_CONTENT_TYPE :
		return "PR_X400_CONTENT_TYPE";
	case BRUTUS_BRUTUS_PR_SUBJECT_PREFIX :
		return "PR_SUBJECT_PREFIX/BRUTUS_PR_SUBJECT_PREFIX_A";
	case BRUTUS_BRUTUS_PR_SUBJECT_PREFIX_W :
		return "PR_SUBJECT_PREFIX_W";
	case BRUTUS_BRUTUS_PR_NON_RECEIPT_REASON :
		return "PR_NON_RECEIPT_REASON";
	case BRUTUS_BRUTUS_PR_RECEIVED_BY_ENTRYID :
		return "PR_RECEIVED_BY_ENTRYID";
	case BRUTUS_BRUTUS_PR_RECEIVED_BY_NAME :
		return "PR_RECEIVED_BY_NAME/BRUTUS_PR_RECEIVED_BY_NAME_A";
	case BRUTUS_BRUTUS_PR_RECEIVED_BY_NAME_W :
		return "PR_RECEIVED_BY_NAME_W";
	case BRUTUS_BRUTUS_PR_SENT_REPRESENTING_ENTRYID :
		return "PR_SENT_REPRESENTING_ENTRYID";
	case BRUTUS_BRUTUS_PR_SENT_REPRESENTING_NAME :
		return "PR_SENT_REPRESENTING_NAME/BRUTUS_PR_SENT_REPRESENTING_NAME_A";
	case BRUTUS_BRUTUS_PR_SENT_REPRESENTING_NAME_W :
		return "PR_SENT_REPRESENTING_NAME_W";
	case BRUTUS_BRUTUS_PR_RCVD_REPRESENTING_ENTRYID :
		return "PR_RCVD_REPRESENTING_ENTRYID";
	case BRUTUS_BRUTUS_PR_RCVD_REPRESENTING_NAME :
		return "PR_RCVD_REPRESENTING_NAME/BRUTUS_PR_RCVD_REPRESENTING_NAME_A";
	case BRUTUS_BRUTUS_PR_RCVD_REPRESENTING_NAME_W :
		return "PR_RCVD_REPRESENTING_NAME_W";
	case BRUTUS_BRUTUS_PR_REPORT_ENTRYID :
		return "PR_REPORT_ENTRYID";
	case BRUTUS_BRUTUS_PR_READ_RECEIPT_ENTRYID :
		return "PR_READ_RECEIPT_ENTRYID";
	case BRUTUS_BRUTUS_PR_MESSAGE_SUBMISSION_ID :
		return "PR_MESSAGE_SUBMISSION_ID";
	case BRUTUS_BRUTUS_PR_PROVIDER_SUBMIT_TIME :
		return "PR_PROVIDER_SUBMIT_TIME";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SUBJECT :
		return "PR_ORIGINAL_SUBJECT/BRUTUS_PR_ORIGINAL_SUBJECT_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SUBJECT_W :
		return "PR_ORIGINAL_SUBJECT_W";
	case BRUTUS_BRUTUS_PR_DISC_VAL :
		return "PR_DISC_VAL";
	case BRUTUS_BRUTUS_PR_ORIG_MESSAGE_CLASS :
		return "PR_ORIG_MESSAGE_CLASS/BRUTUS_PR_ORIG_MESSAGE_CLASS_A";
	case BRUTUS_BRUTUS_PR_ORIG_MESSAGE_CLASS_W :
		return "PR_ORIG_MESSAGE_CLASS_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_AUTHOR_ENTRYID :
		return "PR_ORIGINAL_AUTHOR_ENTRYID";
	case BRUTUS_BRUTUS_PR_ORIGINAL_AUTHOR_NAME :
		return "PR_ORIGINAL_AUTHOR_NAME/BRUTUS_PR_ORIGINAL_AUTHOR_NAME_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_AUTHOR_NAME_W :
		return "PR_ORIGINAL_AUTHOR_NAME_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SUBMIT_TIME :
		return "PR_ORIGINAL_SUBMIT_TIME";
	case BRUTUS_BRUTUS_PR_REPLY_RECIPIENT_ENTRIES :
		return "PR_REPLY_RECIPIENT_ENTRIES";
	case BRUTUS_BRUTUS_PR_REPLY_RECIPIENT_NAMES :
		return "PR_REPLY_RECIPIENT_NAMES/BRUTUS_PR_REPLY_RECIPIENT_NAMES_A";
	case BRUTUS_BRUTUS_PR_REPLY_RECIPIENT_NAMES_W :
		return "PR_REPLY_RECIPIENT_NAMES_W";
	case BRUTUS_BRUTUS_PR_RECEIVED_BY_SEARCH_KEY :
		return "PR_RECEIVED_BY_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_RCVD_REPRESENTING_SEARCH_KEY :
		return "PR_RCVD_REPRESENTING_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_READ_RECEIPT_SEARCH_KEY :
		return "PR_READ_RECEIPT_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_REPORT_SEARCH_KEY :
		return "PR_REPORT_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_ORIGINAL_DELIVERY_TIME :
		return "PR_ORIGINAL_DELIVERY_TIME";
	case BRUTUS_BRUTUS_PR_ORIGINAL_AUTHOR_SEARCH_KEY :
		return "PR_ORIGINAL_AUTHOR_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_MESSAGE_TO_ME :
		return "PR_MESSAGE_TO_ME";
	case BRUTUS_BRUTUS_PR_MESSAGE_CC_ME :
		return "PR_MESSAGE_CC_ME";
	case BRUTUS_BRUTUS_PR_MESSAGE_RECIP_ME :
		return "PR_MESSAGE_RECIP_ME";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENDER_NAME :
		return "PR_ORIGINAL_SENDER_NAME/BRUTUS_PR_ORIGINAL_SENDER_NAME_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENDER_NAME_W :
		return "PR_ORIGINAL_SENDER_NAME_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENDER_ENTRYID :
		return "PR_ORIGINAL_SENDER_ENTRYID";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENDER_SEARCH_KEY :
		return "PR_ORIGINAL_SENDER_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_NAME :
		return "PR_ORIGINAL_SENT_REPRESENTING_NAME/BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_NAME_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_NAME_W :
		return "PR_ORIGINAL_SENT_REPRESENTING_NAME_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_ENTRYID :
		return "PR_ORIGINAL_SENT_REPRESENTING_ENTRYID";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY :
		return "PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_START_DATE :
		return "PR_START_DATE";
	case BRUTUS_BRUTUS_PR_END_DATE :
		return "PR_END_DATE";
	case BRUTUS_BRUTUS_PR_OWNER_APPT_ID :
		return "PR_OWNER_APPT_ID";
	case BRUTUS_BRUTUS_PR_RESPONSE_REQUESTED :
		return "PR_RESPONSE_REQUESTED";
	case BRUTUS_BRUTUS_PR_SENT_REPRESENTING_ADDRTYPE :
		return "PR_SENT_REPRESENTING_ADDRTYPE/BRUTUS_PR_SENT_REPRESENTING_ADDRTYPE_A";
	case BRUTUS_BRUTUS_PR_SENT_REPRESENTING_ADDRTYPE_W :
		return "PR_SENT_REPRESENTING_ADDRTYPE_W";
	case BRUTUS_BRUTUS_PR_SENT_REPRESENTING_EMAIL_ADDRESS :
		return "PR_SENT_REPRESENTING_EMAIL_ADDRESS/BRUTUS_PR_SENT_REPRESENTING_EMAIL_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_SENT_REPRESENTING_EMAIL_ADDRESS_W :
		return "PR_SENT_REPRESENTING_EMAIL_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENDER_ADDRTYPE :
		return "PR_ORIGINAL_SENDER_ADDRTYPE/BRUTUS_PR_ORIGINAL_SENDER_ADDRTYPE_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENDER_ADDRTYPE_W :
		return "PR_ORIGINAL_SENDER_ADDRTYPE_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENDER_EMAIL_ADDRESS :
		return "PR_ORIGINAL_SENDER_EMAIL_ADDRESS/BRUTUS_PR_ORIGINAL_SENDER_EMAIL_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENDER_EMAIL_ADDRESS_W :
		return "PR_ORIGINAL_SENDER_EMAIL_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE :
		return "PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE/BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_W :
		return "PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS :
		return "PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS/BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_W :
		return "PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_CONVERSATION_TOPIC :
		return "PR_CONVERSATION_TOPIC/BRUTUS_PR_CONVERSATION_TOPIC_A";
	case BRUTUS_BRUTUS_PR_CONVERSATION_TOPIC_W :
		return "PR_CONVERSATION_TOPIC_W";
	case BRUTUS_BRUTUS_PR_CONVERSATION_INDEX :
		return "PR_CONVERSATION_INDEX";
	case BRUTUS_BRUTUS_PR_ORIGINAL_DISPLAY_BCC :
		return "PR_ORIGINAL_DISPLAY_BCC/BRUTUS_PR_ORIGINAL_DISPLAY_BCC_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_DISPLAY_BCC_W :
		return "PR_ORIGINAL_DISPLAY_BCC_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_DISPLAY_CC :
		return "PR_ORIGINAL_DISPLAY_CC/BRUTUS_PR_ORIGINAL_DISPLAY_CC_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_DISPLAY_CC_W :
		return "PR_ORIGINAL_DISPLAY_CC_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_DISPLAY_TO :
		return "PR_ORIGINAL_DISPLAY_TO/BRUTUS_PR_ORIGINAL_DISPLAY_TO_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_DISPLAY_TO_W :
		return "PR_ORIGINAL_DISPLAY_TO_W";
	case BRUTUS_BRUTUS_PR_RECEIVED_BY_ADDRTYPE :
		return "PR_RECEIVED_BY_ADDRTYPE/BRUTUS_PR_RECEIVED_BY_ADDRTYPE_A";
	case BRUTUS_BRUTUS_PR_RECEIVED_BY_ADDRTYPE_W :
		return "PR_RECEIVED_BY_ADDRTYPE_W";
	case BRUTUS_BRUTUS_PR_RECEIVED_BY_EMAIL_ADDRESS :
		return "PR_RECEIVED_BY_EMAIL_ADDRESS/BRUTUS_PR_RECEIVED_BY_EMAIL_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_RECEIVED_BY_EMAIL_ADDRESS_W :
		return "PR_RECEIVED_BY_EMAIL_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_RCVD_REPRESENTING_ADDRTYPE :
		return "PR_RCVD_REPRESENTING_ADDRTYPE/BRUTUS_PR_RCVD_REPRESENTING_ADDRTYPE_A";
	case BRUTUS_BRUTUS_PR_RCVD_REPRESENTING_ADDRTYPE_W :
		return "PR_RCVD_REPRESENTING_ADDRTYPE_W";
	case BRUTUS_BRUTUS_PR_RCVD_REPRESENTING_EMAIL_ADDRESS :
		return "PR_RCVD_REPRESENTING_EMAIL_ADDRESS/BRUTUS_PR_RCVD_REPRESENTING_EMAIL_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_RCVD_REPRESENTING_EMAIL_ADDRESS_W :
		return "PR_RCVD_REPRESENTING_EMAIL_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_AUTHOR_ADDRTYPE :
		return "PR_ORIGINAL_AUTHOR_ADDRTYPE/BRUTUS_PR_ORIGINAL_AUTHOR_ADDRTYPE_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_AUTHOR_ADDRTYPE_W :
		return "PR_ORIGINAL_AUTHOR_ADDRTYPE_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS :
		return "PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS/BRUTUS_PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_W :
		return "PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE :
		return "PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE/BRUTUS_PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_A";
	case BRUTUS_BRUTUS_PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_W :
		return "PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_W";
	case BRUTUS_BRUTUS_PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS :
		return "PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS/BRUTUS_PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_W :
		return "PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_TRANSPORT_MESSAGE_HEADERS :
		return "PR_TRANSPORT_MESSAGE_HEADERS/BRUTUS_PR_TRANSPORT_MESSAGE_HEADERS_A";
	case BRUTUS_BRUTUS_PR_TRANSPORT_MESSAGE_HEADERS_W :
		return "PR_TRANSPORT_MESSAGE_HEADERS_W";
	case BRUTUS_BRUTUS_PR_DELEGATION :
		return "PR_DELEGATION";
	case BRUTUS_BRUTUS_PR_TNEF_CORRELATION_KEY :
		return "PR_TNEF_CORRELATION_KEY";
	case BRUTUS_BRUTUS_PR_BODY :
		return "PR_BODY/BRUTUS_PR_BODY_A";
	case BRUTUS_BRUTUS_PR_BODY_W :
		return "PR_BODY_W";
	case BRUTUS_BRUTUS_PR_REPORT_TEXT :
		return "PR_REPORT_TEXT/BRUTUS_PR_REPORT_TEXT_A";
	case BRUTUS_BRUTUS_PR_REPORT_TEXT_W :
		return "PR_REPORT_TEXT_W";
	case BRUTUS_BRUTUS_PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY :
		return "PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY";
	case BRUTUS_BRUTUS_PR_REPORTING_DL_NAME :
		return "PR_REPORTING_DL_NAME";
	case BRUTUS_BRUTUS_PR_REPORTING_MTA_CERTIFICATE :
		return "PR_REPORTING_MTA_CERTIFICATE";
	case BRUTUS_BRUTUS_PR_RTF_SYNC_BODY_CRC :
		return "PR_RTF_SYNC_BODY_CRC";
	case BRUTUS_BRUTUS_PR_RTF_SYNC_BODY_COUNT :
		return "PR_RTF_SYNC_BODY_COUNT";
	case BRUTUS_BRUTUS_PR_RTF_SYNC_BODY_TAG :
		return "PR_RTF_SYNC_BODY_TAG/BRUTUS_PR_RTF_SYNC_BODY_TAG_A";
	case BRUTUS_BRUTUS_PR_RTF_SYNC_BODY_TAG_W :
		return "PR_RTF_SYNC_BODY_TAG_W";
	case BRUTUS_BRUTUS_PR_RTF_COMPRESSED :
		return "PR_RTF_COMPRESSED";
	case BRUTUS_BRUTUS_PR_RTF_SYNC_PREFIX_COUNT :
		return "PR_RTF_SYNC_PREFIX_COUNT";
	case BRUTUS_BRUTUS_PR_RTF_SYNC_TRAILING_COUNT :
		return "PR_RTF_SYNC_TRAILING_COUNT";
	case BRUTUS_BRUTUS_PR_ORIGINALLY_INTENDED_RECIP_ENTRYID :
		return "PR_ORIGINALLY_INTENDED_RECIP_ENTRYID";
	case BRUTUS_BRUTUS_PR_CONTENT_INTEGRITY_CHECK :
		return "PR_CONTENT_INTEGRITY_CHECK";
	case BRUTUS_BRUTUS_PR_EXPLICIT_CONVERSION :
		return "PR_EXPLICIT_CONVERSION";
	case BRUTUS_BRUTUS_PR_IPM_RETURN_REQUESTED :
		return "PR_IPM_RETURN_REQUESTED";
	case BRUTUS_BRUTUS_PR_MESSAGE_TOKEN :
		return "PR_MESSAGE_TOKEN";
	case BRUTUS_BRUTUS_PR_NDR_REASON_CODE :
		return "PR_NDR_REASON_CODE";
	case BRUTUS_BRUTUS_PR_NDR_DIAG_CODE :
		return "PR_NDR_DIAG_CODE";
	case BRUTUS_BRUTUS_PR_NON_RECEIPT_NOTIFICATION_REQUESTED :
		return "PR_NON_RECEIPT_NOTIFICATION_REQUESTED";
	case BRUTUS_BRUTUS_PR_DELIVERY_POINT :
		return "PR_DELIVERY_POINT";
	case BRUTUS_BRUTUS_PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED :
		return "PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED";
	case BRUTUS_BRUTUS_PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT :
		return "PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT";
	case BRUTUS_BRUTUS_PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY :
		return "PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY";
	case BRUTUS_BRUTUS_PR_PHYSICAL_DELIVERY_MODE :
		return "PR_PHYSICAL_DELIVERY_MODE";
	case BRUTUS_BRUTUS_PR_PHYSICAL_DELIVERY_REPORT_REQUEST :
		return "PR_PHYSICAL_DELIVERY_REPORT_REQUEST";
	case BRUTUS_BRUTUS_PR_PHYSICAL_FORWARDING_ADDRESS :
		return "PR_PHYSICAL_FORWARDING_ADDRESS";
	case BRUTUS_BRUTUS_PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED :
		return "PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED";
	case BRUTUS_BRUTUS_PR_PHYSICAL_FORWARDING_PROHIBITED :
		return "PR_PHYSICAL_FORWARDING_PROHIBITED";
	case BRUTUS_BRUTUS_PR_PHYSICAL_RENDITION_ATTRIBUTES :
		return "PR_PHYSICAL_RENDITION_ATTRIBUTES";
	case BRUTUS_BRUTUS_PR_PROOF_OF_DELIVERY :
		return "PR_PROOF_OF_DELIVERY";
	case BRUTUS_BRUTUS_PR_PROOF_OF_DELIVERY_REQUESTED :
		return "PR_PROOF_OF_DELIVERY_REQUESTED";
	case BRUTUS_BRUTUS_PR_RECIPIENT_CERTIFICATE :
		return "PR_RECIPIENT_CERTIFICATE";
	case BRUTUS_BRUTUS_PR_RECIPIENT_NUMBER_FOR_ADVICE :
		return "PR_RECIPIENT_NUMBER_FOR_ADVICE/BRUTUS_PR_RECIPIENT_NUMBER_FOR_ADVICE_A";
	case BRUTUS_BRUTUS_PR_RECIPIENT_NUMBER_FOR_ADVICE_W :
		return "PR_RECIPIENT_NUMBER_FOR_ADVICE_W";
	case BRUTUS_BRUTUS_PR_RECIPIENT_TYPE :
		return "PR_RECIPIENT_TYPE";
	case BRUTUS_BRUTUS_PR_REGISTERED_MAIL_TYPE :
		return "PR_REGISTERED_MAIL_TYPE";
	case BRUTUS_BRUTUS_PR_REPLY_REQUESTED :
		return "PR_REPLY_REQUESTED";
	case BRUTUS_BRUTUS_PR_REQUESTED_DELIVERY_METHOD :
		return "PR_REQUESTED_DELIVERY_METHOD";
	case BRUTUS_BRUTUS_PR_SENDER_ENTRYID :
		return "PR_SENDER_ENTRYID";
	case BRUTUS_BRUTUS_PR_SENDER_NAME :
		return "PR_SENDER_NAME/BRUTUS_PR_SENDER_NAME_A";
	case BRUTUS_BRUTUS_PR_SENDER_NAME_W :
		return "PR_SENDER_NAME_W";
	case BRUTUS_BRUTUS_PR_SUPPLEMENTARY_INFO :
		return "PR_SUPPLEMENTARY_INFO/BRUTUS_PR_SUPPLEMENTARY_INFO_A";
	case BRUTUS_BRUTUS_PR_SUPPLEMENTARY_INFO_W :
		return "PR_SUPPLEMENTARY_INFO_W";
	case BRUTUS_BRUTUS_PR_TYPE_OF_MTS_USER :
		return "PR_TYPE_OF_MTS_USER";
	case BRUTUS_BRUTUS_PR_SENDER_SEARCH_KEY :
		return "PR_SENDER_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_SENDER_ADDRTYPE :
		return "PR_SENDER_ADDRTYPE/BRUTUS_PR_SENDER_ADDRTYPE_A";
	case BRUTUS_BRUTUS_PR_SENDER_ADDRTYPE_W :
		return "PR_SENDER_ADDRTYPE_W";
	case BRUTUS_BRUTUS_PR_SENDER_EMAIL_ADDRESS :
		return "PR_SENDER_EMAIL_ADDRESS/BRUTUS_PR_SENDER_EMAIL_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_SENDER_EMAIL_ADDRESS_W :
		return "PR_SENDER_EMAIL_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_CURRENT_VERSION :
		return "PR_CURRENT_VERSION";
	case BRUTUS_BRUTUS_PR_DELETE_AFTER_SUBMIT :
		return "PR_DELETE_AFTER_SUBMIT";
	case BRUTUS_BRUTUS_PR_DISPLAY_BCC :
		return "PR_DISPLAY_BCC/BRUTUS_PR_DISPLAY_BCC_A";
	case BRUTUS_BRUTUS_PR_DISPLAY_BCC_W :
		return "PR_DISPLAY_BCC_W";
	case BRUTUS_BRUTUS_PR_DISPLAY_CC :
		return "PR_DISPLAY_CC/BRUTUS_PR_DISPLAY_CC_A";
	case BRUTUS_BRUTUS_PR_DISPLAY_CC_W :
		return "PR_DISPLAY_CC_W";
	case BRUTUS_BRUTUS_PR_DISPLAY_TO :
		return "PR_DISPLAY_TO/BRUTUS_PR_DISPLAY_TO_A";
	case BRUTUS_BRUTUS_PR_DISPLAY_TO_W :
		return "PR_DISPLAY_TO_W";
	case BRUTUS_BRUTUS_PR_PARENT_DISPLAY :
		return "PR_PARENT_DISPLAY/BRUTUS_PR_PARENT_DISPLAY_A";
	case BRUTUS_BRUTUS_PR_PARENT_DISPLAY_W :
		return "PR_PARENT_DISPLAY_W";
	case BRUTUS_BRUTUS_PR_MESSAGE_DELIVERY_TIME :
		return "PR_MESSAGE_DELIVERY_TIME";
	case BRUTUS_BRUTUS_PR_MESSAGE_FLAGS :
		return "PR_MESSAGE_FLAGS";
	case BRUTUS_BRUTUS_PR_MESSAGE_SIZE :
		return "PR_MESSAGE_SIZE";
	case BRUTUS_BRUTUS_PR_PARENT_ENTRYID :
		return "PR_PARENT_ENTRYID";
	case BRUTUS_BRUTUS_PR_SENTMAIL_ENTRYID :
		return "PR_SENTMAIL_ENTRYID";
	case BRUTUS_BRUTUS_PR_CORRELATE :
		return "PR_CORRELATE";
	case BRUTUS_BRUTUS_PR_CORRELATE_MTSID :
		return "PR_CORRELATE_MTSID";
	case BRUTUS_BRUTUS_PR_DISCRETE_VALUES :
		return "PR_DISCRETE_VALUES";
	case BRUTUS_BRUTUS_PR_RESPONSIBILITY :
		return "PR_RESPONSIBILITY";
	case BRUTUS_BRUTUS_PR_SPOOLER_STATUS :
		return "PR_SPOOLER_STATUS";
	case BRUTUS_BRUTUS_PR_TRANSPORT_STATUS :
		return "PR_TRANSPORT_STATUS";
	case BRUTUS_BRUTUS_PR_MESSAGE_RECIPIENTS :
		return "PR_MESSAGE_RECIPIENTS";
	case BRUTUS_BRUTUS_PR_MESSAGE_ATTACHMENTS :
		return "PR_MESSAGE_ATTACHMENTS";
	case BRUTUS_BRUTUS_PR_SUBMIT_FLAGS :
		return "PR_SUBMIT_FLAGS";
	case BRUTUS_BRUTUS_PR_RECIPIENT_STATUS :
		return "PR_RECIPIENT_STATUS";
	case BRUTUS_BRUTUS_PR_TRANSPORT_KEY :
		return "PR_TRANSPORT_KEY";
	case BRUTUS_BRUTUS_PR_MSG_STATUS :
		return "PR_MSG_STATUS";
	case BRUTUS_BRUTUS_PR_MESSAGE_DOWNLOAD_TIME :
		return "PR_MESSAGE_DOWNLOAD_TIME";
	case BRUTUS_BRUTUS_PR_CREATION_VERSION :
		return "PR_CREATION_VERSION";
	case BRUTUS_BRUTUS_PR_MODIFY_VERSION :
		return "PR_MODIFY_VERSION";
	case BRUTUS_BRUTUS_PR_HASATTACH :
		return "PR_HASATTACH";
	case BRUTUS_BRUTUS_PR_BODY_CRC :
		return "PR_BODY_CRC";
	case BRUTUS_BRUTUS_PR_NORMALIZED_SUBJECT :
		return "PR_NORMALIZED_SUBJECT/BRUTUS_PR_NORMALIZED_SUBJECT_A";
	case BRUTUS_BRUTUS_PR_NORMALIZED_SUBJECT_W :
		return "PR_NORMALIZED_SUBJECT_W";
	case BRUTUS_BRUTUS_PR_RTF_IN_SYNC :
		return "PR_RTF_IN_SYNC";
	case BRUTUS_BRUTUS_PR_ATTACH_SIZE :
		return "PR_ATTACH_SIZE";
	case BRUTUS_BRUTUS_PR_ATTACH_NUM :
		return "PR_ATTACH_NUM";
	case BRUTUS_BRUTUS_PR_PREPROCESS :
		return "PR_PREPROCESS";
	case BRUTUS_BRUTUS_PR_ORIGINATING_MTA_CERTIFICATE :
		return "PR_ORIGINATING_MTA_CERTIFICATE";
	case BRUTUS_BRUTUS_PR_PROOF_OF_SUBMISSION :
		return "PR_PROOF_OF_SUBMISSION";
	case BRUTUS_BRUTUS_PR_ENTRYID :
		return "PR_ENTRYID";
	case BRUTUS_BRUTUS_PR_OBJECT_TYPE :
		return "PR_OBJECT_TYPE";
	case BRUTUS_BRUTUS_PR_ICON :
		return "PR_ICON";
	case BRUTUS_BRUTUS_PR_MINI_ICON :
		return "PR_MINI_ICON";
	case BRUTUS_BRUTUS_PR_STORE_ENTRYID :
		return "PR_STORE_ENTRYID";
	case BRUTUS_BRUTUS_PR_STORE_RECORD_KEY :
		return "PR_STORE_RECORD_KEY";
	case BRUTUS_BRUTUS_PR_RECORD_KEY :
		return "PR_RECORD_KEY";
	case BRUTUS_BRUTUS_PR_MAPPING_SIGNATURE :
		return "PR_MAPPING_SIGNATURE";
	case BRUTUS_BRUTUS_PR_ACCESS_LEVEL :
		return "PR_ACCESS_LEVEL";
	case BRUTUS_BRUTUS_PR_INSTANCE_KEY :
		return "PR_INSTANCE_KEY";
	case BRUTUS_BRUTUS_PR_ROW_TYPE :
		return "PR_ROW_TYPE";
	case BRUTUS_BRUTUS_PR_ACCESS :
		return "PR_ACCESS";
	case BRUTUS_BRUTUS_PR_ROWID :
		return "PR_ROWID";
	case BRUTUS_BRUTUS_PR_DISPLAY_NAME :
		return "PR_DISPLAY_NAME/BRUTUS_PR_DISPLAY_NAME_A";
	case BRUTUS_BRUTUS_PR_DISPLAY_NAME_W :
		return "PR_DISPLAY_NAME_W";
	case BRUTUS_BRUTUS_PR_ADDRTYPE :
		return "PR_ADDRTYPE/BRUTUS_PR_ADDRTYPE_A";
	case BRUTUS_BRUTUS_PR_ADDRTYPE_W :
		return "PR_ADDRTYPE_W";
	case BRUTUS_BRUTUS_PR_EMAIL_ADDRESS :
		return "PR_EMAIL_ADDRESS/BRUTUS_PR_EMAIL_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_EMAIL_ADDRESS_W :
		return "PR_EMAIL_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_COMMENT :
		return "PR_COMMENT/BRUTUS_PR_COMMENT_A";
	case BRUTUS_BRUTUS_PR_COMMENT_W :
		return "PR_COMMENT_W";
	case BRUTUS_BRUTUS_PR_DEPTH :
		return "PR_DEPTH";
	case BRUTUS_BRUTUS_PR_PROVIDER_DISPLAY :
		return "PR_PROVIDER_DISPLAY/BRUTUS_PR_PROVIDER_DISPLAY_A";
	case BRUTUS_BRUTUS_PR_PROVIDER_DISPLAY_W :
		return "PR_PROVIDER_DISPLAY_W";
	case BRUTUS_BRUTUS_PR_CREATION_TIME :
		return "PR_CREATION_TIME";
	case BRUTUS_BRUTUS_PR_LAST_MODIFICATION_TIME :
		return "PR_LAST_MODIFICATION_TIME";
	case BRUTUS_BRUTUS_PR_RESOURCE_FLAGS :
		return "PR_RESOURCE_FLAGS";
	case BRUTUS_BRUTUS_PR_PROVIDER_DLL_NAME :
		return "PR_PROVIDER_DLL_NAME/BRUTUS_PR_PROVIDER_DLL_NAME_A";
	case BRUTUS_BRUTUS_PR_PROVIDER_DLL_NAME_W :
		return "PR_PROVIDER_DLL_NAME_W";
	case BRUTUS_BRUTUS_PR_SEARCH_KEY :
		return "PR_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_PROVIDER_UID :
		return "PR_PROVIDER_UID";
	case BRUTUS_BRUTUS_PR_PROVIDER_ORDINAL :
		return "PR_PROVIDER_ORDINAL";
	case BRUTUS_BRUTUS_PR_FORM_VERSION :
		return "PR_FORM_VERSION/BRUTUS_PR_FORM_VERSION_A";
	case BRUTUS_BRUTUS_PR_FORM_VERSION_W :
		return "PR_FORM_VERSION_W";
	case BRUTUS_BRUTUS_PR_FORM_CLSID :
		return "PR_FORM_CLSID";
	case BRUTUS_BRUTUS_PR_FORM_CONTACT_NAME :
		return "PR_FORM_CONTACT_NAME/BRUTUS_PR_FORM_CONTACT_NAME_A";
	case BRUTUS_BRUTUS_PR_FORM_CONTACT_NAME_W :
		return "PR_FORM_CONTACT_NAME_W";
	case BRUTUS_BRUTUS_PR_FORM_CATEGORY :
		return "PR_FORM_CATEGORY/BRUTUS_PR_FORM_CATEGORY_A";
	case BRUTUS_BRUTUS_PR_FORM_CATEGORY_W :
		return "PR_FORM_CATEGORY_W";
	case BRUTUS_BRUTUS_PR_FORM_CATEGORY_SUB :
		return "PR_FORM_CATEGORY_SUB/BRUTUS_PR_FORM_CATEGORY_SUB_A";
	case BRUTUS_BRUTUS_PR_FORM_CATEGORY_SUB_W :
		return "PR_FORM_CATEGORY_SUB_W";
	case BRUTUS_BRUTUS_PR_FORM_HOST_MAP :
		return "PR_FORM_HOST_MAP";
	case BRUTUS_BRUTUS_PR_FORM_HIDDEN :
		return "PR_FORM_HIDDEN";
	case BRUTUS_BRUTUS_PR_FORM_DESIGNER_NAME :
		return "PR_FORM_DESIGNER_NAME/BRUTUS_PR_FORM_DESIGNER_NAME_A";
	case BRUTUS_BRUTUS_PR_FORM_DESIGNER_NAME_W :
		return "PR_FORM_DESIGNER_NAME_W";
	case BRUTUS_BRUTUS_PR_FORM_DESIGNER_GUID :
		return "PR_FORM_DESIGNER_GUID";
	case BRUTUS_BRUTUS_PR_FORM_MESSAGE_BEHAVIOR :
		return "PR_FORM_MESSAGE_BEHAVIOR";
	case BRUTUS_BRUTUS_PR_DEFAULT_STORE :
		return "PR_DEFAULT_STORE";
	case BRUTUS_BRUTUS_PR_STORE_SUPPORT_MASK :
		return "PR_STORE_SUPPORT_MASK";
	case BRUTUS_BRUTUS_PR_STORE_STATE :
		return "PR_STORE_STATE";
	case BRUTUS_BRUTUS_PR_IPM_SUBTREE_SEARCH_KEY :
		return "PR_IPM_SUBTREE_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_IPM_OUTBOX_SEARCH_KEY :
		return "PR_IPM_OUTBOX_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_IPM_WASTEBASKET_SEARCH_KEY :
		return "PR_IPM_WASTEBASKET_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_IPM_SENTMAIL_SEARCH_KEY :
		return "PR_IPM_SENTMAIL_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_MDB_PROVIDER :
		return "PR_MDB_PROVIDER";
	case BRUTUS_BRUTUS_PR_RECEIVE_FOLDER_SETTINGS :
		return "PR_RECEIVE_FOLDER_SETTINGS";
	case BRUTUS_BRUTUS_PR_VALID_FOLDER_MASK :
		return "PR_VALID_FOLDER_MASK";
	case BRUTUS_BRUTUS_PR_IPM_SUBTREE_ENTRYID :
		return "PR_IPM_SUBTREE_ENTRYID";
	case BRUTUS_BRUTUS_PR_IPM_OUTBOX_ENTRYID :
		return "PR_IPM_OUTBOX_ENTRYID";
	case BRUTUS_BRUTUS_PR_IPM_WASTEBASKET_ENTRYID :
		return "PR_IPM_WASTEBASKET_ENTRYID";
	case BRUTUS_BRUTUS_PR_IPM_SENTMAIL_ENTRYID :
		return "PR_IPM_SENTMAIL_ENTRYID";
	case BRUTUS_BRUTUS_PR_VIEWS_ENTRYID :
		return "PR_VIEWS_ENTRYID";
	case BRUTUS_BRUTUS_PR_COMMON_VIEWS_ENTRYID :
		return "PR_COMMON_VIEWS_ENTRYID";
	case BRUTUS_BRUTUS_PR_FINDER_ENTRYID :
		return "PR_FINDER_ENTRYID";
	case BRUTUS_BRUTUS_PR_CONTAINER_FLAGS :
		return "PR_CONTAINER_FLAGS";
	case BRUTUS_BRUTUS_PR_FOLDER_TYPE :
		return "PR_FOLDER_TYPE";
	case BRUTUS_BRUTUS_PR_CONTENT_COUNT :
		return "PR_CONTENT_COUNT";
	case BRUTUS_BRUTUS_PR_CONTENT_UNREAD :
		return "PR_CONTENT_UNREAD";
	case BRUTUS_BRUTUS_PR_CREATE_TEMPLATES :
		return "PR_CREATE_TEMPLATES";
	case BRUTUS_BRUTUS_PR_DETAILS_TABLE :
		return "PR_DETAILS_TABLE";
	case BRUTUS_BRUTUS_PR_SEARCH :
		return "PR_SEARCH";
	case BRUTUS_BRUTUS_PR_SELECTABLE :
		return "PR_SELECTABLE";
	case BRUTUS_BRUTUS_PR_SUBFOLDERS :
		return "PR_SUBFOLDERS";
	case BRUTUS_BRUTUS_PR_STATUS :
		return "PR_STATUS";
	case BRUTUS_BRUTUS_PR_ANR :
		return "PR_ANR/BRUTUS_PR_ANR_A";
	case BRUTUS_BRUTUS_PR_ANR_W :
		return "PR_ANR_W";
	case BRUTUS_BRUTUS_PR_CONTENTS_SORT_ORDER :
		return "PR_CONTENTS_SORT_ORDER";
	case BRUTUS_BRUTUS_PR_CONTAINER_HIERARCHY :
		return "PR_CONTAINER_HIERARCHY";
	case BRUTUS_BRUTUS_PR_CONTAINER_CONTENTS :
		return "PR_CONTAINER_CONTENTS";
	case BRUTUS_BRUTUS_PR_FOLDER_ASSOCIATED_CONTENTS :
		return "PR_FOLDER_ASSOCIATED_CONTENTS";
	case BRUTUS_BRUTUS_PR_DEF_CREATE_DL :
		return "PR_DEF_CREATE_DL";
	case BRUTUS_BRUTUS_PR_DEF_CREATE_MAILUSER :
		return "PR_DEF_CREATE_MAILUSER";
	case BRUTUS_BRUTUS_PR_CONTAINER_CLASS :
		return "PR_CONTAINER_CLASS/BRUTUS_PR_CONTAINER_CLASS_A";
	case BRUTUS_BRUTUS_PR_CONTAINER_CLASS_W :
		return "PR_CONTAINER_CLASS_W";
	case BRUTUS_BRUTUS_PR_CONTAINER_MODIFY_VERSION :
		return "PR_CONTAINER_MODIFY_VERSION";
	case BRUTUS_BRUTUS_PR_AB_PROVIDER_ID :
		return "PR_AB_PROVIDER_ID";
	case BRUTUS_BRUTUS_PR_DEFAULT_VIEW_ENTRYID :
		return "PR_DEFAULT_VIEW_ENTRYID";
	case BRUTUS_BRUTUS_PR_ASSOC_CONTENT_COUNT :
		return "PR_ASSOC_CONTENT_COUNT";
	case BRUTUS_BRUTUS_PR_OUTLOOK_CALENDAR :
		return "PR_OUTLOOK_CALENDAR";
	case BRUTUS_BRUTUS_PR_OUTLOOK_CONTACTS :
		return "PR_OUTLOOK_CONTACTS";
	case BRUTUS_BRUTUS_PR_OUTLOOK_JOURNAL :
		return "PR_OUTLOOK_JOURNAL";
	case BRUTUS_BRUTUS_PR_OUTLOOK_NOTES :
		return "PR_OUTLOOK_NOTES";
	case BRUTUS_BRUTUS_PR_OUTLOOK_TASKS :
		return "PR_OUTLOOK_TASKS";
	case BRUTUS_BRUTUS_PR_OUTLOOK_ONLINE_REMINDERS :
		return "PR_OUTLOOK_ONLINE_REMINDERS";
	case BRUTUS_BRUTUS_PR_OUTLOOK_OFFLINE_REMINDERS :
		return "PR_OUTLOOK_OFFLINE_REMINDERS";
	case BRUTUS_BRUTUS_PR_OUTLOOK_DRAFTS :
		return "PR_OUTLOOK_DRAFTS";
	case BRUTUS_BRUTUS_PR_ATTACHMENT_X400_PARAMETERS :
		return "PR_ATTACHMENT_X400_PARAMETERS";
	case BRUTUS_BRUTUS_PR_ATTACH_DATA_OBJ :
		return "PR_ATTACH_DATA_OBJ";
	case BRUTUS_BRUTUS_PR_ATTACH_DATA_BIN :
		return "PR_ATTACH_DATA_BIN";
	case BRUTUS_BRUTUS_PR_ATTACH_ENCODING :
		return "PR_ATTACH_ENCODING";
	case BRUTUS_BRUTUS_PR_ATTACH_EXTENSION :
		return "PR_ATTACH_EXTENSION/BRUTUS_PR_ATTACH_EXTENSION_A";
	case BRUTUS_BRUTUS_PR_ATTACH_EXTENSION_W :
		return "PR_ATTACH_EXTENSION_W";
	case BRUTUS_BRUTUS_PR_ATTACH_FILENAME :
		return "PR_ATTACH_FILENAME/BRUTUS_PR_ATTACH_FILENAME_A";
	case BRUTUS_BRUTUS_PR_ATTACH_FILENAME_W :
		return "PR_ATTACH_FILENAME_W";
	case BRUTUS_BRUTUS_PR_ATTACH_METHOD :
		return "PR_ATTACH_METHOD";
	case BRUTUS_BRUTUS_PR_ATTACH_LONG_FILENAME :
		return "PR_ATTACH_LONG_FILENAME/BRUTUS_PR_ATTACH_LONG_FILENAME_A";
	case BRUTUS_BRUTUS_PR_ATTACH_LONG_FILENAME_W :
		return "PR_ATTACH_LONG_FILENAME_W";
	case BRUTUS_BRUTUS_PR_ATTACH_PATHNAME :
		return "PR_ATTACH_PATHNAME/BRUTUS_PR_ATTACH_PATHNAME_A";
	case BRUTUS_BRUTUS_PR_ATTACH_PATHNAME_W :
		return "PR_ATTACH_PATHNAME_W";
	case BRUTUS_BRUTUS_PR_ATTACH_RENDERING :
		return "PR_ATTACH_RENDERING";
	case BRUTUS_BRUTUS_PR_ATTACH_TAG :
		return "PR_ATTACH_TAG";
	case BRUTUS_BRUTUS_PR_RENDERING_POSITION :
		return "PR_RENDERING_POSITION";
	case BRUTUS_BRUTUS_PR_ATTACH_TRANSPORT_NAME :
		return "PR_ATTACH_TRANSPORT_NAME/BRUTUS_PR_ATTACH_TRANSPORT_NAME_A";
	case BRUTUS_BRUTUS_PR_ATTACH_TRANSPORT_NAME_W :
		return "PR_ATTACH_TRANSPORT_NAME_W";
	case BRUTUS_BRUTUS_PR_ATTACH_LONG_PATHNAME :
		return "PR_ATTACH_LONG_PATHNAME/BRUTUS_PR_ATTACH_LONG_PATHNAME_A";
	case BRUTUS_BRUTUS_PR_ATTACH_LONG_PATHNAME_W :
		return "PR_ATTACH_LONG_PATHNAME_W";
	case BRUTUS_BRUTUS_PR_ATTACH_MIME_TAG :
		return "PR_ATTACH_MIME_TAG/BRUTUS_PR_ATTACH_MIME_TAG_A";
	case BRUTUS_BRUTUS_PR_ATTACH_MIME_TAG_W :
		return "PR_ATTACH_MIME_TAG_W";
	case BRUTUS_BRUTUS_PR_ATTACH_ADDITIONAL_INFO :
		return "PR_ATTACH_ADDITIONAL_INFO";
	case BRUTUS_BRUTUS_PR_DISPLAY_TYPE :
		return "PR_DISPLAY_TYPE";
	case BRUTUS_BRUTUS_PR_TEMPLATEID :
		return "PR_TEMPLATEID";
	case BRUTUS_BRUTUS_PR_PRIMARY_CAPABILITY :
		return "PR_PRIMARY_CAPABILITY";
	case BRUTUS_BRUTUS_PR_7BIT_DISPLAY_NAME :
		return "PR_7BIT_DISPLAY_NAME";
	case BRUTUS_BRUTUS_PR_ACCOUNT :
		return "PR_ACCOUNT/BRUTUS_PR_ACCOUNT_A";
	case BRUTUS_BRUTUS_PR_ACCOUNT_W :
		return "PR_ACCOUNT_W";
	case BRUTUS_BRUTUS_PR_ALTERNATE_RECIPIENT :
		return "PR_ALTERNATE_RECIPIENT";
	case BRUTUS_BRUTUS_PR_CALLBACK_TELEPHONE_NUMBER :
		return "PR_CALLBACK_TELEPHONE_NUMBER/BRUTUS_PR_CALLBACK_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_CALLBACK_TELEPHONE_NUMBER_W :
		return "PR_CALLBACK_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_CONVERSION_PROHIBITED :
		return "PR_CONVERSION_PROHIBITED";
	case BRUTUS_BRUTUS_PR_DISCLOSE_RECIPIENTS :
		return "PR_DISCLOSE_RECIPIENTS";
	case BRUTUS_BRUTUS_PR_GENERATION :
		return "PR_GENERATION/BRUTUS_PR_GENERATION_A";
	case BRUTUS_BRUTUS_PR_GENERATION_W :
		return "PR_GENERATION_W";
	case BRUTUS_BRUTUS_PR_GIVEN_NAME :
		return "PR_GIVEN_NAME/BRUTUS_PR_GIVEN_NAME_A";
	case BRUTUS_BRUTUS_PR_GIVEN_NAME_W :
		return "PR_GIVEN_NAME_W";
	case BRUTUS_BRUTUS_PR_GOVERNMENT_ID_NUMBER :
		return "PR_GOVERNMENT_ID_NUMBER/BRUTUS_PR_GOVERNMENT_ID_NUMBER_A";
	case BRUTUS_BRUTUS_PR_GOVERNMENT_ID_NUMBER_W :
		return "PR_GOVERNMENT_ID_NUMBER_W";
	case BRUTUS_BRUTUS_PR_BUSINESS_TELEPHONE_NUMBER :
		return "PR_BUSINESS_TELEPHONE_NUMBER/BRUTUS_PR_BUSINESS_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_BUSINESS_TELEPHONE_NUMBER_W :
		return "PR_BUSINESS_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_HOME_TELEPHONE_NUMBER :
		return "PR_HOME_TELEPHONE_NUMBER/BRUTUS_PR_HOME_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_HOME_TELEPHONE_NUMBER_W :
		return "PR_HOME_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_INITIALS :
		return "PR_INITIALS/BRUTUS_PR_INITIALS_A";
	case BRUTUS_BRUTUS_PR_INITIALS_W :
		return "PR_INITIALS_W";
	case BRUTUS_BRUTUS_PR_KEYWORD :
		return "PR_KEYWORD/BRUTUS_PR_KEYWORD_A";
	case BRUTUS_BRUTUS_PR_KEYWORD_W :
		return "PR_KEYWORD_W";
	case BRUTUS_BRUTUS_PR_LANGUAGE :
		return "PR_LANGUAGE/BRUTUS_PR_LANGUAGE_A";
	case BRUTUS_BRUTUS_PR_LANGUAGE_W :
		return "PR_LANGUAGE_W";
	case BRUTUS_BRUTUS_PR_LOCATION :
		return "PR_LOCATION/BRUTUS_PR_LOCATION_A";
	case BRUTUS_BRUTUS_PR_LOCATION_W :
		return "PR_LOCATION_W";
	case BRUTUS_BRUTUS_PR_MAIL_PERMISSION :
		return "PR_MAIL_PERMISSION";
	case BRUTUS_BRUTUS_PR_MHS_COMMON_NAME :
		return "PR_MHS_COMMON_NAME/BRUTUS_PR_MHS_COMMON_NAME_A";
	case BRUTUS_BRUTUS_PR_MHS_COMMON_NAME_W :
		return "PR_MHS_COMMON_NAME_W";
	case BRUTUS_BRUTUS_PR_ORGANIZATIONAL_ID_NUMBER :
		return "PR_ORGANIZATIONAL_ID_NUMBER/BRUTUS_PR_ORGANIZATIONAL_ID_NUMBER_A";
	case BRUTUS_BRUTUS_PR_ORGANIZATIONAL_ID_NUMBER_W :
		return "PR_ORGANIZATIONAL_ID_NUMBER_W";
	case BRUTUS_BRUTUS_PR_SURNAME :
		return "PR_SURNAME/BRUTUS_PR_SURNAME_A";
	case BRUTUS_BRUTUS_PR_SURNAME_W :
		return "PR_SURNAME_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_ENTRYID :
		return "PR_ORIGINAL_ENTRYID";
	case BRUTUS_BRUTUS_PR_ORIGINAL_DISPLAY_NAME :
		return "PR_ORIGINAL_DISPLAY_NAME/BRUTUS_PR_ORIGINAL_DISPLAY_NAME_A";
	case BRUTUS_BRUTUS_PR_ORIGINAL_DISPLAY_NAME_W :
		return "PR_ORIGINAL_DISPLAY_NAME_W";
	case BRUTUS_BRUTUS_PR_ORIGINAL_SEARCH_KEY :
		return "PR_ORIGINAL_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_POSTAL_ADDRESS :
		return "PR_POSTAL_ADDRESS/BRUTUS_PR_POSTAL_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_POSTAL_ADDRESS_W :
		return "PR_POSTAL_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_COMPANY_NAME :
		return "PR_COMPANY_NAME/BRUTUS_PR_COMPANY_NAME_A";
	case BRUTUS_BRUTUS_PR_COMPANY_NAME_W :
		return "PR_COMPANY_NAME_W";
	case BRUTUS_BRUTUS_PR_TITLE :
		return "PR_TITLE/BRUTUS_PR_TITLE_A";
	case BRUTUS_BRUTUS_PR_TITLE_W :
		return "PR_TITLE_W";
	case BRUTUS_BRUTUS_PR_DEPARTMENT_NAME :
		return "PR_DEPARTMENT_NAME/BRUTUS_PR_DEPARTMENT_NAME_A";
	case BRUTUS_BRUTUS_PR_DEPARTMENT_NAME_W :
		return "PR_DEPARTMENT_NAME_W";
	case BRUTUS_BRUTUS_PR_OFFICE_LOCATION :
		return "PR_OFFICE_LOCATION/BRUTUS_PR_OFFICE_LOCATION_A";
	case BRUTUS_BRUTUS_PR_OFFICE_LOCATION_W :
		return "PR_OFFICE_LOCATION_W";
	case BRUTUS_BRUTUS_PR_PRIMARY_TELEPHONE_NUMBER :
		return "PR_PRIMARY_TELEPHONE_NUMBER/BRUTUS_PR_PRIMARY_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_PRIMARY_TELEPHONE_NUMBER_W :
		return "PR_PRIMARY_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_BUSINESS2_TELEPHONE_NUMBER :
		return "PR_BUSINESS2_TELEPHONE_NUMBER/BRUTUS_PR_BUSINESS2_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_BUSINESS2_TELEPHONE_NUMBER_W :
		return "PR_BUSINESS2_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_MOBILE_TELEPHONE_NUMBER :
		return "PR_MOBILE_TELEPHONE_NUMBER/BRUTUS_PR_MOBILE_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_MOBILE_TELEPHONE_NUMBER_W :
		return "PR_MOBILE_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_RADIO_TELEPHONE_NUMBER :
		return "PR_RADIO_TELEPHONE_NUMBER/BRUTUS_PR_RADIO_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_RADIO_TELEPHONE_NUMBER_W :
		return "PR_RADIO_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_CAR_TELEPHONE_NUMBER :
		return "PR_CAR_TELEPHONE_NUMBER/BRUTUS_PR_CAR_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_CAR_TELEPHONE_NUMBER_W :
		return "PR_CAR_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_OTHER_TELEPHONE_NUMBER :
		return "PR_OTHER_TELEPHONE_NUMBER/BRUTUS_PR_OTHER_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_OTHER_TELEPHONE_NUMBER_W :
		return "PR_OTHER_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_TRANSMITABLE_DISPLAY_NAME :
		return "PR_TRANSMITABLE_DISPLAY_NAME/BRUTUS_PR_TRANSMITABLE_DISPLAY_NAME_A";
	case BRUTUS_BRUTUS_PR_TRANSMITABLE_DISPLAY_NAME_W :
		return "PR_TRANSMITABLE_DISPLAY_NAME_W";
	case BRUTUS_BRUTUS_PR_PAGER_TELEPHONE_NUMBER :
		return "PR_PAGER_TELEPHONE_NUMBER/BRUTUS_PR_PAGER_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_PAGER_TELEPHONE_NUMBER_W :
		return "PR_PAGER_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_USER_CERTIFICATE :
		return "PR_USER_CERTIFICATE";
	case BRUTUS_BRUTUS_PR_PRIMARY_FAX_NUMBER :
		return "PR_PRIMARY_FAX_NUMBER/BRUTUS_PR_PRIMARY_FAX_NUMBER_A";
	case BRUTUS_BRUTUS_PR_PRIMARY_FAX_NUMBER_W :
		return "PR_PRIMARY_FAX_NUMBER_W";
	case BRUTUS_BRUTUS_PR_BUSINESS_FAX_NUMBER :
		return "PR_BUSINESS_FAX_NUMBER/BRUTUS_PR_BUSINESS_FAX_NUMBER_A";
	case BRUTUS_BRUTUS_PR_BUSINESS_FAX_NUMBER_W :
		return "PR_BUSINESS_FAX_NUMBER_W";
	case BRUTUS_BRUTUS_PR_HOME_FAX_NUMBER :
		return "PR_HOME_FAX_NUMBER/BRUTUS_PR_HOME_FAX_NUMBER_A";
	case BRUTUS_BRUTUS_PR_HOME_FAX_NUMBER_W :
		return "PR_HOME_FAX_NUMBER_W";
	case BRUTUS_BRUTUS_PR_COUNTRY :
		return "PR_COUNTRY/BRUTUS_PR_COUNTRY_A";
	case BRUTUS_BRUTUS_PR_COUNTRY_W :
		return "PR_COUNTRY_W";
	case BRUTUS_BRUTUS_PR_LOCALITY :
		return "PR_LOCALITY/BRUTUS_PR_LOCALITY_A";
	case BRUTUS_BRUTUS_PR_LOCALITY_W :
		return "PR_LOCALITY_W";
	case BRUTUS_BRUTUS_PR_STATE_OR_PROVINCE :
		return "PR_STATE_OR_PROVINCE/BRUTUS_PR_STATE_OR_PROVINCE_A";
	case BRUTUS_BRUTUS_PR_STATE_OR_PROVINCE_W :
		return "PR_STATE_OR_PROVINCE_W";
	case BRUTUS_BRUTUS_PR_STREET_ADDRESS :
		return "PR_STREET_ADDRESS/BRUTUS_PR_STREET_ADDRESS_A";
	case BRUTUS_BRUTUS_PR_STREET_ADDRESS_W :
		return "PR_STREET_ADDRESS_W";
	case BRUTUS_BRUTUS_PR_POSTAL_CODE :
		return "PR_POSTAL_CODE/BRUTUS_PR_POSTAL_CODE_A";
	case BRUTUS_BRUTUS_PR_POSTAL_CODE_W :
		return "PR_POSTAL_CODE_W";
	case BRUTUS_BRUTUS_PR_POST_OFFICE_BOX :
		return "PR_POST_OFFICE_BOX/BRUTUS_PR_POST_OFFICE_BOX_A";
	case BRUTUS_BRUTUS_PR_POST_OFFICE_BOX_W :
		return "PR_POST_OFFICE_BOX_W";
	case BRUTUS_BRUTUS_PR_TELEX_NUMBER :
		return "PR_TELEX_NUMBER/BRUTUS_PR_TELEX_NUMBER_A";
	case BRUTUS_BRUTUS_PR_TELEX_NUMBER_W :
		return "PR_TELEX_NUMBER_W";
	case BRUTUS_BRUTUS_PR_ISDN_NUMBER :
		return "PR_ISDN_NUMBER/BRUTUS_PR_ISDN_NUMBER_A";
	case BRUTUS_BRUTUS_PR_ISDN_NUMBER_W :
		return "PR_ISDN_NUMBER_W";
	case BRUTUS_BRUTUS_PR_ASSISTANT_TELEPHONE_NUMBER :
		return "PR_ASSISTANT_TELEPHONE_NUMBER/BRUTUS_PR_ASSISTANT_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_ASSISTANT_TELEPHONE_NUMBER_W :
		return "PR_ASSISTANT_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_HOME2_TELEPHONE_NUMBER :
		return "PR_HOME2_TELEPHONE_NUMBER/BRUTUS_PR_HOME2_TELEPHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_HOME2_TELEPHONE_NUMBER_W :
		return "PR_HOME2_TELEPHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_ASSISTANT :
		return "PR_ASSISTANT/BRUTUS_PR_ASSISTANT_A";
	case BRUTUS_BRUTUS_PR_ASSISTANT_W :
		return "PR_ASSISTANT_W";
	case BRUTUS_BRUTUS_PR_SEND_RICH_INFO :
		return "PR_SEND_RICH_INFO";
	case BRUTUS_BRUTUS_PR_WEDDING_ANNIVERSARY :
		return "PR_WEDDING_ANNIVERSARY";
	case BRUTUS_BRUTUS_PR_BIRTHDAY :
		return "PR_BIRTHDAY";
	case BRUTUS_BRUTUS_PR_HOBBIES :
		return "PR_HOBBIES/BRUTUS_PR_HOBBIES_A";
	case BRUTUS_BRUTUS_PR_HOBBIES_W :
		return "PR_HOBBIES_W";
	case BRUTUS_BRUTUS_PR_MIDDLE_NAME :
		return "PR_MIDDLE_NAME/BRUTUS_PR_MIDDLE_NAME_A";
	case BRUTUS_BRUTUS_PR_MIDDLE_NAME_W :
		return "PR_MIDDLE_NAME_W";
	case BRUTUS_BRUTUS_PR_DISPLAY_NAME_PREFIX :
		return "PR_DISPLAY_NAME_PREFIX/BRUTUS_PR_DISPLAY_NAME_PREFIX_A";
	case BRUTUS_BRUTUS_PR_DISPLAY_NAME_PREFIX_W :
		return "PR_DISPLAY_NAME_PREFIX_W";
	case BRUTUS_BRUTUS_PR_PROFESSION :
		return "PR_PROFESSION/BRUTUS_PR_PROFESSION_A";
	case BRUTUS_BRUTUS_PR_PROFESSION_W :
		return "PR_PROFESSION_W";
	case BRUTUS_BRUTUS_PR_PREFERRED_BY_NAME :
		return "PR_PREFERRED_BY_NAME/BRUTUS_PR_PREFERRED_BY_NAME_A";
	case BRUTUS_BRUTUS_PR_PREFERRED_BY_NAME_W :
		return "PR_PREFERRED_BY_NAME_W";
	case BRUTUS_BRUTUS_PR_SPOUSE_NAME :
		return "PR_SPOUSE_NAME/BRUTUS_PR_SPOUSE_NAME_A";
	case BRUTUS_BRUTUS_PR_SPOUSE_NAME_W :
		return "PR_SPOUSE_NAME_W";
	case BRUTUS_BRUTUS_PR_COMPUTER_NETWORK_NAME :
		return "PR_COMPUTER_NETWORK_NAME/BRUTUS_PR_COMPUTER_NETWORK_NAME_A";
	case BRUTUS_BRUTUS_PR_COMPUTER_NETWORK_NAME_W :
		return "PR_COMPUTER_NETWORK_NAME_W";
	case BRUTUS_BRUTUS_PR_CUSTOMER_ID :
		return "PR_CUSTOMER_ID/BRUTUS_PR_CUSTOMER_ID_A";
	case BRUTUS_BRUTUS_PR_CUSTOMER_ID_W :
		return "PR_CUSTOMER_ID_W";
	case BRUTUS_BRUTUS_PR_TTYTDD_PHONE_NUMBER :
		return "PR_TTYTDD_PHONE_NUMBER/BRUTUS_PR_TTYTDD_PHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_TTYTDD_PHONE_NUMBER_W :
		return "PR_TTYTDD_PHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_FTP_SITE :
		return "PR_FTP_SITE/BRUTUS_PR_FTP_SITE_A";
	case BRUTUS_BRUTUS_PR_FTP_SITE_W :
		return "PR_FTP_SITE_W";
	case BRUTUS_BRUTUS_PR_GENDER :
		return "PR_GENDER";
	case BRUTUS_BRUTUS_PR_MANAGER_NAME :
		return "PR_MANAGER_NAME/BRUTUS_PR_MANAGER_NAME_A";
	case BRUTUS_BRUTUS_PR_MANAGER_NAME_W :
		return "PR_MANAGER_NAME_W";
	case BRUTUS_BRUTUS_PR_NICKNAME :
		return "PR_NICKNAME/BRUTUS_PR_NICKNAME_A";
	case BRUTUS_BRUTUS_PR_NICKNAME_W :
		return "PR_NICKNAME_W";
	case BRUTUS_BRUTUS_PR_PERSONAL_HOME_PAGE :
		return "PR_PERSONAL_HOME_PAGE/BRUTUS_PR_PERSONAL_HOME_PAGE_A";
	case BRUTUS_BRUTUS_PR_PERSONAL_HOME_PAGE_W :
		return "PR_PERSONAL_HOME_PAGE_W";
	case BRUTUS_BRUTUS_PR_BUSINESS_HOME_PAGE :
		return "PR_BUSINESS_HOME_PAGE/BRUTUS_PR_BUSINESS_HOME_PAGE_A";
	case BRUTUS_BRUTUS_PR_BUSINESS_HOME_PAGE_W :
		return "PR_BUSINESS_HOME_PAGE_W";
	case BRUTUS_BRUTUS_PR_CONTACT_VERSION :
		return "PR_CONTACT_VERSION";
	case BRUTUS_BRUTUS_PR_CONTACT_ENTRYIDS :
		return "PR_CONTACT_ENTRYIDS";
	case BRUTUS_BRUTUS_PR_CONTACT_ADDRTYPES :
		return "PR_CONTACT_ADDRTYPES/BRUTUS_PR_CONTACT_ADDRTYPES_A";
	case BRUTUS_BRUTUS_PR_CONTACT_ADDRTYPES_W :
		return "PR_CONTACT_ADDRTYPES_W";
	case BRUTUS_BRUTUS_PR_CONTACT_DEFAULT_ADDRESS_INDEX :
		return "PR_CONTACT_DEFAULT_ADDRESS_INDEX";
	case BRUTUS_BRUTUS_PR_CONTACT_EMAIL_ADDRESSES :
		return "PR_CONTACT_EMAIL_ADDRESSES/BRUTUS_PR_CONTACT_EMAIL_ADDRESSES_A";
	case BRUTUS_BRUTUS_PR_CONTACT_EMAIL_ADDRESSES_W :
		return "PR_CONTACT_EMAIL_ADDRESSES_W";
	case BRUTUS_BRUTUS_PR_COMPANY_MAIN_PHONE_NUMBER :
		return "PR_COMPANY_MAIN_PHONE_NUMBER/BRUTUS_PR_COMPANY_MAIN_PHONE_NUMBER_A";
	case BRUTUS_BRUTUS_PR_COMPANY_MAIN_PHONE_NUMBER_W :
		return "PR_COMPANY_MAIN_PHONE_NUMBER_W";
	case BRUTUS_BRUTUS_PR_CHILDRENS_NAMES :
		return "PR_CHILDRENS_NAMES/BRUTUS_PR_CHILDRENS_NAMES_A";
	case BRUTUS_BRUTUS_PR_CHILDRENS_NAMES_W :
		return "PR_CHILDRENS_NAMES_W";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_CITY :
		return "PR_HOME_ADDRESS_CITY/BRUTUS_PR_HOME_ADDRESS_CITY_A";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_CITY_W :
		return "PR_HOME_ADDRESS_CITY_W";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_COUNTRY :
		return "PR_HOME_ADDRESS_COUNTRY/BRUTUS_PR_HOME_ADDRESS_COUNTRY_A";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_COUNTRY_W :
		return "PR_HOME_ADDRESS_COUNTRY_W";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_POSTAL_CODE :
		return "PR_HOME_ADDRESS_POSTAL_CODE/BRUTUS_PR_HOME_ADDRESS_POSTAL_CODE_A";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_POSTAL_CODE_W :
		return "PR_HOME_ADDRESS_POSTAL_CODE_W";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_STATE_OR_PROVINCE :
		return "PR_HOME_ADDRESS_STATE_OR_PROVINCE/BRUTUS_PR_HOME_ADDRESS_STATE_OR_PROVINCE_A";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_STATE_OR_PROVINCE_W :
		return "PR_HOME_ADDRESS_STATE_OR_PROVINCE_W";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_STREET :
		return "PR_HOME_ADDRESS_STREET/BRUTUS_PR_HOME_ADDRESS_STREET_A";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_STREET_W :
		return "PR_HOME_ADDRESS_STREET_W";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_POST_OFFICE_BOX :
		return "PR_HOME_ADDRESS_POST_OFFICE_BOX/BRUTUS_PR_HOME_ADDRESS_POST_OFFICE_BOX_A";
	case BRUTUS_BRUTUS_PR_HOME_ADDRESS_POST_OFFICE_BOX_W :
		return "PR_HOME_ADDRESS_POST_OFFICE_BOX_W";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_CITY :
		return "PR_OTHER_ADDRESS_CITY/BRUTUS_PR_OTHER_ADDRESS_CITY_A";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_CITY_W :
		return "PR_OTHER_ADDRESS_CITY_W";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_COUNTRY :
		return "PR_OTHER_ADDRESS_COUNTRY/BRUTUS_PR_OTHER_ADDRESS_COUNTRY_A";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_COUNTRY_W :
		return "PR_OTHER_ADDRESS_COUNTRY_W";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_POSTAL_CODE :
		return "PR_OTHER_ADDRESS_POSTAL_CODE/BRUTUS_PR_OTHER_ADDRESS_POSTAL_CODE_A";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_POSTAL_CODE_W :
		return "PR_OTHER_ADDRESS_POSTAL_CODE_W";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_STATE_OR_PROVINCE :
		return "PR_OTHER_ADDRESS_STATE_OR_PROVINCE/BRUTUS_PR_OTHER_ADDRESS_STATE_OR_PROVINCE_A";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_STATE_OR_PROVINCE_W :
		return "PR_OTHER_ADDRESS_STATE_OR_PROVINCE_W";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_STREET :
		return "PR_OTHER_ADDRESS_STREET/BRUTUS_PR_OTHER_ADDRESS_STREET_A";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_STREET_W :
		return "PR_OTHER_ADDRESS_STREET_W";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_POST_OFFICE_BOX :
		return "PR_OTHER_ADDRESS_POST_OFFICE_BOX/BRUTUS_PR_OTHER_ADDRESS_POST_OFFICE_BOX_A";
	case BRUTUS_BRUTUS_PR_OTHER_ADDRESS_POST_OFFICE_BOX_W :
		return "PR_OTHER_ADDRESS_POST_OFFICE_BOX_W";
	case BRUTUS_BRUTUS_PR_STORE_PROVIDERS :
		return "PR_STORE_PROVIDERS";
	case BRUTUS_BRUTUS_PR_AB_PROVIDERS :
		return "PR_AB_PROVIDERS";
	case BRUTUS_BRUTUS_PR_TRANSPORT_PROVIDERS :
		return "PR_TRANSPORT_PROVIDERS";
	case BRUTUS_BRUTUS_PR_DEFAULT_PROFILE :
		return "PR_DEFAULT_PROFILE";
	case BRUTUS_BRUTUS_PR_AB_SEARCH_PATH :
		return "PR_AB_SEARCH_PATH";
	case BRUTUS_BRUTUS_PR_AB_DEFAULT_DIR :
		return "PR_AB_DEFAULT_DIR";
	case BRUTUS_BRUTUS_PR_AB_DEFAULT_PAB :
		return "PR_AB_DEFAULT_PAB";
	case BRUTUS_BRUTUS_PR_FILTERING_HOOKS :
		return "PR_FILTERING_HOOKS";
	case BRUTUS_BRUTUS_PR_SERVICE_NAME :
		return "PR_SERVICE_NAME/BRUTUS_PR_SERVICE_NAME_A";
	case BRUTUS_BRUTUS_PR_SERVICE_NAME_W :
		return "PR_SERVICE_NAME_W";
	case BRUTUS_BRUTUS_PR_SERVICE_DLL_NAME :
		return "PR_SERVICE_DLL_NAME/BRUTUS_PR_SERVICE_DLL_NAME_A";
	case BRUTUS_BRUTUS_PR_SERVICE_DLL_NAME_W :
		return "PR_SERVICE_DLL_NAME_W";
	case BRUTUS_BRUTUS_PR_SERVICE_ENTRY_NAME :
		return "PR_SERVICE_ENTRY_NAME";
	case BRUTUS_BRUTUS_PR_SERVICE_UID :
		return "PR_SERVICE_UID";
	case BRUTUS_BRUTUS_PR_SERVICE_EXTRA_UIDS :
		return "PR_SERVICE_EXTRA_UIDS";
	case BRUTUS_BRUTUS_PR_SERVICES :
		return "PR_SERVICES";
	case BRUTUS_BRUTUS_PR_SERVICE_SUPPORT_FILES :
		return "PR_SERVICE_SUPPORT_FILES/BRUTUS_PR_SERVICE_SUPPORT_FILES_A";
	case BRUTUS_BRUTUS_PR_SERVICE_SUPPORT_FILES_W :
		return "PR_SERVICE_SUPPORT_FILES_W";
	case BRUTUS_BRUTUS_PR_SERVICE_DELETE_FILES :
		return "PR_SERVICE_DELETE_FILES/BRUTUS_PR_SERVICE_DELETE_FILES_A";
	case BRUTUS_BRUTUS_PR_SERVICE_DELETE_FILES_W :
		return "PR_SERVICE_DELETE_FILES_W";
	case BRUTUS_BRUTUS_PR_AB_SEARCH_PATH_UPDATE :
		return "PR_AB_SEARCH_PATH_UPDATE";
	case BRUTUS_BRUTUS_PR_PROFILE_NAME :
		return "PR_PROFILE_NAME/BRUTUS_PR_PROFILE_NAME_A";
	case BRUTUS_BRUTUS_PR_PROFILE_NAME_W :
		return "PR_PROFILE_NAME_W";
	case BRUTUS_BRUTUS_PR_IDENTITY_DISPLAY :
		return "PR_IDENTITY_DISPLAY/BRUTUS_PR_IDENTITY_DISPLAY_A";
	case BRUTUS_BRUTUS_PR_IDENTITY_DISPLAY_W :
		return "PR_IDENTITY_DISPLAY_W";
	case BRUTUS_BRUTUS_PR_IDENTITY_ENTRYID :
		return "PR_IDENTITY_ENTRYID";
	case BRUTUS_BRUTUS_PR_RESOURCE_METHODS :
		return "PR_RESOURCE_METHODS";
	case BRUTUS_BRUTUS_PR_RESOURCE_TYPE :
		return "PR_RESOURCE_TYPE";
	case BRUTUS_BRUTUS_PR_STATUS_CODE :
		return "PR_STATUS_CODE";
	case BRUTUS_BRUTUS_PR_IDENTITY_SEARCH_KEY :
		return "PR_IDENTITY_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_OWN_STORE_ENTRYID :
		return "PR_OWN_STORE_ENTRYID";
	case BRUTUS_BRUTUS_PR_RESOURCE_PATH :
		return "PR_RESOURCE_PATH/BRUTUS_PR_RESOURCE_PATH_A";
	case BRUTUS_BRUTUS_PR_RESOURCE_PATH_W :
		return "PR_RESOURCE_PATH_W";
	case BRUTUS_BRUTUS_PR_STATUS_STRING :
		return "PR_STATUS_STRING/BRUTUS_PR_STATUS_STRING_A";
	case BRUTUS_BRUTUS_PR_STATUS_STRING_W :
		return "PR_STATUS_STRING_W";
	case BRUTUS_BRUTUS_PR_X400_DEFERRED_DELIVERY_CANCEL :
		return "PR_X400_DEFERRED_DELIVERY_CANCEL";
	case BRUTUS_BRUTUS_PR_HEADER_FOLDER_ENTRYID :
		return "PR_HEADER_FOLDER_ENTRYID";
	case BRUTUS_BRUTUS_PR_REMOTE_PROGRESS :
		return "PR_REMOTE_PROGRESS";
	case BRUTUS_BRUTUS_PR_REMOTE_PROGRESS_TEXT :
		return "PR_REMOTE_PROGRESS_TEXT/BRUTUS_PR_REMOTE_PROGRESS_TEXT_A";
	case BRUTUS_BRUTUS_PR_REMOTE_PROGRESS_TEXT_W :
		return "PR_REMOTE_PROGRESS_TEXT_W";
	case BRUTUS_BRUTUS_PR_REMOTE_VALIDATE_OK :
		return "PR_REMOTE_VALIDATE_OK";
	case BRUTUS_BRUTUS_PR_CONTROL_FLAGS :
		return "PR_CONTROL_FLAGS";
	case BRUTUS_BRUTUS_PR_CONTROL_STRUCTURE :
		return "PR_CONTROL_STRUCTURE";
	case BRUTUS_BRUTUS_PR_CONTROL_TYPE :
		return "PR_CONTROL_TYPE";
	case BRUTUS_BRUTUS_PR_DELTAX :
		return "PR_DELTAX";
	case BRUTUS_BRUTUS_PR_DELTAY :
		return "PR_DELTAY";
	case BRUTUS_BRUTUS_PR_XPOS :
		return "PR_XPOS";
	case BRUTUS_BRUTUS_PR_YPOS :
		return "PR_YPOS";
	case BRUTUS_BRUTUS_PR_CONTROL_ID :
		return "PR_CONTROL_ID";
	case BRUTUS_BRUTUS_PR_INITIAL_DETAILS_PANE :
		return "PR_INITIAL_DETAILS_PANE";
	case BRUTUS_BRUTUS_PR_PROFILE_VERSION :
		return "PR_PROFILE_VERSION";
	case BRUTUS_BRUTUS_PR_PROFILE_CONFIG_FLAGS :
		return "PR_PROFILE_CONFIG_FLAGS";
	case BRUTUS_BRUTUS_PR_PROFILE_HOME_SERVER :
		return "PR_PROFILE_HOME_SERVER";
	case BRUTUS_BRUTUS_PR_PROFILE_HOME_SERVER_DN :
		return "PR_PROFILE_HOME_SERVER_DN";
	case BRUTUS_BRUTUS_PR_PROFILE_HOME_SERVER_ADDRS :
		return "PR_PROFILE_HOME_SERVER_ADDRS";
	case BRUTUS_BRUTUS_PR_PROFILE_USER :
		return "PR_PROFILE_USER";
	case BRUTUS_BRUTUS_PR_PROFILE_CONNECT_FLAGS :
		return "PR_PROFILE_CONNECT_FLAGS";
	case BRUTUS_BRUTUS_PR_PROFILE_TRANSPORT_FLAGS :
		return "PR_PROFILE_TRANSPORT_FLAGS";
	case BRUTUS_BRUTUS_PR_PROFILE_UI_STATE :
		return "PR_PROFILE_UI_STATE";
	case BRUTUS_BRUTUS_PR_PROFILE_UNRESOLVED_NAME :
		return "PR_PROFILE_UNRESOLVED_NAME";
	case BRUTUS_BRUTUS_PR_PROFILE_UNRESOLVED_SERVER :
		return "PR_PROFILE_UNRESOLVED_SERVER";
	case BRUTUS_BRUTUS_PR_PROFILE_BINDING_ORDER :
		return "PR_PROFILE_BINDING_ORDER";
	case BRUTUS_BRUTUS_PR_PROFILE_MAX_RESTRICT :
		return "PR_PROFILE_MAX_RESTRICT";
	case BRUTUS_BRUTUS_PR_PROFILE_AB_FILES_PATH :
		return "PR_PROFILE_AB_FILES_PATH";
	case BRUTUS_BRUTUS_PR_PROFILE_OFFLINE_STORE_PATH :
		return "PR_PROFILE_OFFLINE_STORE_PATH";
	case BRUTUS_BRUTUS_PR_PROFILE_OFFLINE_INFO :
		return "PR_PROFILE_OFFLINE_INFO";
	case BRUTUS_BRUTUS_PR_PROFILE_ADDR_INFO :
		return "PR_PROFILE_ADDR_INFO";
	case BRUTUS_BRUTUS_PR_PROFILE_OPTIONS_DATA :
		return "PR_PROFILE_OPTIONS_DATA";
	case BRUTUS_BRUTUS_PR_PROFILE_SECURE_MAILBOX :
		return "PR_PROFILE_SECURE_MAILBOX";
	case BRUTUS_BRUTUS_PR_DISABLE_WINSOCK :
		return "PR_DISABLE_WINSOCK";
	case BRUTUS_BRUTUS_PR_OST_ENCRYPTION :
		return "PR_OST_ENCRYPTION";
	case BRUTUS_BRUTUS_PR_PROFILE_OPEN_FLAGS :
		return "PR_PROFILE_OPEN_FLAGS";
	case BRUTUS_BRUTUS_PR_PROFILE_TYPE :
		return "PR_PROFILE_TYPE";
	case BRUTUS_BRUTUS_PR_PROFILE_MAILBOX :
		return "PR_PROFILE_MAILBOX";
	case BRUTUS_BRUTUS_PR_PROFILE_SERVER :
		return "PR_PROFILE_SERVER";
	case BRUTUS_BRUTUS_PR_PROFILE_SERVER_DN :
		return "PR_PROFILE_SERVER_DN";
	case BRUTUS_BRUTUS_PR_PROFILE_FAVFLD_DISPLAY_NAME :
		return "PR_PROFILE_FAVFLD_DISPLAY_NAME";
	case BRUTUS_BRUTUS_PR_PROFILE_FAVFLD_COMMENT :
		return "PR_PROFILE_FAVFLD_COMMENT";
	case BRUTUS_BRUTUS_PR_PROFILE_ALLPUB_DISPLAY_NAME :
		return "PR_PROFILE_ALLPUB_DISPLAY_NAME";
	case BRUTUS_BRUTUS_PR_PROFILE_ALLPUB_COMMENT :
		return "PR_PROFILE_ALLPUB_COMMENT";
	case BRUTUS_BRUTUS_PR_USER_ENTRYID :
		return "PR_USER_ENTRYID";
	case BRUTUS_BRUTUS_PR_USER_NAME :
		return "PR_USER_NAME";
	case BRUTUS_BRUTUS_PR_MAILBOX_OWNER_ENTRYID :
		return "PR_MAILBOX_OWNER_ENTRYID";
	case BRUTUS_BRUTUS_PR_MAILBOX_OWNER_NAME :
		return "PR_MAILBOX_OWNER_NAME";
	case BRUTUS_BRUTUS_PR_OOF_STATE :
		return "PR_OOF_STATE";
	case BRUTUS_BRUTUS_PR_HIERARCHY_SERVER :
		return "PR_HIERARCHY_SERVER";
	case BRUTUS_BRUTUS_PR_SCHEDULE_FOLDER_ENTRYID :
		return "PR_SCHEDULE_FOLDER_ENTRYID";
	case BRUTUS_BRUTUS_PR_IPM_DAF_ENTRYID :
		return "PR_IPM_DAF_ENTRYID";
	case BRUTUS_BRUTUS_PR_NON_IPM_SUBTREE_ENTRYID :
		return "PR_NON_IPM_SUBTREE_ENTRYID";
	case BRUTUS_BRUTUS_PR_EFORMS_REGISTRY_ENTRYID :
		return "PR_EFORMS_REGISTRY_ENTRYID";
	case BRUTUS_BRUTUS_PR_SPLUS_FREE_BUSY_ENTRYID :
		return "PR_SPLUS_FREE_BUSY_ENTRYID";
	case BRUTUS_BRUTUS_PR_OFFLINE_ADDRBOOK_ENTRYID :
		return "PR_OFFLINE_ADDRBOOK_ENTRYID";
	case BRUTUS_BRUTUS_PR_EFORMS_FOR_LOCALE_ENTRYID :
		return "PR_EFORMS_FOR_LOCALE_ENTRYID";
	case BRUTUS_BRUTUS_PR_FREE_BUSY_FOR_LOCAL_SITE_ENTRYID :
		return "PR_FREE_BUSY_FOR_LOCAL_SITE_ENTRYID";
	case BRUTUS_BRUTUS_PR_ADDRBOOK_FOR_LOCAL_SITE_ENTRYID :
		return "PR_ADDRBOOK_FOR_LOCAL_SITE_ENTRYID";
	case BRUTUS_BRUTUS_PR_OFFLINE_MESSAGE_ENTRYID :
		return "PR_OFFLINE_MESSAGE_ENTRYID";
	case BRUTUS_BRUTUS_PR_IPM_FAVORITES_ENTRYID :
		return "PR_IPM_FAVORITES_ENTRYID";
	case BRUTUS_BRUTUS_PR_IPM_PUBLIC_FOLDERS_ENTRYID :
		return "PR_IPM_PUBLIC_FOLDERS_ENTRYID";
	case BRUTUS_BRUTUS_PR_GW_MTSIN_ENTRYID :
		return "PR_GW_MTSIN_ENTRYID";
	case BRUTUS_BRUTUS_PR_GW_MTSOUT_ENTRYID :
		return "PR_GW_MTSOUT_ENTRYID";
	case BRUTUS_BRUTUS_PR_TRANSFER_ENABLED :
		return "PR_TRANSFER_ENABLED";
	case BRUTUS_BRUTUS_PR_TEST_LINE_SPEED :
		return "PR_TEST_LINE_SPEED";
	case BRUTUS_BRUTUS_PR_HIERARCHY_SYNCHRONIZER :
		return "PR_HIERARCHY_SYNCHRONIZER";
	case BRUTUS_BRUTUS_PR_CONTENTS_SYNCHRONIZER :
		return "PR_CONTENTS_SYNCHRONIZER";
	case BRUTUS_BRUTUS_PR_COLLECTOR :
		return "PR_COLLECTOR";
	case BRUTUS_BRUTUS_PR_FAST_TRANSFER :
		return "PR_FAST_TRANSFER";
	case BRUTUS_BRUTUS_PR_STORE_OFFLINE :
		return "PR_STORE_OFFLINE";
	case BRUTUS_BRUTUS_PR_IN_TRANSIT :
		return "PR_IN_TRANSIT";
	case BRUTUS_BRUTUS_PR_REPLICATION_STYLE :
		return "PR_REPLICATION_STYLE";
	case BRUTUS_BRUTUS_PR_REPLICATION_SCHEDULE :
		return "PR_REPLICATION_SCHEDULE";
	case BRUTUS_BRUTUS_PR_REPLICATION_MESSAGE_PRIORITY :
		return "PR_REPLICATION_MESSAGE_PRIORITY";
	case BRUTUS_BRUTUS_PR_OVERALL_MSG_AGE_LIMIT :
		return "PR_OVERALL_MSG_AGE_LIMIT";
	case BRUTUS_BRUTUS_PR_REPLICATION_ALWAYS_INTERVAL :
		return "PR_REPLICATION_ALWAYS_INTERVAL";
	case BRUTUS_BRUTUS_PR_REPLICATION_MSG_SIZE :
		return "PR_REPLICATION_MSG_SIZE";
	case BRUTUS_BRUTUS_PR_SOURCE_KEY :
		return "PR_SOURCE_KEY";
	case BRUTUS_BRUTUS_PR_PARENT_SOURCE_KEY :
		return "PR_PARENT_SOURCE_KEY";
	case BRUTUS_BRUTUS_PR_CHANGE_KEY :
		return "PR_CHANGE_KEY";
	case BRUTUS_BRUTUS_PR_PREDECESSOR_CHANGE_LIST :
		return "PR_PREDECESSOR_CHANGE_LIST";
	case BRUTUS_BRUTUS_PR_FOLDER_CHILD_COUNT :
		return "PR_FOLDER_CHILD_COUNT";
	case BRUTUS_BRUTUS_PR_RIGHTS :
		return "PR_RIGHTS";
	case BRUTUS_BRUTUS_PR_ACL_TABLE :
		return "PR_ACL_TABLE";
	case BRUTUS_BRUTUS_PR_RULES_TABLE :
		return "PR_RULES_TABLE";
	case BRUTUS_BRUTUS_PR_HAS_RULES :
		return "PR_HAS_RULES";
	case BRUTUS_BRUTUS_PR_ADDRESS_BOOK_ENTRYID :
		return "PR_ADDRESS_BOOK_ENTRYID";
	case BRUTUS_BRUTUS_PR_ACL_DATA :
		return "PR_ACL_DATA";
	case BRUTUS_BRUTUS_PR_RULES_DATA :
		return "PR_RULES_DATA";
	case BRUTUS_BRUTUS_PR_FOLDER_DESIGN_FLAGS :
		return "PR_FOLDER_DESIGN_FLAGS";
	case BRUTUS_BRUTUS_PR_DESIGN_IN_PROGRESS :
		return "PR_DESIGN_IN_PROGRESS";
	case BRUTUS_BRUTUS_PR_SECURE_ORIGINATION :
		return "PR_SECURE_ORIGINATION";
	case BRUTUS_BRUTUS_PR_PUBLISH_IN_ADDRESS_BOOK :
		return "PR_PUBLISH_IN_ADDRESS_BOOK";
	case BRUTUS_BRUTUS_PR_RESOLVE_METHOD :
		return "PR_RESOLVE_METHOD";
	case BRUTUS_BRUTUS_PR_ADDRESS_BOOK_DISPLAY_NAME :
		return "PR_ADDRESS_BOOK_DISPLAY_NAME";
	case BRUTUS_BRUTUS_PR_EFORMS_LOCALE_ID :
		return "PR_EFORMS_LOCALE_ID";
	case BRUTUS_BRUTUS_PR_REPLICA_LIST :
		return "PR_REPLICA_LIST";
	case BRUTUS_BRUTUS_PR_OVERALL_AGE_LIMIT :
		return "PR_OVERALL_AGE_LIMIT";
	case BRUTUS_BRUTUS_PR_PUBLIC_FOLDER_ENTRYID :
		return "PR_PUBLIC_FOLDER_ENTRYID";
	case BRUTUS_BRUTUS_PR_HAS_NAMED_PROPERTIES :
		return "PR_HAS_NAMED_PROPERTIES";
	case BRUTUS_BRUTUS_PR_CREATOR_NAME :
		return "PR_CREATOR_NAME";
	case BRUTUS_BRUTUS_PR_CREATOR_ENTRYID :
		return "PR_CREATOR_ENTRYID";
	case BRUTUS_BRUTUS_PR_LAST_MODIFIER_NAME :
		return "PR_LAST_MODIFIER_NAME";
	case BRUTUS_BRUTUS_PR_LAST_MODIFIER_ENTRYID :
		return "PR_LAST_MODIFIER_ENTRYID";
	case BRUTUS_BRUTUS_PR_HAS_DAMS :
		return "PR_HAS_DAMS";
	case BRUTUS_BRUTUS_PR_RULE_TRIGGER_HISTORY :
		return "PR_RULE_TRIGGER_HISTORY";
	case BRUTUS_BRUTUS_PR_MOVE_TO_STORE_ENTRYID :
		return "PR_MOVE_TO_STORE_ENTRYID";
	case BRUTUS_BRUTUS_PR_MOVE_TO_FOLDER_ENTRYID :
		return "PR_MOVE_TO_FOLDER_ENTRYID";
	case BRUTUS_BRUTUS_PR_REPLICA_SERVER :
		return "PR_REPLICA_SERVER";
	case BRUTUS_BRUTUS_PR_DEFERRED_SEND_NUMBER :
		return "PR_DEFERRED_SEND_NUMBER";
	case BRUTUS_BRUTUS_PR_DEFERRED_SEND_UNITS :
		return "PR_DEFERRED_SEND_UNITS";
	case BRUTUS_BRUTUS_PR_EXPIRY_NUMBER :
		return "PR_EXPIRY_NUMBER";
	case BRUTUS_BRUTUS_PR_EXPIRY_UNITS :
		return "PR_EXPIRY_UNITS";
	case BRUTUS_BRUTUS_PR_DEFERRED_SEND_TIME :
		return "PR_DEFERRED_SEND_TIME";
	case BRUTUS_BRUTUS_PR_GW_ADMIN_OPERATIONS :
		return "PR_GW_ADMIN_OPERATIONS";
	case BRUTUS_BRUTUS_PR_P1_CONTENT :
		return "PR_P1_CONTENT";
	case BRUTUS_BRUTUS_PR_P1_CONTENT_TYPE :
		return "PR_P1_CONTENT_TYPE";
	case BRUTUS_BRUTUS_PR_CLIENT_ACTIONS :
		return "PR_CLIENT_ACTIONS";
	case BRUTUS_BRUTUS_PR_DAM_ORIGINAL_ENTRYID :
		return "PR_DAM_ORIGINAL_ENTRYID";
	case BRUTUS_BRUTUS_PR_DAM_BACK_PATCHED :
		return "PR_DAM_BACK_PATCHED";
	case BRUTUS_BRUTUS_PR_RULE_ERROR :
		return "PR_RULE_ERROR";
	case BRUTUS_BRUTUS_PR_RULE_ACTION_TYPE :
		return "PR_RULE_ACTION_TYPE";
	case BRUTUS_BRUTUS_PR_RULE_ACTION_NUMBER :
		return "PR_RULE_ACTION_NUMBER";
	case BRUTUS_BRUTUS_PR_RULE_FOLDER_ENTRYID :
		return "PR_RULE_FOLDER_ENTRYID";
	case BRUTUS_BRUTUS_PR_CONFLICT_ENTRYID :
		return "PR_CONFLICT_ENTRYID";
	case BRUTUS_BRUTUS_PR_MESSAGE_LOCALE_ID :
		return "PR_MESSAGE_LOCALE_ID";
	case BRUTUS_BRUTUS_PR_STORAGE_QUOTA_LIMIT :
		return "PR_STORAGE_QUOTA_LIMIT";
	case BRUTUS_BRUTUS_PR_EXCESS_STORAGE_USED :
		return "PR_EXCESS_STORAGE_USED";
	case BRUTUS_BRUTUS_PR_SVR_GENERATING_QUOTA_MSG :
		return "PR_SVR_GENERATING_QUOTA_MSG";
	case BRUTUS_BRUTUS_PR_DELEGATED_BY_RULE :
		return "PR_DELEGATED_BY_RULE";
	case BRUTUS_BRUTUS_PR_IN_CONFLICT :
		return "PR_IN_CONFLICT";
	case BRUTUS_BRUTUS_PR_LONGTERM_ENTRYID_FROM_TABLE :
		return "PR_LONGTERM_ENTRYID_FROM_TABLE";
	case BRUTUS_BRUTUS_PR_ORIGINATOR_NAME :
		return "PR_ORIGINATOR_NAME";
	case BRUTUS_BRUTUS_PR_ORIGINATOR_ADDR :
		return "PR_ORIGINATOR_ADDR";
	case BRUTUS_BRUTUS_PR_ORIGINATOR_ADDRTYPE :
		return "PR_ORIGINATOR_ADDRTYPE";
	case BRUTUS_BRUTUS_PR_ORIGINATOR_ENTRYID :
		return "PR_ORIGINATOR_ENTRYID";
	case BRUTUS_BRUTUS_PR_ARRIVAL_TIME :
		return "PR_ARRIVAL_TIME";
	case BRUTUS_BRUTUS_PR_TRACE_INFO :
		return "PR_TRACE_INFO";
	case BRUTUS_BRUTUS_PR_INTERNAL_TRACE_INFO :
		return "PR_INTERNAL_TRACE_INFO";
	case BRUTUS_BRUTUS_PR_SUBJECT_TRACE_INFO :
		return "PR_SUBJECT_TRACE_INFO";
	case BRUTUS_BRUTUS_PR_RECIPIENT_NUMBER :
		return "PR_RECIPIENT_NUMBER";
	case BRUTUS_BRUTUS_PR_MTS_SUBJECT_ID :
		return "PR_MTS_SUBJECT_ID";
	case BRUTUS_BRUTUS_PR_REPORT_DESTINATION_NAME :
		return "PR_REPORT_DESTINATION_NAME";
	case BRUTUS_BRUTUS_PR_REPORT_DESTINATION_ENTRYID :
		return "PR_REPORT_DESTINATION_ENTRYID";
	case BRUTUS_BRUTUS_PR_CONTENT_SEARCH_KEY :
		return "PR_CONTENT_SEARCH_KEY";
	case BRUTUS_BRUTUS_PR_FOREIGN_ID :
		return "PR_FOREIGN_ID";
	case BRUTUS_BRUTUS_PR_FOREIGN_REPORT_ID :
		return "PR_FOREIGN_REPORT_ID";
	case BRUTUS_BRUTUS_PR_FOREIGN_SUBJECT_ID :
		return "PR_FOREIGN_SUBJECT_ID";
	case BRUTUS_BRUTUS_PR_MEMBER_ID :
		return "PR_MEMBER_ID";
	case BRUTUS_BRUTUS_PR_MEMBER_NAME :
		return "PR_MEMBER_NAME";
	case BRUTUS_BRUTUS_PR_MEMBER_RIGHTS :
		return "PR_MEMBER_RIGHTS";
	case BRUTUS_BRUTUS_PR_RULE_ID :
		return "PR_RULE_ID";
	case BRUTUS_BRUTUS_PR_RULE_IDS :
		return "PR_RULE_IDS";
	case BRUTUS_BRUTUS_PR_RULE_SEQUENCE :
		return "PR_RULE_SEQUENCE";
	case BRUTUS_BRUTUS_PR_RULE_STATE :
		return "PR_RULE_STATE";
	case BRUTUS_BRUTUS_PR_RULE_USER_FLAGS :
		return "PR_RULE_USER_FLAGS";
	case BRUTUS_BRUTUS_PR_RULE_CONDITION :
		return "PR_RULE_CONDITION";
	case BRUTUS_BRUTUS_PR_RULE_ACTIONS :
		return "PR_RULE_ACTIONS";
	case BRUTUS_BRUTUS_PR_RULE_PROVIDER :
		return "PR_RULE_PROVIDER";
	case BRUTUS_BRUTUS_PR_RULE_NAME :
		return "PR_RULE_NAME";
	case BRUTUS_BRUTUS_PR_RULE_LEVEL :
		return "PR_RULE_LEVEL";
	case BRUTUS_BRUTUS_PR_RULE_PROVIDER_DATA :
		return "PR_RULE_PROVIDER_DATA";
	case BRUTUS_BRUTUS_PR_NT_USER_NAME :
		return "PR_NT_USER_NAME";
	case BRUTUS_BRUTUS_PR_LOCALE_ID :
		return "PR_LOCALE_ID";
	case BRUTUS_BRUTUS_PR_LAST_LOGON_TIME :
		return "PR_LAST_LOGON_TIME";
	case BRUTUS_BRUTUS_PR_LAST_LOGOFF_TIME :
		return "PR_LAST_LOGOFF_TIME";
	case BRUTUS_BRUTUS_PR_STORAGE_LIMIT_INFORMATION :
		return "PR_STORAGE_LIMIT_INFORMATION";
	case BRUTUS_BRUTUS_PR_FOLDER_FLAGS :
		return "PR_FOLDER_FLAGS";
	case BRUTUS_BRUTUS_PR_LAST_ACCESS_TIME :
		return "PR_LAST_ACCESS_TIME";
	case BRUTUS_BRUTUS_PR_RESTRICTION_COUNT :
		return "PR_RESTRICTION_COUNT";
	case BRUTUS_BRUTUS_PR_CATEG_COUNT :
		return "PR_CATEG_COUNT";
	case BRUTUS_BRUTUS_PR_CACHED_COLUMN_COUNT :
		return "PR_CACHED_COLUMN_COUNT";
	case BRUTUS_BRUTUS_PR_NORMAL_MSG_W_ATTACH_COUNT :
		return "PR_NORMAL_MSG_W_ATTACH_COUNT";
	case BRUTUS_BRUTUS_PR_ASSOC_MSG_W_ATTACH_COUNT :
		return "PR_ASSOC_MSG_W_ATTACH_COUNT";
	case BRUTUS_BRUTUS_PR_RECIPIENT_ON_NORMAL_MSG_COUNT :
		return "PR_RECIPIENT_ON_NORMAL_MSG_COUNT";
	case BRUTUS_BRUTUS_PR_RECIPIENT_ON_ASSOC_MSG_COUNT :
		return "PR_RECIPIENT_ON_ASSOC_MSG_COUNT";
	case BRUTUS_BRUTUS_PR_ATTACH_ON_NORMAL_MSG_COUNT :
		return "PR_ATTACH_ON_NORMAL_MSG_COUNT";
	case BRUTUS_BRUTUS_PR_ATTACH_ON_ASSOC_MSG_COUNT :
		return "PR_ATTACH_ON_ASSOC_MSG_COUNT";
	case BRUTUS_BRUTUS_PR_NORMAL_MESSAGE_SIZE :
		return "PR_NORMAL_MESSAGE_SIZE";
	case BRUTUS_BRUTUS_PR_NORMAL_MESSAGE_SIZE_EXTENDED :
		return "PR_NORMAL_MESSAGE_SIZE_EXTENDED";
	case BRUTUS_BRUTUS_PR_ASSOC_MESSAGE_SIZE :
		return "PR_ASSOC_MESSAGE_SIZE";
	case BRUTUS_BRUTUS_PR_ASSOC_MESSAGE_SIZE_EXTENDED :
		return "PR_ASSOC_MESSAGE_SIZE_EXTENDED";
	case BRUTUS_BRUTUS_PR_FOLDER_PATHNAME :
		return "PR_FOLDER_PATHNAME";
	case BRUTUS_BRUTUS_PR_OWNER_COUNT :
		return "PR_OWNER_COUNT";
	case BRUTUS_BRUTUS_PR_CONTACT_COUNT :
		return "PR_CONTACT_COUNT";
	case BRUTUS_BRUTUS_PR_MESSAGE_SIZE_EXTENDED :
		return "PR_MESSAGE_SIZE_EXTENDED";
	case BRUTUS_BRUTUS_PR_OFFLINE_FLAGS :
		return "PR_OFFLINE_FLAGS";
	case BRUTUS_BRUTUS_PR_SYNCHRONIZE_FLAGS :
		return "PR_SYNCHRONIZE_FLAGS";
	case BRUTUS_BRUTUS_PR_DELETED_MESSAGE_SIZE_EXTENDED :
		return "PR_DELETED_MESSAGE_SIZE_EXTENDED";
	case BRUTUS_BRUTUS_PR_DELETED_NORMAL_MESSAGE_SIZE_EXTENDED :
		return "PR_DELETED_NORMAL_MESSAGE_SIZE_EXTENDED";
	case BRUTUS_BRUTUS_PR_DELETED_ASSOC_MESSAGE_SIZE_EXTENDED :
		return "PR_DELETED_ASSOC_MESSAGE_SIZE_EXTENDED";
	case BRUTUS_BRUTUS_PR_DELETED_MSG_COUNT :
		return "PR_DELETED_MSG_COUNT";
	case BRUTUS_BRUTUS_PR_DELETED_ASSOC_MSG_COUNT :
		return "PR_DELETED_ASSOC_MSG_COUNT";
	case BRUTUS_BRUTUS_PR_CODE_PAGE_ID :
		return "PR_CODE_PAGE_ID";
	case BRUTUS_BRUTUS_PR_SORT_LOCALE_ID :
		return "PR_SORT_LOCALE_ID";
	case BRUTUS_BRUTUS_PR_DELETED_FOLDER_COUNT :
		return "PR_DELETED_FOLDER_COUNT";
	case BRUTUS_BRUTUS_PR_EXTENDED_ACL_DATA :
		return "PR_EXTENDED_ACL_DATA";
	case BRUTUS_BRUTUS_PR_INTERNET_ARTICLE_NUMBER :
		return "PR_INTERNET_ARTICLE_NUMBER";
	case BRUTUS_BRUTUS_PR_REPLICA_VERSION :
		return "PR_REPLICA_VERSION";
	case BRUTUS_BRUTUS_PR_INTERNET_CPID :
		return "PR_INTERNET_CPID";
	case BRUTUS_BRUTUS_PR_MESSAGE_CODEPAGE :
		return "PR_MESSAGE_CODEPAGE";
	case BRUTUS_BRUTUS_PR_ICON_INDEX :
		return "PR_ICON_INDEX";
	case BRUTUS_BRUTUS_PR_DISABLE_FULL_FIDELITY :
		return "PR_DISABLE_FULL_FIDELITY";
	case BRUTUS_BRUTUS_PR_INTERNET_MESSAGE_ID :
		return "PR_INTERNET_MESSAGE_ID";
	case BRUTUS_BRUTUS_PR_SenderFlags :
		return "PR_SenderFlags";
	case BRUTUS_BRUTUS_PR_SentRepresentingFlags :
		return "PR_SentRepresentingFlags";
	case BRUTUS_BRUTUS_PR_RcvdByFlags :
		return "Pr_RcvdByFlags";
	case BRUTUS_BRUTUS_PR_RcvdRepresentingFlags :
		return "PR_RcvdRepresentingFlags";
	default :
		g_snprintf(unknown_tag, sizeof(unknown_tag), "Unknown_%sproperty_{%s,0x%.4hX}",
			   ((BRUTUS_PROP_ID(PropTag) >= 0x8000) && (BRUTUS_PROP_ID(PropTag) <= 0xFFFE)) ? "Named_" : "",
			   brutus_proptype_to_str(BRUTUS_PROP_TYPE(PropTag)),
			   (unsigned short)(PropTag >> 16));
		return unknown_tag;
	}
}
