/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Implementation file for Brutus CORBA utility functions.
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <orbit/orbit.h>

#include "brutus.h"
#include "brutus_corba.h"
#include "brutus_string.h"

#include <idl_output/BrutusCheck.h>
#include <idl_output/IAttach.h>
#include <idl_output/IExchangeModifyTable.h>
#include <idl_output/IMAPIFormAdviseSink.h>
#include <idl_output/IMAPIProp.h>
#include <idl_output/IMSLogon.h>
#include <idl_output/IProfAdmin.h>
#include <idl_output/ITableData.h>
#include <idl_output/defines.h>
#include <idl_output/BrutusLogOn.h>
#include <idl_output/IDistList.h>
#include <idl_output/IExchangeRuleAction.h>
#include <idl_output/IMAPIFormContainer.h>
#include <idl_output/IMAPISession.h>
#include <idl_output/IMSProvider.h>
#include <idl_output/IProfSect.h>
#include <idl_output/ITnef.h>
#include <idl_output/uid.h>
#include <idl_output/BrutusSession.h>
#include <idl_output/IExchangeExportChanges.h>
#include <idl_output/IMAPIAdviseSink.h>
#include <idl_output/IMAPIFormFactory.h>
#include <idl_output/IMAPIStatus.h>
#include <idl_output/IMailUser.h>
#include <idl_output/IPropData.h>
#include <idl_output/IXPLogon.h>
#include <idl_output/IABContainer.h>
#include <idl_output/IExchangeFavorites.h>
#include <idl_output/IMAPIContainer.h>
#include <idl_output/IMAPIFormInfo.h>
#include <idl_output/IMAPISupport.h>
#include <idl_output/IMessage.h>
#include <idl_output/IProviderAdmin.h>
#include <idl_output/IXPProvider.h>
#include <idl_output/IABLogon.h>
#include <idl_output/IExchangeImportContentsChanges.h>
#include <idl_output/IMAPIControl.h>
#include <idl_output/IMAPIFormMgr.h>
#include <idl_output/IMAPITable.h>
#include <idl_output/IMsgServiceAdmin.h>
#include <idl_output/IABProvider.h>
#include <idl_output/IExchangeImportHierarchyChanges.h>
#include <idl_output/IMAPIFolder.h>
#include <idl_output/IMAPIMessageSite.h>
#include <idl_output/IMAPIViewAdviseSink.h>
#include <idl_output/IMsgStore.h>
#include <idl_output/ISpoolerHook.h>
#include <idl_output/log_server.h>
#include <idl_output/IAddrBook.h>
#include <idl_output/IExchangeManageStore.h>
#include <idl_output/IMAPIForm.h>
#include <idl_output/IMAPIProgress.h>
#include <idl_output/IMAPIViewContext.h>
#include <idl_output/IPersistMessage.h>
#include <idl_output/return_codes.h>
#include <idl_output/IStream.h>

static inline size_t
brutus_corba_strlen(const CORBA_char *s)
{
	size_t l = 0;

	for (; (CORBA_char)'\0' != *s; s++, l++)
		;

	return l;
}

CORBA_ORB
brutus_create_orb(int argc,
		  const char **argv,
		  const gboolean ssl,
		  const char *logon_init_ref,
		  const char *logon_obj_server_key,
		  const char *server_host_name,
		  const guint16 server_port,
		  const char *orb_name,
		  CORBA_Environment *ev)
{
	CORBA_ORB orb = CORBA_OBJECT_NIL;
	int init_argc = (argc ? argc : 1) + 5;
	char *init_ref_logon_arg = NULL;
	char **init_argv = (char**)malloc(sizeof(char*) * (init_argc));
	if (!init_argv)
		return CORBA_OBJECT_NIL;

	// sanity checks
	if (!logon_init_ref)
		goto out;
	if (!logon_obj_server_key)
		goto out;
	if (!server_host_name)
		goto out;
	if (!server_port)
		goto out;
	if (!orb_name)
		goto out;

	memset((void*)init_argv, 0, init_argc);

	// copy the original arguments
	if (argc)
		memcpy((void*)init_argv, (void*)argv, sizeof(char*) * (argc));
	else { // argv[0] must be present due to ORBit2 being clever and skipping the first argument
		argc = 1;
		init_argv[0] = strdup("DUMMY_ARGV0");
	}

	// BrutusLogOn
	init_ref_logon_arg = brutus_build_init_ref_opt(ssl,
						       logon_init_ref,
						       logon_obj_server_key,
						       server_host_name,
						       server_port);
	if (!init_ref_logon_arg)
		goto out;

	init_argv[argc] = init_ref_logon_arg;

	// IPv4 - needed to interoperate with TAO
	init_argv[argc+1] = "--ORBIIOPIPv4=1";

	// IPv6 - could be needed to interoperate with TAO
	init_argv[argc+2] = "--ORBIIOPIPv6=1";

	// Explicitly force ORBit2 to be non-local
	init_argv[argc+3] = "--ORBLocalOnly=0";

	// Force ORBit2 to use the IP address in the all generated IORs
	init_argv[argc+4] = "--ORBNetID=ipaddr";

	// initialize the ORB
	orb = CORBA_ORB_init(&init_argc, init_argv, (char*)orb_name, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		orb = CORBA_OBJECT_NIL;
	}

out:
	free(init_argv[0]);
	free(init_ref_logon_arg);
	free(init_argv);

	return orb;
}

PortableServer_POA
brutus_get_root_poa(const CORBA_ORB Orb,
		    CORBA_Environment *ev)
{
	if (CORBA_OBJECT_NIL == Orb)
		return CORBA_OBJECT_NIL;

	PortableServer_POA root_poa = CORBA_OBJECT_NIL;

	root_poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(Orb, "RootPOA", ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		return CORBA_OBJECT_NIL;
	}

	// force multithreaded POA
	ORBit_ObjectAdaptor_set_thread_hint((ORBit_ObjectAdaptor)root_poa,
					    ORBIT_THREAD_HINT_PER_OBJECT);

	return root_poa;
}

PortableServer_POA
brutus_create_poa(const char *Name,
		  PortableServer_POAManager PoaManager,
		  PortableServer_POA ParentPOA,
		  CORBA_Environment *ev)
{
	if (CORBA_OBJECT_NIL == ParentPOA)
		return CORBA_OBJECT_NIL;

	CORBA_PolicyList pl;
	pl._maximum = 0;
	pl._length = 0;
	pl._buffer = NULL;

	PortableServer_POA retv = CORBA_OBJECT_NIL;
	retv = PortableServer_POA_create_POA(ParentPOA,
					     Name,
					     PoaManager,
					     &pl,
					     ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		return CORBA_OBJECT_NIL;
	}

	// force multithreaded POA
	ORBit_ObjectAdaptor_set_thread_hint((ORBit_ObjectAdaptor)retv,
					    ORBIT_THREAD_HINT_PER_OBJECT);

	return retv;
}

void
brutus_free_orbit_object(CORBA_Object obj,
			 PortableServer_POA poa,
			 CORBA_Environment *ev)
{
	PortableServer_ObjectId *objid = NULL;

	objid = PortableServer_POA_reference_to_id(poa, obj, ev);
	if (ORBIT2_EX(ev))
		return;

	/* Servant: deactivation - will invoke __destroy destructor */
	PortableServer_POA_deactivate_object(poa, objid, ev);
	if (ORBIT2_EX(ev))
		goto out;

	/* free associated memory */
	CORBA_Object_release(obj, ev);
	if (ORBIT2_EX(ev))
		goto out;

out:
	CORBA_free(objid);
}


void
brutus_free_poa(PortableServer_POA poa,
		CORBA_Environment *ev)
{
	PortableServer_POAManager poa_manager = CORBA_OBJECT_NIL;

	poa_manager = PortableServer_POA__get_the_POAManager(poa, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		return;
	}

	PortableServer_POAManager_State state = PortableServer_POAManager_get_state(poa_manager, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		return;
	}
	if (PortableServer_POAManager_ACTIVE == state) {
		PortableServer_POAManager_deactivate(poa_manager, CORBA_TRUE,
						     CORBA_TRUE, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			return;
		}
	}

	CORBA_Object_release((CORBA_Object) poa_manager, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		return;
	}

	PortableServer_POA_destroy(poa, CORBA_TRUE, CORBA_TRUE, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		return;
	}

	CORBA_Object_release((CORBA_Object) poa, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		return;
	}
}

void
brutus_deactivate_poa(PortableServer_POA poa,
		      CORBA_Environment *ev)
{
	PortableServer_POAManager poa_manager = CORBA_OBJECT_NIL;

	poa_manager = PortableServer_POA__get_the_POAManager(poa, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		return;
	}

	PortableServer_POAManager_deactivate(poa_manager, 
					     CORBA_TRUE,
					     CORBA_TRUE,
					     ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		return;
	}

	CORBA_Object_release((CORBA_Object) poa_manager, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		return;
	}
}

gboolean
brutus_object_ref_to_file(CORBA_ORB orb,
			  CORBA_Object servant,
			  gchar *filename,
			  CORBA_Environment *ev)
{
	gboolean retv = FALSE;
	CORBA_char *objref = NULL;
	int file_fd = -1;
	FILE *file = NULL;
	size_t c = 0;

	file_fd = g_open(filename, O_CREAT | O_WRONLY | O_SYNC | O_TRUNC, S_IRUSR | S_IWUSR);
	if (-1 == file_fd)
		return FALSE;

	file = fdopen(file_fd, "w");
	if (!file)
		return FALSE;

	objref = CORBA_ORB_object_to_string(orb, servant, ev);
	if (ORBIT2_EX(ev))
		goto out;

	// must work even if sizeof(char) != sizeof(CORBA_char)
	c = fwrite((const void*)objref, sizeof(CORBA_char), brutus_corba_strlen(objref)+1, file);
	if (c == (1 + brutus_corba_strlen(objref)))
		retv = TRUE;
	CORBA_free(objref);

out:
	fflush(file);
	fclose(file);

	return retv;
}

CORBA_Object
brutus_object_ref_from_file(CORBA_ORB orb,
			    gchar *filename,
			    CORBA_Environment *ev)
{
	CORBA_Object obj = CORBA_OBJECT_NIL;
	CORBA_char *objref = NULL;
	FILE *file = NULL;
	struct stat st;
	size_t c = 0;

	file = g_fopen(filename, "r");
	if (!file)
		return CORBA_OBJECT_NIL;

	if (g_stat(filename, &st))
		goto out;

	objref = g_malloc0((st.st_size) * sizeof(gchar));
	if (!objref)
		goto out;

	// must work even if sizeof(char) != sizeof(CORBA_char)
	c = fread((void*)objref, sizeof(CORBA_char), (size_t)(st.st_size/sizeof(CORBA_char)), file);
	if (c != st.st_size)
		goto out;

	obj = (CORBA_Object)CORBA_ORB_string_to_object(orb, objref, ev);

out:
	g_free(objref);
	fclose(file);

	return obj;
}

void
brutus_release_object(const BRUTUS_BrutusTC type_code,
		      CORBA_Object *object,
		      PortableServer_POA poa,
		      CORBA_Environment *ev)
{
	CORBA_boolean cb = CORBA_TRUE;

	cb = CORBA_Object_is_nil(*object, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		*object = CORBA_OBJECT_NIL;
		goto out;
	}
	if (cb)
		goto out;

	switch (type_code) {
	case BRUTUS_IABContainer_tc :
		BRUTUS_IABContainer_Destroy((BRUTUS_IABContainer)*object, 0, ev);
		break;
	case BRUTUS_IABLogon_tc :
		BRUTUS_IABLogon_Destroy((BRUTUS_IABLogon)*object, 0, ev);
		break;
	case BRUTUS_IABProvider_tc :
		BRUTUS_IABProvider_Destroy((BRUTUS_IABProvider)*object, 0, ev);
		break;
	case BRUTUS_IAddrBook_tc :
		BRUTUS_IAddrBook_Destroy((BRUTUS_IAddrBook)*object, 0, ev);
		break;
	case BRUTUS_IAttach_tc :
		BRUTUS_IAttach_Destroy((BRUTUS_IAttach)*object, 0, ev);
		break;
	case BRUTUS_IDistList_tc :
		BRUTUS_IDistList_Destroy((BRUTUS_IDistList)*object, 0, ev);
		break;
	case BRUTUS_IExchangeExportChanges_tc :
		BRUTUS_IExchangeExportChanges_Destroy((BRUTUS_IExchangeExportChanges)*object, 0, ev);
		break;
	case BRUTUS_IExchangeFavorites_tc :
		BRUTUS_IExchangeFavorites_Destroy((BRUTUS_IExchangeFavorites)*object, 0, ev);
		break;
	case BRUTUS_IExchangeImportContentsChanges_tc :
		BRUTUS_IExchangeImportContentsChanges_Destroy((BRUTUS_IExchangeImportContentsChanges)*object, 0, ev);
		break;
	case BRUTUS_IExchangeImportHierarchyChanges_tc :
		BRUTUS_IExchangeImportHierarchyChanges_Destroy((BRUTUS_IExchangeImportHierarchyChanges)*object, 0, ev);
		break;
	case BRUTUS_IExchangeManageStore_tc :
		BRUTUS_IExchangeManageStore_Destroy((BRUTUS_IExchangeManageStore)*object, 0, ev);
		break;
	case BRUTUS_IExchangeModifyTable_tc :
		BRUTUS_IExchangeModifyTable_Destroy((BRUTUS_IExchangeModifyTable)*object, 0, ev);
		break;
	case BRUTUS_IExchangeRuleAction_tc :
		BRUTUS_IExchangeRuleAction_Destroy((BRUTUS_IExchangeRuleAction)*object, 0, ev);
		break;
	case BRUTUS_IMAPIContainer_tc :
		BRUTUS_IMAPIContainer_Destroy((BRUTUS_IMAPIContainer)*object, 0, ev);
		break;
	case BRUTUS_IMAPIControl_tc :
		BRUTUS_IMAPIControl_Destroy((BRUTUS_IMAPIControl)*object, 0, ev);
		break;
	case BRUTUS_IMAPIFolder_tc :
		BRUTUS_IMAPIFolder_Destroy((BRUTUS_IMAPIFolder)*object, 0, ev);
		break;
	case BRUTUS_IMAPIForm_tc :
		BRUTUS_IMAPIForm_Destroy((BRUTUS_IMAPIForm)*object, 0, ev);
		break;
	case BRUTUS_IMAPIFormContainer_tc :
		BRUTUS_IMAPIFormContainer_Destroy((BRUTUS_IMAPIFormContainer)*object, 0, ev);
		break;
	case BRUTUS_IMAPIFormFactory_tc :
		BRUTUS_IMAPIFormFactory_Destroy((BRUTUS_IMAPIFormFactory)*object, 0, ev);
		break;
	case BRUTUS_IMAPIFormInfo_tc :
		BRUTUS_IMAPIFormInfo_Destroy((BRUTUS_IMAPIFormInfo)*object, 0, ev);
		break;
	case BRUTUS_IMAPIFormMgr_tc :
		BRUTUS_IMAPIFormMgr_Destroy((BRUTUS_IMAPIFormMgr)*object, 0, ev);
		break;
	case BRUTUS_IMAPIMessageSite_tc :
		BRUTUS_IMAPIMessageSite_Destroy((BRUTUS_IMAPIMessageSite)*object, 0, ev);
		break;
	case BRUTUS_IMAPIProgress_tc :
		BRUTUS_IMAPIProgress_Destroy((BRUTUS_IMAPIProgress)*object, 0, ev);
		break;
	case BRUTUS_IMAPIProp_tc :
		BRUTUS_IMAPIProp_Destroy((BRUTUS_IMAPIProp)*object, 0, ev);
		break;
	case BRUTUS_IMAPISession_tc :
		BRUTUS_IMAPISession_Destroy((BRUTUS_IMAPISession)*object, 0, ev);
		break;
	case BRUTUS_IMAPIStatus_tc :
		BRUTUS_IMAPIStatus_Destroy((BRUTUS_IMAPIStatus)*object, 0, ev);
		break;
	case BRUTUS_IMAPISupport_tc :
		BRUTUS_IMAPISupport_Destroy((BRUTUS_IMAPISupport)*object, 0, ev);
		break;
	case BRUTUS_IMAPITable_tc :
		BRUTUS_IMAPITable_Destroy((BRUTUS_IMAPITable)*object, 0, ev);
		break;
	case BRUTUS_IMAPIViewAdviseSink_tc :
		BRUTUS_IMAPIViewAdviseSink_Destroy((BRUTUS_IMAPIViewAdviseSink)*object, 0, ev);
		break;
	case BRUTUS_IMAPIViewContext_tc :
		BRUTUS_IMAPIViewContext_Destroy((BRUTUS_IMAPIViewContext)*object, 0, ev);
		break;
	case BRUTUS_IMSLogon_tc :
		BRUTUS_IMSLogon_Destroy((BRUTUS_IMSLogon)*object, 0, ev);
		break;
	case BRUTUS_IMSProvider_tc :
		BRUTUS_IMSProvider_Destroy((BRUTUS_IMSProvider)*object, 0, ev);
		break;
	case BRUTUS_IMailUser_tc :
		BRUTUS_IMailUser_Destroy((BRUTUS_IMailUser)*object, 0, ev);
		break;
	case BRUTUS_IMessage_tc :
		BRUTUS_IMessage_Destroy((BRUTUS_IMessage)*object, 0, ev);
		break;
	case BRUTUS_IMsgServiceAdmin_tc :
		BRUTUS_IMsgServiceAdmin_Destroy((BRUTUS_IMsgServiceAdmin)*object, 0, ev);
		break;
	case BRUTUS_IMsgStore_tc :
		BRUTUS_IMsgStore_Destroy((BRUTUS_IMsgStore)*object, 0, ev);
		break;
	case BRUTUS_IPersistMessage_tc :
		BRUTUS_IPersistMessage_Destroy((BRUTUS_IPersistMessage)*object, 0, ev);
		break;
	case BRUTUS_IProfAdmin_tc :
		BRUTUS_IProfAdmin_Destroy((BRUTUS_IProfAdmin)*object, 0, ev);
		break;
	case BRUTUS_IProfSect_tc :
		BRUTUS_IProfSect_Destroy((BRUTUS_IProfSect)*object, 0, ev);
		break;
	case BRUTUS_IPropData_tc :
		BRUTUS_IPropData_Destroy((BRUTUS_IPropData)*object, 0, ev);
		break;
	case BRUTUS_IProviderAdmin_tc :
		BRUTUS_IProviderAdmin_Destroy((BRUTUS_IProviderAdmin)*object, 0, ev);
		break;
	case BRUTUS_ISequentialStream_tc :
		BRUTUS_ISequentialStream_Destroy((BRUTUS_ISequentialStream)*object, 0, ev);
		break;
	case BRUTUS_ISpoolerHook_tc :
		BRUTUS_ISpoolerHook_Destroy((BRUTUS_ISpoolerHook)*object, 0, ev);
		break;
	case BRUTUS_IStream_tc :
		BRUTUS_IStream_Destroy((BRUTUS_IStream)*object, 0, ev);
		break;
	case BRUTUS_ITableData_tc :
		BRUTUS_ITableData_Destroy((BRUTUS_ITableData)*object, 0, ev);
		break;
	case BRUTUS_ITnef_tc :
		BRUTUS_ITnef_Destroy((BRUTUS_ITnef)*object, 0, ev);
		break;
	case BRUTUS_IXPLogon_tc :
		BRUTUS_IXPLogon_Destroy((BRUTUS_IXPLogon)*object, 0, ev);
		break;
	case BRUTUS_IXPProvider_tc :
		BRUTUS_IXPProvider_Destroy((BRUTUS_IXPProvider)*object, 0, ev);
		break;
	case BRUTUS_IUnknown_tc :
		BRUTUS_IUnknown_Destroy((BRUTUS_IUnknown)*object, 0, ev);
		break;
	case BRUTUS_BrutusCheck_tc :
	case BRUTUS_BrutusLogOn_tc :
	case BRUTUS_BrutusSession_tc :
	case BRUTUS_MAPIFunctions_tc :
	case BRUTUS_IMAPIAdviseSink_tc :
	case BRUTUS_IMAPIFormAdviseSink_tc :
	case BRUTUS_log_server_tc :
	case BRUTUS_SRestrictionContainer_tc :
	case BRUTUS_SAndRestriction_tc :
	case BRUTUS_SOrRestriction_tc :
	case BRUTUS_SNotRestriction_tc :
	case BRUTUS_SContentRestriction_tc :
	case BRUTUS_SPropertyRestriction_tc :
	case BRUTUS_SComparePropsRestriction_tc :
	case BRUTUS_SBitMaskRestriction_tc :
	case BRUTUS_SSizeRestriction_tc :
	case BRUTUS_SExistRestriction_tc :
	case BRUTUS_SSubRestriction_tc :
	case BRUTUS_SCommentRestriction_tc :
		brutus_free_orbit_object(*object, poa, ev);
		break;
	default :
		d("");
		g_assert(FALSE);
	};
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_Object_release(*object, ev);
	PRINT_IF_ORBIT2_EX(ev);
	*object = CORBA_OBJECT_NIL;

out:
	return;
}

void
brutus_free_seq_string_content(BRUTUS_seq_string *strings)
{
	unsigned int n;

	if (!strings)
		return;

	for (n = 0; n < strings->_maximum; n++)
		CORBA_free(strings->_buffer[n]);

	/* Why, oh why, is this so ??? Most likely an ORBit2 bug. We'll see a leak when it's fixed :-( */
//	CORBA_free(strings->_buffer);

	strings->_maximum = 0;
	strings->_length = 0;
	strings->_buffer = NULL;
}

BRUTUS_BRESULT
brutus_read_from_mapi_stream(BRUTUS_ISequentialStream stream,
			     BRUTUS_seq_octet **data,
			     CORBA_unsigned_long len,
			     CORBA_Environment *ev)
{
	const CORBA_unsigned_long recv_limit = (CORBA_unsigned_long)ORBit_get_giop_recv_limit() - 48; // 48 = large enough message header size
	CORBA_unsigned_long chunk_size;
	BRUTUS_seq_octet *chunk = NULL;
	BRUTUS_BRESULT retv = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	CORBA_unsigned_long n = 0;

	dfmt("ORBit2 receive limit = %d", recv_limit);

	*data = BRUTUS_seq_octet__alloc();
	if (!(*data))
		return BRUTUS_BRUTUS_MAPI_E_NOT_ENOUGH_MEMORY;

	(*data)->_buffer = BRUTUS_seq_octet_allocbuf(len);
	if (!(*data)->_buffer) {
		CORBA_free(*data);
		*data = NULL;
		return BRUTUS_BRUTUS_MAPI_E_NOT_ENOUGH_MEMORY;
	}
	(*data)->_maximum = len;
	(*data)->_length = 0;
	CORBA_sequence_set_release(*data, CORBA_TRUE);

	retv = BRUTUS_BRUTUS_MAPI_E_NOT_ENOUGH_MEMORY;
	while (len) {
		if (len > recv_limit) {
			chunk_size = recv_limit;
			len -= recv_limit;
		} else {
			chunk_size = len;
			len = 0;
		}

		retv = BRUTUS_ISequentialStream_Read(stream,
						     &chunk,
						     chunk_size,
						     ev);
		if (ORBIT2_EX(ev))
			goto out;
		if (BRUTUS_BRUTUS_S_OK != retv)
			goto out;
		if (chunk_size != chunk->_length) {
			retv = BRUTUS_BRUTUS_MAPI_W_PARTIAL_COMPLETION;
			goto out;
		}

		for (n = 0; n < chunk_size; n++)
			(*data)->_buffer[(*data)->_length+n] = chunk->_buffer[n];
		(*data)->_length += chunk_size;
		CORBA_free(chunk);
		chunk = NULL;
	}

out:
	CORBA_free(chunk);

	return retv;
}
