/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Implementation file for Brutus utility functions.
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* system includes */
#include <errno.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <uuid/uuid.h>
#include <libedataserver/e-source-list.h>
#include <libedataserver/md5-utils.h>

/* evolution-brutus includes */
#include <idl_output/return_codes.h>
#include <server/brutus_conv.h>
#include <brutus-keyring/brutus-keyring.h>
#include <brutus-keyring/brutus-keyringd.h>

#include "brutus_corba.h"
#include "brutus_util.h"

GStaticMutex user_query_mutex = G_STATIC_MUTEX_INIT;

#define BRUTUS_REQ_MAJOR (0)
#define BRUTUS_REQ_MINOR (9)
#define BRUTUS_REQ_MICRO (33)
#define BRUTUS_REQ_RELEASE_TAG (0)

gboolean
brutus_server_version_OK(int major,
			 int minor,
			 int micro,
			 int release_tag)
{
	return ((BRUTUS_REQ_MAJOR == major)
		&& (BRUTUS_REQ_MINOR == minor)
		&& (BRUTUS_REQ_MICRO == micro)
		&& (BRUTUS_REQ_RELEASE_TAG <= release_tag));
}

void brutus_for_each_free(gpointer data,
			  gpointer user_data)
{
	free(data);
}

void brutus_for_each_g_free(gpointer data,
			    gpointer user_data)
{
	g_free(data);
}

int brutus_timezone_to_gmt_offset(long time_zone_offset)
{
	int tz = (-1) * time_zone_offset;

	return (((tz/60/60) * 100) + (tz/60 % 60));
}

gboolean
brutus_add_account(const ECalSourceType e_type,
		   const char *account_name,
		   const char *mapi_profile,
		   const char *exchange_server,
		   const char *windows_user,
		   const char *exchange_mailbox,
		   const char *exchange_mailbox_dn,
		   const char *exchange_mailbox_email,
		   const char *windows_domain,
		   const char *brutus_server,
		   const char *brutus_port,
		   const gboolean use_ssl,
		   int auto_check_timeout)
{
	gboolean retv = FALSE;
	gboolean unref_group = FALSE;
	ESourceList *source_list = NULL;
	ESourceGroup *group = NULL;
	ESource *source = NULL;
	char *relative_uri = NULL;
	char *timeout = NULL;
	long port;

	port = strtol(brutus_port, (char **)NULL, 10);
	if ((EINVAL == errno ) || (ERANGE == errno)) {
		d("Invalid Brutus port number");
		goto out;
	}

	if (-1 == auto_check_timeout)
		auto_check_timeout = 10;
	timeout = g_strdup_printf("%d", auto_check_timeout);
	if (!timeout) {
		d("No memory");
		goto out;
	}

	// get list of groups
	if (!e_cal_get_sources(&source_list, e_type, NULL)) {
		d("Could not get source list from GConf");
		goto out;
	}

	// create group
	group = e_source_list_peek_group_by_name(source_list, "Brutus");
	if (!group) {
		group = e_source_group_new("Brutus", "brutus://");
		if (!group) {
			d("No memory");
			goto out;
		}
		unref_group = TRUE;

		if (!e_source_list_add_group(source_list, group, -1)) {
			d("Could not add Brutus source group");
			goto out;
		} else {
			e_source_list_sync(source_list, NULL);
		}
	}

	// create relative uri
	relative_uri = g_strdup_printf ("%s@%s:%s:__%s",
					windows_user,
					exchange_server,
					exchange_mailbox,
					(E_CAL_SOURCE_TYPE_EVENT == e_type) ? "EVENTS" : "TODOS");
	if (!relative_uri) {
		d("No memory");
		goto out;
	}

	// check for presence of source
	source = e_source_group_peek_source_by_name(group, account_name);
	if (source) {
		if (!g_ascii_strcasecmp(mapi_profile, e_source_get_property(source, "mapi_profile"))) {
			retv = TRUE;
			goto out;
		}
	} else { // create source
		source = e_source_new(account_name, relative_uri);
		if (!source) {
			d("Could not create new source");
			goto out;
		}
	}

	// set properties
	e_source_set_color(source, 0x00FF00); // green
	e_source_set_property(source, "mapi_profile", mapi_profile);
	e_source_set_property(source, "exchange_server", exchange_server);
	e_source_set_property(source, "windows_user", windows_user);
	e_source_set_property(source, "exchange_mailbox", exchange_mailbox);
	e_source_set_property(source, "exchange_mailbox_dn", (exchange_mailbox_dn ? exchange_mailbox_dn : ""));
	e_source_set_property(source, "exchange_mailbox_email", (exchange_mailbox_email ? exchange_mailbox_email : ""));
	e_source_set_property(source, "windows_domain", windows_domain);
	e_source_set_property(source, "brutus_server", brutus_server);
	e_source_set_property(source, "brutus_port", brutus_port);
	e_source_set_property(source, "use_ssl", use_ssl ? "1" : "0");
	e_source_set_property(source, "auto_check_timeout", timeout);

	if (!e_source_group_add_source(group, source, -1)) {
		d("Could not add source to group");
		goto out;
	}

	if (!e_source_list_sync(source_list, NULL)) {
		d("Could not sync source list");
		goto out;
	}

	retv = TRUE;

out:
	g_free(timeout);
	g_free(relative_uri);

	if (source)
		g_object_unref(source);

	if (group && unref_group)
		g_object_unref(group);

	if (source_list)
		g_object_unref(source_list);

	return retv;
}


static CORBA_ORB
create_orb(CORBA_Environment *ev)
{
	CORBA_ORB orb = CORBA_OBJECT_NIL;
	int init_argc = 4;
	char **init_argv = (char**)malloc(sizeof(char*) * (init_argc));
	if (!init_argv)
		return CORBA_OBJECT_NIL;

	memset((void*)init_argv, 0, init_argc);
	init_argv[0] = strdup("DUMMY_ARGV0");

	// IPv4 - needed to interoperate with clients
	init_argv[1] = "--ORBIIOPIPv4=1";

	// IPv6 - better to switch it off if we do not use it
	init_argv[2] = "--ORBIIOPIPv6=0";

	// Explicitly force ORBit2 to be local only
	init_argv[3] = "--ORBLocalOnly=1";

	// initialize the ORB
	orb = CORBA_ORB_init(&init_argc, init_argv, "brutus-util_orb_orbit-io-thread", ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		orb = CORBA_OBJECT_NIL;
	}

	free(init_argv[0]);
	free(init_argv);

	return orb;
}

static BRUTUS_BrutusKeyring
get_keyring_ref(CORBA_ORB orb)
{
	char *home_dir = getenv("HOME");
	gchar *ref_path = NULL;
	BRUTUS_BrutusKeyring keyring = CORBA_OBJECT_NIL;
	CORBA_boolean cb = CORBA_FALSE;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	cb = CORBA_Object_is_nil((CORBA_Object)orb, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb) {
		d("ORB is NIL");
		goto out;
	}

	ref_path = g_strconcat(home_dir, "/", BRUTUS_KEYRINGD_REF_FILE_NAME, NULL);
	if (!ref_path)
		goto out;
	keyring = brutus_object_ref_from_file(orb, ref_path, ev);
	g_free(ref_path);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	cb = CORBA_Object_is_nil((CORBA_Object)keyring, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb) {
		d("Could not get the Brutus Keyring reference");
		goto out;
	}

out:
	CORBA_exception_free(ev);

	return keyring;

}

static void
brutus_forget_account_password(const char *mapi_profile)
{
	BRUTUS_BrutusKeyring keyring = CORBA_OBJECT_NIL;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_ORB orb = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);
	orb = create_orb(ev);
	keyring = get_keyring_ref(orb);
	cb = CORBA_Object_is_nil((CORBA_Object)keyring, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb)
		goto out;

	BRUTUS_BrutusKeyring_delSecret(keyring, mapi_profile, ev);

out:
	CORBA_Object_release((CORBA_Object)keyring, ev);
	CORBA_Object_release((CORBA_Object)orb, ev);
	CORBA_exception_free(ev);

	return;
}

static void
brutus_set_account_password(const char *mapi_profile,
			    const char *password)
{
	BRUTUS_BrutusKeyring_KeyringResult keyring_retv = BRUTUS_BrutusKeyring_failed;
	BRUTUS_BrutusKeyring_KeyringData secret;
	BRUTUS_BrutusKeyring keyring = CORBA_OBJECT_NIL;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_ORB orb = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	orb = create_orb(ev);
	keyring = get_keyring_ref(orb);
	cb = CORBA_Object_is_nil((CORBA_Object)keyring, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb)
		goto out;

	if (!password) 
		goto out;
	else {
		secret._maximum = strlen(password)+1;
		secret._length = secret._maximum;
		secret._buffer = BRUTUS_BrutusKeyring_KeyringData_allocbuf(secret._maximum);
		if (!secret._buffer)
			goto out;
	}

	keyring_retv = BRUTUS_BrutusKeyring_setSecret(keyring, mapi_profile, &secret, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BrutusKeyring_success == keyring_retv)

out:
	if (secret._buffer)
		CORBA_free(secret._buffer);

	CORBA_Object_release((CORBA_Object)keyring, ev);
	CORBA_Object_release((CORBA_Object)orb, ev);
	CORBA_exception_free(ev);

	return;
}

/*
 * This will retrieve the Brutus account password for a specified MAPI profile.
 */
static char*
brutus_get_account_password(const char *mapi_profile)
{
	BRUTUS_BrutusKeyring_KeyringResult keyring_retv = BRUTUS_BrutusKeyring_failed;
	BRUTUS_BrutusKeyring_KeyringData *secret = NULL;
	BRUTUS_BrutusKeyring keyring = CORBA_OBJECT_NIL;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_ORB orb = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];
	char *retv = NULL;
	int status;
	char *argv[] = {
		LIBEXECDIR "/brutus-query-password",
		"-set_passphrase",
		NULL,
	};

	CORBA_exception_init(ev);
	orb = create_orb(ev);
	keyring = get_keyring_ref(orb);
	cb = CORBA_Object_is_nil((CORBA_Object)keyring, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb)
		goto out;

	keyring_retv = BRUTUS_BrutusKeyring_getSecret(keyring, mapi_profile, &secret, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BrutusKeyring_set_passphrase == keyring_retv) {
		g_spawn_sync(NULL,
			     argv,
			     NULL,
			     G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
			     NULL,
			     NULL,
			     NULL,
			     NULL,
			     &status,
			     NULL);
		if (EXIT_SUCCESS != status)
			goto out;
	}
	keyring_retv = BRUTUS_BrutusKeyring_getSecret(keyring, mapi_profile, &secret, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BrutusKeyring_success == keyring_retv) 
		retv = strdup((const char*)secret->_buffer);

out:
	if (secret)
		CORBA_free(secret);

	CORBA_Object_release((CORBA_Object)keyring, ev);
	CORBA_Object_release((CORBA_Object)orb, ev);
	CORBA_exception_free(ev);

	return retv;
}

static char*
brutus_get_password(const char *prompt,
		    const char *mapi_profile)
{
	char *password = NULL;
	int status;
	char *argv[] = {
		LIBEXECDIR "/brutus-query-password",
		"-set_password",
		(char*)prompt,
		(char*)mapi_profile,
		NULL,
	};

	// only one password query dialog at any one time
	g_static_mutex_lock(&user_query_mutex);

	d("libexecdir = " LIBEXECDIR);
	password = brutus_get_account_password(mapi_profile); // from keyring
	if (!password) { // not present, query user
		d("Could not retrieve password from keyring - querying user");
		g_spawn_sync(NULL,
			     argv,
			     NULL,
			     G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
			     NULL,
			     NULL,
			     NULL,
			     NULL,
			     &status,
			     NULL);
		if (EXIT_SUCCESS != status)
			goto out;

		password = brutus_get_account_password(mapi_profile); // try the keyring again
	}

out:
	g_static_mutex_unlock(&user_query_mutex);
	return password;
}

gboolean
brutus_logon_loop(BRUTUS_BrutusLogOn brutus_logon,
		  BRUTUS_BrutusCheck lifeline,
		  BRUTUS_IMAPISession *mapi_session,
		  CORBA_unsigned_long *instance_id,
		  char **mapi_profile,
		  const char *prompt,
		  const char *err_prompt,
		  const char *windows_user,
		  const char *windows_domain,
		  const char *exchange_mailbox,
		  const char *exchange_mailbox_dn,
		  const char *exchange_mailbox_email,
		  const char *exchange_server)
{
	gboolean retv = FALSE;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_MAPI_E_LOGON_FAILED;
	char *password = NULL;
	char guid_str[37] = { '\0' };
	uuid_t guid = { 0 };
	char *mailbox_opts[4] = { NULL };
	size_t idx = 0;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	mailbox_opts[0] = g_strdup(exchange_mailbox);
	mailbox_opts[1] = g_strdup_printf("=%s", exchange_mailbox);
	mailbox_opts[2] = (exchange_mailbox_dn && strlen(exchange_mailbox_dn)) ? g_strdup(exchange_mailbox_dn) : NULL;
	mailbox_opts[3] = (exchange_mailbox_email && strlen(exchange_mailbox_email)) ? g_strdup_printf("SMTP:%s", exchange_mailbox_email) : NULL;

	/* get the password */
	password = brutus_get_password(prompt, (const char*)(*mapi_profile));
	if (!password) {
		d("User cancelled or password was all whitespace");
		goto out;
	}

	do { // loop indefinitely if password is bad, break for other errors
		idx = 0;
		do { // loop while resolving the mailbox
			if (idx >= sizeof(mailbox_opts))
				break;

			d(mailbox_opts[idx]);
			br = BRUTUS_BrutusLogOn_Logon(brutus_logon,
						      lifeline,
						      (const char*)(*mapi_profile),
						      "",
						      windows_user,
						      windows_domain,
						      password,
						      mailbox_opts[idx],
						      exchange_server,
						      BRUTUS_BRUTUS_MAPI_EXPLICIT_PROFILE,
						      0,
						      instance_id,
						      mapi_session,
						      ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto out;
			}
			d(brutus_bresult_to_str(br));
			switch (br) {
			case BRUTUS_BRUTUS_MAPI_E_UNKNOWN_MAILBOX : // could not resolve mailbox
			case BRUTUS_BRUTUS_MAPI_E_AMBIGUOUS_RECIP : // could not resolve mailbox
			case BRUTUS_BRUTUS_MAPI_E_VERSION : // no support
				idx++;
				continue;
			default :
				break;
			}
		} while (FALSE);
		switch (br) {
		case BRUTUS_BRUTUS_LIFELINE_TIMEOUT :
			d("Lifeline timeout");
			goto out;
		case BRUTUS_BRUTUS_EXCEPTION_FROM_CLIENT :
			d("Possible exception in lifeline servant");
			goto out;
		case BRUTUS_BRUTUS_INTERNAL_ERROR :
			d("Brutus Server internal error");
			goto out;
		default :
			d(brutus_bresult_to_str(br));
			break;
		}
		if (BRUTUS_BRUTUS_MAPI_E_LOGON_FAILED == br) {
			g_free(password);
			password = NULL;
			brutus_forget_account_password((const char*)(*mapi_profile)); // forget bad password

			password = brutus_get_password(err_prompt, (const char*)(*mapi_profile));
			if (!password) { // user canceled
				d("User cancelled or password was all whitespace");
				goto out;
			}
			continue;
		}
		if (BRUTUS_BRUTUS_MAPI_E_NO_ACCESS == br) {
			brutus_forget_account_password((const char*)(*mapi_profile)); // forget bad profile
			goto create_profile;
		}

		break;
	} while (TRUE);
	if (BRUTUS_BRUTUS_S_OK == br) {
		retv = TRUE;
		goto out;
	}

	dfmt("BrutusLogOn->Logon() returned %s", brutus_bresult_to_str(br));
	if (BRUTUS_BRUTUS_MAPI_E_NOT_INITIALIZED == br)
		goto out;

create_profile:
	if (BRUTUS_BRUTUS_MAPI_E_NO_ACCESS == br) { // should never happen...
		d("MAPI profile name collision, creating a new one...");
		uuid_generate(guid);
		uuid_unparse(guid, guid_str);
		CORBA_free(*mapi_profile);
		*mapi_profile = CORBA_string_dup(guid_str);

		brutus_forget_account_password((const char*)(*mapi_profile)); // forget bad password
		brutus_set_account_password((const char*)(*mapi_profile),
					    password);

		dfmt("The new MAPI profile is \"%s\"", *mapi_profile);
	}

	dfmt("Attempting logon with MAPI profile \"%s\"", *mapi_profile);
	do {
		br = BRUTUS_BrutusLogOn_Logon(brutus_logon,
					      lifeline,
					      (const char*)(*mapi_profile),
					      "",
					      windows_user,
					      windows_domain,
					      password,
					      exchange_mailbox,
					      exchange_server,
					      BRUTUS_BRUTUS_MAPI_EXPLICIT_PROFILE,
					      0,
					      instance_id,
					      mapi_session,
					      ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));

			g_free(password);
			password = NULL;
			brutus_forget_account_password((const char*)(*mapi_profile)); // forget bad password

			password = brutus_get_password(err_prompt, (const char*)(*mapi_profile));
			if (!password) { // user canceled
				d("User cancelled or password was all whitespace");
				goto out;
			}
		} else
			break;
	} while (TRUE);

out:
	if (BRUTUS_BRUTUS_S_OK == br) {
		dfmt("\"%s\" has logged on", windows_user);
		retv = TRUE;
	}

	CORBA_exception_free(ev);

	g_free(password);
	g_free(mailbox_opts[0]);
	g_free(mailbox_opts[1]);
	g_free(mailbox_opts[2]);
	g_free(mailbox_opts[3]);

	return retv;
}

static inline gboolean
brutus_digest_str_is_null(const digest_str_t digest_str)
{
	return (('\0' == digest_str[0]) ? TRUE : FALSE);
}

static inline gboolean
brutus_digest_is_null(const digest_t digest)
{
	static const digest_t null_digest = { 0 };

	return (memcmp((const void*)null_digest, (const void*)digest, sizeof(digest_t)) ? FALSE : TRUE);
}

static inline void
brutus_digest_to_str(const digest_t digest,
		     digest_str_t digest_str)
{
	int i;

	memset((void*)digest_str, '\0', sizeof(digest_str_t));

	if (brutus_digest_is_null(digest))
		return;

	for (i = 0; i < sizeof(digest_t); i++)
		snprintf(&digest_str[2*i], 3, "%02x", (guint8)digest[i]);
}

void
brutus_get_digest_str(const char *str,
		      digest_str_t digest_str)
{
	digest_t digest = { 0 };

	md5_get_digest(str, strlen(str), digest);
	brutus_digest_to_str(digest, digest_str);
}

gchar *
brutus_strip_trailing_and_leading_whitespace(const char *str)
{
	gchar *retv = NULL;
	int pos = 0;

	if (!str)
		return NULL;

	// fast-forward to first non-space character
	while (isspace(*str)) 
		str++;

	retv = g_strdup(str);
	pos = strlen(str) - 1; // last character

	while (0 <= pos) {
		if (isspace(retv[pos]))
			retv[pos] = '\0';
		else
			break;
		pos--;
	}

	return retv;
}

