/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Servant implementation file for a Brutus proxy daemon.
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *    MA 02111-1307 USA
 */

#include "brutus-proxy.h"

#include <BrutusCheck_impl.c>
#include <IMAPIAdviseSink_impl.c>
#include <types_impl.c>

#include <stdio.h>

struct MAPIAccount {
	CORBA_char *mailbox;
	CORBA_char *server;
	CORBA_char *profile;
	BRUTUS_IMAPISession session;
	CORBA_unsigned_long instance_id;
};

static gboolean
brutus_account_find_match(const GPtrArray *accounts,
			  const CORBA_char *mailbox, 
			  const CORBA_char *server, 
			  guint *idx)
{
	guint n = 0;
	struct MAPIAccount *account = NULL;

	if (!accounts)
		return FALSE;

	for (n = 0; n < accounts->len; n++) {
		account = (struct MAPIAccount*)g_ptr_array_index(accounts, n);
		if (!strcmp(mailbox, account->mailbox)
		    && !strcmp(server, account->server)) {
			*idx = n;
			return TRUE;
		}
	}

	return FALSE;
}

static gboolean
brutus_account_find_session(GPtrArray *accounts,
			    const CORBA_char *mailbox, 
			    const CORBA_char *server, 
			    guint *idx)
{
	guint n = 0;
	struct MAPIAccount *account = NULL;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_Environment ev[1];
	CORBA_exception_init(ev);

	if (!accounts)
		return FALSE;

	for (n = 0; n < accounts->len; n++) {
		account = (struct MAPIAccount*)g_ptr_array_index(accounts, n);
		
		if (!strcmp(mailbox, account->mailbox) && !strcmp(server, account->server)) {
			cb = CORBA_Object_is_nil((CORBA_Object)account->session, ev);
			if (!ORBIT2_EX(ev) && !cb) {
				BRUTUS_IMAPISession_ping(account->session, ev);
				if (!ORBIT2_EX(ev)) { 
					CORBA_exception_free(ev);
					*idx = n;
					return TRUE;
				} else {
					CORBA_Object_release(account->session, ev);
					account->session = CORBA_OBJECT_NIL;
				}
			}
			break;
		}
	}
	CORBA_exception_free(ev);

	return FALSE;
}

static void 
brutus_free_accounts(GPtrArray *accounts)
{
	guint n = 0;
	struct MAPIAccount *account = NULL;
	CORBA_boolean cb = CORBA_TRUE;	
	CORBA_Environment ev[1];

	if (!accounts)
		return;

	CORBA_exception_init(ev);

	for (n = 0; n < accounts->len; n++) {
		account = (struct MAPIAccount*)g_ptr_array_index(accounts, n);

		CORBA_free(account->mailbox);
		CORBA_free(account->server);
		CORBA_free(account->profile);
		cb = CORBA_Object_is_nil((CORBA_Object)account->session, ev);
		if (!ORBIT2_EX(ev) && !cb) {
			BRUTUS_IMAPISession_Destroy(account->session, account->instance_id, ev);
			CORBA_Object_release(account->session, ev);
		}
		g_free(account);
	}
	g_ptr_array_free(accounts, TRUE);

	CORBA_exception_free(ev);
}

/* Will write all accounts to disk */
static void
brutus_store_accounts(GPtrArray *accounts)
{
	guint n = 0;
	struct MAPIAccount *account = NULL;
	FILE *accounts_file = NULL;
	int accounts_file_fd = -1;
	gchar *accounts_file_path = g_strconcat(getenv("HOME"), "/", BRUTUSD_ACCOUNTS_FILE_NAME, NULL);
	if (!accounts_file_path)
		goto out;

	if (!accounts || !accounts->len) {
		unlink(accounts_file_path);
		g_free(accounts_file_path);
		return;
	}

	accounts_file_fd = g_open(accounts_file_path, O_CREAT | O_WRONLY | O_SYNC | O_TRUNC, S_IRUSR | S_IWUSR);
	g_free(accounts_file_path);

	if (-1 == accounts_file_fd) 
		goto out;
	accounts_file = fdopen(accounts_file_fd, "w");

	for (n = 0; n < accounts->len; n++) {
		account = (struct MAPIAccount*)g_ptr_array_index(accounts, n);
	
		// mailbox
		fputs(account->mailbox, accounts_file);
		fputs("\n", accounts_file);

		// exchange server
		fputs(account->server, accounts_file);
		fputs("\n", accounts_file);

		// profile name
		fputs(account->profile, accounts_file);
		fputs("\n", accounts_file);
	}

out:
	fflush(accounts_file);
	fclose(accounts_file);
}

/* size of internal read buffer - maximum length of a mailbox or profile name */
#define ACCOUNT_ENTRY_SIZE 300

/* Will read all accounts from disk */
static void
brutus_read_accounts(GPtrArray **accounts)
{
	int c = 1; /* Don't let this initially be EOF */
	int count = 0;
	struct MAPIAccount *account = NULL;
	char buf[ACCOUNT_ENTRY_SIZE+1] = { '\0' };
	FILE *accounts_file = NULL;
	int accounts_file_fd = -1;
	gchar *accounts_file_path = g_strconcat(getenv("HOME"), "/", BRUTUSD_ACCOUNTS_FILE_NAME, NULL);
	if (!accounts_file_path)
		goto err;

	if (*accounts)
		brutus_free_accounts(*accounts);
	*accounts = NULL;

	accounts_file_fd = g_open(accounts_file_path, O_CREAT | O_RDONLY | O_SYNC , S_IRUSR | S_IWUSR);
	g_free(accounts_file_path);

	if (-1 == accounts_file_fd) 
		goto err;

	accounts_file = fdopen(accounts_file_fd, "r");

	// count entries
	while (c != EOF) {
		if ('\n' == c) 
			count++;
		c = getc(accounts_file);
	}
	if (3 > count) // return if too few
		goto err;
	{ // check for divisibility with 3
		div_t q = div(count, 3);
		if (q.rem) 
			goto err;
		else
			count = q.quot;
	}
	rewind(accounts_file);

	*accounts = g_ptr_array_sized_new(count);
	if (!*accounts) 
		goto out;

	/* 256 comes from 255+\n+\0 */
	/* 65 comes from 64+\n+\0 */
	while (count--) {
		account = g_new(struct MAPIAccount, 1);
		if (!account)
			goto err;

		// mailbox
		if (!fgets(buf, ACCOUNT_ENTRY_SIZE, accounts_file))
			goto err;
		if (256 <= strlen(buf)) // corrupt mailbox name. 
			goto err;
		buf[strlen(buf)-1] = '\0'; // we know the last character is \n so we terminate the string at that
		account->mailbox = CORBA_string_dup(buf);
		memset((void*)buf, '\0', ACCOUNT_ENTRY_SIZE);

		// exchange server
		if (!fgets(buf, ACCOUNT_ENTRY_SIZE, accounts_file))
			goto err;
		if (256 <= strlen(buf)) // corrupt server name. 
			goto err;
		buf[strlen(buf)-1] = '\0';
		account->server = CORBA_string_dup(buf);
		memset((void*)buf, '\0', ACCOUNT_ENTRY_SIZE);

		// profile name
		if (!fgets(buf, ACCOUNT_ENTRY_SIZE, accounts_file))
			goto err;
		if (65 <= strlen(buf)) // corrupt profile name
			goto err;
		buf[strlen(buf)-1] = '\0';
		account->profile = CORBA_string_dup(buf);
		memset((void*)buf, '\0', ACCOUNT_ENTRY_SIZE);

		account->session = CORBA_OBJECT_NIL;
		account->instance_id = 0;

		g_ptr_array_add(*accounts, (gpointer)account);
	}

out:
	fclose(accounts_file);
	return;

err:
	brutus_free_accounts(*accounts);
	*accounts = g_ptr_array_new();

	fclose(accounts_file);
	brutus_store_accounts(NULL);

	return;
}


/*** App-specific servant structures ***/

#if !defined(_typedef_impl_POA_BRUTUS_BrutusProxy_)
#define _typedef_impl_POA_BRUTUS_BrutusProxy_ 1
typedef struct {
	POA_BRUTUS_BrutusProxy servant;
	PortableServer_POA poa;

	/* ------ add private attributes here ------ */

	GPtrArray *accounts;

	BRUTUS_BrutusCheck lifeline;

	/* 
	 * Initially in mustOfferState. 
	 * Thereafter always in idleState or offerInProgressState. 
	 */
	BRUTUS_BrutusProxy_QueryState query_state;
	GMutex *state_mutex;

	GMutex *mutex;

	/* ------ ---------- end ------------ ------ */

} impl_POA_BRUTUS_BrutusProxy;
#endif


/*** Implementation stub prototypes ***/

#if !defined(_decl_impl_BRUTUS_BrutusProxy__destroy_)
#define _decl_impl_BRUTUS_BrutusProxy__destroy_ 1
static void impl_BRUTUS_BrutusProxy__destroy(impl_POA_BRUTUS_BrutusProxy *servant,
					     CORBA_Environment *ev);
#endif

#if !defined(_decl_impl_BRUTUS_BrutusProxy_wrapObject_)
#define _decl_impl_BRUTUS_BrutusProxy_wrapObject_ 1
static void
impl_BRUTUS_BrutusProxy_wrapObject(impl_POA_BRUTUS_BrutusProxy *servant,
				   const BRUTUS_BrutusTC type_code,
				   const CORBA_Object filling,
				   CORBA_Object *wrapper,
				   CORBA_Environment *ev);
#endif

#if !defined(_decl_impl_BRUTUS_BrutusProxy_killWrapper_)
#define _decl_impl_BRUTUS_BrutusProxy_killWrapper_ 1
static void
impl_BRUTUS_BrutusProxy_killWrapper(impl_POA_BRUTUS_BrutusProxy *servant,
				    const CORBA_Object Wrapper,
				    CORBA_Environment *ev);
#endif

#if !defined(_decl_impl_BRUTUS_BrutusProxy_querySession_)
#define _decl_impl_BRUTUS_BrutusProxy_querySession_ 1
static BRUTUS_BrutusProxy_QueryState
impl_BRUTUS_BrutusProxy_querySession(impl_POA_BRUTUS_BrutusProxy *servant,
				     const CORBA_char *mailbox,
				     const CORBA_char *server,
				     BRUTUS_IMAPISession *session,
				     CORBA_Environment *ev);
#endif

#if !defined(_decl_impl_BRUTUS_BrutusProxy_offerSession_)
#define _decl_impl_BRUTUS_BrutusProxy_offerSession_ 1
static void
impl_BRUTUS_BrutusProxy_offerSession(impl_POA_BRUTUS_BrutusProxy *servant,
				     const BRUTUS_IMAPISession session,
				     const CORBA_char *mailbox,
				     const CORBA_char *server,
				     const CORBA_char *profile,
				     CORBA_Environment *ev);
#endif

#if !defined(_decl_impl_BRUTUS_BrutusProxy_getProfile_)
#define _decl_impl_BRUTUS_BrutusProxy_getProfile_ 1
static void
impl_BRUTUS_BrutusProxy_getProfile(impl_POA_BRUTUS_BrutusProxy *servant,
				   const CORBA_char *mailbox,
				   const CORBA_char *server,
				   CORBA_string* profile,
				   CORBA_Environment *ev);
#endif


/*** epv structures ***/

#if !defined(_impl_BRUTUS_BrutusProxy_base_epv_)
#define _impl_BRUTUS_BrutusProxy_base_epv_ 1
static PortableServer_ServantBase__epv impl_BRUTUS_BrutusProxy_base_epv = {
	NULL,             /* _private data */
	(gpointer) & impl_BRUTUS_BrutusProxy__destroy, /* finalize routine */
	NULL,             /* default_POA routine */
};
#endif

#if !defined(_impl_BRUTUS_BrutusProxy_epv_)
#define _impl_BRUTUS_BrutusProxy_epv_ 1
static POA_BRUTUS_BrutusProxy__epv impl_BRUTUS_BrutusProxy_epv = {
	NULL, /* _private */
	(gpointer)&impl_BRUTUS_BrutusProxy_wrapObject,
	(gpointer)&impl_BRUTUS_BrutusProxy_killWrapper,
	(gpointer)&impl_BRUTUS_BrutusProxy_querySession,
	(gpointer)&impl_BRUTUS_BrutusProxy_offerSession,
	(gpointer)&impl_BRUTUS_BrutusProxy_getProfile,
};
#endif


/*** vepv structures ***/

#if !defined(_impl_BRUTUS_BrutusProxy_vepv_)
#define _impl_BRUTUS_BrutusProxy_vepv_ 1
static POA_BRUTUS_BrutusProxy__vepv impl_BRUTUS_BrutusProxy_vepv = {
	&impl_BRUTUS_BrutusProxy_base_epv,
	&impl_BRUTUS_BrutusProxy_epv,
};
#endif


/*** Stub implementations ***/

#if !defined(_impl_BRUTUS_BrutusProxy__create_)
#define _impl_BRUTUS_BrutusProxy__create_ 1
static BRUTUS_BrutusProxy impl_BRUTUS_BrutusProxy__create(PortableServer_POA poa,
							  CORBA_Environment *ev)
{
	BRUTUS_BrutusProxy retval;
	impl_POA_BRUTUS_BrutusProxy *newservant;
	PortableServer_ObjectId *objid;

	newservant = g_new0(impl_POA_BRUTUS_BrutusProxy, 1);
	newservant->servant.vepv = &impl_BRUTUS_BrutusProxy_vepv;
	newservant->poa = (PortableServer_POA) CORBA_Object_duplicate((CORBA_Object)poa, ev);
	POA_BRUTUS_BrutusProxy__init((PortableServer_Servant)newservant, ev);

	/* Before servant is going to be activated all
	 * private attributes must be initialized.  */
	/* ------ init private attributes here ------ */

	/* read all accounts */
	newservant->accounts = NULL;
	brutus_read_accounts(&newservant->accounts);

	/* instantiate the proxy lifeline */
	newservant->lifeline = impl_BRUTUS_BrutusCheck__create(CORBA_OBJECT_NIL, FALSE, newservant->poa, ev);

	/* initialize state and locking */
	newservant->query_state = BRUTUS_BrutusProxy_mustOfferState;
	newservant->state_mutex = g_mutex_new();
	newservant->mutex = g_mutex_new();

	/* ------ ---------- end ------------- ------ */

	objid = PortableServer_POA_activate_object(poa, newservant, ev);
	CORBA_free(objid);
	retval = PortableServer_POA_servant_to_reference(poa, newservant, ev);

	return retval;
}
#endif

#if !defined(_impl_BRUTUS_BrutusProxy__destroy_)
#define _impl_BRUTUS_BrutusProxy__destroy_ 1
static void
impl_BRUTUS_BrutusProxy__destroy(impl_POA_BRUTUS_BrutusProxy *servant,
				 CORBA_Environment *ev)
{
	CORBA_boolean cb = CORBA_TRUE;	

	CORBA_Object_release ((CORBA_Object) servant->poa, ev);

	/* No further remote method calls are delegated to 
	 * servant and you may free your private attributes. */
	/* ------ free private attributes here ------ */

	brutus_free_accounts(servant->accounts);

	cb = CORBA_Object_is_nil((CORBA_Object)servant->lifeline, ev);
	if (!ORBIT2_EX(ev) && !cb) 
		CORBA_Object_release(servant->lifeline, ev);

	g_mutex_free(servant->state_mutex);
	g_mutex_free(servant->mutex);

	/* ------ ---------- end ------------- ------ */

	POA_BRUTUS_BrutusProxy__fini((PortableServer_Servant)servant, ev);

	g_free (servant);
}
#endif

#if !defined(_impl_BRUTUS_BrutusProxy_wrapObject_)
#define _impl_BRUTUS_BrutusProxy_wrapObject_ 1
static void
impl_BRUTUS_BrutusProxy_wrapObject(impl_POA_BRUTUS_BrutusProxy *servant,
				   const BRUTUS_BrutusTC type_code,
				   const CORBA_Object filling,
				   CORBA_Object *wrapper,
				   CORBA_Environment *ev)
{
	*wrapper = CORBA_OBJECT_NIL;

	switch (type_code) {
	case BRUTUS_BrutusCheck_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_BrutusCheck__create((BRUTUS_BrutusCheck)filling, TRUE, servant->poa, ev);
		break;
	case BRUTUS_BrutusLogOn_tc :
		break;
	case BRUTUS_BrutusSession_tc :
		break;
	case BRUTUS_IABContainer_tc :
		break;
	case BRUTUS_IABLogon_tc :
		break;
	case BRUTUS_IABProvider_tc :
		break;
	case BRUTUS_IAddrBook_tc :
		break;
	case BRUTUS_IAttach_tc :
		break;
	case BRUTUS_IDistList_tc :
		break;
	case BRUTUS_IExchangeExportChanges_tc :
		break;
	case BRUTUS_IExchangeFavorites_tc :
		break;
	case BRUTUS_IExchangeImportContentsChanges_tc :
		break;
	case BRUTUS_IExchangeImportHierarchyChanges_tc :
		break;
	case BRUTUS_IExchangeManageStore_tc :
		break;
	case BRUTUS_IExchangeModifyTable_tc :
		break;
	case BRUTUS_IExchangeRuleAction_tc :
		break;
	case BRUTUS_IMAPIAdviseSink_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_IMAPIAdviseSink__create((BRUTUS_IMAPIAdviseSink)filling, servant->poa, ev);
		break;
	case BRUTUS_IMAPIContainer_tc :
		break;
	case BRUTUS_IMAPIControl_tc :
		break;
	case BRUTUS_IMAPIFolder_tc :
		break;
	case BRUTUS_IMAPIForm_tc :
		break;
	case BRUTUS_IMAPIFormAdviseSink_tc :
		break;
	case BRUTUS_IMAPIFormContainer_tc :
		break;
	case BRUTUS_IMAPIFormFactory_tc :
		break;
	case BRUTUS_IMAPIFormInfo_tc :
		break;
	case BRUTUS_IMAPIFormMgr_tc :
		break;
	case BRUTUS_IMAPIMessageSite_tc :
		break;
	case BRUTUS_IMAPIProgress_tc :
		break;
	case BRUTUS_IMAPIProp_tc :
		break;
	case BRUTUS_IMAPISession_tc :
		break;
	case BRUTUS_IMAPIStatus_tc :
		break;
	case BRUTUS_IMAPISupport_tc :
		break;
	case BRUTUS_IMAPITable_tc :
		break;
	case BRUTUS_IMAPIViewAdviseSink_tc :
		break;
	case BRUTUS_IMAPIViewContext_tc :
		break;
	case BRUTUS_IMSLogon_tc :
		break;
	case BRUTUS_IMSProvider_tc :
		break;
	case BRUTUS_IMailUser_tc :
		break;
	case BRUTUS_IMessage_tc :
		break;
	case BRUTUS_IMsgServiceAdmin_tc :
		break;
	case BRUTUS_IMsgStore_tc :
		break;
	case BRUTUS_IPersistMessage_tc :
		break;
	case BRUTUS_IProfAdmin_tc :
		break;
	case BRUTUS_IProfSect_tc :
		break;
	case BRUTUS_IPropData_tc :
		break;
	case BRUTUS_IProviderAdmin_tc :
		break;
	case BRUTUS_ISequentialStream_tc :
		break;
	case BRUTUS_ISpoolerHook_tc :
		break;
	case BRUTUS_IStream_tc :
		break;
	case BRUTUS_ITableData_tc :
		break;
	case BRUTUS_ITnef_tc :
		break;
	case BRUTUS_IXPLogon_tc :
		break;
	case BRUTUS_IXPProvider_tc :
		break;
	case BRUTUS_MAPIFunctions_tc :
		break;
	case BRUTUS_log_server_tc :
		break;
	case BRUTUS_SRestrictionContainer_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SRestrictionContainer__create((BRUTUS_SRestrictionContainer)filling, servant->poa, ev);
		break;
	case BRUTUS_SAndRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SAndRestriction__create((BRUTUS_SAndRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_SOrRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SOrRestriction__create((BRUTUS_SOrRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_SNotRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SNotRestriction__create((BRUTUS_SNotRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_SContentRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SContentRestriction__create((BRUTUS_SContentRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_SPropertyRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SPropertyRestriction__create((BRUTUS_SPropertyRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_SComparePropsRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SComparePropsRestriction__create((BRUTUS_SComparePropsRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_SBitMaskRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SBitMaskRestriction__create((BRUTUS_SBitMaskRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_SSizeRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SSizeRestriction__create((BRUTUS_SSizeRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_SExistRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SExistRestriction__create((BRUTUS_SExistRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_SSubRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SSubRestriction__create((BRUTUS_SSubRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_SCommentRestriction_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_SCommentRestriction__create((BRUTUS_SCommentRestriction)filling, servant->poa, ev);
		break;
	case BRUTUS_IUnknown_tc :
		*wrapper = (CORBA_Object)impl_BRUTUS_IUnknown__create((BRUTUS_IUnknown)filling, servant->poa, ev);
		break;
	default :
                CORBA_exception_set_system(ev, ex_CORBA_BAD_PARAM, CORBA_COMPLETED_NO);
	};

	return;
}
#endif

#if !defined(_impl_BRUTUS_BrutusProxy_killWrapper_)
#define _impl_BRUTUS_BrutusProxy_killWrapper_ 1
static void
impl_BRUTUS_BrutusProxy_killWrapper(impl_POA_BRUTUS_BrutusProxy *servant,
				    const CORBA_Object Wrapper,
				    CORBA_Environment *ev)
{
	PortableServer_ObjectId *objid = NULL;

	objid = PortableServer_POA_reference_to_id(servant->poa, (CORBA_Object)Wrapper, ev);
	if (ORBIT2_EX(ev)) {
		PROXY_LOG("Invalid object reference");
		if (objid)
			CORBA_free(objid);
		return;
	}
	if (!objid)
		return;
		
	PortableServer_POA_deactivate_object(servant->poa, objid, ev);
	CORBA_free(objid);
	if (ORBIT2_EX(ev)) {
		PROXY_LOG("Could not deactivaet object");
		return;
	}

	CORBA_Object_release((CORBA_Object)Wrapper, ev);
	if (ORBIT2_EX(ev)) {
		PROXY_LOG("Could not release object");
		return;
	}
}
#endif

#if !defined(_impl_BRUTUS_BrutusProxy_querySession_)
#define _impl_BRUTUS_BrutusProxy_querySession_ 1
static BRUTUS_BrutusProxy_QueryState
impl_BRUTUS_BrutusProxy_querySession(impl_POA_BRUTUS_BrutusProxy *servant,
				     const CORBA_char *mailbox,
				     const CORBA_char *server,
				     BRUTUS_IMAPISession *session,
				     CORBA_Environment *ev)
{
	BRUTUS_BrutusProxy_QueryState retv = BRUTUS_BrutusProxy_mustOfferState;
	guint n = 0;

	*session = CORBA_OBJECT_NIL;

	g_mutex_lock(servant->state_mutex); 

	if (BRUTUS_BrutusProxy_offerInProgressState == servant->query_state) {
		g_mutex_unlock(servant->state_mutex); 
		return BRUTUS_BrutusProxy_offerInProgressState;
	}

	// the initial value of query_state
	if (BRUTUS_BrutusProxy_mustOfferState == servant->query_state) {
		servant->query_state = BRUTUS_BrutusProxy_offerInProgressState;
		g_mutex_unlock(servant->state_mutex); 
		return BRUTUS_BrutusProxy_mustOfferState;
	}

	// state is idleState - this will prevent others from 
	// querying before this request is resolved
	servant->query_state = BRUTUS_BrutusProxy_offerInProgressState; // will make others wait

	g_mutex_unlock(servant->state_mutex); 


	g_mutex_lock(servant->mutex); 
	if (brutus_account_find_session(servant->accounts, mailbox, server, &n)) {
		struct MAPIAccount *account = g_ptr_array_index(servant->accounts, n);
		*session = (BRUTUS_IMAPISession)CORBA_Object_duplicate(account->session, ev);
		retv = BRUTUS_BrutusProxy_queryOK;

		g_mutex_lock(servant->state_mutex); 
		servant->query_state = BRUTUS_BrutusProxy_idleState;
		g_mutex_unlock(servant->state_mutex); 
	} else {
		*session = CORBA_OBJECT_NIL;
		retv = BRUTUS_BrutusProxy_mustOfferState;

		g_mutex_lock(servant->state_mutex); 
		servant->query_state = BRUTUS_BrutusProxy_offerInProgressState;
		g_mutex_unlock(servant->state_mutex); 
	}
	g_mutex_unlock(servant->mutex);

	return retv;
}
#endif

#if !defined(_impl_BRUTUS_BrutusProxy_offerSession_)
#define _impl_BRUTUS_BrutusProxy_offerSession_ 1
static void
impl_BRUTUS_BrutusProxy_offerSession(impl_POA_BRUTUS_BrutusProxy *servant,
				     const BRUTUS_IMAPISession session,
				     const CORBA_char *mailbox,
				     const CORBA_char *server,
				     const CORBA_char *profile,
				     CORBA_Environment *ev)
{
	guint n = 0;
	CORBA_boolean cb = CORBA_TRUE;	
	struct MAPIAccount *account = NULL;

	g_mutex_lock(servant->mutex);

	/* check for NIL offer and go to idleState if so */
	cb = CORBA_Object_is_nil((CORBA_Object)session, ev);
	if (ORBIT2_EX(ev) || cb) 
		goto out;

 	/* check for valid offer - go to idleState if not*/
	BRUTUS_IMAPISession_ping(session, ev);
	if (ORBIT2_EX(ev))  // offer is invalid
		goto out;

	if (brutus_account_find_match(servant->accounts, mailbox, server, &n)) {
		account = g_ptr_array_index(servant->accounts, n);

		// force profile update
		if (account->profile)
			CORBA_free(account->profile);
		account->profile = CORBA_string_dup(profile);

		account->session = (BRUTUS_IMAPISession)CORBA_Object_duplicate(session, ev);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Could not duplicate object");
			account->session = CORBA_OBJECT_NIL;
			brutus_store_accounts(servant->accounts);
			goto out;
		}

		account->instance_id = BRUTUS_IMAPISession_addLifeline(account->session, servant->lifeline, ev);
		if (ORBIT2_EX(ev)) {// it will eventually get deleted in a failed brutus_account_find_session()
			PRINT_ORBIT2_EX(ev);
			PROXY_LOG("Could not add lifeline");
		}
	} else { // new account
		account = g_new(struct MAPIAccount, 1);
		if (!account)
			goto out;

		account->session = (BRUTUS_IMAPISession)CORBA_Object_duplicate(session, ev);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Could not duplicate object");
			account->session = CORBA_OBJECT_NIL;
		}

		account->mailbox = CORBA_string_dup(mailbox);
		account->server = CORBA_string_dup(server);
		account->profile = CORBA_string_dup(profile);

		g_ptr_array_add(servant->accounts, account);
	}
	brutus_store_accounts(servant->accounts);

out:
	g_mutex_unlock(servant->mutex);

	g_mutex_lock(servant->state_mutex); 
	servant->query_state = BRUTUS_BrutusProxy_idleState;
	g_mutex_unlock(servant->state_mutex); 

	return;
}
#endif

#if !defined(_impl_BRUTUS_BrutusProxy_getProfile_)
#define _impl_BRUTUS_BrutusProxy_getProfile_ 1
static void
impl_BRUTUS_BrutusProxy_getProfile(impl_POA_BRUTUS_BrutusProxy *servant,
				   const CORBA_char *mailbox,
				   const CORBA_char *server,
				   CORBA_string *profile,
				   CORBA_Environment *ev)
{
	guint n = 0;

	g_mutex_lock(servant->mutex);

	if (brutus_account_find_match(servant->accounts, mailbox, server, &n)) {
		struct MAPIAccount *account = g_ptr_array_index(servant->accounts, n);
		*profile = CORBA_string_dup(account->profile);
	} else 
		*profile = CORBA_string_dup("");

	g_mutex_unlock(servant->mutex);
}
#endif

