/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Implementation file for a Brutus proxy daemon.
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <signal.h>
#include <dirent.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gstdio.h>

#include "brutusd.h"
#include "log_macros.h"
FILE *log_file = NULL; // global log stream
#include "brutus-proxy_impl.c"

static gboolean proxy_down = FALSE;
static CORBA_ORB global_orb = CORBA_OBJECT_NIL;

static inline size_t
corba_strlen(const CORBA_char *s)
{
	size_t l = 0;

	for (; (CORBA_char)'\0' != *s; s++, l++)
		;

	return l;
}

// shutdown global ORB
static void
orb_shutdown(int sig)
{

	if (CORBA_OBJECT_NIL != global_orb) {
		CORBA_Environment ev[1];

		CORBA_exception_init(ev);
		CORBA_ORB_shutdown(global_orb, TRUE, ev);
		CORBA_exception_free(ev);
	}
	global_orb = CORBA_OBJECT_NIL;
}

static void
wait_for_child(int sig)
{
	int status = 0;

	while (TRUE) {
		wait(&status);
		if (WIFCONTINUED(status))
			continue;

		proxy_down = TRUE;
		break;
	}

	// kill self
	kill(0, SIGTERM);
}

static CORBA_ORB
create_orb(CORBA_Environment *ev)
{
	CORBA_ORB orb = CORBA_OBJECT_NIL;
	int init_argc = 5;
	char **init_argv = (char**)malloc(sizeof(char*) * (init_argc));
	if (!init_argv)
		return CORBA_OBJECT_NIL;

	memset((void*)init_argv, 0, init_argc);
	init_argv[0] = strdup("DUMMY_ARGV0");

	// IPv4 - needed to interoperate with TAO
	init_argv[1] = "--ORBIIOPIPv4=1";

	// IPv6 - could be needed to interoperate with TAO
	init_argv[2] = "--ORBIIOPIPv6=1";

	// Explicitly force ORBit2 to be non-local
	init_argv[3] = "--ORBLocalOnly=0";

	// Force ORBit2 to use the IP address in the all generated IORs
	init_argv[4] = "--ORBNetID=ipaddr";

	// initialize the ORB
	orb = CORBA_ORB_init(&init_argc, init_argv, "proxy_orb_orbit-io-thread", ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		orb = CORBA_OBJECT_NIL;
	}

	free(init_argv[0]);
	free(init_argv);

	return orb;
}

static gboolean
object_ref_to_file(CORBA_ORB orb,
		   CORBA_Object servant,
		   gchar *filename,
		   CORBA_Environment *ev)
{
	gboolean retv = FALSE;
	CORBA_char *objref = NULL;
	int file_fd = -1;
	FILE *file = NULL;
	size_t c = 0;

	file_fd = g_open(filename, O_CREAT | O_WRONLY | O_SYNC | O_TRUNC, S_IRUSR | S_IWUSR);
	if (-1 == file_fd)
		return FALSE;
	if (lockf(file_fd, F_TLOCK, 0))
		goto out_no_lock;

	file = fdopen(file_fd, "w");
	if (!file)
		goto out;

	objref = CORBA_ORB_object_to_string(orb, servant, ev);
	if (ORBIT2_EX(ev) || !objref)
		goto out;

	// must work even if (sizeof(char) != sizeof(CORBA_char))
	c = fwrite((const void*)objref, sizeof(CORBA_char), corba_strlen(objref)+1, file);
	if (c == (1 + corba_strlen(objref)))
		retv = TRUE;

out:
	lockf(file_fd, F_ULOCK, 0);
	CORBA_free(objref);

out_no_lock:
	if (file)
		fclose(file);
	else
		close(file_fd);

	return retv;
}

static int
wait_for_fork(void)
{
	int count = 10000;
	int retv = -1;
	int fd = -1;
	struct timespec ts = {
		.tv_sec = 0,
		.tv_nsec = 100000000,
	};
	struct stat st = { 0 };

	gchar *path = g_strconcat(getenv("HOME"), "/", BRUTUSD_REF_FILE_NAME, NULL);
	if (!path)
		return EXIT_FAILURE;

	// exit when data has been written to the ior file and no more data is being written.
	while (count--) {
		nanosleep(&ts, NULL);

		retv = stat(path, &st);
		if ((!retv) && (st.st_size)) {
			fd = g_open(path, O_RDONLY, 0);
			lockf(fd, F_LOCK, 0);  // wait for completion
			lockf(fd, F_ULOCK, 0);
			close(fd);
			break;
		}
	}
	g_free(path);

	if ((retv || (!st.st_size)))
		return EXIT_FAILURE;
	else
		return EXIT_SUCCESS;
}

// has evolution (or anything else for that matter) locked the lock file?
static gboolean
proceed_with_execution(void)
{
	gboolean retv = FALSE;
	char *lock_file_path = NULL;
	int fd = -1;

	lock_file_path = g_strconcat(getenv("HOME"), "/", BRUTUSD_LOCK_FILE_NAME, NULL);
	if (!lock_file_path)
		return FALSE;

	fd = g_open(lock_file_path, O_CREAT | O_WRONLY | O_SYNC, S_IRUSR | S_IWUSR);
	g_free(lock_file_path);
	if (-1 == fd)
		return FALSE;

	if (lockf(fd, F_TLOCK, 0)) // returns TRUE if Evolution has locked the file
		retv = TRUE;
	lockf(fd, F_ULOCK, 0); // be nice...
	close(fd);

	return retv;
}

// will return when the lock is released on the lock file
static void
wait_for_evolution(void *data)
{
	char *lock_file_path = NULL;
	int fd = -1;
	pid_t pid = *((pid_t*)data);

	lock_file_path = g_strconcat(getenv("HOME"), "/", BRUTUSD_LOCK_FILE_NAME, NULL);
	if (!lock_file_path)
		return;

	fd = g_open(lock_file_path, O_CREAT | O_WRONLY | O_SYNC, S_IRUSR | S_IWUSR);
	g_free(lock_file_path);
	if (-1 == fd)
		return;

	lockf(fd, F_LOCK, 0);
	lockf(fd, F_ULOCK, 0); // be nice...
	close(fd);

	// kill brutusd
	if (!proxy_down)
		kill(pid, SIGTERM);

	return;
}

static void
set_proxy_down(int sig)
{
	proxy_down = TRUE;

	// kill self
	kill(0, SIGTERM);
}


int
main(int argc, char *argv[])
{
	PortableServer_POA root_poa = CORBA_OBJECT_NIL;
	BRUTUS_BrutusProxy proxy = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];
	int retv = EXIT_FAILURE;
	pid_t pid;
	pid_t pid_of_proxy;
	pid_t sid;
	int ref_file_fd = -1;
	int pid_file_fd = -1;
	FILE *pid_file = NULL;
	gchar *pid_file_path = NULL;
	gchar *ref_file_path = NULL;

	ref_file_path = g_strconcat(getenv("HOME"), "/", BRUTUSD_REF_FILE_NAME, NULL);
	if (!ref_file_path)
		exit(EXIT_FAILURE);

	/* create application directory */
	{
		gchar *dir_path = g_strconcat(getenv("HOME"), "/", BRUTUSD_DIR_NAME, NULL);
		int retv = g_mkdir_with_parents(dir_path, 0700);
		g_free(dir_path);

		if ((retv) && (EEXIST != errno))
			exit(EXIT_FAILURE);
	}

	/* create ref file */
	{
		ref_file_fd = g_open(ref_file_path, O_CREAT | O_WRONLY | O_SYNC | O_TRUNC, S_IRUSR | S_IWUSR);
		if (-1 == ref_file_fd)
			exit(EXIT_FAILURE);
		close(ref_file_fd);
	}

	/* reset accounts file? */
	if ((2 <= argc)) {
		if (!strcmp("-reset", argv[1])) {
			gchar *accounts_file_path = g_strconcat(getenv("HOME"), "/", BRUTUSD_ACCOUNTS_FILE_NAME, NULL);
			if (!accounts_file_path)
				exit(EXIT_FAILURE);
			unlink(accounts_file_path);
			exit(EXIT_SUCCESS);
		}
	}

	if (!proceed_with_execution())
		exit(EXIT_FAILURE); // the daemon was not started by Evolution or Evolution failed to lock to lock the file...

	/* Fork off the parent process to create the lock listener */
	pid = fork();
	if (pid < 0) {
		g_free(ref_file_path);
		exit(EXIT_FAILURE);
	}
	if (pid > 0) {
		g_free(ref_file_path);
		exit(wait_for_fork()); // ior file will be created by proxy daemon in second fork
	}

	/*
	 * We are now the lock listener...
	 */

	/* Create SID for lock listener */
	sid = setsid();
	if (sid < 0)
		goto out;

	/* Change the working directory */
	if ((chdir("/")) < 0)
		goto out;

	/* Fork off the parent process to create the proxy daemon */
	pid_of_proxy = fork();
	if (pid_of_proxy < 0)
		exit(EXIT_FAILURE);
	if (pid_of_proxy > 0) {
		signal(SIGUSR1, set_proxy_down);
		signal(SIGCHLD, wait_for_child);
		goto exit_listener; // we are the listener
	}
	pid = getpid(); // we are the proxy daemon

	/*
	 * We are now the proxy daemon...
	 */

	/* Change the working directory */
	if ((chdir("/")) < 0)
		goto out;

	// handle signals
	signal(SIGUSR1, SIG_IGN);
	signal(SIGINT,  orb_shutdown);
	signal(SIGTERM, orb_shutdown);
	signal(SIGSEGV, orb_shutdown);
	signal(SIGUSR2, orb_shutdown);
	signal(SIGILL, orb_shutdown);
	signal(SIGQUIT, orb_shutdown);

	/* Close standard file descriptors */
	close(STDIN_FILENO);
	close(STDOUT_FILENO);
	close(STDERR_FILENO);

	/* reset the file mode mask */
	umask(0);

	/* write pid to file */
	{
		pid_file_path = g_strconcat(getenv("HOME"), "/", BRUTUSD_PID_FILE_NAME, NULL);
		if (!pid_file_path)
			exit(EXIT_FAILURE);

		pid_file_fd = g_open(pid_file_path, O_CREAT | O_WRONLY | O_SYNC, S_IRUSR | S_IWUSR);
		if (-1 == pid_file_fd) {
			g_free(pid_file_path);
			exit(EXIT_FAILURE);
		}
		if (flock(pid_file_fd, LOCK_EX | LOCK_NB)) {
			close(pid_file_fd);
			g_free(pid_file_path);
			exit(EXIT_SUCCESS);
		}

		/* we now have a cooperative exclusive lock on the file */
		if (ftruncate(pid_file_fd, 0)) {
			close(pid_file_fd);
			g_free(pid_file_path);
			exit(EXIT_FAILURE);
		}

		/* write pid */
		pid_file = fdopen(pid_file_fd, "w");
		g_fprintf(pid_file, "%d", pid);
		fflush(pid_file);
	}

	/* open log file */
	if (getenv("BRUTUSD_LOG")) {
		int log_file_fd = -1;
		gchar *log_file_path = g_strconcat(getenv("HOME"), "/", BRUTUSD_LOG_FILE_NAME, NULL);
		if (!log_file_path)
			goto out;

		log_file_fd = g_open(log_file_path, O_CREAT | O_WRONLY | O_SYNC | O_TRUNC, S_IRUSR | S_IWUSR);
		g_free(log_file_path);

		if (-1 == log_file_fd)
			goto out;

		log_file = fdopen(log_file_fd, "w");
	}

	/*
	  Initialization goes here...
	*/

	CORBA_exception_init(ev);

	global_orb = create_orb(ev);
	if (ORBIT2_EX(ev)) {
		PROXY_LOG("ORB creation failed - exiting");
		goto out;
	}
	PROXY_LOG("ORB created");

	{
		PortableServer_POAManager poa_manager = CORBA_OBJECT_NIL;

		root_poa = (PortableServer_POA) CORBA_ORB_resolve_initial_references(global_orb, "RootPOA", ev);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Could not resolve RootPOA - exiting");
			goto out;
		}

		poa_manager = PortableServer_POA__get_the_POAManager(root_poa, ev);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Could not get the POAManager - exiting");
			goto out;
		}

		PortableServer_POAManager_activate(poa_manager, ev);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Could not activate RootPOA - exiting");
			goto out;
		}

		CORBA_Object_release((CORBA_Object) poa_manager, ev);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Exception caught when releasing POAManager - exiting");
			goto out;
		}
	}

	/* create proxy servant */
	proxy = impl_BRUTUS_BrutusProxy__create(root_poa, ev);
	if (ORBIT2_EX(ev)) {
		PROXY_LOG("Exception caught when creating BrutusProxy object - exiting");
		goto out;
	}
	if (CORBA_OBJECT_NIL == proxy) {
		PROXY_LOG("BrutusProxy object is NIL - exiting");
		goto out;
	}

	/* write ior to file */
	{
		if (!ref_file_path) {
			PROXY_LOG("No memory - exiting");
			goto out;
		}

		object_ref_to_file(global_orb, proxy, ref_file_path, ev);
	}

	/* loop until killed */
	PROXY_LOG("starting");
	CORBA_ORB_run(global_orb, ev);
	PROXY_LOG("shutting down...");

	/* cleanup */
	{
		PortableServer_ObjectId *objid = NULL;

		objid = PortableServer_POA_reference_to_id(root_poa, (CORBA_Object)proxy, ev);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Exception caught from reference_to_id() - exiting");
			if (objid)
				CORBA_free(objid);
			goto out;
		}

		PortableServer_POA_deactivate_object(root_poa, objid, ev);
		CORBA_free(objid);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Exception caught from deactivate_object() - exiting");
			goto out;
		}

		CORBA_Object_release((CORBA_Object)proxy, ev);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Exception caught from release() - exiting");
			goto out;
		}

		PortableServer_POA_destroy(root_poa, TRUE, FALSE, ev);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Exception caught from destroy() - exiting");
			goto out;
		}

		CORBA_Object_release((CORBA_Object)root_poa, ev);
		if (ORBIT2_EX(ev)) {
			PROXY_LOG("Exception caught from release() - exiting");
			goto out;
		}

		/* ORB: tear down the ORB */
		if (CORBA_OBJECT_NIL != global_orb) {
			CORBA_ORB_destroy(global_orb, ev);
			if (ORBIT2_EX(ev)) {
				PROXY_LOG("Exception caught from destroy() - exiting");
				goto out;
			}
			CORBA_Object_release((CORBA_Object)global_orb, ev);
			if (ORBIT2_EX(ev)) {
				PROXY_LOG("Exception caught from release() - exiting");
				goto out;
			}
		}
	}
	retv = EXIT_SUCCESS;

out:
	CORBA_exception_free(ev);

	PROXY_LOG("shutdown complete");
	if (log_file)
		fclose(log_file);

	if (pid_file) {
		flock(pid_file_fd, LOCK_UN | LOCK_NB);
		unlink(pid_file_path);
		g_free(pid_file_path);
		fclose(pid_file);
	}
	if (ref_file_path) {
		if (truncate(ref_file_path, 0))
			retv = EXIT_FAILURE;
	}
	g_free(ref_file_path);

	// signal the listener that we are down
	kill(0, SIGUSR1);

	exit(retv);

exit_listener:
	g_free(ref_file_path);

	wait_for_evolution((void*)&pid_of_proxy);

	exit(EXIT_SUCCESS);
}

