/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus e-cal backend class.
 *
 *    Authors:
 *        Jules Colding <colding@omesc.com>
 *
 *    Copyright (C) 2006-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* system includes */
#include <glib.h>
#include <glib/gstdio.h>
#include <glib/gi18n-lib.h>
#include <uuid/uuid.h>
#include <string.h>
#include <fcntl.h>
#include <libedataserver/e-iconv.h>
#include <libedataserver/md5-utils.h>
#include <libedataserver/e-xml-hash-utils.h>
#include <libecal/e-cal-time-util.h>
#include <libedata-cal/e-cal-backend-util.h>

/* pure brutus includes */
#include <idl_output/BrutusLogOn.h>
#include <idl_output/return_codes.h>
#include <idl_output/defines.h>
#include <idl_output/types.h>
#include <idl_output/uid.h>
#include <servant_impl/BrutusCheck_impl.c>

/* evolution-brutus includes */
#include <session/brutusd.h>
#include <server/brutus.h>
#include <server/brutus_util.h>
#include <server/brutus_cpid.h>
#include <server/brutus_conv.h>
#include <server/brutus_corba.h>
#include <server/brutus_mapi.h>
#include <server/brutus_string.h>

/* e-cal brutus includes */
#include "e-cal-backend-brutus.h"

#ifdef UTC_TIME_FORMAT_STRING
#undef UTC_TIME_FORMAT_STRING
#endif
#define UTC_TIME_FORMAT_STRING "%Y-%m-%dT%H:%M:%SZ"

#ifdef X_PROP_RECORD_KEY
#undef X_PROP_RECORD_KEY
#endif
#define X_PROP_RECORD_KEY "X-PR_RECORD_KEY"

/* auto-check timeout interval in minutes */
#ifdef AUTO_CHECK_TIMEOUT_DEFAULT
#undef AUTO_CHECK_TIMEOUT_DEFAULT
#endif
#define AUTO_CHECK_TIMEOUT_DEFAULT 10

static ECalBackendClass *parent_class = NULL;

// GObject methods
static void e_cal_backend_brutus_dispose(GObject *object);
static void e_cal_backend_brutus_finalize(GObject *object);

// ECalBackendSync methods
static ECalBackendSyncStatus e_cal_backend_brutus_is_read_only_sync(ECalBackendSync *backend,
								    EDataCal *cal,
								    gboolean *read_only);

static ECalBackendSyncStatus e_cal_backend_brutus_get_cal_address_sync(ECalBackendSync *backend,
								       EDataCal *cal,
								       char **address);

static ECalBackendSyncStatus e_cal_backend_brutus_get_alarm_email_address_sync(ECalBackendSync *backend,
									       EDataCal *cal,
									       char **address);

static ECalBackendSyncStatus e_cal_backend_brutus_get_ldap_attribute_sync(ECalBackendSync *backend,
									  EDataCal *cal,
									  char **attribute);

static ECalBackendSyncStatus e_cal_backend_brutus_get_static_capabilities_sync(ECalBackendSync *backend,
									       EDataCal *cal,
									       char **capabilities);

static ECalBackendSyncStatus e_cal_backend_brutus_open_sync(ECalBackendSync *backend,
							    EDataCal *cal,
							    gboolean only_if_exists,
							    const char *username,
							    const char *password);

static ECalBackendSyncStatus e_cal_backend_brutus_remove_sync(ECalBackendSync *backend,
							      EDataCal *cal);

static ECalBackendSyncStatus e_cal_backend_brutus_create_object_sync(ECalBackendSync *backend,
								     EDataCal *cal,
								     char **calobj,
								     char **uid);

static ECalBackendSyncStatus e_cal_backend_brutus_modify_object_sync(ECalBackendSync *backend,
								     EDataCal *cal,
								     const char *calobj,
								     CalObjModType mod,
								     char **old_object,
								     char **new_object);

static ECalBackendSyncStatus e_cal_backend_brutus_remove_object_sync(ECalBackendSync *backend,
								     EDataCal *cal,
								     const char *uid,
								     const char *rid,
								     CalObjModType mod,
								     char **old_object,
								     char **object);

static ECalBackendSyncStatus e_cal_backend_brutus_discard_alarm_sync(ECalBackendSync *backend,
								     EDataCal *cal,
								     const char *uid,
								     const char *auid);

static ECalBackendSyncStatus e_cal_backend_brutus_receive_objects_sync(ECalBackendSync *backend,
								       EDataCal *cal,
								       const char *calobj);

static ECalBackendSyncStatus e_cal_backend_brutus_send_objects_sync(ECalBackendSync *backend,
								    EDataCal *cal,
								    const char *calobj,
								    GList **users,
								    char **modified_calobj);

static ECalBackendSyncStatus e_cal_backend_brutus_get_default_object_sync(ECalBackendSync *backend,
									  EDataCal *cal,
									  char **object);

static ECalBackendSyncStatus e_cal_backend_brutus_get_object_sync(ECalBackendSync *backend,
								  EDataCal *cal,
								  const char *uid,
								  const char *rid,
								  char **object);

static ECalBackendSyncStatus e_cal_backend_brutus_get_object_list_sync(ECalBackendSync *backend,
								       EDataCal *cal,
								       const char *sexp,
								       GList **objects);

static ECalBackendSyncStatus e_cal_backend_brutus_get_attachment_list_sync(ECalBackendSync *backend,
									   EDataCal *cal,
									   const char *uid,
									   const char *rid,
									   GSList **attachments);

static ECalBackendSyncStatus e_cal_backend_brutus_get_timezone_sync(ECalBackendSync *backend,
								    EDataCal *cal,
								    const char *tzid,
								    char **object);

static ECalBackendSyncStatus e_cal_backend_brutus_add_timezone_sync(ECalBackendSync *backend,
								    EDataCal *cal,
								    const char *tzobj);

static ECalBackendSyncStatus e_cal_backend_brutus_set_default_timezone_sync(ECalBackendSync *backend,
									    EDataCal *cal,
									    const char *tzid);

static ECalBackendSyncStatus e_cal_backend_brutus_get_changes_sync(ECalBackendSync *backend,
								   EDataCal *cal,
								   const char *change_id,
								   GList **adds,
								   GList **modifies,
								   GList **deletes);

static ECalBackendSyncStatus e_cal_backend_brutus_get_freebusy_sync(ECalBackendSync *backend,
								    EDataCal *cal,
								    GList *users,
								    time_t start,
								    time_t end,
								    GList **freebusy);

static gboolean e_cal_backend_brutus_is_loaded_sync(ECalBackend *backend);

static void e_cal_backend_brutus_start_query_sync(ECalBackend *backend,
						  EDataCalView *query);

static icaltimezone *e_cal_backend_brutus_internal_get_default_timezone_sync(ECalBackend *backend);

static icaltimezone *e_cal_backend_brutus_internal_get_timezone_sync(ECalBackend *backend,
								     const char *tzid);

static CalMode e_cal_backend_brutus_get_mode_sync(ECalBackend *backend);

static void e_cal_backend_brutus_set_mode_sync(ECalBackend *backend,
					       CalMode mode);

/*******************************
 *   static helper functions   *
 ******************************/

static void
brutus_adjust_to_server_time_zone(time_t *tt,
				  long server_time_zone_offset)
{
	*tt += server_time_zone_offset * 60;
}

/* static void */
/* recurid_to_filetime(const ECalComponentRange *recurid, */
/*		    const long server_time_zone_offset, */
/*		    BRUTUS_FILETIME * const ft) */
/* { */
/*	time_t tt = icaltime_as_timet_with_zone(*(recurid->datetime.value), NULL); */
/* #ifdef BRUTUS_DEBUG */
/*	char server_utc_time_str[100] = { '\0' }; */
/*	brutus_time_t_to_string(tt, */
/*				sizeof(server_utc_time_str), */
/*				UTC_TIME_FORMAT_STRING, */
/*				server_utc_time_str); */
/*	d(server_utc_time_str); */
/* #endif */
/*	brutus_adjust_to_server_time_zone(&tt, server_time_zone_offset); */
/*	brutus_time_t_to_filetime(tt, ft); */
/* } */

/* static void */
/* filetime_to_recurid(const BRUTUS_FILETIME *ft, */
/*		    ECalComponentRange * const recurid) */
/* { */
/*	time_t tt = brutus_filetime_to_time_t(*ft); */
/*	struct icaltimetype ical_time_utc = icaltime_from_timet_with_zone(tt, 0, icaltimezone_get_utc_timezone()); */

/*	recurid->datetime.value = g_new0(struct icaltimetype, 1); */
/*	memcpy((void*)recurid->datetime.value, (void*) &ical_time_utc, sizeof(struct icaltimetype)); */

/*	recurid->datetime.tzid = g_strdup("UTC"); */
/*	recurid->datetime.value->is_date = FALSE; */
/*	recurid->type = E_CAL_COMPONENT_RANGE_SINGLE; */
/* } */

static gboolean
brutus_orb_init(ECalBackendBrutusPrivate *priv,
		const char *brutus_server,
		const guint16 brutus_port,
		const icalcomponent_kind kind)
{
	gboolean retv = FALSE;
	static guint orb_num = 0;
	char *orb_name = NULL;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	// Hmm... I am unsure as to whether the ECal sub system will invoke the same backend
	// object more than once, so I put this here to avoid identical ORB names for certainty.
	orb_num++;
	orb_name = g_strdup_printf("%d-%s",
				   orb_num,
				   (ICAL_VEVENT_COMPONENT == kind) ? "Brutus-EVENTS-ORB_orbit-io-thread" : "Brutus-TODOS-ORB_orbit-io-thread");
	if (!orb_name) {
		d("No memory");
		goto out;
	}

	d("Invoking ORB_Init()");
	priv->orb = brutus_create_orb(0,
				      NULL,
				      priv->use_ssliop,
				      "BrutusLogOn",
				      BRUTUS_BrutusLogOn_IOR_TABLE_KEY,
				      brutus_server,
				      brutus_port,
				      orb_name,
				      ev);
	g_free(orb_name);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	cb = CORBA_Object_is_nil((CORBA_Object)priv->orb, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb) {
		d("Could not initialize the ORB");
		goto out;
	}

	d("Getting root POA");
	priv->root_poa = brutus_get_root_poa(priv->orb, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	cb = CORBA_Object_is_nil((CORBA_Object)priv->root_poa, ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (cb) {
		d("Could not get the root POA");
		goto out;
	}
	retv = TRUE;

out:
	CORBA_exception_free(ev);

	return retv;
}

/* only invoked from brutus_(dis)connect() and from finalize() when the backend is locked */
static void
brutus_mapi_shutdown(ECalBackendBrutus *brutus_backend)
{
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	d("");
	// the server-side MAPI folder
	cb = CORBA_Object_is_nil(priv->mapi_folder, ev);
	if (!ORBIT2_EX(ev) && !cb) {
		BRUTUS_IMAPIFolder_Destroy(priv->mapi_folder, 0, ev);
		PRINT_IF_ORBIT2_EX(ev);

		CORBA_Object_release(priv->mapi_folder, ev);
	}
	priv->mapi_folder = CORBA_OBJECT_NIL;

	/* mapi session and instance ID */
	cb = CORBA_Object_is_nil(priv->mapi_session, ev);
	if (!ORBIT2_EX(ev) && !cb) {
		BRUTUS_IMAPISession_Destroy(priv->mapi_session, priv->instance_id, ev);
		PRINT_IF_ORBIT2_EX(ev);

		CORBA_Object_release(priv->mapi_session, ev);
	}
	priv->mapi_session = CORBA_OBJECT_NIL;
	priv->instance_id = 0;

	CORBA_exception_free(ev);
}

static void
brutus_free_lifeline(BRUTUS_BrutusProxy proxy,
		     struct BrutusCheckProxy *lifeline)
{
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_Environment ev[1];
	CORBA_exception_init(ev);

	cb = CORBA_Object_is_nil((CORBA_Object)lifeline->wrapper, ev);
	if (!ORBIT2_EX(ev) && !cb) {
		BRUTUS_BrutusProxy_killWrapper(proxy, lifeline->wrapper, ev);
		CORBA_Object_release((CORBA_Object)lifeline->wrapper, ev);
	}

	cb = CORBA_Object_is_nil((CORBA_Object)lifeline->filling, ev);
	if (!ORBIT2_EX(ev) && !cb)
		CORBA_Object_release((CORBA_Object)lifeline->filling, ev);

	CORBA_exception_free(ev);
	lifeline->wrapper = CORBA_OBJECT_NIL;
	lifeline->filling = CORBA_OBJECT_NIL;
}

/* the backend must be locked when this function is invoked */
static gboolean
brutus_connect(ECalBackendBrutus *brutus_backend)
{
	static gboolean got_time_zones = FALSE;
	char *prompt = NULL;
	char *err_prompt = NULL;
	char *windows_domain = NULL;
	char *mailbox_dn = NULL;
	char *mailbox_email = NULL;
	ESource *source = NULL;
	gboolean retv = FALSE;
	BRUTUS_OUTLOOK_FOLDER mapi_folder_kind = BRUTUS_OUTLOOK_NONE;
	icalcomponent_kind kind;
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	BRUTUS_BrutusLogOn brutus_logon = CORBA_OBJECT_NIL;
	BRUTUS_BrutusLogOn_BrutusVersion brutus_server_version = { 0 };
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	/* check connection status */
	retv = brutus_is_connected(priv->brutus_server, priv->mapi_session);
	if (retv)
		goto out; // connected
	else
		brutus_mapi_shutdown(brutus_backend); // reset MAPI

#ifndef HAVE_ORBIT2_TIMEOUT
	if (!brutus_server_ping(priv->brutus_server)) {
		retv = FALSE;
		goto exit; // no reason to even try...
	}
#endif
	source = e_cal_backend_get_source(E_CAL_BACKEND(brutus_backend));
	if (!source)
		goto out;

	windows_domain = brutus_strip_trailing_and_leading_whitespace(e_source_get_property(source, "windows_domain"));
	if (!windows_domain)
		windows_domain = g_strdup("");
	mailbox_dn = brutus_strip_trailing_and_leading_whitespace(e_source_get_property(source, "exchange_mailbox_dn"));
	mailbox_email = brutus_strip_trailing_and_leading_whitespace(e_source_get_property(source, "exchange_mailbox_email"));

	/* we are not connected - proceed */
	d("Attempting to query a session from the proxy");
	{
		BRUTUS_BrutusProxy_QueryState query_state = BRUTUS_BrutusProxy_offerInProgressState;
		struct timespec t = {
			.tv_sec = 0,
			.tv_nsec = 10000000 // 0.01 second
		};

		d(e_source_get_property(source, "exchange_mailbox"));
		d(e_source_get_property(source, "exchange_mailbox_dn"));
		d(e_source_get_property(source, "exchange_mailbox_email"));
		d(e_source_get_property(source, "exchange_server"));
		do {
			query_state = BRUTUS_BrutusProxy_querySession(priv->brutus_proxy,
								      e_source_get_property(source, "exchange_mailbox"),
								      e_source_get_property(source, "exchange_server"),
								      &priv->mapi_session,
								      ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto proceed;
			}
			switch (query_state) {
			case BRUTUS_BrutusProxy_mustOfferState :
				goto proceed;
			case BRUTUS_BrutusProxy_offerInProgressState :
				nanosleep(&t, NULL); // sleep for 1 second
				break;
			case BRUTUS_BrutusProxy_queryOK : // got session
				break;
			default:
				d("Unknown query response");
				goto proceed;
			}
		} while (BRUTUS_BrutusProxy_queryOK != query_state);

		// check session for validity
		cb = CORBA_Object_is_nil((CORBA_Object)priv->mapi_session, ev);
		if (!ORBIT2_EX(ev) && !cb) {
			BRUTUS_IMAPISession_ping(priv->mapi_session, ev);
			if (!ORBIT2_EX(ev)) {
				priv->instance_id = BRUTUS_IMAPISession_addLifeline(priv->mapi_session, priv->lifeline.wrapper, ev);
				if (ORBIT2_EX(ev)) {
					d("Could not add lifeline");
					goto proceed;
				}

				d("Query successful");
				retv = TRUE; // we are connected, no query lock to release
				goto out_get_folder;
			} else { // could not ping
				CORBA_Object_release((CORBA_Object)priv->mapi_session, ev);
				priv->mapi_session = CORBA_OBJECT_NIL;
			}
		}
	}

proceed:
	d("Could not get session from proxy, must log on explicitly");
	BRUTUS_BrutusProxy_getProfile(priv->brutus_proxy,
				      e_source_get_property(source, "exchange_mailbox"),
				      e_source_get_property(source, "exchange_server"),
				      &priv->mapi_profile,
				      ev);
	if (ORBIT2_EX(ev))
		PRINT_ORBIT2_EX(ev);
	else if (!strlen(priv->mapi_profile)) {
		d("No stored MAPI Profile - generating a new one");
		char guid_str[37] = { 0 };
		uuid_t guid = { 0 };

		CORBA_free(priv->mapi_profile);

		uuid_generate(guid);
		uuid_unparse(guid, guid_str);
		priv->mapi_profile = CORBA_string_dup(guid_str);
	}
	dfmt("Using this MAPI Profile: \"%s\"", priv->mapi_profile);

	d("Getting reference to BrutusLogOn object");
	{
		brutus_logon = (BRUTUS_BrutusLogOn)CORBA_ORB_resolve_initial_references(priv->orb, "BrutusLogOn", ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			brutus_logon = CORBA_OBJECT_NIL;
		}

		cb = CORBA_Object_is_nil(brutus_logon, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			cb = TRUE;
		}
		if (cb) {
			d("Could not resolve (resolve_initial_references) BrutusLogOn, falling back to corbaloc...");
			brutus_logon = (BRUTUS_BrutusLogOn)CORBA_ORB_string_to_object(priv->orb, priv->brutus_logon_str, ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto out_release_lock;
			}

			cb = CORBA_Object_is_nil(brutus_logon, ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				cb = TRUE;
			}
			if (cb) {
				d("Could not resolve (string_to_object) BrutusLogOn, NIL object returned");
				goto out_release_lock;
			} else
				d("BrutusLogOn resolved");
		}
		d("BrutusLogOn reference has been acquired");

		BRUTUS_BrutusLogOn_GetVersion(brutus_logon, &brutus_server_version, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_release_lock;
		}
		if (!brutus_server_version_OK((int)brutus_server_version.major,
					      (int)brutus_server_version.minor,
					      (int)brutus_server_version.micro,
					      (int)brutus_server_version.release_tag)) {
			e_cal_backend_notify_error(E_CAL_BACKEND(brutus_backend), _("Unsupported version of Brutus Server. You must have Brutus Server version "BRUTUS_SERVER_REQ_VERSION_STRING));
			d("Unsupported version of Brutus Server. You must have Brutus Server version "BRUTUS_SERVER_REQ_VERSION_STRING);
			goto out_release_lock;
		}
		d("Brutus Server version check completed");
	}
	prompt = g_strdup_printf(_("Please enter Windows domain password for \"%s\\%s\" (Exchange mailbox: %s)"), 
				 windows_domain,
				 e_source_get_property(source, "windows_user"),
				 e_source_get_property(source, "exchange_mailbox"));
	err_prompt = g_strdup_printf(_("Authentication attempt failed - Please reenter Windows domain password for \"%s\\%s\" (Exchange mailbox: %s)"), 
				     windows_domain, 
				     e_source_get_property(source, "windows_user"),
				     e_source_get_property(source, "exchange_mailbox"));
	if (!prompt || !err_prompt)
		d("No memory");
	else {
		retv = brutus_logon_loop(brutus_logon,
					 priv->lifeline.wrapper,
					 &priv->mapi_session,
					 &priv->instance_id,
					 &priv->mapi_profile,
					 prompt,
					 err_prompt,
					 e_source_get_property(source, "windows_user"),
					 windows_domain,
					 e_source_get_property(source, "exchange_mailbox"),
					 mailbox_dn,
					 mailbox_email,
					 e_source_get_property(source, "exchange_server"));
	}

out_release_lock:
	if (retv) {
		d("Offering IMAPISession to Brutus proxy");
		BRUTUS_BrutusProxy_offerSession(priv->brutus_proxy,
						priv->mapi_session,
						e_source_get_property(source, "exchange_mailbox"),
						e_source_get_property(source, "exchange_server"),
						priv->mapi_profile,
						ev);
		PRINT_IF_ORBIT2_EX(ev);
	} else {
		d("No IMAPISession offer - releasing query lock");
		BRUTUS_BrutusProxy_offerSession(priv->brutus_proxy,
						CORBA_OBJECT_NIL,
						"",
						"",
						"",
						ev);
		PRINT_IF_ORBIT2_EX(ev);
	}

out_get_folder:
	// get the right mapi folder
	if (retv) {
		retv = FALSE;

		kind = e_cal_backend_get_kind(E_CAL_BACKEND(brutus_backend));
		switch (kind) {
		case ICAL_VEVENT_COMPONENT :
			mapi_folder_kind = BRUTUS_OUTLOOK_CALENDAR;
			break;
		case ICAL_VTODO_COMPONENT :
			mapi_folder_kind = BRUTUS_OUTLOOK_TASKS;
			break;
		default :
			d("Unknown ECal backend kind");
			goto out;
		}

		br = BRUTUS_IMAPISession_GetOutlookFolder(priv->mapi_session,
							  mapi_folder_kind,
							  CORBA_OBJECT_NIL,
							  &priv->mapi_folder,
							  ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			goto out;
		}
		retv = TRUE;
	}
out:
	g_free(prompt);
	g_free(err_prompt);
	g_free(windows_domain);
	g_free(mailbox_dn);
	g_free(mailbox_email);

	cb = CORBA_Object_is_nil(brutus_logon, ev);
	if (!ORBIT2_EX(ev) && !cb)
		CORBA_Object_release(brutus_logon, ev);

	do { // get time zone information
		if (!got_time_zones && retv) {
			struct timeval tv;
			struct timezone tz;
			gchar *tmp_exchange_server = NULL;
			BRUTUS_TIME_OF_DAY_INFO exchange_tod;

			tmp_exchange_server = g_strjoin(NULL, "\\\\", e_source_get_property(source, "exchange_server"), NULL);
			d(tmp_exchange_server);
			if (!tmp_exchange_server)
				break;

			cb = BRUTUS_IMAPISession_GetServerTime(priv->mapi_session, tmp_exchange_server, &exchange_tod, ev);
			g_free(tmp_exchange_server);

			if (cb && (exchange_tod.tod_timezone != -1)) {
				if (gettimeofday(&tv, &tz)) // must not fail!
					break;

				priv->exchange_time_zone = (long)exchange_tod.tod_timezone;
				priv->client_time_zone = (long)tz.tz_minuteswest;

				dfmt("Exchange server time zone %ld", priv->exchange_time_zone);
				dfmt("Client server time zone %ld", priv->client_time_zone);
				got_time_zones = TRUE;
			}
		}
	} while (0);

#ifndef HAVE_ORBIT2_TIMEOUT
exit:
#endif
	CORBA_exception_free(ev);

	if (!retv)
		e_cal_backend_notify_error(E_CAL_BACKEND(brutus_backend), _("Could not connect to Brutus server"));

	return retv;
}

static gboolean
brutus_backend_construct(ECalBackendBrutus *brutus_backend)
{
	gboolean retv = FALSE;
	ESource *source = NULL;
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	icalcomponent_kind kind;
	char *err_msg = NULL;
	CORBA_boolean cb = CORBA_TRUE;
	CORBA_Environment ev[1];

	d("");
	CORBA_exception_init(ev);

	source = e_cal_backend_get_source(E_CAL_BACKEND(brutus_backend));
	if (!source)
		goto out;

	kind = e_cal_backend_get_kind(E_CAL_BACKEND(brutus_backend));

	if (!e_source_get_property(source, "mapi_profile")
	    || !e_source_get_property(source, "exchange_server")
	    || !e_source_get_property(source, "windows_user")
	    || !e_source_get_property(source, "exchange_mailbox")
	    || !e_source_get_property(source, "brutus_server")
	    || !e_source_get_property(source, "brutus_port")
	    || !e_source_get_property(source, "use_ssl")) {
		if (ICAL_VEVENT_COMPONENT == kind)
			err_msg = _("Calendar account not configured");
		else if (ICAL_VTODO_COMPONENT == kind)
			err_msg = _("Tasks account not configured");
		else
			err_msg = _("Account not configured");
		goto out;
	}

	priv->brutus_server = g_strdup(e_source_get_property(source, "brutus_server"));
	if (!priv->brutus_server) {
		err_msg = _("No memory");
		goto out;
	}

	// at least EDS_1_8
#if (EDS_EPOCH >= 18)
	if (ICAL_VEVENT_COMPONENT == kind)
		priv->ecal_source_type = E_CAL_SOURCE_TYPE_EVENT;
	else if (ICAL_VTODO_COMPONENT == kind)
		priv->ecal_source_type = E_CAL_SOURCE_TYPE_TODO;
	else {
		err_msg = _("Unknown calendar type");
		goto out;
	}
#endif

	/* SSLIOP */
#ifdef BRUTUS_HAVE_SSL
	priv->use_ssliop = e_source_get_property(source, "use_ssl") ? TRUE : FALSE;
#endif

	if (!priv->cache) { // create kind-dependent cache file
#if (EDS_EPOCH <= 16)
		priv->cache = e_cal_backend_cache_new(e_cal_backend_get_uri(E_CAL_BACKEND(brutus_backend)));
#else
		priv->cache = e_cal_backend_cache_new(e_cal_backend_get_uri(E_CAL_BACKEND(brutus_backend)), priv->ecal_source_type);
#endif
		if (!priv->cache) {
			err_msg = _("Could not create cache file");
			goto out;
		}
	}

	d("Initializing CORBA");
	{
		gboolean orb_ok = FALSE;
		guint64 brutus_port = 951;
		const char *brutus_server = e_source_get_property(source, "brutus_server");

		if (!brutus_get_xbit_number(16, e_source_get_property(source, "brutus_port"), &brutus_port)) {
			err_msg = _("Invalid Brutus port in url");
			goto out;
		}

		priv->brutus_logon_str = brutus_build_corbaloc_string(brutus_server,
								      BRUTUS_BrutusLogOn_IOR_TABLE_KEY,
								      priv->use_ssliop,
								      brutus_port);
		orb_ok = brutus_orb_init(priv,
					 brutus_server,
					 brutus_port,
					 kind);
		if (!orb_ok) {
			err_msg = _("Could not initiate ORB");
			goto out;
		}
	}

	d("Getting root POAManager, activating the RootPOA and instantiating the lifeline");
	{
		PortableServer_POAManager root_poa_manager = CORBA_OBJECT_NIL;
		gboolean use_local_proxy = !(ORBit_proto_use("IPv4") || ORBit_proto_use("IPv6"));

		root_poa_manager = PortableServer_POA__get_the_POAManager(priv->root_poa, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		}

		cb = CORBA_Object_is_nil((CORBA_Object)root_poa_manager, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		}
		if (cb) {
			d("Could not get the root POAManager");
			goto out_ex;
		}
		PortableServer_POAManager_activate(root_poa_manager, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		}

		CORBA_Object_release((CORBA_Object)root_poa_manager, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		}

		// get brutus proxy
		{
			char *home_dir = getenv("HOME");
			gchar *ref_path = g_strconcat(home_dir, "/", BRUTUSD_REF_FILE_NAME, NULL);

			priv->brutus_proxy = brutus_object_ref_from_file(priv->orb, ref_path, ev);
			g_free(ref_path);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto out_ex;
			}
			cb = CORBA_Object_is_nil((CORBA_Object)priv->brutus_proxy, ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto out_ex;
			}
			if (cb) {
				d("Could not get the Brutus proxy reference");
				goto out_ex;
			}
		}

		// the lifeline
		priv->lifeline.filling = impl_BRUTUS_BrutusCheck__create(priv->root_poa, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		}
		if (use_local_proxy) {
			d("Using local network capable Brutus proxy");
			BRUTUS_BrutusProxy_wrapObject(priv->brutus_proxy,
						      BRUTUS_BrutusCheck_tc,
						      priv->lifeline.filling,
						      &priv->lifeline.wrapper,
						      ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto out_ex;
			}
		} else {
			/* this will enable us to use "wrapper" unconditionally */
			priv->lifeline.wrapper = CORBA_Object_duplicate(priv->lifeline.filling, ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto out_ex;
			}
		}
		d("Testing primary BrutusCheck object...");
		BRUTUS_BrutusCheck_ping(priv->lifeline.filling, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		} else
			d("priv->lifeline.filling - OK");
		d("Testing BrutusCheck object wrapper...");
		BRUTUS_BrutusCheck_ping(priv->lifeline.wrapper, ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out_ex;
		} else
			d("priv->lifeline.wrapper - OK");


		// construct the name of the attachment cache directory (from Groupwise backend)
		{
			int n;
			char *dir_name = NULL;
			char *mangled_uri = NULL;

			mangled_uri = g_strdup(e_cal_backend_get_uri(E_CAL_BACKEND(brutus_backend)));

			// mangle the URI to not contain invalid characters
			for (n = 0; n < strlen(mangled_uri); n++) {
				switch (mangled_uri[n]) {
				case ':' :
				case '/' :
					mangled_uri[n] = '_';
				}
			}

			dir_name = g_build_filename(g_get_home_dir(),
						    ".evolution/cache/calendar",
						    mangled_uri,
						    NULL);
			g_free(mangled_uri);
			priv->cache_dir = g_filename_to_uri(dir_name, NULL, NULL);
			g_free(dir_name);
		}
		d(priv->cache_dir);

		goto no_err; // all is fine

	out_ex:
		if (ICAL_VEVENT_COMPONENT == kind)
			err_msg = _("Could not construct the Calendar backend - CORBA exception caught");
		else if (ICAL_VTODO_COMPONENT == kind)
			err_msg = _("Could not construct the Tasks backend - CORBA exception caught");
		else
			err_msg = _("Could not construct the ECal backend - CORBA exception caught");
		goto out;
	}

no_err:
	retv = TRUE;

out:
	CORBA_exception_free(ev);
	if (err_msg)
		e_cal_backend_notify_error(E_CAL_BACKEND(brutus_backend), err_msg);

	return retv;
}

//
// Modified from e-cal-backend-groupwise.c
//
static const char*
brutus_get_x_property_val(const char *x_prop_name,
			  ECalComponent *ecal_comp)
{
	const char *x_name = NULL;
	const char *x_val = NULL;
	icalproperty *x_prop = NULL;

	x_prop = icalcomponent_get_first_property(e_cal_component_get_icalcomponent(ecal_comp), ICAL_X_PROPERTY);
	while (x_prop) {
		x_name = icalproperty_get_x_name(x_prop);
		x_val = icalproperty_get_x(x_prop);

		if (!strcmp(x_name, x_prop_name))
			return x_val;

		x_prop = icalcomponent_get_next_property(e_cal_component_get_icalcomponent(ecal_comp), ICAL_X_PROPERTY);
	}

	return NULL;
}

static char*
brutus_get_md5_hash(const char *str)
{
	unsigned int i;
	guchar digest[16] = { 0 }; // md5 hash of str
	gchar digest_str[33] = { '\0' }; // stringified version of digest

	md5_get_digest(str, strlen(str), digest);
	for (i = 0; i < sizeof(digest); i++)
		snprintf(&digest_str[2*i], 3, "%02x", (guint8)digest[i]);

	return g_strdup(digest_str);
}

static GSList*
brutus_get_attachment_list(ECalBackendBrutus *brutus_backend,
			   BRUTUS_IMessage mapi_message,
			   const char *msg_id)
{
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	GSList *ecal_attachment_list = NULL;
	gchar *attach_file_url = NULL;
	gchar *file_name = NULL;
	gchar *md5_uid = NULL;
	char *tmp = NULL;
	int fd = -1;
	CORBA_long n = 0;
	CORBA_long attach_num;
	CORBA_unsigned_long r = 0;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_IMAPITable attachment_table = CORBA_OBJECT_NIL;
	BRUTUS_seq_SPropValue *attach_prop_values = NULL;
	BRUTUS_IAttach mapi_attach =  CORBA_OBJECT_NIL;
	BRUTUS_seq_octet *attach_data = NULL;
	BRUTUS_SRowSet *row_set = NULL;
	static BRUTUS_SSortOrderSet sort_order = {            // for the contents table
		.cCategories = 0,
		.cExpanded = 0,
		.aSort._maximum = 0,
		.aSort._length = 0,
		.aSort._buffer = NULL,
	};
	static BRUTUS_BDEFINE attach_props[1] = {            // for the attachment table
		/* 0 */ BRUTUS_BRUTUS_PR_ATTACH_NUM,
	};
	static BRUTUS_SPropTagArray attach_proptag_array = {
		._maximum = 1,
		._length = 1,
		._buffer = attach_props,
	};
	static BRUTUS_BDEFINE attach_object_props[3] = {     // for the attachment object
		/* 0 */ BRUTUS_BRUTUS_PR_ATTACH_FILENAME,
		/* 1 */ BRUTUS_BRUTUS_PR_ATTACH_LONG_FILENAME,
		/* 2 */ BRUTUS_BRUTUS_PR_ATTACH_METHOD,
		/* 3 */ /* BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray attach_object_proptag_array = {
		._maximum = 3,
		._length = 3,
		._buffer = attach_object_props,
	};

	CORBA_Environment ev[1];

	br = BRUTUS_IMessage_GetAttachmentTable(mapi_message,
						0,
						&attachment_table,
						ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto exit;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		d("Could not query message object for attachment table");
		goto exit;
	}

	br = brutus_QueryAllRows(attachment_table,
				 &attach_proptag_array,
				 CORBA_OBJECT_NIL,
				 &sort_order,
				 0,
				 &row_set,
				 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto exit;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto exit;
	}

	// iterate over content (columns)
	// See http://windowssdk.msdn.microsoft.com/library/default.asp?url=/library/en-us/MAPI/html/_mapi1book_opening_an_attachment.asp
	for (r = 0; r < row_set->_length; r++) {

		n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_ATTACH_NUM);
		if (-1 == n) {
			d("No PR_ATTACH_NUM");
			continue;
		}
		attach_num = row_set->_buffer[r]._buffer[n].Value._u.l;

		br = BRUTUS_IMessage_OpenAttach(mapi_message,
						attach_num,
						"",
						0, //  read-only
						&mapi_attach,
						ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			continue;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			continue;
		}

		// get attachment properties
		br = BRUTUS_IMAPIProp_GetProps(mapi_attach,
					       &attach_object_proptag_array,
					       0,
					       &attach_prop_values,
					       ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto next_attachment;
		}
		if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
			d(brutus_bresult_to_str(br));
			goto next_attachment;
		}

		// print attachment method
		n = brutus_get_spropvalue_index(attach_prop_values, BRUTUS_BRUTUS_PR_ATTACH_METHOD);
		if (-1 != n) {
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(attach_prop_values->_buffer[n].ulPropTag)) { // PR_ATTACH_METHOD
				switch (attach_prop_values->_buffer[n].Value._u.l) {
				case BRUTUS_BRUTUS_NO_ATTACHMENT :
					d("PR_ATTACH_METHOD: NO_ATTACHMENT");
					break;
				case BRUTUS_BRUTUS_ATTACH_BY_VALUE :
					d("PR_ATTACH_METHOD: ATTACH_BY_VALUE");
					break;
				case BRUTUS_BRUTUS_ATTACH_BY_REFERENCE :
					d("PR_ATTACH_METHOD: ATTACH_BY_REFERENCE");
					break;
				case BRUTUS_BRUTUS_ATTACH_BY_REF_RESOLVE :
					d("PR_ATTACH_METHOD: ATTACH_BY_REF_RESOLVE");
					break;
				case BRUTUS_BRUTUS_ATTACH_BY_REF_ONLY :
					d("PR_ATTACH_METHOD: ATTACH_BY_REF_ONLY");
					break;
				case BRUTUS_BRUTUS_ATTACH_EMBEDDED_MSG :
					d("PR_ATTACH_METHOD: ATTACH_EMBEDDED_MS");
					break;
				case BRUTUS_BRUTUS_ATTACH_OLE :
					d("PR_ATTACH_METHOD: ATTACH_OLE");
					break;
				default :
					d("PR_ATTACH_METHOD: Unknown");
					goto next_attachment;
				}
			}
		}

		//
		// stream attachment to file
		//
		tmp = NULL;
		n = brutus_get_spropvalue_index(attach_prop_values, BRUTUS_BRUTUS_PR_ATTACH_LONG_FILENAME);
		if (-1 != n) {
			tmp = attach_prop_values->_buffer[n].Value._u.lpszA;
			d(attach_prop_values->_buffer[n].Value._u.lpszA);
		} else {
			n = brutus_get_spropvalue_index(attach_prop_values, BRUTUS_BRUTUS_PR_ATTACH_FILENAME);
			if (-1 != n) {
				tmp = attach_prop_values->_buffer[n].Value._u.lpszA;
				d(attach_prop_values->_buffer[n].Value._u.lpszA);
			}
		}
		if (!tmp)
			goto next_attachment;

		// read attachment data from stream
		attach_data = brutus_read_binary_stream(BRUTUS_BRUTUS_PR_ATTACH_DATA_BIN, mapi_attach);
		if (!attach_data)
			goto next_attachment;

		// use the md5 hash of the uid as the name would otherwise get too long
		md5_uid = brutus_get_md5_hash(msg_id);
		if (!md5_uid)
			goto next_attachment;

		// url of cache file
		attach_file_url = g_strconcat(priv->cache_dir,
					      "/",
					      md5_uid,
					      "-",
					      tmp,  // filename
					      NULL);
		file_name = g_filename_from_uri(attach_file_url, NULL, NULL);

		// write data to file
		fd = g_open(file_name, O_RDWR | O_CREAT | O_TRUNC, 0600);
		if (fd == -1) {
			d("could not serialize attachments");
			goto next_attachment;
		}
		if (-1 == write(fd, attach_data->_buffer, attach_data->_length)) {
			g_warning ("attachment write failed");
			goto next_attachment;
		}
		close(fd);
		fd = -1;

		ecal_attachment_list = g_slist_append(ecal_attachment_list, attach_file_url);

	next_attachment:
		brutus_release_object(BRUTUS_IAttach_tc, &mapi_attach, CORBA_OBJECT_NIL, ev);
		PRINT_IF_ORBIT2_EX(ev);

		g_free(md5_uid);
		md5_uid = NULL;

		g_free(file_name);
		file_name = NULL;

		CORBA_free(attach_prop_values);
		attach_prop_values = NULL;

		CORBA_free(attach_data);
		attach_data = NULL;
	}

exit:
	brutus_release_object(BRUTUS_IMAPITable_tc, &attachment_table, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_free(row_set);

	return ecal_attachment_list;
}

static ECalBackendSyncStatus
brutus_populate_calendar_cache(ECalBackendBrutus *brutus_backend,
			       gboolean from_scratch)
{
	ECalBackendSyncStatus retv = GNOME_Evolution_Calendar_OtherError;
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	ECalComponent *ecal_comp_server = NULL; // retrieved from the server
	ECalComponent *ecal_comp_cache = NULL;  // retrieved from the cache
	ESource *source = NULL;
#ifndef EDS_1_4
	ECalComponentId *ecal_id = NULL;
#endif
	ECalComponentText ecal_text;
	ECalComponentDateTime ecal_dt;
	ECalComponentAlarm *ecal_alarm = NULL;
	ECalComponentAlarmTrigger ecal_trigger;
	ECalComponentAttendee *attendee = NULL;
	ECalComponentOrganizer *organizer = NULL;
	icalproperty *icalprop;
	struct icaltimetype ical_time;
	struct icaltimetype ical_time_utc;
	GList *g_list = NULL;
	GList *g_list_iterator = NULL;
	GSList gs_list;
	GSList *gs_list_p = NULL;
	GSList *ecal_attachment_list = NULL;
	time_t server_utc_time;
	time_t cache_utc_time;
	time_t last_cache_update = 0;   // last time the cache was updated
	time_t msg_modification_time = 1; // greater than last_summary_update to make sure that changes are caught during the first run
	time_t tmp_time;
	char server_utc_time_str[100] = { '\0' };
	GPtrArray *seen_events = NULL;   // PR_RECORD_KEY array of events with corresponding remote message object
	GPtrArray *expunge_events = NULL; // events infos to be deleted due to missing corresponding remote message object
	gboolean exists = FALSE;
	gboolean errors = TRUE;
	gboolean do_break = FALSE;
	gboolean all_day_event;
	gboolean has_attach = FALSE;
	char *tmp = NULL;
	char *pr_record_key = NULL;
	char *msg_id = NULL;
	const char *uid = NULL;
	char *ecal_comp_str = NULL;
	const char *charset = NULL;
	int priority = 0;
	CORBA_long n = 0;
	CORBA_long m = 0;
	CORBA_long k = 0;
	CORBA_long g = 0;
	CORBA_long i = 0;
	CORBA_long t = 0;
	CORBA_long s = 0;
	CORBA_unsigned_long r = 0;
	CORBA_unsigned_long processed = 0;
	CORBA_unsigned_long row_count = 0;
	BRUTUS_BDEFINE obj_type;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_IMAPITable content_table = CORBA_OBJECT_NIL;   // folder content
	BRUTUS_IMessage mapi_message = CORBA_OBJECT_NIL;      // event message object
	BRUTUS_SRowSet *row_set = NULL;
	BRUTUS_SRowSet *recp_row_set = NULL;
	BRUTUS_seq_SPropValue *prop_values = NULL;
	BRUTUS_seq_SPropValue *ex_prop_values = NULL;
	BRUTUS_seq_SPropValue *named_prop_values = NULL;
	CORBA_Environment ev[1];
	BRUTUS_seq_NamedPropertyTag named_prop_tags = {       // named properties
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	static BRUTUS_BDEFINE content_props[4] = {            // for the contents table
		/* 0 */ BRUTUS_BRUTUS_PR_ENTRYID,                     // short-term entryid
		/* 1 */ BRUTUS_BRUTUS_PR_MSG_STATUS,                  // message status
		/* 2 */ BRUTUS_BRUTUS_PR_LAST_MODIFICATION_TIME,      // message modification time
		/* 3 */ BRUTUS_BRUTUS_PR_RECORD_KEY,                  // provider unique ID
	};
	static BRUTUS_SPropTagArray content_proptag_array = { // for the contents table
		._maximum = 4,
		._length = 4,
		._buffer = content_props,
	};
	static BRUTUS_SSortOrderSet sort_order = {            // for the contents table
		.cCategories = 0,
		.cExpanded = 0,
		.aSort._maximum = 0,
		.aSort._length = 0,
		.aSort._buffer = NULL,
	};
	static BRUTUS_BDEFINE recp_props[7] = {               // for the recipients table
		/* 0 */ BRUTUS_BRUTUS_PR_RECIPIENT_TYPE,             // MAPI_TO (required attendee), MAPI_CC (optional attendee), MAPI_BCC (resource)
		/* 1 */ BRUTUS_BRUTUS_PR_DISPLAY_NAME,
		/* 2 */ BRUTUS_BRUTUS_PR_EMAIL_ADDRESS,
		/* 3 */ BRUTUS_BRUTUS_PR_SMTP_ADDRESS,
		/* 4 */ BRUTUS_BRUTUS_PR_ADDRTYPE,                   // (==EX => email=PR_SMTP_ADDRESS) (==SMTP => email=PR_EMAIL_ADDRESS)
		/* 5 */ BRUTUS_BRUTUS_PR_RECIPIENT_FLAGS,            // (==3 => Organizer) (==1 => Attendee)
		/* 6 */ BRUTUS_BRUTUS_PR_RECIPIENT_TRACKSTATUS,      // none, tentative, accepted or declined
		/* 7 */ /* BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray recp_proptag_array = {
		._maximum = 7,
		._length = 7,
		._buffer = recp_props,
	};
	static BRUTUS_BDEFINE msg_props[8] = {                // for a calendar object
		/* 0  */ BRUTUS_BRUTUS_PR_ENTRYID,                   // long-term entryid
		/* 1  */ BRUTUS_BRUTUS_PR_OBJECT_TYPE,               // must be MAPI_MESSAGE
		/* 2  */ BRUTUS_BRUTUS_PR_MESSAGE_FLAGS,             // flags
		/* 3  */ BRUTUS_BRUTUS_PR_SUBJECT,                   // subject
		/* 4  */ BRUTUS_BRUTUS_PR_CREATION_TIME,             // creation time
		/* 5  */ BRUTUS_BRUTUS_PR_SENSITIVITY,               // sensitivity - none, personal, private or company_confidential (ECalComponentClassification)
		/* 6  */ BRUTUS_BRUTUS_PR_IMPORTANCE,                // importance to the user - low, high or normal
		/* 7  */ BRUTUS_BRUTUS_PR_INTERNET_CPID,             // Equivalent to RFC 2822 "Content-Type".
		/* 8  *//*  BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray msg_proptag_array = {     // for message object properties
		._maximum = 8,
		._length = 8,
		._buffer = msg_props,
	};
	static BRUTUS_BDEFINE ex_msg_props[1] = {             // for the message object
		/* 0 */ BRUTUS_BRUTUS_PR_BODY,
		/* 1 *//*  BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray ex_msg_proptag_array = {
		._maximum = 1,
		._length = 1,
		._buffer = ex_msg_props,
	};

	CORBA_exception_init(ev);

	e_file_cache_freeze_changes(E_FILE_CACHE(priv->cache));
	e_cal_backend_notify_view_progress(E_CAL_BACKEND(brutus_backend),
					   _("Fetching Exchange Calendar Information"),
					   0);

	/* get last cache update time */
	if (!from_scratch) { // irrelevant if updating from scratch
		cache_utc_time = 0;
		tmp = (char*)e_cal_backend_cache_get_server_utc_time(priv->cache);
		if (tmp)
			brutus_string_to_time_t((const char*)tmp,
						UTC_TIME_FORMAT_STRING,
						&cache_utc_time);
		else
			from_scratch = TRUE;
	}

	/* timestamp the cache */
	source = e_cal_backend_get_source(E_CAL_BACKEND(brutus_backend));
	if (!source) {
		d("Could not get ESource");
		goto out;
	}
	server_utc_time = brutus_get_server_utc_time(priv->brutus_server,
						     priv->mapi_session, 
						     e_source_get_property(source, "exchange_server"));
	brutus_time_t_to_string(server_utc_time,
				sizeof(server_utc_time_str),
				UTC_TIME_FORMAT_STRING,
				server_utc_time_str);
	e_cal_backend_cache_put_server_utc_time(priv->cache, server_utc_time_str);

	// get contents table
	br = BRUTUS_IMAPIFolder_GetContentsTable(priv->mapi_folder,
						 0,
						 &content_table,
						 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		d("Possibly this error: http://support.microsoft.com/default.aspx?scid=kb\%3Ben-us\%3B176001");
		goto out;
	}

	br = BRUTUS_IMAPITable_GetRowCount(content_table,
					   0,
					   &row_count,
					   ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}
	if (!row_count)
		goto out;

	br = brutus_PrepareTable(content_table,
				 &content_proptag_array,
				 CORBA_OBJECT_NIL,
				 &sort_order,
				 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}

	if (from_scratch) // using PR_LAST_MODIFICATION_TIME of content table to detect changes
		last_cache_update = 0;
	else
		last_cache_update = cache_utc_time;

	// for later use for figuring out if any events has been deleted behind our back
	seen_events = g_ptr_array_sized_new(row_count);
	if (!seen_events)
		goto out;

	do {
		br = brutus_QuerySomeRows(content_table,
					  (CORBA_long)BRUTUS_IMAPITABLE_ROW_COUNT_MAX,
					  &row_set,
					  ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			goto out;
		}

		for (r = 0; r < row_set->_length; r++) {

			// progress bar
			e_cal_backend_notify_view_progress(E_CAL_BACKEND(brutus_backend),
							   _("Fetching Exchange Calendar Information"),
							   (100*processed)/row_count);
			// to catch goto's
			errors = TRUE;

			free(pr_record_key);
			pr_record_key = NULL;

			// add to the array of pr_record_key for existing remote messages
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_RECORD_KEY);
			if (-1 != n) {
				pr_record_key = brutus_print_spropvalue(&row_set->_buffer[r]._buffer[n]);
				g_ptr_array_add(seen_events, (gpointer)g_strdup(pr_record_key));
			} else {
				d("No PR_RECORD_KEY");
				continue;
			}

			// check whether the message has been modified since the last cache update. If from_scratch is TRUE this check is skipped
			if (!from_scratch) {
				n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_LAST_MODIFICATION_TIME);
				if (-1 != n) {
					msg_modification_time = brutus_filetime_to_time_t(row_set->_buffer[r]._buffer[n].Value._u.ft);
					if (last_cache_update > msg_modification_time)
						continue;
				}
			}

			// get short-term ENTRYID
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_ENTRYID);
			if (-1 == n) {
				d("No ENTRYID");
				continue;
			}
			brutus_sbinary_to_entryid(&row_set->_buffer[r]._buffer[n].Value._u.bin, &eid);
			if (!eid)
				continue;

			// get normal properties from message object
			br = BRUTUS_IMAPISession_QueryMessage(priv->mapi_session,
							      eid,
							      &msg_proptag_array,
							      &ex_msg_proptag_array,
							      &recp_proptag_array,
							      &prop_values,
							      &ex_prop_values,
							      &recp_row_set,
							      ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				CORBA_free(eid);
				eid = NULL;
				goto next;
			}
			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				CORBA_free(eid);
				eid = NULL;
				goto next;
			}

			/*
			 * initiate the named property tags - BRUTUS_PS_PROPSET_APPOINTMENT
			 */
			CORBA_free(named_prop_tags._buffer);
			named_prop_tags._maximum = 6;
			named_prop_tags._length = named_prop_tags._maximum;
			named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);

			// start time
			named_prop_tags._buffer[0].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_START_DATE);
			named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[0].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_START_DATE);

			// end time
			named_prop_tags._buffer[1].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_END_DATE);
			named_prop_tags._buffer[1].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[1].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_END_DATE);

			// location
			named_prop_tags._buffer[2].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_LOCATION);
			named_prop_tags._buffer[2].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[2].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_LOCATION);

			// show time as
			named_prop_tags._buffer[3].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_SHOW_TIME_AS);
			named_prop_tags._buffer[3].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[3].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_SHOW_TIME_AS);

			// duration
			named_prop_tags._buffer[4].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_DURATION);
			named_prop_tags._buffer[4].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[4].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_DURATION);

			// all-day event?
			named_prop_tags._buffer[5].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_ALL_DAY);
			named_prop_tags._buffer[5].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[5].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_ALL_DAY);

			br = BRUTUS_IMAPISession_OpenEntry(priv->mapi_session,
							   eid,
							   "",
							   BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
							   &obj_type,
							   &mapi_message,
							   ev);
			CORBA_free(eid);
			eid = NULL;
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto next;
			}
			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				goto next;
			}
			br = BRUTUS_IMessage_GetNamedProps(mapi_message,
							   BRUTUS_BRUTUS_PS_PROPSET_APPOINTMENT,
							   &named_prop_tags,
							   0,
							   &named_prop_values,
							   ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto next;
			}
			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				goto next;
			}

			// get long-term ENTRYID
			brutus_sbinary_to_entryid(&prop_values->_buffer[0].Value._u.bin, &eid); // PR_ENTRYID
			if (!eid)
				goto next;

			msg_id = brutus_entryid_to_string(eid);
			CORBA_free(eid);
			eid = NULL;
			if (!msg_id)
				goto next;

			// create new/modified ECalComponent object
			ecal_comp_server = e_cal_component_new();
			if (!ecal_comp_server)
				goto next;
			e_cal_component_set_new_vtype(ecal_comp_server, E_CAL_COMPONENT_EVENT);
			e_cal_component_set_uid(ecal_comp_server, msg_id);

			// does it exist in the cache?
			ecal_comp_cache = e_cal_backend_cache_get_component(priv->cache, msg_id, NULL);
			if (ecal_comp_cache)
				exists = TRUE;  // present in cache - modified event
			else
				exists = FALSE; // not in cache - new event

			// add PR_RECORD_KEY to ECalComponent object
			if (pr_record_key) {
				icalprop = icalproperty_new_x(pr_record_key);
				if (!icalprop)
					goto next;

				icalproperty_set_x_name(icalprop, X_PROP_RECORD_KEY);
				icalcomponent_add_property(e_cal_component_get_icalcomponent(ecal_comp_server), icalprop);
				free(pr_record_key);
				pr_record_key = NULL;
			}

			// MIME charset
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[7].ulPropTag)) // PR_INTERNET_CPID
				charset = brutus_cpid_to_label(prop_values->_buffer[7].Value._u.l);
			charset = charset ? e_iconv_charset_name(charset) : NULL;

			// add MIME charset to ECalComponent object
			if (charset) {
				tmp = g_strdup_printf("Content-Type: text/calendar; charset=%s", charset);
				if (!tmp)
					goto next;

				icalprop = icalproperty_new_x(tmp);
				if (!icalprop)
					goto next;

				icalproperty_set_x_name(icalprop, "X-LIC-MIMECONTENTTYPE");
				icalcomponent_add_property(e_cal_component_get_icalcomponent(ecal_comp_server), icalprop);
				g_free(tmp);
				tmp = NULL;
				charset = NULL;
			}

			// message flags - kept for possible future use
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_MSG_STATUS); // required column in folder content tables
			if (-1 != n) {
				if (BRUTUS_BRUTUS_MSGSTATUS_ANSWERED & row_set->_buffer[r]._buffer[n].Value._u.l)
					;

				if (BRUTUS_BRUTUS_MSGSTATUS_DELMARKED & row_set->_buffer[r]._buffer[n].Value._u.l)
					;

				if (BRUTUS_BRUTUS_MSGSTATUS_DRAFT & row_set->_buffer[r]._buffer[n].Value._u.l)
					;

				if (BRUTUS_BRUTUS_MSGSTATUS_HIGHLIGHTED & row_set->_buffer[r]._buffer[n].Value._u.l)
					;
			}
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[2].ulPropTag)) { // PR_MESSAGE_FLAGS
				if (BRUTUS_BRUTUS_MSGFLAG_READ & prop_values->_buffer[2].Value._u.l)
					;

				if (BRUTUS_BRUTUS_MSGFLAG_HASATTACH & prop_values->_buffer[2].Value._u.l)
					has_attach = TRUE;
				else
					has_attach = FALSE;
			}

			// organizer/attendees
			if ((recp_row_set->_length) // anyone?
			    && (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(recp_row_set->_buffer[0]._buffer[0].ulPropTag))) { // we don't know if STORE_SUBMIT_OK is set
				for (k = 0; k < recp_row_set->_length; k++) { // loop all recipients

					// reset failed values from 'continue' statement
					if (attendee) {
						g_free(attendee);
						attendee = NULL;
					}
					if (organizer) {
						g_free(organizer);
						organizer = NULL;
					}

					// locate properties
					n = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_RECIPIENT_TYPE);
					m = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_DISPLAY_NAME);
					i = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_ADDRTYPE); // might be -1
					t = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_RECIPIENT_FLAGS);
					s = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_RECIPIENT_TRACKSTATUS);

					// organizer or attendee?
					switch (recp_row_set->_buffer[k]._buffer[t].Value._u.l) {
					case 1 :
						attendee = g_new0(ECalComponentAttendee, 1);
						if (!attendee)
							continue;
						break;
					case 3 :
						organizer = g_new0(ECalComponentOrganizer, 1);
						if (!organizer)
							continue;
						break;
					default :
						dfmt("Unknown participant role %d", recp_row_set->_buffer[k]._buffer[t].Value._u.l);
						continue;
					}

					// (==SMTP => email=PR_EMAIL_ADDRESS) - (==EX => email=PR_SMTP_ADDRESS)
					if (-1 == i) {
						dfmt("No usable email address type for attendee \"%s\"", recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA);
						g = m;  // using display name - best effort...
					} else if (!g_ascii_strcasecmp("SMTP", recp_row_set->_buffer[k]._buffer[i].Value._u.lpszA)) {
						g = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_EMAIL_ADDRESS);
					} else if (!g_ascii_strcasecmp("EX", recp_row_set->_buffer[k]._buffer[i].Value._u.lpszA)) {
						g = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_SMTP_ADDRESS);
					} else {
						dfmt("No usable email address for attendee \"%s\"", recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA);
						g = m; // using display name - best effort...
					}
					if (-1 == g)
						g = m;

					if (attendee) {
						if (g != m)
							attendee->value = g_strconcat("MAILTO:", recp_row_set->_buffer[k]._buffer[g].Value._u.lpszA, NULL); // email address
						else
							attendee->value = g_strdup(recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA); // PR_DISPLAY_NAME
						if (!attendee->value)
							continue;

						if (BRUTUS_BRUTUS_MAPI_TO & recp_row_set->_buffer[k]._buffer[n].Value._u.l) { // requried attendee
							attendee->cutype = ICAL_CUTYPE_INDIVIDUAL;
							attendee->role = ICAL_ROLE_REQPARTICIPANT;
							attendee->rsvp = TRUE; // reply expected
						} else if (BRUTUS_BRUTUS_MAPI_CC & recp_row_set->_buffer[k]._buffer[n].Value._u.l) { // optional attendee
							attendee->cutype = ICAL_CUTYPE_INDIVIDUAL;
							attendee->role = ICAL_ROLE_OPTPARTICIPANT;
							attendee->rsvp = FALSE; // reply not expected
						} else if (BRUTUS_BRUTUS_MAPI_BCC & recp_row_set->_buffer[k]._buffer[n].Value._u.l) { // resource
							attendee->cutype = ICAL_CUTYPE_RESOURCE;
							attendee->role = ICAL_ROLE_NONPARTICIPANT; // what else?
							attendee->rsvp = TRUE; // reply expected
						} else {
							d("Unknown participant type");
							attendee->cutype = ICAL_CUTYPE_UNKNOWN;
							attendee->role = ICAL_ROLE_NONE; // what else?
							attendee->rsvp = FALSE; // reply expected
						}

						switch (recp_row_set->_buffer[k]._buffer[s].Value._u.l) {
						case BRUTUS_BRUTUS_olResponseNone :
							attendee->status = ICAL_PARTSTAT_NEEDSACTION;
							break;
						case BRUTUS_BRUTUS_olResponseTentative :
							attendee->status = ICAL_PARTSTAT_TENTATIVE;
							break;
						case BRUTUS_BRUTUS_olResponseAccepted :
							attendee->status = ICAL_PARTSTAT_ACCEPTED;
							break;
						case BRUTUS_BRUTUS_olResponseDeclined :
							attendee->status = ICAL_PARTSTAT_DECLINED;
							break;
						case BRUTUS_BRUTUS_olResponseNotResponded :
							attendee->status = ICAL_PARTSTAT_NONE;
							break;
						case BRUTUS_BRUTUS_olResponseOrganized :
							d("olResponseOrganized should not happen for any but the organizer");
							attendee->status = ICAL_PARTSTAT_NONE;
							break;
						default:
							d("Unknown response status");
							attendee->status = ICAL_PARTSTAT_NONE;
						}

						attendee->cn = g_strdup(recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA); // PR_DISPLAY_NAME
						if (!attendee->cn)
							continue;

						gs_list_p = g_slist_append(gs_list_p, attendee);
						attendee = NULL;
					} else { // set the organizer
						organizer->value = NULL;
						if (g != m) {
							if (recp_row_set->_buffer[k]._buffer[g].Value._u.lpszA)
								organizer->value = g_strconcat("MAILTO:", recp_row_set->_buffer[k]._buffer[g].Value._u.lpszA, NULL); // email address
						} else {
							if (recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA)
								organizer->value = g_strdup(recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA); // PR_DISPLAY_NAME
						}
						if (!organizer->value)
							continue;

						organizer->cn = g_strdup(recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA); // PR_DISPLAY_NAME
						if (!organizer->cn)
							continue;

						e_cal_component_set_organizer(ecal_comp_server, organizer);
						organizer = NULL;
					}
				}

				// set the attendee list
				if (gs_list_p) {
					e_cal_component_set_attendee_list(ecal_comp_server, gs_list_p);
					gs_list_p = NULL;
				}
			}
			// free failed attendee/organizer entries from early loop exit
			if (attendee) {
				g_free(attendee);
				attendee = NULL;
			}
			if (organizer) {
				g_free(organizer);
				organizer = NULL;
			}

			// Subject:
			tmp = NULL;
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[3].ulPropTag)) // PR_SUBJECT
				tmp = prop_values->_buffer[3].Value._u.lpszA;
			if (tmp) {
				ecal_text.altrep = NULL;
				ecal_text.value = g_strdup(tmp);
				tmp = NULL;
				if (!ecal_text.value)
					goto rollback;
				e_cal_component_set_summary(ecal_comp_server, &ecal_text);
			}

			// Description:
			tmp = NULL;
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(ex_prop_values->_buffer[0].ulPropTag)) // PR_BODY
				tmp = ex_prop_values->_buffer[0].Value._u.lpszA;
			if (tmp) {
				ecal_text.altrep = NULL;
				ecal_text.value = g_strdup(tmp);
				tmp = NULL;
				if (!ecal_text.value)
					goto rollback;

				gs_list.data = &ecal_text;
				gs_list.next = NULL;

				e_cal_component_set_description_list(ecal_comp_server, &gs_list);
			}

			// all day event?
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[5].ulPropTag)) // PR_OUTLOOK_EVENT_ALL_DAY
				all_day_event = named_prop_values->_buffer[5].Value._u.b ? TRUE : FALSE;
			else
				all_day_event = FALSE;

			// creation date
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[4].ulPropTag)) { // PR_CREATION_TIME
				tmp_time = brutus_filetime_to_time_t(prop_values->_buffer[4].Value._u.ft);
				ical_time_utc = icaltime_from_timet_with_zone(tmp_time, 0, icaltimezone_get_utc_timezone());
				if (priv->default_timezone) {
					ical_time = icaltime_convert_to_zone(ical_time_utc, priv->default_timezone);
					icaltime_set_timezone(&ical_time, priv->default_timezone);
					e_cal_component_set_created(ecal_comp_server, &ical_time);
					e_cal_component_set_dtstamp(ecal_comp_server, &ical_time);
				} else {
					e_cal_component_set_created (ecal_comp_server, &ical_time_utc);
					e_cal_component_set_dtstamp (ecal_comp_server, &ical_time_utc);
				}
			}

			// start time
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[0].ulPropTag)) { // PR_OUTLOOK_EVENT_START_DATE
				tmp_time = brutus_filetime_to_time_t(named_prop_values->_buffer[0].Value._u.ft) + (all_day_event ? ((-60) * priv->exchange_time_zone) : 0);
				ical_time_utc = icaltime_from_timet_with_zone(tmp_time, 0, icaltimezone_get_utc_timezone());

				ecal_dt.value = &ical_time_utc;
				ecal_dt.value->is_date = all_day_event ? TRUE : FALSE;
				ecal_dt.tzid = all_day_event ? NULL : g_strdup("UTC");
				if (!all_day_event && !ecal_dt.tzid)
					goto rollback;

				e_cal_component_set_dtstart(ecal_comp_server, &ecal_dt);
			}

			// end/due time
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[1].ulPropTag)) { // PR_OUTLOOK_EVENT_END_DATE
				tmp_time = brutus_filetime_to_time_t(named_prop_values->_buffer[1].Value._u.ft) + (all_day_event ? ((-60) * priv->exchange_time_zone) : 0);
				ical_time_utc = icaltime_from_timet_with_zone(tmp_time, 0, icaltimezone_get_utc_timezone());

				ecal_dt.value = &ical_time_utc;
				ecal_dt.value->is_date = all_day_event ? TRUE : FALSE;
				ecal_dt.tzid = all_day_event ? NULL : g_strdup("UTC");
				if (!all_day_event && !ecal_dt.tzid)
					goto rollback;

				e_cal_component_set_dtend(ecal_comp_server, &ecal_dt);
			}

			// location
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[2].ulPropTag)) // PR_OUTLOOK_EVENT_LOCATION
				e_cal_component_set_location(ecal_comp_server, named_prop_values->_buffer[2].Value._u.lpszA);

			// free/busy
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[3].ulPropTag)) { // PR_OUTLOOK_EVENT_SHOW_TIME_AS
				switch (named_prop_values->_buffer[3].Value._u.l) {
				case BRUTUS_BRUTUS_olFree :
				case BRUTUS_BRUTUS_olTentative :
					d("Free");
					e_cal_component_set_transparency(ecal_comp_server, E_CAL_COMPONENT_TRANSP_TRANSPARENT);
					break;
				case BRUTUS_BRUTUS_olBusy :
				case BRUTUS_BRUTUS_olOutOfOffice :
					d("Busy");
					e_cal_component_set_transparency(ecal_comp_server, E_CAL_COMPONENT_TRANSP_OPAQUE);
					break;
				default :
					d("Unknown free/busy status");
					e_cal_component_set_transparency(ecal_comp_server, E_CAL_COMPONENT_TRANSP_UNKNOWN);
					break;
				}
			}

			// Classification
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[5].ulPropTag)) { // PR_SENSITIVITY
				switch (prop_values->_buffer[5].Value._u.l) {
				case BRUTUS_BRUTUS_SENSITIVITY_NONE :
					d("None");
					e_cal_component_set_classification(ecal_comp_server, E_CAL_COMPONENT_CLASS_PUBLIC);
					break;
				case BRUTUS_BRUTUS_SENSITIVITY_COMPANY_CONFIDENTIAL :
					d("Confidential");
					e_cal_component_set_classification(ecal_comp_server, E_CAL_COMPONENT_CLASS_CONFIDENTIAL);
					break;
				case BRUTUS_BRUTUS_SENSITIVITY_PRIVATE :
				case BRUTUS_BRUTUS_SENSITIVITY_PERSONAL :
					d("Private");
					e_cal_component_set_classification(ecal_comp_server, E_CAL_COMPONENT_CLASS_PRIVATE);
					break;
				default :
					d("Unknown classification");
					e_cal_component_set_classification(ecal_comp_server, E_CAL_COMPONENT_CLASS_UNKNOWN);
					break;
				}
			}

			// Priority
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[6].ulPropTag)) { // PR_IMPORTANCE
				switch (prop_values->_buffer[6].Value._u.l) {
				case BRUTUS_BRUTUS_IMPORTANCE_LOW :
					d("Low");
					priority = 7;
					e_cal_component_set_priority(ecal_comp_server, &priority);
					break;
				case BRUTUS_BRUTUS_IMPORTANCE_NORMAL :
					d("Normal");
					priority = 5;
					e_cal_component_set_priority(ecal_comp_server, &priority);
					break;
				case BRUTUS_BRUTUS_IMPORTANCE_HIGH :
					d("High");
					priority = 3;
					e_cal_component_set_priority(ecal_comp_server, &priority);
					break;
				default :
					d("Unknown priority");
					priority = 0;
					e_cal_component_set_priority(ecal_comp_server, &priority);
					break;
				}
			}

			/*
			 * initiate the named property tags - BRUTUS_PS_PROPSET_COMMON
			 */
			CORBA_free(named_prop_tags._buffer);
			named_prop_tags._maximum = 3;
			named_prop_tags._length = named_prop_tags._maximum;
			named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);
			if (!named_prop_tags._buffer)
				goto rollback;

			// alarm?
			named_prop_tags._buffer[0].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_SET);
			named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[0].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_SET);

			// when to go off as minutes before start time
			named_prop_tags._buffer[1].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE);
			named_prop_tags._buffer[1].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[1].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE);

			// when to go off at a specific time
			named_prop_tags._buffer[2].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_DATE);
			named_prop_tags._buffer[2].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[2].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_DATE);

			br = BRUTUS_IMessage_GetNamedProps(mapi_message,
							   BRUTUS_BRUTUS_PS_PROPSET_COMMON,
							   &named_prop_tags,
							   0,
							   &named_prop_values,
							   ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto rollback;
			}
			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				goto rollback;
			}

			// set alarm if present
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[0].ulPropTag)) { // PR_OUTLOOK_COMMON_REMINDER_SET
				if (named_prop_values->_buffer[0].Value._u.b) {
					ecal_alarm = e_cal_component_alarm_new();
					e_cal_component_alarm_set_action(ecal_alarm, E_CAL_COMPONENT_ALARM_DISPLAY);

					if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[1].ulPropTag)) { // PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE
						ecal_trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START;
						ecal_trigger.u.rel_duration = icaldurationtype_from_int(60*named_prop_values->_buffer[1].Value._u.l);

					} else if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[2].ulPropTag)) { // PR_OUTLOOK_COMMON_REMINDER_DATE
						ecal_trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE;
						tmp_time = brutus_filetime_to_time_t(named_prop_values->_buffer[2].Value._u.ft);
						ecal_trigger.u.abs_time = icaltime_from_timet_with_zone(tmp_time, 0, icaltimezone_get_utc_timezone());
					} else {
						d("No alarm time");
						goto rollback;
					}
					e_cal_component_alarm_set_trigger(ecal_alarm, ecal_trigger);
					e_cal_component_add_alarm(ecal_comp_server, ecal_alarm);
				}
			}

			/*
			 * initiate the named property tags - BRUTUS_PS_PUBLIC_STRINGS
			 */
			CORBA_free(named_prop_tags._buffer);
			named_prop_tags._maximum = 1;
			named_prop_tags._length = named_prop_tags._maximum;
			named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);
			if (!named_prop_tags._buffer)
				goto rollback;

			// Categories
			named_prop_tags._buffer[0].type = BRUTUS_BRUTUS_PT_MV_STRING8;
			named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_STRING;
			named_prop_tags._buffer[0].name._u.str_name = CORBA_string_dup(BRUTUS_BRUTUS_PR_MNID_STRING_KEYWORDS);

			br = BRUTUS_IMessage_GetNamedProps(mapi_message,
							   BRUTUS_BRUTUS_PS_PUBLIC_STRINGS,
							   &named_prop_tags,
							   0,
							   &named_prop_values,
							   ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto rollback;
			}
			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				goto rollback;
			}

			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[0].ulPropTag)) { // Keywords/Categories
				for (n = 0; n < named_prop_values->_buffer[0].Value._u.MVszA._length; n++)
					gs_list_p = g_slist_append(gs_list_p, named_prop_values->_buffer[0].Value._u.MVszA._buffer[n]);

				if ((named_prop_values->_buffer[0].Value._u.MVszA._length) && (!gs_list_p))
					goto rollback;

				if (gs_list_p) {
					e_cal_component_set_categories_list(ecal_comp_server, gs_list_p);
					g_slist_free(gs_list_p);
					gs_list_p = NULL;
				}
			}

			//
			// attachments
			//
			if (has_attach) { // one or more attachments present
				has_attach = FALSE;

				ecal_attachment_list = brutus_get_attachment_list(brutus_backend, mapi_message, msg_id);
				if (ecal_attachment_list)
					e_cal_component_set_attachment_list(ecal_comp_server, ecal_attachment_list);
				else
					goto rollback;
			}

			// all is well
			errors = FALSE;
			goto next;

		rollback:
			d("*************  ROLLBACK * ROLLBACK * ROLLBACK  **************");
			if (ecal_comp_server) {
				g_object_unref(ecal_comp_server);
				ecal_comp_server = NULL;
			}
			if (exists) {
				ecal_comp_str = e_cal_component_get_as_string(ecal_comp_cache);
#ifdef EDS_1_4
				e_cal_backend_notify_object_removed(E_CAL_BACKEND(brutus_backend), msg_id, ecal_comp_str, NULL);
				e_cal_backend_cache_remove_component(priv->cache, msg_id, NULL);
#else
				ecal_id = e_cal_component_get_id(ecal_comp_cache);

				e_cal_backend_notify_object_removed(E_CAL_BACKEND(brutus_backend), ecal_id, ecal_comp_str, NULL);
				e_cal_backend_cache_remove_component(priv->cache, msg_id, NULL);

				e_cal_component_free_id(ecal_id);
#endif
				g_free(ecal_comp_str);
			}
		next:
			brutus_release_object(BRUTUS_IMessage_tc, &mapi_message, CORBA_OBJECT_NIL, ev);
			PRINT_IF_ORBIT2_EX(ev);

			CORBA_free(named_prop_tags._buffer);
			named_prop_tags._maximum = 0;
			named_prop_tags._length = named_prop_tags._maximum;
			named_prop_tags._buffer = NULL;

			g_free(tmp);
			tmp = NULL;

			free(msg_id);
			msg_id = NULL;

			CORBA_free(eid);
			eid = NULL;

			CORBA_free(prop_values);
			prop_values = NULL;

			CORBA_free(ex_prop_values);
			ex_prop_values = NULL;

			CORBA_free(recp_row_set);
			recp_row_set = NULL;

			if (gs_list_p) {
				g_slist_free(gs_list_p);
				gs_list_p = NULL;
			}

			if (!errors) {
				// ensure that the components are fully consistent
				e_cal_component_commit_sequence(ecal_comp_server);
				if (exists)
					e_cal_component_commit_sequence(ecal_comp_cache);

				// update the cache
				e_cal_backend_cache_put_component(priv->cache, ecal_comp_server);

				// notify the frontend
				if (exists) {
					tmp = e_cal_component_get_as_string(ecal_comp_cache);
					ecal_comp_str = e_cal_component_get_as_string(ecal_comp_server);
					e_cal_backend_notify_object_modified(E_CAL_BACKEND(brutus_backend), (const char*)tmp, (const char*)ecal_comp_str);
					g_free(tmp);
					tmp = NULL;
				} else {
					ecal_comp_str = e_cal_component_get_as_string(ecal_comp_server);
					e_cal_backend_notify_object_created(E_CAL_BACKEND(brutus_backend), (const char*)ecal_comp_str);
				}
				g_free(ecal_comp_str);
				ecal_comp_str = NULL;
			}
			if (ecal_comp_server) {
				g_object_unref(ecal_comp_server);
				ecal_comp_server = NULL;
			}
			if (ecal_comp_cache) {
				g_object_unref(ecal_comp_cache);
				ecal_comp_cache = NULL;
			}
			
			processed++;
		}

		if (!row_set->_length)
			do_break = TRUE;

		CORBA_free(row_set);
		row_set = NULL;

		if (do_break)
			break;
	} while (TRUE);

	// from an early loop exit
	free(pr_record_key);
	pr_record_key = NULL;

	// cleanup for deleted or moved message objects
	expunge_events = g_ptr_array_new();
	if (!expunge_events)
		goto out;

	g_list = e_cal_backend_cache_get_components(priv->cache);
	for (g_list_iterator = g_list; g_list_iterator; g_list_iterator = g_list_next(g_list_iterator)) {
		exists = FALSE;

		ecal_comp_cache = E_CAL_COMPONENT(g_list_iterator->data);
		pr_record_key = (char*)brutus_get_x_property_val(X_PROP_RECORD_KEY, ecal_comp_cache);
		if (!pr_record_key)
			continue;

		for (r = 0; r < seen_events->len; r++) { // compare by pr_record_key
			if (!g_ascii_strcasecmp((gchar*)g_ptr_array_index(seen_events, r), pr_record_key)) {
				exists = TRUE;
				break;
			}
		}
		if (!exists) { // there is no remote corresponding message object for this info and it must be deleted by uid
			e_cal_component_get_uid(ecal_comp_cache, &uid);
			g_ptr_array_add(expunge_events, (gpointer)uid);
		}
	}
	for (r = 0; r < expunge_events->len; r++) {
		msg_id = (gchar*)g_ptr_array_index(expunge_events, r);
		ecal_comp_cache = e_cal_backend_cache_get_component(priv->cache, msg_id, NULL);
		ecal_comp_str = e_cal_component_get_as_string(ecal_comp_cache);
#ifdef EDS_1_4
		e_cal_backend_notify_object_removed(E_CAL_BACKEND(brutus_backend), msg_id, ecal_comp_str, NULL);
		e_cal_backend_cache_remove_component(priv->cache, msg_id, NULL);
#else
		ecal_id = e_cal_component_get_id(ecal_comp_cache);

		e_cal_backend_notify_object_removed(E_CAL_BACKEND(brutus_backend), ecal_id, ecal_comp_str, NULL);
		e_cal_backend_cache_remove_component(priv->cache, msg_id, NULL);

		e_cal_component_free_id(ecal_id);
#endif
		g_free(ecal_comp_str);
	}

	retv = GNOME_Evolution_Calendar_Success;
out:
	e_cal_backend_notify_view_progress(E_CAL_BACKEND(brutus_backend),
					   _("Fetching Exchange Calendar Information"),
					   100);
	e_file_cache_thaw_changes(E_FILE_CACHE(priv->cache));

	brutus_release_object(BRUTUS_IMAPITable_tc, &content_table, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	if (GNOME_Evolution_Calendar_Success != retv)
		e_cal_backend_notify_error(E_CAL_BACKEND(brutus_backend), _("Error updating calendar from server"));

	return retv;
}

static ECalBackendSyncStatus
brutus_populate_todos_cache(ECalBackendBrutus *brutus_backend,
			    gboolean from_scratch)
{
	ECalBackendSyncStatus retv = GNOME_Evolution_Calendar_OtherError;
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	ECalComponent *ecal_comp_server = NULL; // retrieved from the server
	ECalComponent *ecal_comp_cache = NULL;  // retrieved from the cache
	ESource *source = NULL;
#ifndef EDS_1_4
	ECalComponentId *ecal_id = NULL;
#endif
	ECalComponentText ecal_text;
	ECalComponentDateTime ecal_dt;
	ECalComponentAlarm *ecal_alarm = NULL;
	ECalComponentAlarmTrigger ecal_trigger;
	icalproperty *icalprop;
	icalproperty_status completion_status = ICAL_STATUS_NONE;
	struct icaltimetype ical_time;
	struct icaltimetype ical_time_utc;
	GList *g_list = NULL;
	GList *g_list_iterator = NULL;
	GSList gs_list;
	GSList *gs_list_p = NULL;
	GSList *ecal_attachment_list = NULL;
	time_t server_utc_time;
	time_t cache_utc_time;
	time_t last_cache_update = 0;   // last time the cache was updated
	time_t msg_modification_time = 1; // greater than last_summary_update to make sure that changes are caught during the first run
	time_t tmp_time;
	char server_utc_time_str[100] = { '\0' };
	GPtrArray *seen_events = NULL;   // PR_RECORD_KEY array of events with corresponding remote message object
	GPtrArray *expunge_events = NULL; // events infos to be deleted due to missing corresponding remote message object
	gboolean exists = FALSE;
	gboolean errors = TRUE;
	gboolean do_break = FALSE;
	gboolean has_attach = FALSE;
	char *tmp = NULL;
	char *pr_record_key = NULL;
	char *msg_id = NULL;
	const char *uid = NULL;
	char *ecal_comp_str = NULL;
	const char *charset = NULL;
	int priority = 0;
	int percent = 0;
	CORBA_long n = 0;
	CORBA_unsigned_long r = 0;
	CORBA_unsigned_long processed = 0;
	CORBA_unsigned_long row_count = 0;
	BRUTUS_BDEFINE obj_type;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_IMAPITable content_table = CORBA_OBJECT_NIL;   // folder content
	BRUTUS_IMessage mapi_message = CORBA_OBJECT_NIL;      // event message object
	BRUTUS_SRowSet *row_set = NULL;
	BRUTUS_SRowSet *recp_row_set = NULL;
	BRUTUS_seq_SPropValue *prop_values = NULL;
	BRUTUS_seq_SPropValue *ex_prop_values = NULL;
	BRUTUS_seq_SPropValue *named_prop_values = NULL;
	CORBA_Environment ev[1];
	BRUTUS_seq_NamedPropertyTag named_prop_tags = {       // named properties
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	static BRUTUS_BDEFINE content_props[4] = {            // for the contents table
		/* 0 */ BRUTUS_BRUTUS_PR_ENTRYID,                     // short-term entryid
		/* 1 */ BRUTUS_BRUTUS_PR_MSG_STATUS,                  // message status
		/* 2 */ BRUTUS_BRUTUS_PR_LAST_MODIFICATION_TIME,      // message modification time
		/* 3 */ BRUTUS_BRUTUS_PR_RECORD_KEY,                  // provider unique ID
	};
	static BRUTUS_SPropTagArray content_proptag_array = { // for the contents table
		._maximum = 4,
		._length = 4,
		._buffer = content_props,
	};
	static BRUTUS_SSortOrderSet sort_order = {            // for the contents table
		.cCategories = 0,
		.cExpanded = 0,
		.aSort._maximum = 0,
		.aSort._length = 0,
		.aSort._buffer = NULL,
	};
	static BRUTUS_SPropTagArray recp_proptag_array = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	static BRUTUS_BDEFINE msg_props[8] = {                // for a calendar object
		/* 0  */ BRUTUS_BRUTUS_PR_ENTRYID,                   // long-term entryid
		/* 1  */ BRUTUS_BRUTUS_PR_OBJECT_TYPE,               // must be MAPI_MESSAGE
		/* 2  */ BRUTUS_BRUTUS_PR_MESSAGE_FLAGS,             // flags
		/* 3  */ BRUTUS_BRUTUS_PR_SUBJECT,                   // subject
		/* 4  */ BRUTUS_BRUTUS_PR_CREATION_TIME,             // creation time
		/* 5  */ BRUTUS_BRUTUS_PR_SENSITIVITY,               // sensitivity - none, personal, private or company_confidential (ECalComponentClassification)
		/* 6  */ BRUTUS_BRUTUS_PR_IMPORTANCE,                // importance to the user - low, high or normal
		/* 7  */ BRUTUS_BRUTUS_PR_INTERNET_CPID,             // Equivalent to RFC 2822 "Content-Type".
		/* 8  *//*  BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray msg_proptag_array = {     // for message object properties
		._maximum = 8,
		._length = 8,
		._buffer = msg_props,
	};
	static BRUTUS_BDEFINE ex_msg_props[1] = {             // for the message object
		/* 0 */ BRUTUS_BRUTUS_PR_BODY,
		/* 1 *//*  BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray ex_msg_proptag_array = {
		._maximum = 1,
		._length = 1,
		._buffer = ex_msg_props,
	};

	CORBA_exception_init(ev);

	e_file_cache_freeze_changes(E_FILE_CACHE(priv->cache));
	e_cal_backend_notify_view_progress(E_CAL_BACKEND(brutus_backend),
					   _("Fetching Exchange Tasks Information"),
					   0);

	/* get last cache update time */
	if (!from_scratch) { // irrelevant if updating from scratch
		cache_utc_time = 0;
		tmp = (char*)e_cal_backend_cache_get_server_utc_time(priv->cache);
		if (tmp)
			brutus_string_to_time_t((const char*)tmp,
						UTC_TIME_FORMAT_STRING,
						&cache_utc_time);
		else
			from_scratch = TRUE;
	}

	/* timestamp the cache */
	source = e_cal_backend_get_source(E_CAL_BACKEND(brutus_backend));
	if (!source) {
		d("Could not get ESource");
		goto out;
	}
	server_utc_time = brutus_get_server_utc_time(priv->brutus_server, 
						     priv->mapi_session, 
						     e_source_get_property(source, "exchange_server"));
	brutus_time_t_to_string(server_utc_time,
				sizeof(server_utc_time_str),
				UTC_TIME_FORMAT_STRING,
				server_utc_time_str);
	e_cal_backend_cache_put_server_utc_time(priv->cache, server_utc_time_str);

	// get contents table
	br = BRUTUS_IMAPIFolder_GetContentsTable(priv->mapi_folder,
						 0,
						 &content_table,
						 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		d("Possibly this error: http://support.microsoft.com/default.aspx?scid=kb\%3Ben-us\%3B176001");
		goto out;
	}

	br = BRUTUS_IMAPITable_GetRowCount(content_table,
					   0,
					   &row_count,
					   ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}
	if (!row_count)
		goto out;

	br = brutus_PrepareTable(content_table,
				 &content_proptag_array,
				 CORBA_OBJECT_NIL,
				 &sort_order,
				 ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}

	if (from_scratch) // using PR_LAST_MODIFICATION_TIME of content table to detect changes
		last_cache_update = 0;
	else
		last_cache_update = cache_utc_time;

	// for later use for figuring out if any todos has been deleted behind our back
	seen_events = g_ptr_array_sized_new(row_count);
	if (!seen_events)
		goto out;

	do {
		br = brutus_QuerySomeRows(content_table,
					  (CORBA_long)BRUTUS_IMAPITABLE_ROW_COUNT_MAX,
					  &row_set,
					  ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			goto out;
		}

		// iterate over content (columns)
		for (r = 0; r < row_set->_length; r++) {

			// progress bar
			e_cal_backend_notify_view_progress(E_CAL_BACKEND(brutus_backend),
							   _("Fetching Exchange Tasks Information"),
							   (100*processed)/row_count);

			// to catch goto's
			errors = TRUE;

			free(pr_record_key);
			pr_record_key = NULL;

			// add to the array of pr_record_key for existing remote messages
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_RECORD_KEY);
			if (-1 != n) {
				pr_record_key = brutus_print_spropvalue(&row_set->_buffer[r]._buffer[n]);
				g_ptr_array_add(seen_events, (gpointer)g_strdup(pr_record_key));
			} else {
				d("No PR_RECORD_KEY");
				continue;
			}

			// check whether the message has been modified since the last cache update. If from_scratch is TRUE this check is skipped
			if (!from_scratch) {
				n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_LAST_MODIFICATION_TIME);
				if (-1 != n) {
					msg_modification_time = brutus_filetime_to_time_t(row_set->_buffer[r]._buffer[n].Value._u.ft);
					if (last_cache_update > msg_modification_time)
						continue;
				}
			}

			// get short-term ENTRYID
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_ENTRYID);
			if (-1 == n) {
				d("No ENTRYID");
				continue;
			}
			brutus_sbinary_to_entryid(&row_set->_buffer[r]._buffer[n].Value._u.bin, &eid);
			if (!eid)
				continue;

			// get normal properties from message object
			br = BRUTUS_IMAPISession_QueryMessage(priv->mapi_session,
							      eid,
							      &msg_proptag_array,
							      &ex_msg_proptag_array,
							      &recp_proptag_array,
							      &prop_values,
							      &ex_prop_values,
							      &recp_row_set,
							      ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				CORBA_free(eid);
				eid = NULL;
				goto next;
			}
			CORBA_free(recp_row_set);
			recp_row_set = NULL;

			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				CORBA_free(eid);
				eid = NULL;
				goto next;
			}

			/*
			 * initiate the named property tags - BRUTUS_PS_PROPSET_TASK
			 */
			CORBA_free(named_prop_tags._buffer);
			named_prop_tags._maximum = 8;
			named_prop_tags._length = named_prop_tags._maximum;
			named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);

			// completion status
			named_prop_tags._buffer[0].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_COMPLETION_STATUS);
			named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[0].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_COMPLETION_STATUS);

			// percent complete
			named_prop_tags._buffer[1].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_PERCENT_COMPLETE);
			named_prop_tags._buffer[1].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[1].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_PERCENT_COMPLETE);

			// start date
			named_prop_tags._buffer[2].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_START);
			named_prop_tags._buffer[2].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[2].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_START);

			// due date
			named_prop_tags._buffer[3].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_DUE);
			named_prop_tags._buffer[3].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[3].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_DUE);

			// when was it completed?
			named_prop_tags._buffer[4].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_COMPLETED);
			named_prop_tags._buffer[4].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[4].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_COMPLETED);

			// actual effort (not currently used)
			named_prop_tags._buffer[5].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_ACTUAL_EFFORT);
			named_prop_tags._buffer[5].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[5].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_ACTUAL_EFFORT);

			// estimated effort (not currently used)
			named_prop_tags._buffer[6].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_ESTIMATED_EFFORT);
			named_prop_tags._buffer[6].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[6].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_ESTIMATED_EFFORT);

			// completed despite all other indications?
			named_prop_tags._buffer[7].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_IS_COMPLETE);
			named_prop_tags._buffer[7].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[7].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_IS_COMPLETE);

			br = BRUTUS_IMAPISession_OpenEntry(priv->mapi_session,
							   eid,
							   "",
							   BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
							   &obj_type,
							   &mapi_message,
							   ev);
			CORBA_free(eid);
			eid = NULL;
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto next;
			}
			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				goto next;
			}
			br = BRUTUS_IMessage_GetNamedProps(mapi_message,
							   BRUTUS_BRUTUS_PS_PROPSET_TASK,
							   &named_prop_tags,
							   0,
							   &named_prop_values,
							   ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto next;
			}
			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				goto next;
			}

			// get long-term ENTRYID
			brutus_sbinary_to_entryid(&prop_values->_buffer[0].Value._u.bin, &eid); // PR_ENTRYID
			if (!eid)
				goto next;

			msg_id = brutus_entryid_to_string(eid);
			CORBA_free(eid);
			eid = NULL;
			if (!msg_id)
				goto next;

			// create new/modified ECalComponent object
			ecal_comp_server = e_cal_component_new();
			if (!ecal_comp_server)
				goto next;
			e_cal_component_set_new_vtype(ecal_comp_server, E_CAL_COMPONENT_TODO);
			e_cal_component_set_uid(ecal_comp_server, msg_id);

			// does it exist in the cache?
			ecal_comp_cache = e_cal_backend_cache_get_component(priv->cache, msg_id, NULL);
			if (ecal_comp_cache)
				exists = TRUE;  // present in cache - modified event
			else
				exists = FALSE; // not in cache - new event

			// add PR_RECORD_KEY to ECalComponent object
			if (pr_record_key) {
				icalprop = icalproperty_new_x(pr_record_key);
				if (!icalprop)
					goto next;

				icalproperty_set_x_name(icalprop, X_PROP_RECORD_KEY);
				icalcomponent_add_property(e_cal_component_get_icalcomponent(ecal_comp_server), icalprop);
				free(pr_record_key);
				pr_record_key = NULL;
			}

			// MIME charset
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[7].ulPropTag)) // PR_INTERNET_CPID
				charset = brutus_cpid_to_label(prop_values->_buffer[7].Value._u.l);
			charset = charset ? e_iconv_charset_name(charset) : NULL;

			// add MIME charset to ECalComponent object
			if (charset) {
				tmp = g_strdup_printf("Content-Type: text/calendar; charset=%s", charset);
				if (!tmp)
					goto next;

				icalprop = icalproperty_new_x(tmp);
				if (!icalprop)
					goto next;

				icalproperty_set_x_name(icalprop, "X-LIC-MIMECONTENTTYPE");
				icalcomponent_add_property(e_cal_component_get_icalcomponent(ecal_comp_server), icalprop);
				g_free(tmp);
				tmp = NULL;
				charset = NULL;
			}

			// message flags - kept for possible future use
			n = brutus_get_srow_index(&row_set->_buffer[r], BRUTUS_BRUTUS_PR_MSG_STATUS); // required column in folder content tables
			if (-1 != n) {
				if (BRUTUS_BRUTUS_MSGSTATUS_ANSWERED & row_set->_buffer[r]._buffer[n].Value._u.l)
					;

				if (BRUTUS_BRUTUS_MSGSTATUS_DELMARKED & row_set->_buffer[r]._buffer[n].Value._u.l)
					;

				if (BRUTUS_BRUTUS_MSGSTATUS_DRAFT & row_set->_buffer[r]._buffer[n].Value._u.l)
					;

				if (BRUTUS_BRUTUS_MSGSTATUS_HIGHLIGHTED & row_set->_buffer[r]._buffer[n].Value._u.l)
					;
			}
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[2].ulPropTag)) { // PR_MESSAGE_FLAGS
				if (BRUTUS_BRUTUS_MSGFLAG_READ & prop_values->_buffer[2].Value._u.l)
					;

				if (BRUTUS_BRUTUS_MSGFLAG_HASATTACH & prop_values->_buffer[2].Value._u.l)
					has_attach = TRUE;
				else
					has_attach = FALSE;
			}

			// Subject:
			tmp = NULL;
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[3].ulPropTag)) // PR_SUBJECT
				tmp = prop_values->_buffer[3].Value._u.lpszA;
			if (tmp) {
				ecal_text.altrep = NULL;
				ecal_text.value = g_strdup(tmp);
				tmp = NULL;
				if (!ecal_text.value)
					goto rollback;
				e_cal_component_set_summary(ecal_comp_server, &ecal_text);
			}

			// Description:
			tmp = NULL;
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(ex_prop_values->_buffer[0].ulPropTag)) // PR_BODY
				tmp = ex_prop_values->_buffer[0].Value._u.lpszA;
			if (tmp) {
				ecal_text.altrep = NULL;
				ecal_text.value = g_strdup(tmp);
				tmp = NULL;
				if (!ecal_text.value)
					goto rollback;

				gs_list.data = &ecal_text;
				gs_list.next = NULL;

				e_cal_component_set_description_list(ecal_comp_server, &gs_list);
			}

			// creation date
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[4].ulPropTag)) { // PR_CREATION_TIME
				tmp_time = brutus_filetime_to_time_t(prop_values->_buffer[4].Value._u.ft);
				ical_time_utc = icaltime_from_timet_with_zone(tmp_time, 0, icaltimezone_get_utc_timezone());
				if (priv->default_timezone) {
					ical_time = icaltime_convert_to_zone(ical_time_utc, priv->default_timezone);
					icaltime_set_timezone(&ical_time, priv->default_timezone);
					e_cal_component_set_created(ecal_comp_server, &ical_time);
					e_cal_component_set_dtstamp(ecal_comp_server, &ical_time);
				} else {
					e_cal_component_set_created (ecal_comp_server, &ical_time_utc);
					e_cal_component_set_dtstamp (ecal_comp_server, &ical_time_utc);
				}
			}

			// completeness status
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[7].ulPropTag)) { // PR_OUTLOOK_TASK_IS_COMPLETE
				if (named_prop_values->_buffer[7].Value._u.b)
					completion_status = ICAL_STATUS_COMPLETED;
			}
			if ((ICAL_STATUS_COMPLETED != completion_status)
			    && (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[0].ulPropTag))) { // PR_OUTLOOK_TASK_COMPLETION_STATUS
				switch (named_prop_values->_buffer[0].Value._u.l) {
				case BRUTUS_BRUTUS_olTaskNotStarted :
					completion_status = ICAL_STATUS_NEEDSACTION;
					break;
				case BRUTUS_BRUTUS_olTaskInProgress :
					completion_status = ICAL_STATUS_INPROCESS;
					break;
				case BRUTUS_BRUTUS_olTaskComplete :
					completion_status = ICAL_STATUS_COMPLETED;
					break;
				case BRUTUS_BRUTUS_olTaskWaiting :
					completion_status = ICAL_STATUS_NEEDSACTION;
					break;
				case BRUTUS_BRUTUS_olTaskDeferred :
					completion_status = ICAL_STATUS_TENTATIVE;
					break;
				default :
					d("Unknown completion status");
					completion_status = ICAL_STATUS_NONE;
				}
			}
			e_cal_component_set_status(ecal_comp_server, completion_status);
			completion_status = ICAL_STATUS_NONE;

			// percent complete
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[1].ulPropTag)) { // PR_OUTLOOK_TASK_PERCENT_COMPLETE
				percent = (int)(100.0 * named_prop_values->_buffer[1].Value._u.dbl);
				if (percent < 0)
					percent = 0;
				if (percent > 100)
					percent = 100;
				e_cal_component_set_percent(ecal_comp_server, &percent);
			}

			// start time
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[2].ulPropTag)) { // PR_OUTLOOK_TASK_DATE_START
				tmp_time = brutus_filetime_to_time_t(named_prop_values->_buffer[2].Value._u.ft);
				ical_time_utc = icaltime_from_timet_with_zone(tmp_time, 0, icaltimezone_get_utc_timezone());

				ecal_dt.value = &ical_time_utc;
				ecal_dt.value->is_date = FALSE;
				ecal_dt.tzid = g_strdup("UTC");
				if (!ecal_dt.tzid)
					goto rollback;

				e_cal_component_set_dtstart(ecal_comp_server, &ecal_dt);
			}

			// due time
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[3].ulPropTag)) { // PR_OUTLOOK_TASK_DATE_DUE
				tmp_time = brutus_filetime_to_time_t(named_prop_values->_buffer[3].Value._u.ft);
				ical_time_utc = icaltime_from_timet_with_zone(tmp_time, 0, icaltimezone_get_utc_timezone());

				ecal_dt.value = &ical_time_utc;
				ecal_dt.value->is_date = FALSE;
				ecal_dt.tzid = g_strdup("UTC");
				if (!ecal_dt.tzid)
					goto rollback;

				e_cal_component_set_due(ecal_comp_server, &ecal_dt);
			}

			// when it was completed
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[4].ulPropTag)) { // PR_OUTLOOK_TASK_DATE_COMPLETED
				tmp_time = brutus_filetime_to_time_t(named_prop_values->_buffer[4].Value._u.ft);
				ical_time_utc = icaltime_from_timet_with_zone(tmp_time, 0, icaltimezone_get_utc_timezone());
				if (priv->default_timezone) {
					ical_time = icaltime_convert_to_zone(ical_time_utc, priv->default_timezone);
					icaltime_set_timezone(&ical_time, priv->default_timezone);
					e_cal_component_set_completed(ecal_comp_server, &ical_time);
				} else {
					e_cal_component_set_completed(ecal_comp_server, &ical_time_utc);
				}
			}

			// Classification
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[5].ulPropTag)) { // PR_SENSITIVITY
				switch (prop_values->_buffer[5].Value._u.l) {
				case BRUTUS_BRUTUS_SENSITIVITY_NONE :
					d("None");
					e_cal_component_set_classification(ecal_comp_server, E_CAL_COMPONENT_CLASS_PUBLIC);
					break;
				case BRUTUS_BRUTUS_SENSITIVITY_COMPANY_CONFIDENTIAL :
					d("Confidential");
					e_cal_component_set_classification(ecal_comp_server, E_CAL_COMPONENT_CLASS_CONFIDENTIAL);
					break;
				case BRUTUS_BRUTUS_SENSITIVITY_PRIVATE :
				case BRUTUS_BRUTUS_SENSITIVITY_PERSONAL :
					d("Private");
					e_cal_component_set_classification(ecal_comp_server, E_CAL_COMPONENT_CLASS_PRIVATE);
					break;
				default :
					d("Unknown classification");
					e_cal_component_set_classification(ecal_comp_server, E_CAL_COMPONENT_CLASS_UNKNOWN);
					break;
				}
			}

			// Priority
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[6].ulPropTag)) { // PR_IMPORTANCE
				switch (prop_values->_buffer[6].Value._u.l) {
				case BRUTUS_BRUTUS_IMPORTANCE_LOW :
					d("Low");
					priority = 7;
					e_cal_component_set_priority(ecal_comp_server, &priority);
					break;
				case BRUTUS_BRUTUS_IMPORTANCE_NORMAL :
					d("Normal");
					priority = 5;
					e_cal_component_set_priority(ecal_comp_server, &priority);
					break;
				case BRUTUS_BRUTUS_IMPORTANCE_HIGH :
					d("High");
					priority = 3;
					e_cal_component_set_priority(ecal_comp_server, &priority);
					break;
				default :
					d("Unknown priority");
					priority = 0;
					e_cal_component_set_priority(ecal_comp_server, &priority);
					break;
				}
			}

			/*
			 * initiate the named property tags - BRUTUS_PS_PROPSET_COMMON
			 */
			CORBA_free(named_prop_tags._buffer);
			named_prop_tags._maximum = 3;
			named_prop_tags._length = named_prop_tags._maximum;
			named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);
			if (!named_prop_tags._buffer)
				goto rollback;

			// alarm?
			named_prop_tags._buffer[0].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_SET);
			named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[0].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_SET);

			// when to go off as minutes before start time
			named_prop_tags._buffer[1].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE);
			named_prop_tags._buffer[1].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[1].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE);

			// when to go off at a specific time
			named_prop_tags._buffer[2].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_DATE);
			named_prop_tags._buffer[2].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[2].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_DATE);

			br = BRUTUS_IMessage_GetNamedProps(mapi_message,
							   BRUTUS_BRUTUS_PS_PROPSET_COMMON,
							   &named_prop_tags,
							   0,
							   &named_prop_values,
							   ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto rollback;
			}
			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				goto rollback;
			}

			// set alarm if present
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[0].ulPropTag)) { // PR_OUTLOOK_COMMON_REMINDER_SET
				if (named_prop_values->_buffer[0].Value._u.b) {
					ecal_alarm = e_cal_component_alarm_new();
					e_cal_component_alarm_set_action(ecal_alarm, E_CAL_COMPONENT_ALARM_DISPLAY);

					if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[1].ulPropTag)) { // PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE
						ecal_trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START;
						ecal_trigger.u.rel_duration = icaldurationtype_from_int(60*named_prop_values->_buffer[1].Value._u.l);

					} else if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[2].ulPropTag)) { // PR_OUTLOOK_COMMON_REMINDER_DATE
						ecal_trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE;
						tmp_time = brutus_filetime_to_time_t(named_prop_values->_buffer[2].Value._u.ft);
						ecal_trigger.u.abs_time = icaltime_from_timet_with_zone(tmp_time, 0, icaltimezone_get_utc_timezone());
					} else {
						d("No alarm time");
						goto rollback;
					}
					e_cal_component_alarm_set_trigger(ecal_alarm, ecal_trigger);
					e_cal_component_add_alarm(ecal_comp_server, ecal_alarm);
				}
			}

			/*
			 * initiate the named property tags - BRUTUS_PS_PUBLIC_STRINGS
			 */
			CORBA_free(named_prop_tags._buffer);
			named_prop_tags._maximum = 1;
			named_prop_tags._length = named_prop_tags._maximum;
			named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);
			if (!named_prop_tags._buffer)
				goto rollback;

			// Categories
			named_prop_tags._buffer[0].type = BRUTUS_BRUTUS_PT_MV_STRING8;
			named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_STRING;
			named_prop_tags._buffer[0].name._u.str_name = CORBA_string_dup(BRUTUS_BRUTUS_PR_MNID_STRING_KEYWORDS);

			br = BRUTUS_IMessage_GetNamedProps(mapi_message,
							   BRUTUS_BRUTUS_PS_PUBLIC_STRINGS,
							   &named_prop_tags,
							   0,
							   &named_prop_values,
							   ev);
			if (ORBIT2_EX(ev)) {
				PRINT_ORBIT2_EX(ev);
				goto rollback;
			}
			if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
				d(brutus_bresult_to_str(br));
				goto rollback;
			}

			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(named_prop_values->_buffer[0].ulPropTag)) { // Keywords/Categories
				for (n = 0; n < named_prop_values->_buffer[0].Value._u.MVszA._length; n++)
					gs_list_p = g_slist_append(gs_list_p, named_prop_values->_buffer[0].Value._u.MVszA._buffer[n]);

				if ((named_prop_values->_buffer[0].Value._u.MVszA._length) && (!gs_list_p))
					goto rollback;

				if (gs_list_p) {
					e_cal_component_set_categories_list(ecal_comp_server, gs_list_p);
					g_slist_free(gs_list_p);
				}
			}

			//
			// attachments
			//
			if (has_attach) { // one or more attachments present
				has_attach = FALSE;

				ecal_attachment_list = brutus_get_attachment_list(brutus_backend, mapi_message, msg_id);
				if (ecal_attachment_list)
					e_cal_component_set_attachment_list(ecal_comp_server, ecal_attachment_list);
				else
					goto rollback;
			}

			// all is well
			errors = FALSE;
			goto next;

		rollback:
			d("*************  ROLLBACK * ROLLBACK * ROLLBACK  **************");
			if (ecal_comp_server) {
				g_object_unref(ecal_comp_server);
				ecal_comp_server = NULL;
			}
			if (exists) {
				ecal_comp_str = e_cal_component_get_as_string(ecal_comp_cache);
#ifdef EDS_1_4
				e_cal_backend_notify_object_removed(E_CAL_BACKEND(brutus_backend), msg_id, ecal_comp_str, NULL);
				e_cal_backend_cache_remove_component(priv->cache, msg_id, NULL);
#else
				ecal_id = e_cal_component_get_id(ecal_comp_cache);

				e_cal_backend_notify_object_removed(E_CAL_BACKEND(brutus_backend), ecal_id, ecal_comp_str, NULL);
				e_cal_backend_cache_remove_component(priv->cache, msg_id, NULL);

				e_cal_component_free_id(ecal_id);
#endif
				g_free(ecal_comp_str);
			}
		next:
			brutus_release_object(BRUTUS_IMessage_tc, &mapi_message, CORBA_OBJECT_NIL, ev);
			PRINT_IF_ORBIT2_EX(ev);

			CORBA_free(named_prop_tags._buffer);
			named_prop_tags._maximum = 0;
			named_prop_tags._length = named_prop_tags._maximum;
			named_prop_tags._buffer = NULL;

			g_free(tmp);
			tmp = NULL;

			free(msg_id);
			msg_id = NULL;

			CORBA_free(eid);
			eid = NULL;

			CORBA_free(prop_values);
			prop_values = NULL;

			CORBA_free(ex_prop_values);
			ex_prop_values = NULL;

			CORBA_free(recp_row_set);
			recp_row_set = NULL;

			if (!errors) {
				// ensure that the components are fully consistent
				e_cal_component_commit_sequence(ecal_comp_server);
				if (exists)
					e_cal_component_commit_sequence(ecal_comp_cache);

				// update the cache
				e_cal_backend_cache_put_component(priv->cache, ecal_comp_server);

				// notify the frontend
				if (exists) {
					tmp = e_cal_component_get_as_string(ecal_comp_cache);
					ecal_comp_str = e_cal_component_get_as_string(ecal_comp_server);
					e_cal_backend_notify_object_modified(E_CAL_BACKEND(brutus_backend), (const char*)tmp, (const char*)ecal_comp_str);
					g_free(tmp);
					tmp = NULL;
				} else {
					ecal_comp_str = e_cal_component_get_as_string(ecal_comp_server);
					e_cal_backend_notify_object_created(E_CAL_BACKEND(brutus_backend), (const char*)ecal_comp_str);
				}
				g_free(ecal_comp_str);
				ecal_comp_str = NULL;
			}
			if (ecal_comp_server) {
				g_object_unref(ecal_comp_server);
				ecal_comp_server = NULL;
			}
			if (ecal_comp_cache) {
				g_object_unref(ecal_comp_cache);
				ecal_comp_cache = NULL;
			}
			
			processed++;
		}

		if (!row_set->_length)
			do_break = TRUE;

		CORBA_free(row_set);
		row_set = NULL;

		if (do_break)
			break;
	} while (TRUE);

	// from an early loop exit
	free(pr_record_key);
	pr_record_key = NULL;

	// cleanup for deleted or moved message objects
	expunge_events = g_ptr_array_new();
	if (!expunge_events)
		goto out;

	g_list = e_cal_backend_cache_get_components(priv->cache);
	for (g_list_iterator = g_list; g_list_iterator; g_list_iterator = g_list_next(g_list_iterator)) {
		exists = FALSE;

		ecal_comp_cache = E_CAL_COMPONENT(g_list_iterator->data);
		pr_record_key = (char*)brutus_get_x_property_val(X_PROP_RECORD_KEY, ecal_comp_cache);
		if (!pr_record_key)
			continue;

		for (r = 0; r < seen_events->len; r++) { // compare by pr_record_key
			if (!g_ascii_strcasecmp((gchar*)g_ptr_array_index(seen_events, r), pr_record_key)) {
				exists = TRUE;
				break;
			}
		}
		if (!exists) { // there is no remote corresponding message object for this info and it must be deleted by uid
			e_cal_component_get_uid(ecal_comp_cache, &uid);
			g_ptr_array_add(expunge_events, (gpointer)uid);
		}
	}
	for (r = 0; r < expunge_events->len; r++) {
		msg_id = (gchar*)g_ptr_array_index(expunge_events, r);
		ecal_comp_cache = e_cal_backend_cache_get_component(priv->cache, msg_id, NULL);
		ecal_comp_str = e_cal_component_get_as_string(ecal_comp_cache);
#ifdef EDS_1_4
		e_cal_backend_notify_object_removed(E_CAL_BACKEND(brutus_backend), msg_id, ecal_comp_str, NULL);
		e_cal_backend_cache_remove_component(priv->cache, msg_id, NULL);
#else
		ecal_id = e_cal_component_get_id(ecal_comp_cache);

		e_cal_backend_notify_object_removed(E_CAL_BACKEND(brutus_backend), ecal_id, ecal_comp_str, NULL);
		e_cal_backend_cache_remove_component(priv->cache, msg_id, NULL);

		e_cal_component_free_id(ecal_id);
#endif
		g_free(ecal_comp_str);
	}

	retv = GNOME_Evolution_Calendar_Success;

out:
	e_cal_backend_notify_view_progress(E_CAL_BACKEND(brutus_backend),
					   _("Fetching Exchange Tasks Information"),
					   100);
	e_file_cache_thaw_changes(E_FILE_CACHE(priv->cache));

	brutus_release_object(BRUTUS_IMAPITable_tc, &content_table, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	if (GNOME_Evolution_Calendar_Success != retv)
		e_cal_backend_notify_error(E_CAL_BACKEND(brutus_backend), _("Error updating tasks from server"));

	return retv;
}

static ECalBackendSyncStatus
brutus_populate_cache(ECalBackendBrutus *brutus_backend,
		      const gboolean from_scratch)
{
	ECalBackendSyncStatus retv;
	icalcomponent_kind kind;

	kind = e_cal_backend_get_kind(E_CAL_BACKEND(brutus_backend));
	switch (kind) {
	case ICAL_VEVENT_COMPONENT :
		d("populating EVENT cache");
		retv = brutus_populate_calendar_cache(brutus_backend, from_scratch);
		break;
	case ICAL_VTODO_COMPONENT :
		d("populating TODO cache");
		retv = brutus_populate_todos_cache(brutus_backend, from_scratch);
		break;
	default :
		d("");
		retv = GNOME_Evolution_Calendar_NoSuchCal;
		goto out;
	}

out:
	return retv;
}

struct poll_once_data {
	ECalBackendBrutus *brutus_backend;
	gboolean complete_poll;
};

static gboolean
brutus_poll_server_once(gpointer data)
{
	struct poll_once_data *input = (struct poll_once_data *)data;
	ECalBackendBrutus *brutus_backend = NULL;
	ECalBackendBrutusPrivate *priv = NULL;
	gboolean complete_poll = FALSE;

	d("");
	if (!data)
		return FALSE;

	complete_poll = input->complete_poll;

	brutus_backend = (ECalBackendBrutus*)input->brutus_backend;
	g_free(input);

	if (!brutus_backend)
		return FALSE;

	priv = brutus_backend->priv;
	if (!priv)
		return FALSE;

	g_mutex_lock(priv->poll_mutex);

	if (!brutus_is_connected(priv->brutus_server, priv->mapi_session))
		brutus_connect(brutus_backend); // reconnect
	if (!brutus_is_connected(priv->brutus_server, priv->mapi_session))
		goto out;

	brutus_populate_cache(brutus_backend, complete_poll);
out:
	g_mutex_unlock(priv->poll_mutex);
	return FALSE;
}

static gboolean
brutus_poll_server(gpointer data)
{
	gboolean retv = FALSE;
	ECalBackendBrutus *brutus_backend = NULL;
	ECalBackendBrutusPrivate *priv = NULL;

	d("");
	if (!data) {
		priv->timeout_id = 0;
		return FALSE; // stop polling
	}
	brutus_backend = (ECalBackendBrutus*)data;

	priv = brutus_backend->priv;
	if (!priv) {
		priv->timeout_id = 0;
		return FALSE; // stop polling
	}

	// do not execute more than one instance at any one time
	if (!g_mutex_trylock(priv->poll_mutex))
		return TRUE; // skip this poll - a poll is in progress or explicitly blocked

	if (!brutus_is_connected(priv->brutus_server, priv->mapi_session))
		brutus_connect(brutus_backend); // reconnect
	if (!brutus_is_connected(priv->brutus_server, priv->mapi_session))
		goto out; // stop polling

	brutus_populate_cache(brutus_backend, FALSE);
	retv = TRUE;
out:
	if (!retv)
		priv->timeout_id = 0;

	g_mutex_unlock(priv->poll_mutex);
	return retv;
}

static ECalBackendSyncStatus
brutus_activate_cache(ECalBackendBrutus *brutus_backend)
{
	ECalBackendSyncStatus retv = GNOME_Evolution_Calendar_OtherError;
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	const icalcomponent_kind kind = e_cal_backend_get_kind(E_CAL_BACKEND(brutus_backend));
	GList *cache_items = NULL;
	GList *cache_iterator = NULL;
	ECalComponent *ecal_comp = NULL;
	char *ecal_comp_string = NULL;
	struct poll_once_data *poll_data = NULL;


	/* check connection status */
	if (!brutus_is_connected(priv->brutus_server, priv->mapi_session))
		return GNOME_Evolution_Calendar_OfflineUnavailable;

	poll_data = g_new0(struct poll_once_data, 1);
	poll_data->brutus_backend = brutus_backend;

	if (!e_cal_backend_cache_get_marker(priv->cache)) {
		e_cal_backend_cache_set_marker(priv->cache);

		poll_data->complete_poll = TRUE;
		g_timeout_add(1, brutus_poll_server_once, (gpointer)poll_data);

		retv = GNOME_Evolution_Calendar_Success;
	} else {
		/* notify ecal about the objects already in cache */
		cache_items = e_cal_backend_cache_get_components(priv->cache);

		for (cache_iterator = cache_items; cache_iterator; cache_iterator = g_list_next(cache_iterator)) {
			ecal_comp = E_CAL_COMPONENT(cache_iterator->data);
			if (ecal_comp) {
				if (kind == icalcomponent_isa(e_cal_component_get_icalcomponent(ecal_comp))) {
					ecal_comp_string = e_cal_component_get_as_string(ecal_comp);
					if (ecal_comp_string) {
						e_cal_backend_notify_object_created(E_CAL_BACKEND(brutus_backend), ecal_comp_string);
						g_free(ecal_comp_string);
					}
				}
				g_object_unref(ecal_comp);
			}
		}
		if (cache_items)
			g_list_free(cache_items);

		/* get the deltas from the cache */
		poll_data->complete_poll = FALSE;
		g_timeout_add(1, brutus_poll_server_once, (gpointer)poll_data);

		retv = GNOME_Evolution_Calendar_Success;
	}

	// create server poll thread
	priv->timeout_id = g_timeout_add(1000*60*priv->auto_check_timeout, brutus_poll_server, (gpointer)brutus_backend);

	return retv;
}

/*******************************
 *   Calendar backend methods  *
 *******************************/

/* Dispose handler for the backend */
static void
e_cal_backend_brutus_dispose(GObject *object)
{
	d("");
	if (G_OBJECT_CLASS(parent_class)->dispose)
		(* G_OBJECT_CLASS(parent_class)->dispose)(object);
}

/* Finalize handler for the backend */
static void
e_cal_backend_brutus_finalize(GObject *object)
{
	ECalBackendBrutus *brutus_backend;
	ECalBackendBrutusPrivate *priv;

	d("");
	g_return_if_fail(object != NULL);
	g_return_if_fail(E_IS_CAL_BACKEND_BRUTUS(object));

	brutus_backend = E_CAL_BACKEND_BRUTUS(object);
	priv = brutus_backend->priv;


	/* Clean up */
	if (priv) {
		CORBA_boolean cb = CORBA_TRUE;
		CORBA_Environment ev[1];

		CORBA_exception_init(ev);

		g_mutex_lock(priv->poll_mutex); // to block brutus_poll_server()
		if (priv->mapi_profile)
			CORBA_free(priv->mapi_profile);

		if (priv->timeout_id) {
			g_source_remove(priv->timeout_id);
			priv->timeout_id = 0;
		}
		g_mutex_unlock(priv->poll_mutex);

		// free the MAPI session
		d("Shutting down MAPI");
		if (brutus_is_connected(priv->brutus_server, priv->mapi_session))
			brutus_mapi_shutdown(brutus_backend);

		// destroy the lifeline
		brutus_free_lifeline(priv->brutus_proxy,
				     &priv->lifeline);

		// destroy RootPOA
		cb = CORBA_Object_is_nil((CORBA_Object)priv->root_poa, ev);
		if (!ORBIT2_EX(ev) && !cb) {
			CORBA_Object_release((CORBA_Object)priv->root_poa, ev);
			PRINT_IF_ORBIT2_EX(ev);
		}

		// destroy ORB
		cb = CORBA_Object_is_nil((CORBA_Object)priv->orb, ev);
		if (!ORBIT2_EX(ev) && !cb) {
			CORBA_Object_release((CORBA_Object)priv->orb, ev);
			PRINT_IF_ORBIT2_EX(ev);
		}

		CORBA_exception_free(ev);
		d("CORBA is down");

		// mutexes
		g_mutex_free(priv->poll_mutex);
		g_mutex_free(priv->connect_mutex);

		g_free(priv->brutus_logon_str);

		if (priv->cache)
			g_object_unref(priv->cache);

		g_free(priv->email_address);

		g_free(priv->cache_dir);

		if (priv->brutus_server)
			g_free(priv->brutus_server);

		g_free(priv);
		brutus_backend->priv = NULL;
	}

	if (G_OBJECT_CLASS(parent_class)->finalize)
		(* G_OBJECT_CLASS(parent_class)->finalize)(object);
}

/* Object initialization function for the backend */
static void
e_cal_backend_brutus_init(ECalBackendBrutus *brutus_backend,
			  ECalBackendBrutusClass *class)
{
	ECalBackendBrutusPrivate *priv;
	const char *tmp = NULL;
	ESource *source = NULL;

	d("");
	priv = g_new0(ECalBackendBrutusPrivate, 1);
	if (!priv)
		goto out_no_mem;

	brutus_backend->priv = priv;

	/* create the mutex to controll poll blocking */
	priv->poll_mutex = g_mutex_new();
	if (!priv->poll_mutex)
		goto out_no_mem;

	/* create the mutex for thread safety in *_open_sync() */
	priv->connect_mutex = g_mutex_new();
	if (!priv->connect_mutex)
		goto out_no_mem;

	priv->constructed = FALSE;

	priv->read_only = FALSE;

	priv->mode = CAL_MODE_INVALID; // not yet open or even offline

	priv->orb = CORBA_OBJECT_NIL;
	priv->root_poa = CORBA_OBJECT_NIL;

#ifdef BRUTUS_HAVE_SSL
	priv->use_ssliop = TRUE;
#else
	priv->use_ssliop = FALSE;
#endif

	priv->mapi_profile = NULL;
	priv->mapi_session = CORBA_OBJECT_NIL;
	priv->instance_id = 0;

	priv->mapi_folder = CORBA_OBJECT_NIL;

	priv->brutus_proxy = CORBA_OBJECT_NIL;
	priv->lifeline.wrapper = CORBA_OBJECT_NIL;
	priv->lifeline.filling = CORBA_OBJECT_NIL;

	priv->exchange_time_zone = 0;
	priv->client_time_zone = 0;

	priv->auto_check_timeout = AUTO_CHECK_TIMEOUT_DEFAULT;
	source = e_cal_backend_get_source(E_CAL_BACKEND(brutus_backend));
	if (source) {
		tmp = e_source_get_property(source, "auto_check_timeout");
		if (tmp) {
			priv->auto_check_timeout = strtol(tmp, (char **)NULL, 10);
			if ((LONG_MIN == priv->auto_check_timeout)
			    || (LONG_MAX == priv->auto_check_timeout)
			    || (!priv->auto_check_timeout))
				priv->auto_check_timeout = AUTO_CHECK_TIMEOUT_DEFAULT;
		}
	}
	priv->timeout_id = 0;

	priv->brutus_logon_str = NULL;

	priv->email_address = NULL;

	priv->default_timezone = NULL;

	priv->cache_dir = NULL;

	priv->brutus_server = NULL;

	e_cal_backend_sync_set_lock(E_CAL_BACKEND_SYNC(brutus_backend), TRUE);

	d("");
	return;

out_no_mem:
	d("");
	e_cal_backend_notify_error(E_CAL_BACKEND(brutus_backend), _("No memory"));
	return;
}

/* Class initialization function for the Brutus backend */
static void
e_cal_backend_brutus_class_init(ECalBackendBrutusClass *class)
{
	GObjectClass *object_class = (GObjectClass*)class;
	ECalBackendClass *backend_class = (ECalBackendClass*)class;
	ECalBackendSyncClass *sync_class = (ECalBackendSyncClass*)class;

	d("");
	parent_class = g_type_class_peek_parent(class);

	object_class->dispose = e_cal_backend_brutus_dispose;
	object_class->finalize = e_cal_backend_brutus_finalize;

	sync_class->is_read_only_sync = e_cal_backend_brutus_is_read_only_sync;
	sync_class->get_cal_address_sync = e_cal_backend_brutus_get_cal_address_sync;
	sync_class->get_alarm_email_address_sync = e_cal_backend_brutus_get_alarm_email_address_sync;
	sync_class->get_ldap_attribute_sync = e_cal_backend_brutus_get_ldap_attribute_sync;
	sync_class->get_static_capabilities_sync = e_cal_backend_brutus_get_static_capabilities_sync;
	sync_class->open_sync = e_cal_backend_brutus_open_sync;
	sync_class->remove_sync = e_cal_backend_brutus_remove_sync;
	sync_class->create_object_sync = e_cal_backend_brutus_create_object_sync;
	sync_class->modify_object_sync = e_cal_backend_brutus_modify_object_sync;
	sync_class->remove_object_sync = e_cal_backend_brutus_remove_object_sync;
	sync_class->discard_alarm_sync = e_cal_backend_brutus_discard_alarm_sync;
	sync_class->receive_objects_sync = e_cal_backend_brutus_receive_objects_sync;
	sync_class->send_objects_sync = e_cal_backend_brutus_send_objects_sync;
	sync_class->get_default_object_sync = e_cal_backend_brutus_get_default_object_sync;
	sync_class->get_object_sync = e_cal_backend_brutus_get_object_sync;
	sync_class->get_object_list_sync = e_cal_backend_brutus_get_object_list_sync;
	sync_class->get_attachment_list_sync = e_cal_backend_brutus_get_attachment_list_sync;
	sync_class->get_timezone_sync = e_cal_backend_brutus_get_timezone_sync;
	sync_class->add_timezone_sync = e_cal_backend_brutus_add_timezone_sync;
	sync_class->set_default_timezone_sync = e_cal_backend_brutus_set_default_timezone_sync;
	sync_class->get_freebusy_sync = e_cal_backend_brutus_get_freebusy_sync;
	sync_class->get_changes_sync = e_cal_backend_brutus_get_changes_sync;

	backend_class->is_loaded = e_cal_backend_brutus_is_loaded_sync;
	backend_class->start_query = e_cal_backend_brutus_start_query_sync;
	backend_class->get_mode = e_cal_backend_brutus_get_mode_sync;
	backend_class->set_mode = e_cal_backend_brutus_set_mode_sync;
	backend_class->internal_get_default_timezone = e_cal_backend_brutus_internal_get_default_timezone_sync;
	backend_class->internal_get_timezone = e_cal_backend_brutus_internal_get_timezone_sync;
}

GType
e_cal_backend_brutus_get_type(void)
{
	static GType e_cal_backend_brutus_type = 0;

	if (!e_cal_backend_brutus_type) {
		static GTypeInfo info = {
			sizeof(ECalBackendBrutusClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) e_cal_backend_brutus_class_init,
			NULL, NULL,
			sizeof(ECalBackendBrutus),
			0,
			(GInstanceInitFunc) e_cal_backend_brutus_init
		};
		e_cal_backend_brutus_type = g_type_register_static(E_TYPE_CAL_BACKEND_SYNC,
								   "ECalBackendBrutus",
								   &info,
								   0);
	}

	return e_cal_backend_brutus_type;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_is_read_only_sync(ECalBackendSync *backend,
				       EDataCal *cal,
				       gboolean *read_only)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);

	*read_only = priv->read_only;

	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_get_cal_address_sync(ECalBackendSync *backend,
					  EDataCal *cal,
					  char **address)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	BRUTUS_IMsgStore msg_store = CORBA_OBJECT_NIL;
	BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_SPropValue *pv = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	CORBA_Environment ev[1];

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);

	if (priv->email_address) {
		*address = g_strdup(priv->email_address);
		return GNOME_Evolution_Calendar_Success;
	}
	*address = NULL;

	if (!brutus_is_connected(priv->brutus_server, priv->mapi_session))
		return GNOME_Evolution_Calendar_RepositoryOffline;

	CORBA_exception_init(ev);

	br = brutus_get_default_message_store(priv->mapi_session,
					      &msg_store,
					      ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}
	eid = brutus_get_entryid(msg_store, BRUTUS_BRUTUS_PR_USER_ENTRYID);

	pv = brutus_open_object_get_one_prop(priv->mapi_session,
					     CORBA_OBJECT_NIL,
					     eid,
					     BRUTUS_BRUTUS_PR_SMTP_ADDRESS);
	if (pv) {
		priv->email_address = g_strdup(pv->Value._u.lpszA);
		*address = g_strdup(priv->email_address);
	}
	d(priv->email_address);

out:
	brutus_release_object(BRUTUS_IMsgStore_tc, &msg_store, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	CORBA_free(pv);
	CORBA_free(eid);

	return GNOME_Evolution_Calendar_Success;
}

/*
 * TODO: The email alarm address could optionally be different
 * than the email address
 */
static ECalBackendSyncStatus
e_cal_backend_brutus_get_alarm_email_address_sync(ECalBackendSync *backend,
						  EDataCal *cal,
						  char **address)
{
	d("");
	return e_cal_backend_brutus_get_cal_address_sync(backend, cal, address);
}

static ECalBackendSyncStatus
e_cal_backend_brutus_get_ldap_attribute_sync(ECalBackendSync *backend,
					     EDataCal *cal,
					     char **attribute)
{
	d("");
	*attribute = NULL;

	return GNOME_Evolution_Calendar_Success;
}

/*
 * TODO: Revisit these capabilities, especially the alarm and recurrence support.
 */
static ECalBackendSyncStatus
e_cal_backend_brutus_get_static_capabilities_sync(ECalBackendSync *backend,
						  EDataCal *cal,
						  char **capabilities)
{
	d("");

	/* CAL_STATIC_CAPABILITY_NO_ALARM_REPEAT             - The backend does not support storing repeating alarms                                    */
	/* CAL_STATIC_CAPABILITY_NO_AUDIO_ALARMS             - The backend does not support storing audio alarms                                        */
	/* CAL_STATIC_CAPABILITY_NO_DISPLAY_ALARMS           - The backend does not support storing visual alarms                                       */
	/* CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS             - The backend does not support email alarms                                                */
	/* CAL_STATIC_CAPABILITY_NO_PROCEDURE_ALARMS         - The backend does not support alarms which will call other programs                       */
	/* CAL_STATIC_CAPABILITY_NO_TASK_ASSIGNMENT          - The backend does not support task assignment                                             */
	/* CAL_STATIC_CAPABILITY_NO_THISANDFUTURE            - The backend does not support modifying "this and future instances" of a recurrent event  */
	/* CAL_STATIC_CAPABILITY_NO_THISANDPRIOR             - The backend does not support modifying "this and earlier instances" of a recurrent event */
	/* CAL_STATIC_CAPABILITY_NO_TRANSPARENCY             - The backend does not support the event's transparency property                           */
	/* CAL_STATIC_CAPABILITY_ONE_ALARM_ONLY              - The backend does only support storing one alarm                                          */
	/* CAL_STATIC_CAPABILITY_ORGANIZER_MUST_ATTEND       - An event organizer must attend                                                           */
	/* CAL_STATIC_CAPABILITY_ORGANIZER_NOT_EMAIL_ADDRESS - The organizer field of an event is not an email address                                  */
	/* CAL_STATIC_CAPABILITY_REMOVE_ALARMS               - The backend support removing alarms                                                      */
	/* CAL_STATIC_CAPABILITY_SAVE_SCHEDULES              - Do not send meeting invitations or updates. The backend will do this by itself           */
	/* CAL_STATIC_CAPABILITY_NO_CONV_TO_ASSIGN_TASK      - A normal event can not be converted into an assigned task                                */
	/* CAL_STATIC_CAPABILITY_NO_CONV_TO_RECUR            - A normal event can not be converted into a recurring event                               */
	/* CAL_STATIC_CAPABILITY_NO_GEN_OPTIONS              - ?                                                                                        */
	/* CAL_STATIC_CAPABILITY_REQ_SEND_OPTIONS            - ?                                                                                        */
	/* CAL_STATIC_CAPABILITY_RECURRENCES_NO_MASTER       - ?                                                                                        */
	/* CAL_STATIC_CAPABILITY_ORGANIZER_MUST_ACCEPT       - The organizer must accept the event                                                      */
	/* CAL_STATIC_CAPABILITY_DELEGATE_SUPPORTED          - The backend supports delegation                                                          */
	/* CAL_STATIC_CAPABILITY_NO_ORGANIZER                - The backend supports events with no organizer                                            */
	/* CAL_STATIC_CAPABILITY_DELEGATE_TO_MANY            - The backend supports delegation to many                                                  */
	/* CAL_STATIC_CAPABILITY_HAS_UNACCEPTED_MEETING      - ?                                                                                        */

	*capabilities = g_strdup(CAL_STATIC_CAPABILITY_NO_ALARM_REPEAT ","
				 CAL_STATIC_CAPABILITY_NO_AUDIO_ALARMS ","
				 CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS ","
				 CAL_STATIC_CAPABILITY_NO_PROCEDURE_ALARMS ","
				 CAL_STATIC_CAPABILITY_NO_TASK_ASSIGNMENT ","
				 CAL_STATIC_CAPABILITY_NO_THISANDFUTURE ","
				 CAL_STATIC_CAPABILITY_NO_THISANDPRIOR ","
				 CAL_STATIC_CAPABILITY_ONE_ALARM_ONLY ","
				 CAL_STATIC_CAPABILITY_ORGANIZER_NOT_EMAIL_ADDRESS ","
				 CAL_STATIC_CAPABILITY_REMOVE_ALARMS ","
				 CAL_STATIC_CAPABILITY_SAVE_SCHEDULES ","
				 CAL_STATIC_CAPABILITY_NO_CONV_TO_ASSIGN_TASK ","
				 CAL_STATIC_CAPABILITY_NO_CONV_TO_RECUR ","
				 CAL_STATIC_CAPABILITY_REQ_SEND_OPTIONS ","
				 CAL_STATIC_CAPABILITY_DELEGATE_SUPPORTED ","
				 CAL_STATIC_CAPABILITY_HAS_UNACCEPTED_MEETING ","
		);
	if (!*capabilities)
		e_cal_backend_notify_error(E_CAL_BACKEND(backend), _("No memory"));

	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_open_sync(ECalBackendSync *backend,
			       EDataCal *cal,
			       gboolean only_if_exists,
			       const char *username,
			       const char *password)
{
	gboolean connected = FALSE;
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;

	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);

	// serialize open_sync()
	g_mutex_lock(priv->connect_mutex);

	d("");
	if (!priv->constructed)
		priv->constructed = brutus_backend_construct(brutus_backend);
	if (priv->constructed)
		connected = brutus_connect(brutus_backend);
	if (connected) {
		brutus_activate_cache(brutus_backend);
		priv->mode = CAL_MODE_REMOTE;
	}

	g_mutex_unlock(priv->connect_mutex);

	d(connected ? "SUCCESS: Backend has connected to Exchange server" : "FAILURE: Backend has NOT conencted to Exchange server");

	return (connected ? GNOME_Evolution_Calendar_Success : GNOME_Evolution_Calendar_OfflineUnavailable);
}

static ECalBackendSyncStatus
e_cal_backend_brutus_remove_sync(ECalBackendSync *backend,
				 EDataCal *cal)
{
	ECalBackendSyncStatus retv = GNOME_Evolution_Calendar_OtherError;
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);

	g_mutex_lock(priv->poll_mutex);
	if (priv->timeout_id) { // kill polling
		g_source_remove(priv->timeout_id);
		priv->timeout_id = 0;
	}
	g_mutex_unlock(priv->poll_mutex);
	brutus_mapi_shutdown(brutus_backend);

	if (priv->cache) {
		if (e_file_cache_remove(E_FILE_CACHE(priv->cache))) {
			priv->cache = NULL;
			retv = GNOME_Evolution_Calendar_Success;
		}
	} else
		retv = GNOME_Evolution_Calendar_Success;

	return retv;
}

static ECalBackendSyncStatus
brutus_modify_calendar_object(ECalBackendBrutus *brutus_backend,
			      ECalComponent *ecal_comp,
			      BRUTUS_IMessage mapi_message)
{
	ECalBackendSyncStatus retv = GNOME_Evolution_Calendar_OtherError;
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	ECalComponentDateTime ecal_dt;
	ECalComponentTransparency transp;
	ECalComponentAlarm *ecal_alarm = NULL;
	ECalComponentAlarmTrigger ecal_trigger;
	ECalComponentText ecal_text;
	ECalComponentClassification classification;
	int n;
	time_t tt = 0;
	const char *tmp = NULL;
	GList *alarm_list = NULL;
	GSList *category_list = NULL;
	GSList *category_list_p = NULL;
	GSList *description_list = NULL;
	gboolean all_day_event = FALSE;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_SPropProblemArray *mapi_problems = NULL;
	BRUTUS_seq_NamedPropertyTag named_prop_tags = { // named properties
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	BRUTUS_seq_SPropValue msg_prop_values = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	CORBA_Environment ev[1];

	d("");
	CORBA_exception_init(ev);

	// initiate the (un-)used problems array
	mapi_problems = BRUTUS_seq_SPropProblem__alloc();
	if (!mapi_problems)
		goto out;
	mapi_problems->_maximum = 0;
	mapi_problems->_length = 0;
	mapi_problems->_buffer = NULL;

	/*
	 * initiate the named property tags - BRUTUS_PS_PROPSET_APPOINTMENT
	 */
	named_prop_tags._maximum = 6;
	named_prop_tags._length = named_prop_tags._maximum;
	named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);
	if (!named_prop_tags._buffer)
		goto out;

	// initiate SPropValues
	msg_prop_values._maximum = named_prop_tags._maximum;
	msg_prop_values._length = msg_prop_values._maximum;
	msg_prop_values._buffer = BRUTUS_seq_SPropValue_allocbuf(msg_prop_values._maximum);
	if (!msg_prop_values._buffer)
		goto out;

	/*
	 * prepare data for transfer
	 */

	// start time
	named_prop_tags._buffer[0].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_START_DATE);
	named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[0].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_START_DATE);
	{
		msg_prop_values._buffer[0].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_START_DATE;
		msg_prop_values._buffer[0].Value._d = BRUTUS_BRUTUS_PT_SYSTIME;

		e_cal_component_get_dtstart(ecal_comp, &ecal_dt);
		tt = icaltime_as_timet_with_zone(*ecal_dt.value, NULL);
		brutus_adjust_to_server_time_zone(&tt, priv->exchange_time_zone);

		brutus_time_t_to_filetime(tt, &msg_prop_values._buffer[0].Value._u.ft);

	}
	all_day_event = ecal_dt.value->is_date ? TRUE : FALSE;

	// end time
	named_prop_tags._buffer[1].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_END_DATE);
	named_prop_tags._buffer[1].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[1].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_END_DATE);
	{
		msg_prop_values._buffer[1].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_END_DATE;
		msg_prop_values._buffer[1].Value._d = BRUTUS_BRUTUS_PT_SYSTIME;

		e_cal_component_get_dtend(ecal_comp, &ecal_dt);
		tt = icaltime_as_timet_with_zone(*ecal_dt.value, NULL);
		brutus_adjust_to_server_time_zone(&tt, priv->exchange_time_zone);

		brutus_time_t_to_filetime(tt, &msg_prop_values._buffer[1].Value._u.ft);
	}

	// location
	named_prop_tags._buffer[2].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_LOCATION);
	named_prop_tags._buffer[2].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[2].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_LOCATION);
	{
		msg_prop_values._buffer[2].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_LOCATION;
		msg_prop_values._buffer[2].Value._d = BRUTUS_BRUTUS_PT_TSTRING;

		e_cal_component_get_location(ecal_comp, &tmp);
		msg_prop_values._buffer[2].Value._u.lpszA = tmp ? CORBA_string_dup(tmp) : CORBA_string_dup("");
	}

	// show time as
	named_prop_tags._buffer[3].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_SHOW_TIME_AS);
	named_prop_tags._buffer[3].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[3].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_SHOW_TIME_AS);
	{
		msg_prop_values._buffer[3].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_SHOW_TIME_AS;
		msg_prop_values._buffer[3].Value._d = BRUTUS_BRUTUS_PT_LONG;

		e_cal_component_get_transparency(ecal_comp, &transp);
		switch (transp) {
		case E_CAL_COMPONENT_TRANSP_TRANSPARENT :
			msg_prop_values._buffer[3].Value._u.l = BRUTUS_BRUTUS_olFree;
			break;
		case E_CAL_COMPONENT_TRANSP_OPAQUE :
			msg_prop_values._buffer[3].Value._u.l = BRUTUS_BRUTUS_olBusy;
			break;
		case E_CAL_COMPONENT_TRANSP_UNKNOWN :
		default :
			msg_prop_values._buffer[3].Value._u.l = BRUTUS_BRUTUS_olTentative;
			break;
		}
	}

	// all-day event?
	named_prop_tags._buffer[4].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_ALL_DAY);
	named_prop_tags._buffer[4].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[4].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_ALL_DAY);
	{
		msg_prop_values._buffer[4].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_ALL_DAY;
		msg_prop_values._buffer[4].Value._d = BRUTUS_BRUTUS_PT_BOOLEAN;
		msg_prop_values._buffer[4].Value._u.b = all_day_event ? CORBA_TRUE : CORBA_FALSE;
	}

	// recurring?
	named_prop_tags._buffer[5].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_IS_RECURRING);
	named_prop_tags._buffer[5].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[5].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_IS_RECURRING);
	{
		msg_prop_values._buffer[5].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_EVENT_IS_RECURRING;
		msg_prop_values._buffer[5].Value._d = BRUTUS_BRUTUS_PT_BOOLEAN;
		msg_prop_values._buffer[5].Value._u.b = e_cal_component_is_instance(ecal_comp) ? CORBA_TRUE : CORBA_FALSE; // TODO: will always be false for now...
	}

	// set the props
	br = BRUTUS_IMessage_SetNamedProps(mapi_message,
					   BRUTUS_BRUTUS_PS_PROPSET_APPOINTMENT,
					   &named_prop_tags,
					   &msg_prop_values,
					   CORBA_FALSE,
					   &mapi_problems,
					   ev);
	brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}

	/*
	 * initiate the named property tags - BRUTUS_PS_PROPSET_COMMON
	 */
	CORBA_free(named_prop_tags._buffer);
	named_prop_tags._maximum = 4 + (e_cal_component_has_alarms(ecal_comp) ? 1 : 0);
	named_prop_tags._length = named_prop_tags._maximum;
	named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);
	if (!named_prop_tags._buffer)
		goto out;

	// initiate SPropValues
	msg_prop_values._maximum = named_prop_tags._maximum;
	msg_prop_values._length = msg_prop_values._maximum;
	msg_prop_values._buffer = BRUTUS_seq_SPropValue_allocbuf(msg_prop_values._maximum);
	if (!msg_prop_values._buffer)
		goto out;

	/*
	 * prepare data for transfer
	 */

	// Outlook build version
	named_prop_tags._buffer[0].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_INTERNAL_VERSION);
	named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[0].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_INTERNAL_VERSION);
	{
		msg_prop_values._buffer[0].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_INTERNAL_VERSION;
		msg_prop_values._buffer[0].Value._d = BRUTUS_BRUTUS_PT_LONG;
		msg_prop_values._buffer[0].Value._u.l = 110; // Emulating Outlook 2003
	}

	// Outlook textual version
	named_prop_tags._buffer[1].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_VERSION);
	named_prop_tags._buffer[1].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[1].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_VERSION);
	{
		msg_prop_values._buffer[1].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_INTERNAL_VERSION;
		msg_prop_values._buffer[1].Value._d = BRUTUS_BRUTUS_PT_TSTRING;
		msg_prop_values._buffer[1].Value._u.lpszA = CORBA_string_dup("11.0"); // Emulating Outlook 2003
	}

	// Outlook menu flags
	named_prop_tags._buffer[2].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_CONTEXT_MENU_FLAGS);
	named_prop_tags._buffer[2].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[2].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_CONTEXT_MENU_FLAGS);
	{
		msg_prop_values._buffer[2].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_CONTEXT_MENU_FLAGS;
		msg_prop_values._buffer[2].Value._d = BRUTUS_BRUTUS_PT_LONG;
		msg_prop_values._buffer[2].Value._u.l = 369; // Emulating Outlook 2003
	}

	// alarm?
	named_prop_tags._buffer[3].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_SET);
	named_prop_tags._buffer[3].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[3].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_SET);
	{
		msg_prop_values._buffer[3].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_SET;
		msg_prop_values._buffer[3].Value._d = BRUTUS_BRUTUS_PT_BOOLEAN;
		msg_prop_values._buffer[3].Value._u.b = e_cal_component_has_alarms(ecal_comp) ? CORBA_TRUE : CORBA_FALSE;
	}
	do {
		if (!msg_prop_values._buffer[3].Value._u.b)
			break;

		alarm_list = e_cal_component_get_alarm_uids(ecal_comp);
		if (!alarm_list)
			break;
		if (!alarm_list->data)
			break;

		// only use the first alarm
		ecal_alarm = e_cal_component_get_alarm(ecal_comp, (const char*)alarm_list->data);
		e_cal_component_alarm_get_trigger(ecal_alarm, &ecal_trigger);
		e_cal_component_alarm_free(ecal_alarm);

		switch (ecal_trigger.type) {
		case E_CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE :
			// when to go off at a specific time
			named_prop_tags._buffer[4].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_DATE);
			named_prop_tags._buffer[4].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[4].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_DATE);
			{
				msg_prop_values._buffer[4].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_DATE;
				msg_prop_values._buffer[4].Value._d = BRUTUS_BRUTUS_PT_SYSTIME;

				tt = icaltime_as_timet_with_zone(ecal_trigger.u.abs_time, icaltimezone_get_utc_timezone());
				brutus_adjust_to_server_time_zone(&tt, priv->exchange_time_zone);

				brutus_time_t_to_filetime(tt, &msg_prop_values._buffer[4].Value._u.ft);
			}
			break;
		default :
			// when to go off as minutes before start time
			named_prop_tags._buffer[4].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE);
			named_prop_tags._buffer[4].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[4].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE);
			{
				msg_prop_values._buffer[4].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE;
				msg_prop_values._buffer[4].Value._d = BRUTUS_BRUTUS_PT_LONG;

				msg_prop_values._buffer[4].Value._u.l = (CORBA_long) abs(icaldurationtype_as_int(ecal_trigger.u.rel_duration)/60);
			}
		}
	} while (0);
	if (alarm_list)
		cal_obj_uid_list_free(alarm_list);

	// set the props
	br = BRUTUS_IMessage_SetNamedProps(mapi_message,
					   BRUTUS_BRUTUS_PS_PROPSET_COMMON,
					   &named_prop_tags,
					   &msg_prop_values,
					   CORBA_FALSE,
					   &mapi_problems,
					   ev);
	brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}


	/*
	 * initiate the named property tags - BRUTUS_PS_PUBLIC_STRINGS
	 */

	e_cal_component_get_categories_list(ecal_comp, &category_list);
	if (category_list && category_list->data) {
		category_list_p = category_list;

		CORBA_free(named_prop_tags._buffer);
		named_prop_tags._maximum = 1;
		named_prop_tags._length = named_prop_tags._maximum;
		named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);
		if (!named_prop_tags._buffer)
			goto out;

		// initiate SPropValues
		msg_prop_values._maximum = named_prop_tags._maximum;
		msg_prop_values._length = msg_prop_values._maximum;
		msg_prop_values._buffer = BRUTUS_seq_SPropValue_allocbuf(msg_prop_values._maximum);
		if (!msg_prop_values._buffer)
			goto out;

		// Categories
		named_prop_tags._buffer[0].type = BRUTUS_BRUTUS_PT_MV_STRING8;
		named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_STRING;
		named_prop_tags._buffer[0].name._u.str_name = CORBA_string_dup(BRUTUS_BRUTUS_PR_MNID_STRING_KEYWORDS);
		{
			msg_prop_values._buffer[0].ulPropTag = BRUTUS_BRUTUS_PT_MV_STRING8; // the type is at least correct...
			msg_prop_values._buffer[0].Value._d = BRUTUS_BRUTUS_PT_MV_STRING8;

			msg_prop_values._buffer[0].Value._u.MVszA._maximum = g_slist_length(category_list_p);
			msg_prop_values._buffer[0].Value._u.MVszA._length = msg_prop_values._buffer[0].Value._u.MVszA._maximum;
			msg_prop_values._buffer[0].Value._u.MVszA._buffer = BRUTUS_SLPSTRArray_allocbuf(msg_prop_values._buffer[0].Value._u.MVszA._maximum);

			for (n = 0; n < msg_prop_values._buffer[0].Value._u.MVszA._maximum; n++){
				msg_prop_values._buffer[0].Value._u.MVszA._buffer[n] = CORBA_string_dup((const char*)category_list_p->data);
				category_list_p = category_list_p->next;
			}
		}
		dfmt("%p", msg_prop_values._buffer[0].Value._u.MVszA._buffer);
		dfmt("%p", msg_prop_values._buffer[0].Value._u.MVszA._buffer[0]);

		// set the props
		br = BRUTUS_IMessage_SetNamedProps(mapi_message,
						   BRUTUS_BRUTUS_PS_PUBLIC_STRINGS,
						   &named_prop_tags,
						   &msg_prop_values,
						   CORBA_FALSE,
						   &mapi_problems,
						   ev);
		brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			goto out;
		}
	}

	//
	// initiate normal properties
	//

	// initiate final msg prop sequence
	msg_prop_values._maximum = 11;
	msg_prop_values._length = msg_prop_values._maximum;
	msg_prop_values._buffer = BRUTUS_seq_SPropValue_allocbuf(msg_prop_values._maximum);
	if (!msg_prop_values._buffer)
		goto out;

	// message class
	msg_prop_values._buffer[0].ulPropTag = BRUTUS_BRUTUS_PR_MESSAGE_CLASS;
	msg_prop_values._buffer[0].Value._d = BRUTUS_BRUTUS_PT_STRING8;
	msg_prop_values._buffer[0].Value._u.lpszA = CORBA_string_dup("IPM.Appointment");

	// subject
	msg_prop_values._buffer[1].ulPropTag = BRUTUS_BRUTUS_PR_SUBJECT;
	msg_prop_values._buffer[1].Value._d = BRUTUS_BRUTUS_PT_STRING8;
	{
		ecal_text.value = NULL;
		e_cal_component_get_summary(ecal_comp, &ecal_text);
		msg_prop_values._buffer[1].Value._u.lpszA = ecal_text.value ? CORBA_string_dup(ecal_text.value) : CORBA_string_dup("");
	}

	// charset encoding
	msg_prop_values._buffer[2].ulPropTag = BRUTUS_BRUTUS_PR_INTERNET_CPID;
	msg_prop_values._buffer[2].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[2].Value._u.l = brutus_label_to_cpid("UTF-8"); // defaulting to UTF-8 - TODO: support other encodings

	// body text format
	msg_prop_values._buffer[3].ulPropTag = BRUTUS_BRUTUS_PR_MSG_EDITOR_FORMAT;
	msg_prop_values._buffer[3].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[3].Value._u.l = BRUTUS_BRUTUS_EDITOR_FORMAT_PLAINTEXT;

	// force MIME encoding
	msg_prop_values._buffer[4].ulPropTag = BRUTUS_BRUTUS_PR_INETMAIL_OVERRIDE_FORMAT;
	msg_prop_values._buffer[4].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[4].Value._u.l = BRUTUS_BRUTUS_ENCODEMIME;

	// body text
	msg_prop_values._buffer[5].ulPropTag = BRUTUS_BRUTUS_PR_BODY;
	msg_prop_values._buffer[5].Value._d = BRUTUS_BRUTUS_PT_STRING8;
	{
		ecal_text.value = NULL;
		e_cal_component_get_description_list(ecal_comp, &description_list);
		if (description_list && description_list->data)
			ecal_text.value = ((ECalComponentText*)(description_list->data))->value;

		msg_prop_values._buffer[5].Value._u.lpszA = ecal_text.value ? CORBA_string_dup(ecal_text.value) : CORBA_string_dup(" ");
		if (ecal_text.value)
			e_cal_component_free_text_list(description_list);
	}

	// sensitivity
	msg_prop_values._buffer[6].ulPropTag = BRUTUS_BRUTUS_PR_SENSITIVITY;
	msg_prop_values._buffer[6].Value._d = BRUTUS_BRUTUS_PT_LONG;
	{
		e_cal_component_get_classification (ecal_comp, &classification);
		switch (classification) {
		case E_CAL_COMPONENT_CLASS_NONE :
		case E_CAL_COMPONENT_CLASS_PUBLIC :
		case E_CAL_COMPONENT_CLASS_UNKNOWN :
			msg_prop_values._buffer[6].Value._u.l = BRUTUS_BRUTUS_SENSITIVITY_NONE;
			break;
		case E_CAL_COMPONENT_CLASS_PRIVATE :
			msg_prop_values._buffer[6].Value._u.l = BRUTUS_BRUTUS_SENSITIVITY_PRIVATE;
			break;
		case E_CAL_COMPONENT_CLASS_CONFIDENTIAL :
			msg_prop_values._buffer[6].Value._u.l = BRUTUS_BRUTUS_SENSITIVITY_COMPANY_CONFIDENTIAL;
			break;
		} // intentionally no default - let the compiler complain if more options are created
	}

	// start time
	msg_prop_values._buffer[7].ulPropTag = BRUTUS_BRUTUS_PR_START_DATE;
	msg_prop_values._buffer[7].Value._d = BRUTUS_BRUTUS_PT_SYSTIME;
	{
		e_cal_component_get_dtstart(ecal_comp, &ecal_dt);
		tt = icaltime_as_timet_with_zone(*ecal_dt.value, NULL);
		brutus_adjust_to_server_time_zone(&tt, priv->exchange_time_zone);

		brutus_time_t_to_filetime(tt, &msg_prop_values._buffer[7].Value._u.ft);
	}

	// end time
	msg_prop_values._buffer[8].ulPropTag = BRUTUS_BRUTUS_PR_END_DATE;
	msg_prop_values._buffer[8].Value._d = BRUTUS_BRUTUS_PT_SYSTIME;
	{
		e_cal_component_get_dtend(ecal_comp, &ecal_dt);
		tt = icaltime_as_timet_with_zone(*ecal_dt.value, NULL);
		brutus_adjust_to_server_time_zone(&tt, priv->exchange_time_zone);

		brutus_time_t_to_filetime(tt, &msg_prop_values._buffer[8].Value._u.ft);
	}

	// icon index for Outlook
	msg_prop_values._buffer[9].ulPropTag = BRUTUS_BRUTUS_PR_ICON_INDEX;
	msg_prop_values._buffer[9].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[9].Value._u.l = 1024;

	// messaging subsystem priority
	msg_prop_values._buffer[10].ulPropTag = BRUTUS_BRUTUS_PR_PRIORITY;
	msg_prop_values._buffer[10].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[10].Value._u.l = BRUTUS_BRUTUS_PRIO_NORMAL;

	br = BRUTUS_IMessage_SetProps(mapi_message,
				      &msg_prop_values,
				      FALSE,
				      &mapi_problems,
				      ev);
	brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}

	/*
	 * TODO: Attachments
	 */

	br = BRUTUS_IMessage_SaveChanges(mapi_message, BRUTUS_BRUTUS_KEEP_OPEN_READONLY, ev);
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}

	// all is well...
	retv = GNOME_Evolution_Calendar_Success;
out:
	CORBA_exception_free(ev);

	if (category_list)
		e_cal_component_free_categories_list(category_list);

	CORBA_free(mapi_problems);
	CORBA_free(named_prop_tags._buffer);
	brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);

	return retv;
}

static ECalBackendSyncStatus
brutus_modify_todo_object(ECalBackendBrutus *brutus_backend,
			  ECalComponent *ecal_comp,
			  BRUTUS_IMessage mapi_message)
{
	ECalBackendSyncStatus retv = GNOME_Evolution_Calendar_OtherError;
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	ECalComponentText ecal_text;
	ECalComponentDateTime ecal_dt_start;
	ECalComponentDateTime ecal_dt_due;
	ECalComponentAlarm *ecal_alarm = NULL;
	ECalComponentAlarmTrigger ecal_trigger;
	ECalComponentClassification classification;
	icalproperty_status completion_status;
	struct icaltimetype *ical_time_completed = NULL;
	int *percent = NULL;
	int n;
	time_t tt = 0;
	GList *alarm_list = NULL;
	GSList *category_list = NULL;
	GSList *category_list_p = NULL;
	GSList *description_list = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_SPropProblemArray *mapi_problems = NULL;
	BRUTUS_seq_NamedPropertyTag named_prop_tags = { // named properties
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	BRUTUS_seq_SPropValue msg_prop_values = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	CORBA_Environment ev[1];

	d("");
	CORBA_exception_init(ev);

	// initiate the (un-)used problems array
	mapi_problems = BRUTUS_seq_SPropProblem__alloc();
	if (!mapi_problems)
		goto out;
	mapi_problems->_maximum = 0;
	mapi_problems->_length = 0;
	mapi_problems->_buffer = NULL;

	/*
	 * initiate the named property tags - BRUTUS_PS_PROPSET_TASK
	 */
	e_cal_component_get_dtstart(ecal_comp, &ecal_dt_start);
	e_cal_component_get_due(ecal_comp, &ecal_dt_due);
	e_cal_component_get_completed(ecal_comp, &ical_time_completed);

	named_prop_tags._maximum = 3 + !!ecal_dt_start.value + !!ecal_dt_due.value + !!ical_time_completed;
	named_prop_tags._length = named_prop_tags._maximum;
	named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);
	if (!named_prop_tags._buffer)
		goto out;

	// initiate SPropValues
	msg_prop_values._maximum = named_prop_tags._maximum;
	msg_prop_values._length = msg_prop_values._maximum;
	msg_prop_values._buffer = BRUTUS_seq_SPropValue_allocbuf(msg_prop_values._maximum);
	if (!msg_prop_values._buffer)
		goto out;

	/*
	 * prepare data for transfer
	 */

	// status
	named_prop_tags._buffer[0].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_COMPLETION_STATUS);
	named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[0].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_COMPLETION_STATUS);
	{
		msg_prop_values._buffer[0].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_TASK_COMPLETION_STATUS;
		msg_prop_values._buffer[0].Value._d = BRUTUS_BRUTUS_PT_LONG;

		e_cal_component_get_status(ecal_comp, &completion_status);
		switch (completion_status) {
		case ICAL_STATUS_NONE :
			msg_prop_values._buffer[0].Value._u.l = BRUTUS_BRUTUS_olTaskNotStarted;
			break;
		case ICAL_STATUS_INPROCESS :
			msg_prop_values._buffer[0].Value._u.l = BRUTUS_BRUTUS_olTaskInProgress;
			break;
		case ICAL_STATUS_COMPLETED :
			msg_prop_values._buffer[0].Value._u.l = BRUTUS_BRUTUS_olTaskComplete;
			break;
		case ICAL_STATUS_NEEDSACTION :
			msg_prop_values._buffer[0].Value._u.l = BRUTUS_BRUTUS_olTaskWaiting;
			break;
		case ICAL_STATUS_TENTATIVE :
			msg_prop_values._buffer[0].Value._u.l = BRUTUS_BRUTUS_olTaskDeferred;
			break;
		default :
			d("Unknown completion status");
			msg_prop_values._buffer[0].Value._u.l = BRUTUS_BRUTUS_olTaskWaiting;
		}
	}

	// completed?
	named_prop_tags._buffer[1].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_IS_COMPLETE);
	named_prop_tags._buffer[1].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[1].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_IS_COMPLETE);
	{
		msg_prop_values._buffer[1].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_TASK_IS_COMPLETE;
		msg_prop_values._buffer[1].Value._d = BRUTUS_BRUTUS_PT_BOOLEAN;

		// depends on status
		if (BRUTUS_BRUTUS_olTaskComplete == msg_prop_values._buffer[0].Value._u.l)
			msg_prop_values._buffer[1].Value._u.b = CORBA_TRUE;
		else
			msg_prop_values._buffer[1].Value._u.b = CORBA_FALSE;
	}

	// percent complete
	named_prop_tags._buffer[2].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_PERCENT_COMPLETE);
	named_prop_tags._buffer[2].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[2].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_PERCENT_COMPLETE);
	{
		msg_prop_values._buffer[2].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_TASK_PERCENT_COMPLETE;
		msg_prop_values._buffer[2].Value._d = BRUTUS_BRUTUS_PT_DOUBLE;

		e_cal_component_get_percent(ecal_comp, &percent);
		if (percent)
			msg_prop_values._buffer[2].Value._u.dbl = (CORBA_double)(*percent/100.0);
		else
			msg_prop_values._buffer[2].Value._u.dbl = 0.0;
	}
	n = 2;

	// completion time
	if (ical_time_completed) {
		n++;
		named_prop_tags._buffer[n].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_COMPLETED);
		named_prop_tags._buffer[n].name._d = BRUTUS_BRUTUS_ID_INTEGER;
		named_prop_tags._buffer[n].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_COMPLETED);
		{
			msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_COMPLETED;
			msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_SYSTIME;

			tt = icaltime_as_timet_with_zone(*ical_time_completed, NULL);
			brutus_adjust_to_server_time_zone(&tt, priv->exchange_time_zone);

			brutus_time_t_to_filetime(tt, &msg_prop_values._buffer[n].Value._u.ft);
		}
	}

	// start time
	if (ecal_dt_start.value) {
		n++;
		named_prop_tags._buffer[n].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_START);
		named_prop_tags._buffer[n].name._d = BRUTUS_BRUTUS_ID_INTEGER;
		named_prop_tags._buffer[n].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_START);
		{
			msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_START;
			msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_SYSTIME;

			tt = icaltime_as_timet_with_zone(*ecal_dt_start.value, NULL);
			brutus_adjust_to_server_time_zone(&tt, priv->exchange_time_zone);

			brutus_time_t_to_filetime(tt, &msg_prop_values._buffer[n].Value._u.ft);

		}
	}

	// due time
	if (ecal_dt_due.value) {
		n++;
		named_prop_tags._buffer[n].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_DUE);
		named_prop_tags._buffer[n].name._d = BRUTUS_BRUTUS_ID_INTEGER;
		named_prop_tags._buffer[n].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_DUE);
		{
			msg_prop_values._buffer[n].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_TASK_DATE_DUE;
			msg_prop_values._buffer[n].Value._d = BRUTUS_BRUTUS_PT_SYSTIME;

			tt = icaltime_as_timet_with_zone(*ecal_dt_due.value, NULL);
			brutus_adjust_to_server_time_zone(&tt, priv->exchange_time_zone);

			brutus_time_t_to_filetime(tt, &msg_prop_values._buffer[n].Value._u.ft);
		}
	}

	// set the props
	br = BRUTUS_IMessage_SetNamedProps(mapi_message,
					   BRUTUS_BRUTUS_PS_PROPSET_TASK,
					   &named_prop_tags,
					   &msg_prop_values,
					   CORBA_FALSE,
					   &mapi_problems,
					   ev);
	brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}

	/*
	 * initiate the named property tags - BRUTUS_PS_PROPSET_COMMON
	 */
	CORBA_free(named_prop_tags._buffer);
	named_prop_tags._maximum = 4 + (e_cal_component_has_alarms(ecal_comp) ? 1 : 0);
	named_prop_tags._length = named_prop_tags._maximum;
	named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);
	if (!named_prop_tags._buffer)
		goto out;

	// initiate SPropValues
	msg_prop_values._maximum = named_prop_tags._maximum;
	msg_prop_values._length = msg_prop_values._maximum;
	msg_prop_values._buffer = BRUTUS_seq_SPropValue_allocbuf(msg_prop_values._maximum);
	if (!msg_prop_values._buffer)
		goto out;

	/*
	 * prepare data for transfer
	 */

	// Outlook build version
	named_prop_tags._buffer[0].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_INTERNAL_VERSION);
	named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[0].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_INTERNAL_VERSION);
	{
		msg_prop_values._buffer[0].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_INTERNAL_VERSION;
		msg_prop_values._buffer[0].Value._d = BRUTUS_BRUTUS_PT_LONG;
		msg_prop_values._buffer[0].Value._u.l = 110; // Emulating Outlook 2003
	}

	// Outlook textual version
	named_prop_tags._buffer[1].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_VERSION);
	named_prop_tags._buffer[1].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[1].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_VERSION);
	{
		msg_prop_values._buffer[1].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_INTERNAL_VERSION;
		msg_prop_values._buffer[1].Value._d = BRUTUS_BRUTUS_PT_TSTRING;
		msg_prop_values._buffer[1].Value._u.lpszA = CORBA_string_dup("11.0"); // Emulating Outlook 2003
	}

	// Outlook menu flags
	named_prop_tags._buffer[2].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_CONTEXT_MENU_FLAGS);
	named_prop_tags._buffer[2].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[2].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_CONTEXT_MENU_FLAGS);
	{
		msg_prop_values._buffer[2].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_CONTEXT_MENU_FLAGS;
		msg_prop_values._buffer[2].Value._d = BRUTUS_BRUTUS_PT_LONG;
		msg_prop_values._buffer[2].Value._u.l = 272; // Emulating Outlook 2003
	}

	// alarm?
	named_prop_tags._buffer[3].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_SET);
	named_prop_tags._buffer[3].name._d = BRUTUS_BRUTUS_ID_INTEGER;
	named_prop_tags._buffer[3].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_SET);
	{
		msg_prop_values._buffer[3].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_SET;
		msg_prop_values._buffer[3].Value._d = BRUTUS_BRUTUS_PT_BOOLEAN;
		msg_prop_values._buffer[3].Value._u.b = e_cal_component_has_alarms(ecal_comp) ? CORBA_TRUE : CORBA_FALSE;
	}
	do {
		if (!msg_prop_values._buffer[3].Value._u.b)
			break;

		alarm_list = e_cal_component_get_alarm_uids(ecal_comp);
		if (!alarm_list)
			break;
		if (!alarm_list->data)
			break;

		// only use the first alarm
		ecal_alarm = e_cal_component_get_alarm(ecal_comp, (const char*)alarm_list->data);
		e_cal_component_alarm_get_trigger(ecal_alarm, &ecal_trigger);
		e_cal_component_alarm_free(ecal_alarm);

		switch (ecal_trigger.type) {
		case E_CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE :
			// when to go off at a specific time
			named_prop_tags._buffer[4].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_DATE);
			named_prop_tags._buffer[4].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[4].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_DATE);
			{
				msg_prop_values._buffer[4].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_DATE;
				msg_prop_values._buffer[4].Value._d = BRUTUS_BRUTUS_PT_SYSTIME;

				tt = icaltime_as_timet_with_zone(ecal_trigger.u.abs_time, icaltimezone_get_utc_timezone());
				brutus_adjust_to_server_time_zone(&tt, priv->exchange_time_zone);

				brutus_time_t_to_filetime(tt, &msg_prop_values._buffer[4].Value._u.ft);
			}
			break;
		default :
			// when to go off as minutes before start time
			named_prop_tags._buffer[4].type = BRUTUS_PROP_TYPE(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE);
			named_prop_tags._buffer[4].name._d = BRUTUS_BRUTUS_ID_INTEGER;
			named_prop_tags._buffer[4].name._u.int_name = BRUTUS_PROP_ID(BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE);
			{
				msg_prop_values._buffer[4].ulPropTag = BRUTUS_BRUTUS_PR_OUTLOOK_COMMON_REMINDER_MINUTES_BEFORE;
				msg_prop_values._buffer[4].Value._d = BRUTUS_BRUTUS_PT_LONG;

				msg_prop_values._buffer[4].Value._u.l = (CORBA_long) abs(icaldurationtype_as_int(ecal_trigger.u.rel_duration)/60);
			}
		}
	} while (0);
	if (alarm_list)
		cal_obj_uid_list_free(alarm_list);

	// set the props
	br = BRUTUS_IMessage_SetNamedProps(mapi_message,
					   BRUTUS_BRUTUS_PS_PROPSET_COMMON,
					   &named_prop_tags,
					   &msg_prop_values,
					   CORBA_FALSE,
					   &mapi_problems,
					   ev);
	brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}


	/*
	 * initiate the named property tags - BRUTUS_PS_PUBLIC_STRINGS
	 */

	e_cal_component_get_categories_list(ecal_comp, &category_list);
	if (category_list && category_list->data) {
		category_list_p = category_list;

		CORBA_free(named_prop_tags._buffer);
		named_prop_tags._maximum = 1;
		named_prop_tags._length = named_prop_tags._maximum;
		named_prop_tags._buffer = BRUTUS_seq_NamedPropertyTag_allocbuf(named_prop_tags._maximum);
		if (!named_prop_tags._buffer)
			goto out;

		// initiate SPropValues
		msg_prop_values._maximum = named_prop_tags._maximum;
		msg_prop_values._length = msg_prop_values._maximum;
		msg_prop_values._buffer = BRUTUS_seq_SPropValue_allocbuf(msg_prop_values._maximum);
		if (!msg_prop_values._buffer)
			goto out;

		// Categories
		named_prop_tags._buffer[0].type = BRUTUS_BRUTUS_PT_MV_STRING8;
		named_prop_tags._buffer[0].name._d = BRUTUS_BRUTUS_ID_STRING;
		named_prop_tags._buffer[0].name._u.str_name = CORBA_string_dup(BRUTUS_BRUTUS_PR_MNID_STRING_KEYWORDS);
		{
			msg_prop_values._buffer[0].ulPropTag = BRUTUS_BRUTUS_PT_MV_STRING8; // the type is at least correct...
			msg_prop_values._buffer[0].Value._d = BRUTUS_BRUTUS_PT_MV_STRING8;

			msg_prop_values._buffer[0].Value._u.MVszA._maximum = g_slist_length(category_list_p);
			msg_prop_values._buffer[0].Value._u.MVszA._length = msg_prop_values._buffer[0].Value._u.MVszA._maximum;
			msg_prop_values._buffer[0].Value._u.MVszA._buffer = BRUTUS_SLPSTRArray_allocbuf(msg_prop_values._buffer[0].Value._u.MVszA._maximum);

			for (n = 0; n < msg_prop_values._buffer[0].Value._u.MVszA._maximum; n++){
				msg_prop_values._buffer[0].Value._u.MVszA._buffer[n] = CORBA_string_dup((const char*)category_list_p->data);
				category_list_p = category_list_p->next;
			}
		}
		dfmt("%p", msg_prop_values._buffer[0].Value._u.MVszA._buffer);
		dfmt("%p", msg_prop_values._buffer[0].Value._u.MVszA._buffer[0]);

		// set the props
		br = BRUTUS_IMessage_SetNamedProps(mapi_message,
						   BRUTUS_BRUTUS_PS_PUBLIC_STRINGS,
						   &named_prop_tags,
						   &msg_prop_values,
						   CORBA_FALSE,
						   &mapi_problems,
						   ev);
		brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			goto out;
		}
	}

	//
	// initiate normal properties
	//

	// initiate final msg prop sequence
	msg_prop_values._maximum = 9;
	msg_prop_values._length = msg_prop_values._maximum;
	msg_prop_values._buffer = BRUTUS_seq_SPropValue_allocbuf(msg_prop_values._maximum);
	if (!msg_prop_values._buffer)
		goto out;

	// message class
	msg_prop_values._buffer[0].ulPropTag = BRUTUS_BRUTUS_PR_MESSAGE_CLASS;
	msg_prop_values._buffer[0].Value._d = BRUTUS_BRUTUS_PT_STRING8;
	msg_prop_values._buffer[0].Value._u.lpszA = CORBA_string_dup("IPM.Task");

	// subject
	msg_prop_values._buffer[1].ulPropTag = BRUTUS_BRUTUS_PR_SUBJECT;
	msg_prop_values._buffer[1].Value._d = BRUTUS_BRUTUS_PT_STRING8;
	{
		ecal_text.value = NULL;
		e_cal_component_get_summary(ecal_comp, &ecal_text);
		msg_prop_values._buffer[1].Value._u.lpszA = ecal_text.value ? CORBA_string_dup(ecal_text.value) : CORBA_string_dup("");
	}

	// charset encoding
	msg_prop_values._buffer[2].ulPropTag = BRUTUS_BRUTUS_PR_INTERNET_CPID;
	msg_prop_values._buffer[2].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[2].Value._u.l = brutus_label_to_cpid("UTF-8"); // defaulting to UTF-8 - TODO: support other encodings

	// body text format
	msg_prop_values._buffer[3].ulPropTag = BRUTUS_BRUTUS_PR_MSG_EDITOR_FORMAT;
	msg_prop_values._buffer[3].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[3].Value._u.l = BRUTUS_BRUTUS_EDITOR_FORMAT_PLAINTEXT;

	// force MIME encoding
	msg_prop_values._buffer[4].ulPropTag = BRUTUS_BRUTUS_PR_INETMAIL_OVERRIDE_FORMAT;
	msg_prop_values._buffer[4].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[4].Value._u.l = BRUTUS_BRUTUS_ENCODEMIME;

	// body text
	msg_prop_values._buffer[5].ulPropTag = BRUTUS_BRUTUS_PR_BODY;
	msg_prop_values._buffer[5].Value._d = BRUTUS_BRUTUS_PT_STRING8;
	{
		ecal_text.value = NULL;
		e_cal_component_get_description_list(ecal_comp, &description_list);
		if (description_list && description_list->data)
			ecal_text.value = ((ECalComponentText*)(description_list->data))->value;

		msg_prop_values._buffer[5].Value._u.lpszA = ecal_text.value ? CORBA_string_dup(ecal_text.value) : CORBA_string_dup(" ");
		if (ecal_text.value)
			e_cal_component_free_text_list(description_list);
	}

	// sensitivity
	msg_prop_values._buffer[6].ulPropTag = BRUTUS_BRUTUS_PR_SENSITIVITY;
	msg_prop_values._buffer[6].Value._d = BRUTUS_BRUTUS_PT_LONG;
	{
		e_cal_component_get_classification (ecal_comp, &classification);
		switch (classification) {
		case E_CAL_COMPONENT_CLASS_NONE :
		case E_CAL_COMPONENT_CLASS_PUBLIC :
		case E_CAL_COMPONENT_CLASS_UNKNOWN :
			msg_prop_values._buffer[6].Value._u.l = BRUTUS_BRUTUS_SENSITIVITY_NONE;
			break;
		case E_CAL_COMPONENT_CLASS_PRIVATE :
			msg_prop_values._buffer[6].Value._u.l = BRUTUS_BRUTUS_SENSITIVITY_PRIVATE;
			break;
		case E_CAL_COMPONENT_CLASS_CONFIDENTIAL :
			msg_prop_values._buffer[6].Value._u.l = BRUTUS_BRUTUS_SENSITIVITY_COMPANY_CONFIDENTIAL;
			break;
		} // intentionally no default - let the compiler complain if more options are created
	}

	// icon index for Outlook
	msg_prop_values._buffer[7].ulPropTag = BRUTUS_BRUTUS_PR_ICON_INDEX;
	msg_prop_values._buffer[7].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[7].Value._u.l = 1280;

	// messaging subsystem priority
	msg_prop_values._buffer[8].ulPropTag = BRUTUS_BRUTUS_PR_PRIORITY;
	msg_prop_values._buffer[8].Value._d = BRUTUS_BRUTUS_PT_LONG;
	msg_prop_values._buffer[8].Value._u.l = BRUTUS_BRUTUS_PRIO_NORMAL;

	br = BRUTUS_IMessage_SetProps(mapi_message,
				      &msg_prop_values,
				      FALSE,
				      &mapi_problems,
				      ev);
	brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}

	/*
	 * TODO: Attachments
	 */

	br = BRUTUS_IMessage_SaveChanges(mapi_message, BRUTUS_BRUTUS_KEEP_OPEN_READONLY, ev);
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}

	// all is well...
	retv = GNOME_Evolution_Calendar_Success;
out:
	CORBA_exception_free(ev);

	if (category_list)
		e_cal_component_free_categories_list(category_list);

	CORBA_free(mapi_problems);
	CORBA_free(named_prop_tags._buffer);
	brutus_free_seq_spropvalue_content(&msg_prop_values, CORBA_OBJECT_NIL);

	return retv;
}

static ECalBackendSyncStatus
brutus_create_backend_object(ECalBackendBrutus *brutus_backend,
			     ECalComponent *ecal_comp,
			     char **uid)
{
	ECalBackendSyncStatus retv = GNOME_Evolution_Calendar_OtherError;
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	icalcomponent_kind kind;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_IMessage mapi_message = CORBA_OBJECT_NIL;
	BRUTUS_ENTRYID *eid = NULL;
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	*uid = NULL;

	// create event object
	br = BRUTUS_IMAPIFolder_CreateMessage(priv->mapi_folder,
					      "",
					      0,
					      &mapi_message,
					      ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if (BRUTUS_BRUTUS_S_OK != br) {
		d(brutus_bresult_to_str(br));
		goto out;
	}

	kind = e_cal_backend_get_kind(E_CAL_BACKEND(brutus_backend));
	switch (kind) {
	case ICAL_VEVENT_COMPONENT :
		retv = brutus_modify_calendar_object(brutus_backend,
						     ecal_comp,
						     mapi_message);
		break;
	case ICAL_VTODO_COMPONENT :
		retv = brutus_modify_todo_object(brutus_backend,
						 ecal_comp,
						 mapi_message);
		break;
	default :
		d("");
		retv = GNOME_Evolution_Calendar_OtherError;
	}

	// get long-term entry id
	eid = brutus_get_entryid(mapi_message, BRUTUS_BRUTUS_PR_ENTRYID);
	*uid = brutus_entryid_to_string(eid);
	CORBA_free(eid);
	e_cal_component_set_uid(ecal_comp, *uid);

out:
	brutus_release_object(BRUTUS_IMessage_tc, &mapi_message, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	return retv;
}

/*
 * Modified from the Groupwise backend
 */
static ECalBackendSyncStatus
e_cal_backend_brutus_create_object_sync(ECalBackendSync *backend,
					EDataCal *cal,
					char **calobj,
					char **uid)
{
	ECalBackendSyncStatus retv = GNOME_Evolution_Calendar_OtherError;
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	icalcomponent *icalcomp = NULL;
	ECalComponent *ecal_comp = NULL;
	const char *comp_uid;
	struct poll_once_data *poll_data = NULL;

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);
	g_return_val_if_fail((calobj && *calobj), GNOME_Evolution_Calendar_InvalidObject);

	*uid = NULL;

	if (!brutus_is_connected(priv->brutus_server, priv->mapi_session))
		return GNOME_Evolution_Calendar_RepositoryOffline;

	// check component validity
	icalcomp = icalparser_parse_string(*calobj);
	if (!icalcomp)
		return GNOME_Evolution_Calendar_InvalidObject;

	// check component type
	if (e_cal_backend_get_kind(E_CAL_BACKEND (backend)) != icalcomponent_isa(icalcomp)) {
		retv = GNOME_Evolution_Calendar_InvalidObject;
		goto out;
	}

	// check for duplicate
	comp_uid = icalcomponent_get_uid(icalcomp);
	if (comp_uid) {
		ecal_comp = e_cal_backend_cache_get_component(priv->cache, comp_uid, NULL);
		if (ecal_comp) {
			retv = GNOME_Evolution_Calendar_ObjectIdAlreadyExists;
			goto out;
		}
	}

	ecal_comp = e_cal_component_new();
	if (!ecal_comp)
		goto out;
	e_cal_component_set_icalcomponent(ecal_comp, icalcomp);
	icalcomp = NULL;

	retv = brutus_create_backend_object(brutus_backend, ecal_comp, uid);
	*calobj = e_cal_component_get_as_string(ecal_comp);

out:
	if (icalcomp)
		icalcomponent_free(icalcomp);

	if (ecal_comp)
		g_object_unref(ecal_comp);

	// update cache in the background
	if (GNOME_Evolution_Calendar_Success == retv) {
		poll_data = g_new0(struct poll_once_data, 1);

		poll_data->brutus_backend = brutus_backend;
		poll_data->complete_poll = FALSE;
		g_timeout_add(1, brutus_poll_server_once, (gpointer)poll_data);
	}

	return retv;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_modify_object_sync(ECalBackendSync *backend,
					EDataCal *cal,
					const char *calobj,
					CalObjModType mod,
					char **old_object,
					char **new_object)
{
	ECalBackendSyncStatus retv = GNOME_Evolution_Calendar_OtherError;
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	icalcomponent *icalcomp = NULL;
	ECalComponent *ecal_comp = NULL;
	ECalComponent *ecal_comp_cache = NULL;
	icalcomponent_kind kind;
	const char *comp_uid;
	struct poll_once_data *poll_data = NULL;
	BRUTUS_BDEFINE obj_type;
	BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_IMessage mapi_message = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];

	d("");
	CORBA_exception_init(ev);

	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);
	g_return_val_if_fail((calobj && *calobj), GNOME_Evolution_Calendar_InvalidObject);

	*old_object = NULL;
	*new_object = NULL;

	if (!brutus_is_connected(priv->brutus_server, priv->mapi_session))
		return GNOME_Evolution_Calendar_RepositoryOffline;

	// check component validity
	icalcomp = icalparser_parse_string(calobj);
	if (!icalcomp)
		return GNOME_Evolution_Calendar_InvalidObject;

	// check component type
	if (e_cal_backend_get_kind(E_CAL_BACKEND (backend)) != icalcomponent_isa(icalcomp)) {
		retv = GNOME_Evolution_Calendar_InvalidObject;
		goto out;
	}

	// get cached copy
	comp_uid = icalcomponent_get_uid(icalcomp);
	if (comp_uid) {
		ecal_comp_cache = e_cal_backend_cache_get_component(priv->cache, comp_uid, NULL);
		if (!ecal_comp_cache) {
			retv = GNOME_Evolution_Calendar_ObjectNotFound;
			goto out;
		}
	}

	// initiate modified object
	ecal_comp = e_cal_component_new();
	if (!ecal_comp)
		goto out;
	e_cal_component_set_icalcomponent(ecal_comp, icalcomp);
	icalcomp = NULL;

	// open the object on the server for modification
	eid = brutus_string_to_entryid(comp_uid);
	if (!eid)
		goto out;
	br = BRUTUS_IMAPISession_OpenEntry(priv->mapi_session,
					   eid,
					   "",
					   BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
					   &obj_type,
					   &mapi_message,
					   ev);
	CORBA_free(eid);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto out;
	}
	if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
		d(brutus_bresult_to_str(br));
		retv = GNOME_Evolution_Calendar_ObjectNotFound;
		goto out;
	}

	kind = e_cal_backend_get_kind(E_CAL_BACKEND(brutus_backend));
	switch (kind) {
	case ICAL_VEVENT_COMPONENT :
		d("modifying EVENT object");
		retv = brutus_modify_calendar_object(brutus_backend, ecal_comp, mapi_message);
		break;
	case ICAL_VTODO_COMPONENT :
		d("modifying TODO object");
		retv = brutus_modify_todo_object(brutus_backend, ecal_comp, mapi_message);
		break;
	default :
		d("");
		retv = GNOME_Evolution_Calendar_OtherError;
	}

	*old_object = e_cal_component_get_as_string(ecal_comp_cache);
	*new_object = e_cal_component_get_as_string(ecal_comp);

out:
	brutus_release_object(BRUTUS_IMessage_tc, &mapi_message, CORBA_OBJECT_NIL, ev);
	PRINT_IF_ORBIT2_EX(ev);

	CORBA_exception_free(ev);

	if (icalcomp)
		icalcomponent_free(icalcomp);

	if (ecal_comp)
		g_object_unref(ecal_comp);

	// update cache in the background
	if (GNOME_Evolution_Calendar_Success == retv) {
		poll_data = g_new0(struct poll_once_data, 1);

		poll_data->brutus_backend = brutus_backend;
		poll_data->complete_poll = FALSE;
		g_timeout_add(1, brutus_poll_server_once, (gpointer)poll_data);
	}

	return retv;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_remove_object_sync(ECalBackendSync *backend,
					EDataCal *cal,
					const char *uid,
					const char *rid,
					CalObjModType mod,
					char **old_object,
					char **object)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	ECalComponent *ecal_comp_cache = NULL;  // retrieved from the cache
	ECalBackendSyncStatus retv = GNOME_Evolution_Calendar_OtherError;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_seq_ENTRYID msg_id_list = {
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	CORBA_Environment ev[1];
#ifndef EDS_1_4
	ECalComponentId *ecal_id = NULL;
#endif

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);

	*object = NULL;
	*old_object = NULL;

	if (!brutus_is_connected(priv->brutus_server, priv->mapi_session))
		return GNOME_Evolution_Calendar_RepositoryOffline;

	if (!brutus_string_to_seq_entryid(uid, &msg_id_list)) {
		d("No memory or corrupted uid");
		return GNOME_Evolution_Calendar_OtherError;
	}

	CORBA_exception_init(ev);

	switch (mod) {
	case CALOBJ_MOD_ALL :  // all instances
	case CALOBJ_MOD_THIS : // this instance
		br = BRUTUS_IMAPIFolder_DeleteMessages(priv->mapi_folder,
						       &msg_id_list,
						       CORBA_OBJECT_NIL,
						       0,
						       ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			goto out;
		}
		if (BRUTUS_BRUTUS_S_OK != br) {
			d(brutus_bresult_to_str(br));
			goto out;
		}

		ecal_comp_cache = e_cal_backend_cache_get_component(priv->cache, uid, NULL);
		if (ecal_comp_cache) {
			*old_object = e_cal_component_get_as_string(ecal_comp_cache);
#ifdef EDS_1_4
			e_cal_backend_notify_object_removed(E_CAL_BACKEND(brutus_backend), uid, *old_object, NULL);
			e_cal_backend_cache_remove_component(priv->cache, uid, NULL);
#else
			ecal_id = e_cal_component_get_id(ecal_comp_cache);

			e_cal_backend_notify_object_removed(E_CAL_BACKEND(brutus_backend), ecal_id, *old_object, NULL);
			e_cal_backend_cache_remove_component(priv->cache, uid, NULL);

			e_cal_component_free_id(ecal_id);
#endif
		}
		break;
	case CALOBJ_MOD_THISANDPRIOR :  // this and prior instances
	case CALOBJ_MOD_THISANDFUTURE : // this and future instances
	default :
		retv = GNOME_Evolution_Calendar_UnsupportedMethod;
	}

out:
	CORBA_exception_free(ev);
	brutus_free_seq_entryid_content(&msg_id_list);

	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_discard_alarm_sync(ECalBackendSync *backend,
					EDataCal *cal,
					const char *uid,
					const char *auid)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);

	if (!brutus_is_connected(priv->brutus_server, priv->mapi_session))
		return GNOME_Evolution_Calendar_RepositoryOffline;

	return GNOME_Evolution_Calendar_OfflineUnavailable;
}

/*
 * From the HTTP backend
 */
static ECalBackendSyncStatus
e_cal_backend_brutus_receive_objects_sync(ECalBackendSync *backend,
					  EDataCal *cal,
					  const char *calobj)
{
	icalcomponent *icalcomp = NULL;

	d("");
	icalcomp = icalparser_parse_string(calobj);
	if (!icalcomp)
		return GNOME_Evolution_Calendar_InvalidObject;

	icalcomponent_free(icalcomp);

	return GNOME_Evolution_Calendar_PermissionDenied;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_send_objects_sync(ECalBackendSync *backend,
				       EDataCal *cal,
				       const char *calobj,
				       GList **users,
				       char **modified_calobj)
{
	icalcomponent *icalcomp = NULL;

	d("");
	*users = NULL;
	*modified_calobj = NULL;

	icalcomp = icalparser_parse_string(calobj);
	if (!icalcomp)
		return GNOME_Evolution_Calendar_InvalidObject;

	icalcomponent_free(icalcomp);

	return GNOME_Evolution_Calendar_PermissionDenied;
}

/*
 * From the HTTP backend
 */
static ECalBackendSyncStatus
e_cal_backend_brutus_get_default_object_sync(ECalBackendSync *backend,
					     EDataCal *cal,
					     char **object)
{
	icalcomponent *icalcomp = NULL;
	icalcomponent_kind kind;

	d("");
	*object = NULL;

	kind = e_cal_backend_get_kind(E_CAL_BACKEND(backend));
	icalcomp = e_cal_util_new_component(kind);

	*object = g_strdup(icalcomponent_as_ical_string(icalcomp));
	icalcomponent_free(icalcomp);

	return GNOME_Evolution_Calendar_Success;
}

/*
 * Modified from the Groupwise backend
 */
static ECalBackendSyncStatus
e_cal_backend_brutus_get_object_sync(ECalBackendSync *backend,
				     EDataCal *cal,
				     const char *uid,
				     const char *rid,
				     char **object)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	ECalComponent *ecal_comp = NULL;

	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);

	*object = NULL;

	if (!priv->cache)
		return GNOME_Evolution_Calendar_NoSuchCal;

	// search the object in the cache
	ecal_comp = e_cal_backend_cache_get_component(priv->cache, uid, rid);
	if (ecal_comp) {
		if (e_cal_backend_get_kind(E_CAL_BACKEND(backend)) == icalcomponent_isa(e_cal_component_get_icalcomponent(ecal_comp)))
			*object = e_cal_component_get_as_string(ecal_comp);
		g_object_unref(ecal_comp);
	}
	if (*object)
		return GNOME_Evolution_Calendar_Success;

	// callers will never have a uuid that is in the server but not in the cache
	return GNOME_Evolution_Calendar_ObjectNotFound;
}

/*
 * Modified from the HTTP backend
 */
static ECalBackendSyncStatus
e_cal_backend_brutus_get_object_list_sync(ECalBackendSync *backend,
					  EDataCal *cal,
					  const char *sexp,
					  GList **objects)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	GList *components = NULL;
	GList *l = NULL;
	ECalBackendSExp *cbsexp = NULL;

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);

	*objects = NULL;

	if (!priv->cache)
		return GNOME_Evolution_Calendar_NoSuchCal;

	/* process all components in the cache */
	cbsexp = e_cal_backend_sexp_new(sexp);
	if (!cbsexp)
		return GNOME_Evolution_Calendar_InvalidQuery;

	components = e_cal_backend_cache_get_components(priv->cache);
	for (l = components; l != NULL; l = l->next) {
		if (e_cal_backend_sexp_match_comp(cbsexp, E_CAL_COMPONENT(l->data), E_CAL_BACKEND(backend)))
			*objects = g_list_append(*objects, e_cal_component_get_as_string(l->data));
	}

	g_list_foreach(components, (GFunc) g_object_unref, NULL);
	g_list_free(components);
	g_object_unref(cbsexp);

	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_get_attachment_list_sync(ECalBackendSync *backend,
					      EDataCal *cal,
					      const char *uid,
					      const char *rid,
					      GSList **attachments)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	ECalComponent *ecal_comp_cache = NULL;

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);

	*attachments = NULL;

	ecal_comp_cache = e_cal_backend_cache_get_component(priv->cache, uid, NULL);
	if (!ecal_comp_cache)
		return GNOME_Evolution_Calendar_InvalidObject;

	e_cal_component_get_attachment_list(ecal_comp_cache, attachments);
	g_object_unref(ecal_comp_cache);

	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_get_timezone_sync(ECalBackendSync *backend,
				       EDataCal *cal,
				       const char *tzid,
				       char **object)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	const icaltimezone *zone;
	icalcomponent *icalcomp;

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);
	g_return_val_if_fail(tzid, GNOME_Evolution_Calendar_ObjectNotFound);

	/* first try to get the timezone from the cache */
	zone = e_cal_backend_cache_get_timezone(priv->cache, tzid);
	if (!zone) {
		zone = icaltimezone_get_builtin_timezone_from_tzid(tzid);
		if (!zone)
			return GNOME_Evolution_Calendar_ObjectNotFound;
	}

	icalcomp = icaltimezone_get_component((icaltimezone *)zone);
	if (!icalcomp)
		return GNOME_Evolution_Calendar_InvalidObject;

	*object = g_strdup(icalcomponent_as_ical_string(icalcomp));

	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_add_timezone_sync(ECalBackendSync *backend,
				       EDataCal *cal,
				       const char *tzobj)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	icalcomponent *tz_comp;
	icaltimezone *zone;

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);
	g_return_val_if_fail(tzobj, GNOME_Evolution_Calendar_OtherError);

	tz_comp = icalparser_parse_string (tzobj);
	if (!tz_comp)
		return GNOME_Evolution_Calendar_InvalidObject;

	if (icalcomponent_isa(tz_comp) != ICAL_VTIMEZONE_COMPONENT) {
		icalcomponent_free(tz_comp);
		return GNOME_Evolution_Calendar_InvalidObject;
	}

	zone = icaltimezone_new ();
	icaltimezone_set_component(zone, tz_comp);
	e_cal_backend_cache_put_timezone(priv->cache, zone);

	return GNOME_Evolution_Calendar_Success;
}

static ECalBackendSyncStatus
e_cal_backend_brutus_set_default_timezone_sync(ECalBackendSync *backend,
					       EDataCal *cal,
					       const char *tzid)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;

	d(tzid);
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);

	// set default timezone
	priv->default_timezone = icaltimezone_get_builtin_timezone_from_tzid(tzid);
	if (!priv->default_timezone)
		return GNOME_Evolution_Calendar_ObjectNotFound;

	return GNOME_Evolution_Calendar_Success;
}

typedef struct {
	ECalBackendBrutus *backend;
	icalcomponent_kind kind;
	GList *deletes;
	EXmlHash *ehash;
} ECalBackendBrutusComputeChangesData;

static void
e_cal_backend_brutus_compute_changes_foreach_key(const char *key,
						 const char *value,
						 gpointer data)
{
	ECalBackendBrutusComputeChangesData *be_data = data;
	ECalComponent *ecal_comp_cache = NULL;
	ECalComponent *ecal_comp = NULL;

	ecal_comp_cache = e_cal_backend_cache_get_component(be_data->backend->priv->cache, key, NULL);
	if (ecal_comp_cache) {
		g_object_unref(ecal_comp_cache);
		return;
	}

	ecal_comp = e_cal_component_new();
	if (ICAL_VTODO_COMPONENT == be_data->kind)
		e_cal_component_set_new_vtype(ecal_comp, E_CAL_COMPONENT_TODO);
	else
		e_cal_component_set_new_vtype(ecal_comp, E_CAL_COMPONENT_EVENT);

	e_cal_component_set_uid(ecal_comp, key);
	be_data->deletes = g_list_prepend(be_data->deletes, e_cal_component_get_as_string(ecal_comp));

	e_xmlhash_remove(be_data->ehash, key);
}

static ECalBackendSyncStatus
e_cal_backend_brutus_compute_changes(ECalBackendBrutus *brutus_backend,
				     const char *change_id,
				     GList **adds,
				     GList **modifies,
				     GList **deletes)
{
	ECalBackendSyncStatus status;
	ECalBackendCache *cache;
	char *filename;
	EXmlHash *ehash;
	ECalBackendBrutusComputeChangesData be_data;
	GList *i = NULL;
	GList *list = NULL;
	const char *uid = NULL;
	char *calobj = NULL;

	cache = brutus_backend->priv->cache;

	filename = g_strdup_printf("%s/changes-%s.db", brutus_backend->priv->cache_dir, change_id);
	ehash = e_xmlhash_new(filename);
	g_free(filename);

	status = e_cal_backend_brutus_get_object_list_sync(E_CAL_BACKEND_SYNC(brutus_backend), NULL, NULL, &list);
	if (status != GNOME_Evolution_Calendar_Success)
		return status;

	/* Calculate adds and modifies */
	for (i = list; i != NULL; i = g_list_next (i)) {
		e_cal_component_get_uid(i->data, &uid);
		calobj = e_cal_component_get_as_string(i->data);

		g_assert(calobj != NULL);

		/* check what type of change has occurred, if any */
		switch (e_xmlhash_compare(ehash, uid, calobj)) {
		case E_XMLHASH_STATUS_SAME:
			break;
		case E_XMLHASH_STATUS_NOT_FOUND:
			*adds = g_list_prepend(*adds, g_strdup(calobj));
			e_xmlhash_add(ehash, uid, calobj);
			break;
		case E_XMLHASH_STATUS_DIFFERENT:
			*modifies = g_list_prepend(*modifies, g_strdup(calobj));
			e_xmlhash_add(ehash, uid, calobj);
			break;
		}
		g_free(calobj);
		calobj = NULL;
	}
	g_list_foreach(list, (GFunc)g_free, NULL);
	g_list_free(list);

	/* Calculate deletions */
	be_data.backend = brutus_backend;
	be_data.kind = e_cal_backend_get_kind(E_CAL_BACKEND(brutus_backend));
	be_data.deletes = NULL;
	be_data.ehash = ehash;
	e_xmlhash_foreach_key(ehash, (EXmlHashFunc)e_cal_backend_brutus_compute_changes_foreach_key, &be_data);

	*deletes = be_data.deletes;

	e_xmlhash_write(ehash);
	e_xmlhash_destroy(ehash);

	return GNOME_Evolution_Calendar_Success;
}

/*
 * Modified from the Groupwise backend
 */
static ECalBackendSyncStatus
e_cal_backend_brutus_get_changes_sync(ECalBackendSync *backend,
				      EDataCal *cal,
				      const char *change_id,
				      GList **adds,
				      GList **modifies,
				      GList **deletes)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);
	g_return_val_if_fail(change_id, GNOME_Evolution_Calendar_ObjectNotFound);

	return e_cal_backend_brutus_compute_changes(brutus_backend, change_id, adds, modifies, deletes);
}

static icaltimezone *
brutus_resolve_tzid(const char *tzid,
		    gpointer user_data)
{
	icalcomponent *vcalendar_comp = user_data;

	if (!tzid || !tzid[0])
		return NULL;
	else if (!strcmp (tzid, "UTC"))
		return icaltimezone_get_utc_timezone ();

	return icalcomponent_get_timezone (vcalendar_comp, tzid);
}


static gboolean
brutus_free_busy_instance(ECalComponent *comp,
			  time_t instance_start,
			  time_t instance_end,
			  gpointer data)
{
	icalcomponent *vfb = data;
	icalproperty *prop;
	icalparameter *param;
	struct icalperiodtype ipt;
	icaltimezone *utc_zone;

	utc_zone = icaltimezone_get_utc_timezone();

	ipt.start = icaltime_from_timet_with_zone(instance_start, FALSE, utc_zone);
	ipt.end = icaltime_from_timet_with_zone(instance_end, FALSE, utc_zone);
	ipt.duration = icaldurationtype_null_duration();

	/* add busy information to the vfb component */
	prop = icalproperty_new(ICAL_FREEBUSY_PROPERTY);
	icalproperty_set_freebusy(prop, ipt);

	param = icalparameter_new_fbtype(ICAL_FBTYPE_BUSY);
	icalproperty_add_parameter(prop, param);

	icalcomponent_add_property(vfb, prop);

	return TRUE;
}

static icalcomponent *
brutus_create_user_free_busy(ECalBackendBrutus *brutus_backend,
			     const char *address,
			     const char *cn,
			     time_t start,
			     time_t end)
{
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	ECalComponent *ecal_comp = NULL;
	icalcomponent *icalcomp = NULL;
	icalcomponent *vcalendar_comp = NULL;
	icalcomponent *vfb = NULL;
	icalproperty *prop = NULL;
	icalparameter *param = NULL;
	icalproperty_transp transp_val;
	icaltimezone *utc_zone = NULL;
	ECalBackendSExp *obj_sexp = NULL;
	ECalBackendCache *cache = priv->cache;
	GList *list = NULL;
	GList *l = NULL;
	char *query = NULL;
	char *iso_start = NULL;
	char *iso_end = NULL;


	/* create the (unique) VFREEBUSY object that we'll return */
	vfb = icalcomponent_new_vfreebusy();
	if (address) {
		prop = icalproperty_new_organizer(address);
		if (prop && cn) {
			param = icalparameter_new_cn(cn);
			icalproperty_add_parameter(prop, param);
		}
		if (prop) {
			icalcomponent_add_property(vfb, prop);
			prop = NULL;
		}
	}
	utc_zone = icaltimezone_get_utc_timezone();
	icalcomponent_set_dtstart(vfb, icaltime_from_timet_with_zone(start, FALSE, utc_zone));
	icalcomponent_set_dtend(vfb, icaltime_from_timet_with_zone(end, FALSE, utc_zone));

	/* add all objects in the given interval */
	iso_start = isodate_from_time_t(start);
	iso_end = isodate_from_time_t(end);
	query = g_strdup_printf("occur-in-time-range? (make-time \"%s\") (make-time \"%s\")",
				iso_start, iso_end);
	obj_sexp = e_cal_backend_sexp_new(query);
	g_free(query);
	g_free(iso_start);
	g_free(iso_end);

	if (!obj_sexp)
		return vfb;

	list = e_cal_backend_cache_get_components(cache);
	for (l = list; l; l = l->next) {
		ecal_comp = l->data;

		icalcomp = e_cal_component_get_icalcomponent (ecal_comp);
		if (!icalcomp)
			continue;

		/* If the event is TRANSPARENT, skip it. */
		prop = icalcomponent_get_first_property(icalcomp,
							ICAL_TRANSP_PROPERTY);
		if (prop) {
			transp_val = icalproperty_get_transp (prop);
			if ((transp_val == ICAL_TRANSP_TRANSPARENT) ||
			    (transp_val == ICAL_TRANSP_TRANSPARENTNOCONFLICT))
				continue;
		}

		if (!e_cal_backend_sexp_match_comp(obj_sexp, l->data, E_CAL_BACKEND(brutus_backend)))
			continue;

		vcalendar_comp = icalcomponent_get_parent(icalcomp);
		if (!vcalendar_comp)
			vcalendar_comp = icalcomp;
		e_cal_recur_generate_instances (ecal_comp,
						start,
						end,
						brutus_free_busy_instance,
						vfb,
						brutus_resolve_tzid,
						vcalendar_comp,
						e_cal_backend_cache_get_default_timezone(cache));
	}
	g_object_unref(obj_sexp);

	g_list_foreach(list, (GFunc)g_object_unref, NULL);
	g_list_free(list);

	return vfb;
}

/*
 * Modified from the HTTP backend
 */
static ECalBackendSyncStatus
e_cal_backend_brutus_get_freebusy_sync(ECalBackendSync *backend,
				       EDataCal *cal,
				       GList *users,
				       time_t start,
				       time_t end,
				       GList **freebusy)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	gchar *address = NULL;
	gchar *name = NULL;
	icalcomponent *vfb = NULL;
	char *calobj = NULL;
	GList *l = NULL;

	d("");
	g_return_val_if_fail(E_IS_CAL_BACKEND_BRUTUS(brutus_backend), GNOME_Evolution_Calendar_InvalidObject);
	g_return_val_if_fail(((-1 != start) && (-1 != end)), GNOME_Evolution_Calendar_InvalidRange);
	g_return_val_if_fail ((start <= end), GNOME_Evolution_Calendar_InvalidRange);

	*freebusy = NULL;

	if (!priv->cache)
		return GNOME_Evolution_Calendar_NoSuchCal;

	if (!users) {
		if (e_cal_backend_mail_account_get_default(&address, &name)) {
			vfb = brutus_create_user_free_busy(brutus_backend, address, name, start, end);
			calobj = icalcomponent_as_ical_string(vfb);
			*freebusy = g_list_append(*freebusy, g_strdup(calobj));
			icalcomponent_free (vfb);
			g_free(address);
			g_free(name);
		}
	} else {
		for (l = users; l != NULL; l = l->next ) {
			address = l->data;
			if (e_cal_backend_mail_account_is_valid(address, &name)) {
				vfb = brutus_create_user_free_busy(brutus_backend, address, name, start, end);
				calobj = icalcomponent_as_ical_string(vfb);
				*freebusy = g_list_append(*freebusy, g_strdup(calobj));
				icalcomponent_free(vfb);
				g_free(name);
			}
		}
	}

	return GNOME_Evolution_Calendar_Success;
}

static gboolean
e_cal_backend_brutus_is_loaded_sync(ECalBackend *backend)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;

	return (priv->cache ? TRUE : FALSE);
}

/*
 * Modified from e-cal-backend-http.c
 */
static void
e_cal_backend_brutus_start_query_sync(ECalBackend *backend,
				      EDataCalView *query)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;
	GList *l = NULL;
	GList *components = NULL;
	GList *objects = NULL;
	ECalBackendSExp *cbsexp = NULL;

	d(e_data_cal_view_get_text(query));

	if (!priv->cache) {
		e_data_cal_view_notify_done(query, GNOME_Evolution_Calendar_NoSuchCal);
		return;
	}

	/* process all components in the cache */
	cbsexp = e_cal_backend_sexp_new(e_data_cal_view_get_text(query));

	objects = NULL;
	components = e_cal_backend_cache_get_components (priv->cache);
	for (l = components; l; l = l->next) {
		if (e_cal_backend_sexp_match_comp(cbsexp, E_CAL_COMPONENT (l->data), backend)) {
			objects = g_list_append(objects, e_cal_component_get_as_string(l->data));
		}
	}

	e_data_cal_view_notify_objects_added(query, (const GList *)objects);

	g_list_foreach(components, (GFunc)g_object_unref, NULL);
	g_list_free(components);
	g_list_foreach(objects, (GFunc)g_free, NULL);
	g_list_free(objects);
	g_object_unref(cbsexp);

	e_data_cal_view_notify_done(query, GNOME_Evolution_Calendar_Success);
}

static icaltimezone*
e_cal_backend_brutus_internal_get_default_timezone_sync(ECalBackend *backend)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;

	return priv->default_timezone;
}

static icaltimezone*
e_cal_backend_brutus_internal_get_timezone_sync(ECalBackend *backend,
						const char *tzid)
{
	icaltimezone *zone = NULL;

	return zone;
}

static CalMode
e_cal_backend_brutus_get_mode_sync(ECalBackend *backend)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;

	return priv->mode;
}

/*
 * Modified from the Groupwise backend
 */
static void
e_cal_backend_brutus_set_mode_sync(ECalBackend *backend,
				   CalMode mode)
{
	ECalBackendBrutus *brutus_backend = E_CAL_BACKEND_BRUTUS(backend);
	ECalBackendBrutusPrivate *priv = brutus_backend->priv;

	d("");
	if (priv->mode == mode) {
		e_cal_backend_notify_mode(backend,
					  GNOME_Evolution_Calendar_CalListener_MODE_SET,
					  cal_mode_to_corba(mode));
		return;
	}

	switch (mode) {
	case CAL_MODE_REMOTE :// go online
		d("");
		if (GNOME_Evolution_Calendar_Success != e_cal_backend_brutus_open_sync(E_CAL_BACKEND_SYNC(backend), NULL, TRUE, NULL, NULL))
			break;

		priv->read_only = FALSE;
		e_cal_backend_notify_readonly(backend, priv->read_only);

		priv->mode = CAL_MODE_REMOTE;
		e_cal_backend_notify_mode(backend,
					  GNOME_Evolution_Calendar_CalListener_MODE_SET,
					  GNOME_Evolution_Calendar_MODE_REMOTE);

		break;
	case CAL_MODE_LOCAL : // go offline
		d("");

		g_mutex_lock(priv->poll_mutex); // will prevent polling the server while going offline

		priv->mode = CAL_MODE_LOCAL;
		priv->read_only = TRUE;
		e_cal_backend_notify_readonly(backend, priv->read_only);

		if (priv->mapi_profile)
			CORBA_free(priv->mapi_profile);

		if (priv->timeout_id) {
			g_source_remove(priv->timeout_id);
			priv->timeout_id = 0;
		}

		if (brutus_is_connected(priv->brutus_server, priv->mapi_session))
			brutus_mapi_shutdown(brutus_backend);

		e_cal_backend_notify_mode(backend,
					  GNOME_Evolution_Calendar_CalListener_MODE_SET,
					  GNOME_Evolution_Calendar_MODE_LOCAL);

		g_mutex_unlock(priv->poll_mutex);
		break;
	default :
		d("");
		e_cal_backend_notify_mode(backend,
					  GNOME_Evolution_Calendar_CalListener_MODE_NOT_SUPPORTED,
					  cal_mode_to_corba(mode));
	}
}
